c              Last modification on March, 22, 2019 by MMM
C ======================================================================
C ========================== BHDECAY VERSION 2.00 ======================
C ======================================================================
C ======================================================================
C ======== SUBROUTINE BHDECAY FOR HIGGS DECAYS WITHIN THE NMSSM ========
C ======================================================================
C  This subroutine calculates the Higgs decay widths and branching 
C  ratios within the real NMSSM. It is part of the program package
C  NMSSMCALC. Its structure and the structure of the supplementary 
C  files is orientated along the files of the Fortran package HDECAY, 
C  which calculates SM and MSSM Higgs boson decays and branching 
C  ratios. The program will continuously be updated to include the 
C  newest developments. 
C ======================================================================
C ======================================================================
C
C                    ********************************
C                    * Based on HDECAY VERSION 6.10 *
C                    ********************************
C
C
C  This subroutine calculates the total decay widths and the branching 
C  ratios of the neutral and charged Higgs bosons of the 
C  Next-to-Minimal-Supersymmetric Extension of the Standard Model
C  (NMSSM) in the CP-conserving case.
C
C  Notation: 
C  H1, H2, H3 = CP-even Higgs bosons, ordered in ascending mass,
C               i.e. M_H1 < M_H2 < M_H3
C  A1, A2     = CP-odd Higgs bosons, ordered in ascending mass,
C               i.e. M_A1 < M_A2
C  H+, H-     = charged Higgs bosons
C  It includes:
C
C - All the decay channels which are kinematically allowed. 
C
C - All QCD corrections to the fermionic and gluonic decay modes.
C   Most of these corrections are mapped into running masses in a
C   consistent way with some freedom for including higher order terms. 
C
C - Below-threshold three-body decays with off-shell top quarks.
C
C - Double off-shell decays: H1,H2 --> W*W*,Z*Z* --> 4 fermions,
C   which could be important for Higgs masses close to MW or MZ.
C
C - The radiative corrections with full squark mixing.
C
C - All the decays into CHARGINOS, NEUTRALINOS, SLEPTONS 
C   and SQUARKS (with mixing in the stop and sbottom sectors). 
C
C - Chargino, slepton and squark loops in the 2 photon decays and squark
C   loops in the gluonic decays (including QCD corrections). 
C
C  ===================================================================
C  This program package has been written by J. Baglio, R. Grober, 
C  M. Muhlleitner, Dao Thi Nhung, H. Rzehak, M. Spira, J. Streicher and
C  K. Walz. For details on how to use the program package  
c  see:
C  Comput.Phys.Commun. 185 (2014) 12, 3372-3391. For any question,
C  comment, suggestion or complaint, please contact us at:
C
C                         nmssmcalc@itp.kit.edu
C
C ============================= INPUT FIlES ============================
C  The program package needs 2 input files: bhdecay.in and slha.in
C
C  The file slha.in must be in the SLHA format and inlcudes the NMSSM 
C  Higgs and SUSY particle masses and mixing angles, the soft SUSY 
C  breaking scalar masses and trilinear couplings, in particular also 
C  the NMSSM specific parameters lambda, kappa, Akappa, Alambda and 
C  mu_effective. The file is obtained by running CalcMasses.F which
C  provides the NMSSM Higgs boson masses at one-loop level in the 
C  CP-conserving and the CP-violating case. It can, however, also be 
C  created from any NMSSM spectrum calculator providing an output file 
C  in the SLHA format.
C
C  The homepage of the program package NMSSMCALC is given by:
C
C  http://www-itp.particle.uni-karlsruhe.de/~maggie/NMSSMCALC
C
C  The input file bhdecay.in is provided along with the NMSSMCALC 
C  package. It includes the following parameters:
C
C VTB:      CKM PARAMETER |V_TB|
C VTS:      CKM PARAMETER |V_TS|
C VTD:      CKM PARAMETER |V_TD|
C VCB:      CKM PARAMETER |V_CB|
C VCS:      CKM PARAMETER |V_CS|
C VCD:      CKM PARAMETER |V_CD|
C VUB:      CKM PARAMETER |V_UB|
C VUS:      CKM PARAMETER |V_US|
C VUD:      CKM PARAMETER |V_UD|
C NNLO (M): =0: USE O(ALPHA_S) FORMULA FOR POLE MASS --> MSBAR MASS
C           =1: USE O(ALPHA_S**2) FORMULA FOR POLE MASS --> MSBAR MASS
C
C ON-SHELL: =0: INCLUDE OFF_SHELL DECAYS H_i,A_j --> TT*, 
C               A_j --> Z*H_i,W-*H+, H_i --> Z*A_j,W-*H+, 
C               H+ --> W-*A_j, W*H_i, T*B, T*S, T*D
C           =1: EXCLUDE THE OFF-SHELL DECAYS ABOVE
C
C ON-SH-WZ: =0: INCLUDE DOUBLE OFF-SHELL PAIR DECAYS PHI --> W*W*,Z*Z*
C           =-1: INCLUDE DOUBLE OFF-SHELL PAIR DECAYS PHI --> W*W*,Z*Z*
C               BELOW THRESHOLD, BUT ON-SHELL PAIR DECAYS ABOVE
C           =1: INCLUDE ONLY SINGLE OFF-SHELL DECAYS PHI --> W*W,Z*Z
C                BELOW THRESHOLD, BUT ON-SHELL PAIR DECAYS ABOVE
C
C OFF-SUSY: =0: INCLUDE DECAYS (AND LOOPS) INTO SUPERSYMMETRIC PARTICLES
C           =1: EXCLUDE DECAYS (AND LOOPS) INTO SUPERSYMMETRIC PARTICLES
C
C NF-GG:    NUMBER OF LIGHT FLAVORS INCLUDED IN THE GLUONIC DECAYS 
C            PHI --> GG* --> GQQ (3,4 OR 5)
C
C ==================================================================== c
C ================ BEGINNING OF THE MAIN SUBROUTINE ================== c
C ==================================================================== c

      subroutine BHDECAY(input,output)
      implicit double precision (a-h,o-z)

      character*30 input,output

      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
c      COMMON/FLAGS_HDEC/indidec
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/DAVID/qsusy1,qsusy2,loop
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      print*,''
      print*,'**********************************************************
     .***********'
      print*,' 2/ Calculation of the Higgs decay widths and branching ra
     .tios'
      print*,' in the CP-conserving NMSSM.'
      print*,'**********************************************************
     .***********'

      call read_nmssm(tgbet,input)

      call hdec_nmssm(tgbet)

      call write_nmssm(tgbet,output)

 9999  continue

      end

c ==================================================================== c
c ========== reads in the input files bhdecay.in and slha.in ========= c
c ==================================================================== c

      subroutine read_nmssm(tgbet,input)
      implicit double precision (a-h,o-z)
      parameter(ni=87,ninlha=22)
      character*30 input
      double precision minval(1:20),smval(1:24),massval(1:100),
     .                 nmixval(5,5),umixval(2,2),vmixval(2,2),
     .                 stopmixval(2,2),sbotmixval(2,2),staumixval(2,2),
     .                 hmixval(1:10),gaugeval(1:3),msoftval(1:100),
     .                 auval(3,3),adval(3,3),aeval(3,3),yuval(3,3),
     .                 ydval(3,3),yeval(3,3),qvalue(1:20),extval(0:100),
     .                 m_softval(1:100)
      double precision nmhmixval(3,3),nmamixval(2,3),slhahmix(3,3),
     .     slhaamix(2,2),numixval(9,9),snsmixval(9,9),snpmixval(9,9)
     .     ,numassval(9),imnumixval(9,9)
      double precision slhaneut(1:5),slhaxneut(1:5),slhachar(1:2),
     .                 slhaxchar(1:2),
     .                 slhau(2,2),slhav(2,2),slhaz(5,5),
     .                 slhast(2),slhasb(2),slhasu(2),slhasd(2),
     .                 slhase(2),slhasl(2),slhasn(18),slhasnl(2),
     .                 warning(1:10)
      double precision vckmval(4)
      integer   imod(1:2)
      integer check(1:100)
      double precision mbmsbar,mbl,mbu
      character spinfo1*100,spinfo2*100,mincom(1:20)*20,
     .          extcom(0:100)*20,softcom(1:100)*20,hmixcom(1:10)*20,
     .          m_softcom(1:100)*20
      integer modselval(15)
      dimension gmn(5),xmn(5),gmc(2),gmst(2),gmsb(2),gmsl(2),
     .          gmsu(2),gmsd(2),gmse(2),gmsn(18),gmsnl(2),gmnu(9)
      dimension h1brsc(2,2),h1brsn(5,5),h2brsc(2,2),h2brsn(5,5),
     .     h3brsc(2,2),h3brsn(5,5),ha1brsc(2,2),ha1brsn(5,5),
     .     ha2brsc(2,2),ha2brsn(5,5),hcbrsu(2,5),h2brst(2,2),
     .     h2brsb(2,2),h3brst(2,2),h3brsb(2,2),hcbrstb(2,2) 
      dimension ac1(2,2),ac2(2,2),ac3(2,2),ac4(2,2),ac5(2,2),
     .          an1(5,5),an2(5,5),an3(5,5),an4(5,5),an5(5,5),
     .          acnl(2,5),acnr(2,5) 
c -------------- common block given by read_leshouches -------------- c
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval
      COMMON/SLHA_leshouches1_NMSSM/extcom,mincom,softcom,hmixcom
      COMMON/SLHA_leshouches2_NMSSM/minval,extval,smval,massval,
     .                   nmixval,umixval,vmixval,stopmixval,sbotmixval,
     .                   staumixval,hmixval,gaugeval,msoftval,auval,
     .                   adval,aeval,yuval,ydval,yeval,qvalue,imod,numixval
     .                   ,numassval,imnumixval
      COMMON/SLHA_leshouches3_NMSSM/nmhmixval,nmamixval
      COMMON/SLHA_leshouches5_NMSSM/snsmixval,snpmixval
      COMMON/SLHA_checkval_CNMSSM/check
      COMMON/SLHA_leshouches3_HDEC/vckmval
c -------------------------------------------------------------------- c
      COMMON/SD_scaleofewsb/scaleofewsb
c ------------ common blocks needed in BSMHDECAY subroutines --------- c
      COMMON/SLHA_m1_HDEC/am1
      COMMON/SLHA_gaug_NMSSM/slhaneut,slhaxneut,slhachar,slhau,slhav,
     .                      slhaz,slhaxchar
      COMMON/SLHA_sfer_HDEC/slhast,slhasb,slhasu,slhasd,slhase,slhasl,
     .                 slhasn,slhasnl,slhacot,slhasit,slhacob,slhasib,
     .                 slhacol,slhasil
      COMMON/SLHA_hmass_NMSSM/slhamh1,slhamh2,slhamh3,slhama1,slhama2,
     .     slhamc
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/SLHAVAL_HDEC/g1ew,g2ew
      COMMON/MASSES_HDEC/AMS,AMC,AMB,AMT
      COMMON/STRANGE_HDEC/AMSB
      COMMON/PARAM_HDEC/GF,ALPH,AMTAU,AMMUON,AMZ,AMW      
      COMMON/CKMPAR_HDEC/VTB,VTS,VTD,VCB,VCS,VCD,VUB,VUS,VUD
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
      COMMON/BREAKSCALE_HDEC/SUSYSCALE
      COMMON/BREAK_NMSSM/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AU,AD,Alambda,
     .     Akappa,AMU,AM2
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/BREAKGLU_HDEC/AMGLU
      COMMON/SFER1ST_HDEC/AMQL1,AMUR1,AMDR1,AMEL1,AMER1
      COMMON/GLUINO_NMSSM/amgluino,xmsb1,xmsb2,sthb,cthb,
     .     xh1bb(2,2),xh2bb(2,2),xh3bb(2,2),xha1bb(2,2),xha2bb(2,2),
     .     xmst1,xmst2,stht,ctht,xh1tt(2,2),xh2tt(2,2),xh3tt(2,2),
     .     xha1tt(2,2),xha2tt(2,2)
      COMMON/WZWDTH_HDEC/GAMC0,GAMT0,GAMT1,GAMW,GAMZ
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/ALS_HDEC/XLAMBDA,AMC0,AMB0,AMT0,N0
      COMMON/FLAG_HDEC/IHIGGS,NNLO
      COMMON/FERMIOPHOBIC_NMSSM/IFERMPHOB
      COMMON/MODEL_NMSSM/IMODEL
      COMMON/ONSHELL_NMSSM/IONSH,IONWZ,IOFSUSY
      COMMON/OLDFASH_NMSSM/NFGG
      COMMON/WIDTHA1_NMSSM/A1BRB,A1BRL,A1BRM,A1BRS,A1BRC,A1BRT,A1BRG,
     .     A1BRGA,A1BRZGA,A1BRZH1,A1BRZH2,A1BRZH3,A1WDTH
      COMMON/WIDTHA2_NMSSM/A2BRB,A2BRL,A2BRM,A2BRS,A2BRC,A2BRT,A2BRG,
     .     A2BRGA,A2BRZGA,A2BRZH1,A2BRZH2,A2BRZH3,A2BRA1H1,A2BRA1H2,
     .     A2BRA1H3,A2BRHCW,A2WDTH
      COMMON/WIDTHH1_NMSSM/H1BRB,H1BRL,H1BRM,H1BRS,H1BRC,H1BRT,H1BRG,
     .     H1BRGA,H1BRZGA,H1BRW,H1BRZ,H1BRA1A1,H1BRA1Z,H1BRA2Z,H1BRHW,
     .     H1BRA1A2,H1WDTH
      COMMON/WIDTHH2_NMSSM/H2BRB,H2BRL,H2BRM,H2BRS,H2BRC,H2BRT,H2BRG,
     .     H2BRGA,H2BRZGA,H2BRW,H2BRZ,H2BRH1H1,H2BRA1A1,H2BRA1A2,
     .     H2BRA1Z,H2BRA2Z,H2BRHW,H2WDTH
      COMMON/WIDTHH3_NMSSM/H3BRB,H3BRL,H3BRM,H3BRS,H3BRC,H3BRT,H3BRG,
     .     H3BRGA,H3BRZGA,H3BRW,H3BRZ,H3BRH1H1,H3BRH1H2,H3BRH2H2,
     .     H3BRA1A1,H3BRA1A2,H3BRA2A2,H3BRA1Z,H3BRA2Z,H3BRHW,H3WDTH
      COMMON/WIDTHHC_NMSSM/HCBRB,HCBRL,HCBRM,HCBRBU,HCBRS,HCBRC,HCBRT,
     .               HCBRWH1,HCBRWH2,HCBRWH3,HCBRA1,HCBRA2,hcbrd,hcbrdt,
     .               hcbrst,HCWDTH
      COMMON/WISUSY_NMSSM/H1BRSC,H1BRSN,H2BRSC,H2BRSN,H3BRSC,H3BRSN,
     .     HA1BRSC,HA1BRSN,HA2BRSC,HA2BRSN,HCBRSU,H1BRCHT,H2BRCHT,
     .     H3BRCHT,HA1BRCHT,HA2BRCHT,H1BRNET,H2BRNET,H3BRNET,HA1BRNET,
     .     HA2BRNET,HCBRCNT,H1BRSL,H2BRSL,H3BRSL,HCBRSL,HA1BRSL,HA2BRSL,
     .     HA1BRST,HA2BRST,HA1BRSB,HA2BRSB,H2BRSQ,H2BRST,H2BRSB,
     .     H2BRSQT,H3BRSQ,H3BRST,H3BRSB,H3BRSQT,HCBRSQ,HCBRSTB,HCBRSQT,
     .     H1BRSQ,H1BRSQT
      COMMON/WISFER_NMSSM/BH1SLNL,BH1SLEL,BH1SLER,BH1SQUL,BH1SQUR,
     .     BH1SQDL,BH1SQDR,BH1ST(2,2),BH1SB(2,2),BH1STAU(2,2),BH2SLNL,
     .     BH2SLEL,BH2SLER,BH2SQUL,BH2SQUR,BH2SQDL,BH2SQDR,BH2ST(2,2),
     .     BH2SB(2,2),BH2STAU(2,2),BH3SLNL,BH3SLEL,BH3SLER,BH3SQUL,
     .     BH3SQUR,BH3SQDL,BH3SQDR,BH3ST(2,2),BH3SB(2,2),BH3STAU(2,2),
     .     BHA1STAU,BHA1SB,BHA1ST,BHA2STAU,BHA2SB,BHA2ST,BHCSL00,
     .     BHCSL11,BHCSL21,BHCSQ,BHCSTB(2,2),BH1SLNLTAU,BH2SLNLTAU,
     .     BH3SLNLTAU
      COMMON/SMASS_NMSSM/GMN,XMN,GMC,GMST,GMSB,GMSL,GMSU,GMSD,GMSE,
     .     GMSN,GMSNL,GMNU
c      COMMON/FLAGS_HDEC/INDIDEC
      COMMON/SFMIXANG_NMSSM/sdthet,sdtheb,sdthel,cum,sum,cdm,sdm,cem,sem
      common/NMSSMGAUGINOCOUPL/anm(5,5,5),acnm(5,2,2),acnlnm(2,5),
     .     acnrnm(2,5)
      COMMON/VEV_NMSSM/vewsb
      COMMON/RUNCOUPTEST/iruncoup
      COMMON/M1andM2val/am1val,am2val
c ND added 7/2021
c--------------------connect to calcmasses ISS------------------------c
      logical isstrue
      common/issswitch/isstrue
c end ND added 7/2021

      unlikely = -123456789D0

      PI = 4*DATAN(1D0)

      OPEN(NI,FILE='bhdecay.in')

c      READ(NI,101)IHIGGS
c      READ(NI,100)ALSMZ
c      READ(NI,100)AMS
c      READ(NI,100)AMC
c      READ(NI,100)AMB
c      READ(NI,100)AMT
c      READ(NI,100)AMTAU
c      READ(NI,100)AMMUON
c      READ(NI,100)ALPH
c      READ(NI,100)GF
c      READ(NI,100)GAMW
c      READ(NI,100)GAMZ
c      READ(NI,100)AMZ
c      READ(NI,100)AMW
      READ(NI,100)VTB
      READ(NI,100)VTS
      READ(NI,100)VTD
      READ(NI,100)VCB
      READ(NI,100)VCS
      READ(NI,100)VCD
      READ(NI,100)VUB
      READ(NI,100)VUS
      READ(NI,100)VUD
      READ(NI,101)NNLO
      READ(NI,101)IONSH
      READ(NI,101)IONWZ
c      READ(NI,101)IPOLE
      READ(NI,101)IOFSUSY
c      READ(NI,101)INDIDEC
      READ(NI,101)NFGG
      iruncoup = 0

      ihiggs = 7
      gamw = 2.08856D0
      gamz = 2.49581D0
      alph = 137.0359997D0

c      cw2calc = amw**2/amz**2
c      sw2calc = 1-cw2calc
c      cwcalc  = dsqrt(cw2calc)
c      swcalc  = dsqrt(sw2calc)
c      vewsb = 1.D0/dsqrt(dsqrt(2.D0)*gf)
c      g2ew = 2*amw/vewsb
c      g1ew = g2ew*swcalc/cwcalc

c -- initialization of the check array --
      do i1=1,33,1
         check(i1) = 0
      end do

      open(ninlha,file=input)
      call SLHA_read_leshouches_NMSSM(ninlha)

c -- G_F --
         if(smval(2).ne.0.D0) then
            GF = smval(2)
         else
            print*,'Attention the value for G_F is not set. It has to be
     . given.'
         endif
c -- the strong coupling constant alphas_MSbar at the scale MZ --
         if(smval(3).ne.0.D0) then
            alsmz = smval(3)
         else
            print*,'Attention the value for alphas_MSbar at the scale MZ
     . is not set. It has to be given.'
         endif
         alphasmzms = alsmz
c -- Z pole mass --
         if(smval(4).ne.0.D0) then
            AMZ = smval(4)
         else
            print*,'Attention the value for MZ is not set. It has to be 
     .given.'
         endif
c -- W pole mass --
         if(massval(1).ne.0.D0) then
            AMW = massval(1)
         else
            print*,'Attention the value for MW is not set. It has to be 
     .given.'
         endif

c -- the weak mixing angle -- 
      cw2calc = amw**2/amz**2
      sw2calc = 1-cw2calc
      cwcalc  = dsqrt(cw2calc)
      swcalc  = dsqrt(sw2calc)
      vewsb = 1.D0/dsqrt(dsqrt(2.D0)*gf)
      g2ew = 2*amw/vewsb
      g1ew = g2ew*swcalc/cwcalc

c -- the MSbar couplings g1,g2 at the scale Q --
         if(gaugeval(1).ne.0.D0) then
            g1ew  = gaugeval(1)
         endif
         if(gaugeval(2).ne.0.D0) then
            g2ew  = gaugeval(2)*(1-gaugeval(2)**2/96/pi**2*2)
         endif

c         cw2calc = amw**2/amz**2
c         sw2calc = 1-cw2calc
c         cwcalc  = dsqrt(cw2calc)
c         swcalc  = dsqrt(sw2calc)
c -- in case the gauge couplings are not given at the scale Q --
         if(gaugeval(1).eq.0.D0.or.gaugeval(2).eq.0.D0) then
c -- v at the scale Q --         
          if(smval(2).eq.0.D0.and.hmixval(3).ne.unlikely) then
           vewsb = hmixval(3)
           gf = 1/dsqrt(2.D0)/vewsb**2
          else
           vewsb = 1.D0/dsqrt(dsqrt(2.D0)*gf)
          endif
          g2ew = 2*amw/vewsb
          g1ew = g2ew*swcalc/cwcalc
         endif
c -- the scale Q at which the couplings are given --
         if(extval(0).ne.unlikely.and.extval(0).ne.-1.D0) then
            scaleofewsb = extval(0)
            ewsbscale = scaleofewsb
         else
            qvalsum = 0.D0
            isum    = 0
            do i=1,20,1
               qvalsum = qvalsum + qvalue(i)
               if(qvalue(i).ne.0.D0) then
                  isum = isum + 1
               endif
            end do

            if(isum.ne.0) then
               scaleofewsb = qvalsum/dble(isum)
            else
               warning(1) = 1.D0
               print*,'The scaleofewsb and hence also susyscale are not 
     .given.'
            endif
            ewsbscale   = scaleofewsb
         endif
         SUSYSCALE = scaleofewsb

c -- CKM mixing matrix --

      if(vckmval(1).ne.unlikely) then
         vus = vckmval(1)
         vud = 1.D0 - vckmval(1)**2/2.D0
         vcd = dabs(vckmval(1))
         vcs = 1.D0 - vckmval(1)**2/2.D0
         if(vckmval(2).ne.unlikely) then
            vcb = vckmval(1)**2*vckmval(2)
            vts = vckmval(1)**2*vckmval(2)
            if(vckmval(3).ne.unlikely.and.vckmval(4).ne.unlikely) then
               vub = vckmval(1)**3*vckmval(2)*
     .              dsqrt(vckmval(3)**2+vckmval(4)**2)
               vtd = vckmval(1)**3*vckmval(2)*
     .              dsqrt((1.D0-vckmval(3))**2+vckmval(4)**2) 
               vtb = dsqrt(1.D0-vts**2-vtd**2)
            endif
         endif
         rvub= vub/vcb
      endif

c -- neutralino and chargino masses --      

         slhaneut(1) =dabs(massval(28))
         slhaneut(2) =dabs(massval(29))
         slhaneut(3) =dabs(massval(30))
         slhaneut(4) =dabs(massval(31))
         slhaneut(5) =dabs(massval(36))
         slhaxneut(1)=massval(28)
         slhaxneut(2)=massval(29)
         slhaxneut(3)=massval(30)
         slhaxneut(4)=massval(31)
         slhaxneut(5)=massval(36)
         slhachar(1) =dabs(massval(32))
         slhachar(2) =dabs(massval(33))
         slhaxchar(1) =massval(32)
         slhaxchar(2) =massval(33)

c -- the chargino and neutralino mixing matrix elements --
      do i=1,2,1
         do j=1,2,1
            slhau(i,j)=umixval(i,j)
            slhav(i,j)=vmixval(i,j)
         end do
      end do
      do i=1,5,1
         do j=1,5,1
            slhaz(i,j)=nmixval(i,j)
         end do
      end do

c -- sfermion masses --

      slhast(1) = massval(16)
      slhast(2) = massval(17)
      slhasb(1) = massval(14)
      slhasb(2) = massval(15)

      slhasu(1) = massval(8)
      slhasu(2) = massval(9)
      slhasd(1) = massval(10)
      slhasd(2) = massval(11)

      slhase(1) = massval(18)
      slhase(2) = massval(19)
      slhasl(1) = massval(24)
      slhasl(2) = massval(25)

      slhasn(1) = massval(20)
      slhasn(2) = 1.D15
      slhasnl(1) = massval(26)
      slhasnl(2) = 1.D15
c ND added 7/2021
      if (isstrue)then
      slhasn(2) = massval(23)
      slhasn(3) = massval(26)
      slhasn(4) = massval(40)
      slhasn(5) = massval(41)
      slhasn(6) = massval(42)
      slhasn(7) = massval(43)
      slhasn(8) = massval(44)
      slhasn(9) = massval(45)
      slhasn(10) = massval(46)
      slhasn(11) = massval(47)
      slhasn(12) = massval(48)
      slhasn(13) = massval(49)
      slhasn(14) = massval(50)
      slhasn(15) = massval(51)
      slhasn(16) = massval(52)
      slhasn(17) = massval(53)
      slhasn(18) = massval(54)
c -- neutrino masses
      gmnu(1:9)=abs(numassval(1:9))
      endif

c end ND added 7/2021
c -- the sfermion mixing angles --

      slhacot=stopmixval(1,1)
      slhasit=stopmixval(1,2)

      slhacob=sbotmixval(1,1)
      slhasib=sbotmixval(1,2)

      slhacol=staumixval(1,1)
      slhasil=staumixval(1,2)

c maggie added 16/6/11
      sdthet=dacos(stopmixval(1,1))
      sdtheb=dacos(sbotmixval(1,1))
      sdthel=dacos(staumixval(1,1))

      if(stopmixval(1,2).le.0.D0) then
         if(sdthet.ge.0.D0) then
            sdthet = -1.D0*sdthet
         endif
      endif

      if(sbotmixval(1,2).le.0.D0) then
         if(sdtheb.ge.0.D0) then
            sdtheb = -1.D0*sdtheb
         endif
      endif

      if(staumixval(1,2).le.0.D0) then
         if(sdthel.ge.0.D0) then
            sdthel = -1.D0*sdthel
         endif
      endif         

      cum = 1.D0
      sum = 0.D0
      cdm = 1.D0
      sdm = 0.D0
      cem = 1.D0
      sem = 0.D0      

c -- the gluino mass --

      if(massval(27).ne.0.D0) then
         amgluino = massval(27)
      else
         amgluino = msoftval(3)
      endif 

c -- the NMSSM Higgs masses --

      slhamh1 = massval(2)
      slhamh2 = massval(34)
      slhamh3 = massval(35)
      slhamc  = massval(5)

      slhama1 = massval(3)
      if(massval(4).ne.0.D0) then
         slhama2 = massval(4)
      elseif(extval(26).ne.unlikely) then
         slhama2 = extval(26)
      elseif(extval(24).ne.unlikely) then
         slhama2 = dsqrt(extval(24))
      endif

      amh1 = slhamh1
      amh2 = slhamh2
      amh3 = slhamh3
      amch = slhamc
      ama1 = slhama1
      ama2 = slhama2

c -- DRbar value of tanbeta at the scale Q --

      if(hmixval(2).ne.unlikely) then
         TGBET = hmixval(2)
      else
         tgbet = extval(25)
      endif
      b = datan(tgbet)

c -- the NMSSM Higgs mixing matrix elements --
      do i=1,3,1
         do j=1,3,1
            slhahmix(i,j)=nmhmixval(i,j)
c            print*,'slhahmix',slhahmix(i,j),i,j
         end do
      end do

c      sinb = dsqrt(tgbet**2/(1.D0+tgbet**2))
c      cosb = dsqrt(1.D0/(1.D0+tgbet**2))

      slhaamix(1,1)=nmamixval(1,1)/dsin(b)
      check1 = nmamixval(1,2)/dcos(b)

      if(dabs(slhaamix(1,1)-check1).ge.0.01D0) then
         print*,'Problem in P11,P12'
      endif

      slhaamix(1,2)=nmamixval(1,3)
      slhaamix(2,2)=nmamixval(2,3)

      slhaamix(2,1)=nmamixval(2,1)/dsin(b)
      check2 = nmamixval(2,2)/dcos(b)
      if(dabs(slhaamix(2,1)-check2).ge.0.01D0) then
         print*,'Problem in P21,P22'
      endif

c -- the MSSM mixing angle alpha in the Higgs sector --
c -- Attention: It might be that alphaval is not the DRbar value at
c -- the scale Q.
c      slha_alpha = alphaval

c -- the fermion pole masses --

      if(smval(6).ne.0.D0) then
         AMT = smval(6)
         else
            print*,'Attention the value for mt is not set. It has to be 
     .given.'
      endif
      if(smval(24).ne.0.D0) then
         AMC = smval(24)
         else
            print*,'Attention the value for mc is not set. It has to be 
     .given.'
      endif
      if(smval(7).ne.0.D0) then
         AMTAU = smval(7)
         else
            print*,'Attention the value for mtau is not set. It has to b
     .e given.'
      endif
      if(smval(23).ne.0.D0) then
         AMS = smval(23)
         else
            print*,'Attention the value for ms is not set. It has to be 
     .given.'
      endif
      if(smval(13).ne.0.D0) then
         AMMUON = smval(13)
         else
            print*,'Attention the value for mmuon is not set. It has to 
     . be given.'
      endif

c -- the mass mb(mb)_MSbar --
      if(smval(5).ne.0.D0) then
         mbmsbar = smval(5)
      else
         print*,'Attention the value for mbMSbar(mb) is not set. It has 
     .to be given.'
      endif

      fmt = amt
      fmtau = amtau
      fms = ams
      fmc = amc
c -- calculation of the mb pole mass from mb(mb)_MSbar --
      if(smval(5).ne.0.D0) then
       del = 1.d-10
       mbl = mbmsbar
       mbu = 2*mbmsbar
       fmb = (mbl+mbu)/2
       amsb = ams
       amc0=amc
       amt0=amt
       acc=1.d-8
       nloop=3
11     amb=fmb
       amb0=amb
       xlambda=xitla_hdec(nloop,alsmz,acc)
       n0=5
       call alsini_hdec(acc)
c      xmb = runm_hdec(fmb,5)
       xmb = runm_hdec(mbmsbar,5)
       if(xmb.eq.mbmsbar)then
        mbl = fmb
        mbu = fmb
       elseif(xmb.gt.mbmsbar)then
        mbu = fmb
       else
        mbl = fmb
       endif
       fmb = (mbl+mbu)/2
       if(dabs(xmb/mbmsbar-1).gt.del) goto 11
      endif
      amb = fmb

c      print*,'amw,amz,gamw,gamz,ihiggs',amw,amz,gamw,gamz,ihiggs
c      print*,'alsmz',alsmz
c      print*,'s,c,b,t,tau,mu',ams,amc,amb,amt,amtau,ammuon
c      print*,'alph,gf',alph,Gf

c -- If no DRbar value at the scale Q has been given for tanbeta --

      if(hmixval(2).eq.unlikely) then
         if(extval(25).ne.0.D0.and.extval(25).ne.unlikely) then
            TGBET = extval(25)
         elseif(minval(3).ne.0.D0.and.minval(3).ne.unlikely) then
            TGBET = minval(3)
         endif
      endif

c -- The soft SUSY breaking parameters: DRbar values at the scale Q --

      do i=1,100,1
         m_softval(i) = unlikely
      end do

      do i=1,99,1
         if(msoftval(i).ne.unlikely) then
            m_softval(i) = msoftval(i)
            m_softcom(i) = softcom(i)
         elseif(extval(i).ne.unlikely) then
            m_softval(i) = extval(i)
            m_softcom(i) = extcom(i)
         endif
      end do

      if(auval(3,3).ne.unlikely) then
         AU=auval(3,3)
      elseif(extval(11).ne.unlikely) then
         AU=extval(11)
      else
         print*,'Attention: A_top has not been given.'
      endif
      if(adval(3,3).ne.unlikely) then
         AD=adval(3,3)
      elseif(extval(12).ne.unlikely) then
         AD=extval(12)
      else
         print*,'Attention: A_bottom has not been given.'
      endif
      if(aeval(3,3).ne.unlikely) then
         AL=aeval(3,3)
      elseif(extval(13).ne.unlikely) then
         AL=extval(13)
      else
         print*,'Attention: A_tau has not been given.'
      endif

c The NMSSM parameters at the scale Q = M_input 

      if(extval(61).ne.unlikely) then
         xlambdapar = extval(61)
      endif
      if(extval(62).ne.unlikely) then
         xkappa = extval(62)
      endif
      if(extval(63).ne.unlikely) then
         Alam = extval(63)
      endif
      if(extval(64).ne.unlikely) then
         Akap = extval(64)
      endif
      if(extval(65).ne.unlikely) then
         xlamvs = extval(65)
      endif
      if(extval(66).ne.unlikely) then
         xif = extval(66)
      endif
      if(extval(67).ne.unlikely) then
         xis = extval(67)
      endif
      if(extval(68).ne.unlikely) then
         xmuprime = extval(68)
      endif
      if(extval(69).ne.unlikely) then
         amsph2 = extval(69)
      endif
      if(extval(70).ne.unlikely) then
         amsh2 = extval(70)
      endif

      Alambda = Alam
      Akappa = Akap

      amu = xlamvs

c The soft SUSY breaking parameters M1, M2 in the MS_bar scheme 
      am1msbar = m_softval(1)*(1.D0+g1ew**2/16.D0/pi**2*0.D0)
      am2msbar = m_softval(2)*(1.D0+g2ew**2/16.D0/pi**2*2.D0)

      if(am1msbar.ne.0.d0) then 
         am1 = am1msbar
      else
         print*,'Problem: M1 not given.'
      endif
     
      if(am2msbar.ne.0.d0) then
         AM2 = am2msbar
      else
         print*,'Problem: M2 not given.'
      endif

      am1val = msoftval(1)
      am2val = msoftval(2)

      if(m_softval(31).ne.unlikely.and.m_softval(32).ne.unlikely) then
         if(m_softval(31).ne.0.D0.and.m_softval(32).ne.0.D0) then
            AMEL1 = (m_softval(31)+m_softval(32))/2.D0
         elseif(m_softval(31).ne.0.D0) then
            AMEL1 = m_softval(31)
         elseif(m_softval(32).ne.0.D0) then
            AMEL1 = m_softval(32)
         else
            AMEL1 = 0.D0
         endif
      elseif(m_softval(31).ne.unlikely) then
         AMEL1 = m_softval(31)
      elseif(m_softval(32).ne.unlikely) then
         AMEL1 = m_softval(32)
      endif

      if(m_softval(34).ne.unlikely.and.m_softval(35).ne.unlikely) then
         if(m_softval(34).ne.0.D0.and.m_softval(35).ne.0.D0) then
            AMER1 = (m_softval(34)+m_softval(35))/2.D0
         elseif(m_softval(34).ne.0.D0) then
            AMER1 = m_softval(34)
         elseif(m_softval(35).ne.0.D0) then
            AMER1 = m_softval(35)
         else
            AMER1 = 0.D0
         endif
      elseif(m_softval(34).ne.unlikely) then
         AMER1 = m_softval(34)
      elseif(m_softval(35).ne.unlikely) then
         AMER1 = m_softval(35)
      endif

      if(m_softval(41).ne.unlikely.and.m_softval(42).ne.unlikely) then
         if(m_softval(41).ne.0.D0.and.m_softval(42).ne.0.D0) then
            AMQL1 = (m_softval(41)+m_softval(42))/2.D0
         elseif(m_softval(41).ne.0.D0) then
            AMQL1 = m_softval(41)
         elseif(m_softval(42).ne.0.D0) then
            AMQL1 = m_softval(42)
         else
            AMQL1 = 0.D0
         endif
      elseif(m_softval(41).ne.unlikely) then
         AMQL1 = m_softval(41)
      elseif(m_softval(42).ne.unlikely) then
         AMQL1 = m_softval(42)
      endif

      if(m_softval(44).ne.unlikely.and.m_softval(45).ne.unlikely) then
         if(m_softval(44).ne.0.D0.and.m_softval(45).ne.0.D0) then
            AMUR1 = (m_softval(44)+m_softval(45))/2.D0
         elseif(m_softval(44).ne.0.D0) then
            AMUR1 = m_softval(44)
         elseif(m_softval(45).ne.0.D0) then
            AMUR1 = m_softval(45)
         else
            AMUR1 = 0.D0
         endif
      elseif(m_softval(44).ne.unlikely) then
         AMUR1 = m_softval(44)
      elseif(m_softval(45).ne.unlikely) then
         AMUR1 = m_softval(45)
      endif

      if(m_softval(47).ne.unlikely.and.m_softval(48).ne.unlikely) then
         if(m_softval(47).ne.0.D0.and.m_softval(48).ne.0.D0) then
            AMDR1 = (m_softval(47)+m_softval(48))/2.D0
         elseif(m_softval(47).ne.0.D0) then
            AMDR1 = m_softval(47)
         elseif(m_softval(48).ne.0.D0) then
            AMDR1 = m_softval(48)
         else
            AMDR1 = 0.D0
         endif
      elseif(m_softval(47).ne.unlikely) then
         AMDR1 = m_softval(47)
      elseif(m_softval(48).ne.unlikely) then
         AMDR1 = m_softval(48)
      endif

      if(m_softval(33).ne.unlikely) then
         AMEL = m_softval(33)
      endif
      if(m_softval(36).ne.unlikely) then
         AMER = m_softval(36)
      endif
      if(m_softval(43).ne.unlikely) then
         AMSQ = m_softval(43)
      endif
      if(m_softval(46).ne.unlikely) then
         AMUR = m_softval(46)
      endif
      if(m_softval(49).ne.unlikely) then
         AMDR = m_softval(49)
      endif

c --- NMSSM Higgs boson couplings to neutralinos and charginos

      do i=1,5,1
         do j=1,5,1
            do k=1,5,1
               anm(i,j,k) = 1.D0
            end do
         end do
      end do

      do i=1,5,1
         do j=1,2,1
            do k=1,2,1
               acnm(i,j,k) = 1.D0
            end do
         end do
      end do
      
      do i=1,2,1
         do j=1,5,1
            acnlnm(i,j) = 1.D0
            acnrnm(i,j) = 1.D0
         end do
      end do

c --- further parameter settings --- c

      b = datan(tgbet)
      amglu = amgluino

      if(rvub.ne.0.D0) then
         vub=rvub*vcb
      endif
      alph=1.d0/alph
      amsb = ams

      amc0=amc
      amb0=amb
      amt0=amt
      acc=1.d-8
      nloop=3
      xlambda=xitla_hdec(nloop,alsmz,acc)
      n0=5

      call alsini_hdec(acc)
c--decoupling the top quark from alphas
      amt0=3.d8

c--initialize coefficients for polylogarithms
      nber = 18
      call bernini_hdec(nber)

c--check nfgg
      if(nfgg.gt.5.or.nfgg.lt.3)then
       write(6,*)'nf-gg not valid. taking the default nf-gg = 3....'
       nfgg = 3
      endif

100   format(10x,g30.20)
101   format(10x,i30)

7     format(a11,i7)
8     format(a11,i4)
9     format(a11,g15.8)
10    format(a26)

      close(ni)

      return

      end

c ==================================================================== c
c ==== writes out all important parameters, the NMSSM Higgs decay ==== c
c ==== widths and branching ratios in the SLHA format ================ c
c ==================================================================== c

      subroutine write_nmssm(tgbet,output)
      implicit double precision (a-h,o-z)
      double precision brwhigne1(3,5,5)
      double precision brhiggg(3),brhigmm(3),brhigll(3),brhigss(3),
     .     brhigcc(3),brhigbb(3),brhigtt(3),brhigga(3),brhigzga(3),
     .     brhigww(3),brhigzz(3),brhitohpwm(3),brhitohjhk(3,5,5),
     .     brhitoajz(3,5),brwhigch(3,2,2),brwhigne(3,5,5),brwhigcht(3),
     .     brwhignet(3),brwhislt(3),brwhisq(3),brwhisqt(3),
     .     brwhislnl(3),brwhislnltau(3),brwhislel(3),brwhisler(3),
     .     brwhisqul(3),brwhisqur(3),brwhisqdl(3),brwhisqdr(3),
     .     brwhist(3,2,2),brwhisb(3,2,2),brwhistau(3,2,2),whitot(5),
     .     whatot(2),brhignn(3)
      double precision brhagg(2),brhamm(2),brhall(2),brhass(2),
     .     brhacc(2),brhabb(2),brhatt(2),brhaga(2),brhazga(2),
     .     brhaiwphm(2),braitoajhk(2,2,3),brhjaiz(2,3),brwhaich(2,2,2),
     .     brwhaine(2,5,5),brwhaicht(2),brwhainet(2),brwhaisl(2),
     .     brwhaist(2),brwhaisb(2),brhann(2)
      parameter (nout=44,nout2=45)
      dimension gmn(5),xmn(5),gmc(2),gmst(2),gmsb(2),gmsl(2),
     .          gmsu(2),gmsd(2),gmse(2),gmsn(18),gmsnl(2),gmnu(9)
      dimension h1brsc(2,2),h1brsn(5,5),h2brsc(2,2),h2brsn(5,5),
     .     h3brsc(2,2),h3brsn(5,5),ha1brsc(2,2),ha1brsn(5,5),
     .     ha2brsc(2,2),ha2brsn(5,5),hcbrsu(2,5),h2brst(2,2),
     .     h2brsb(2,2),h3brst(2,2),h3brsb(2,2),hcbrstb(2,2) 
      dimension ac1(2,2),ac2(2,2),ac3(2,2),ac4(2,2),ac5(2,2),
     .          an1(5,5),an2(5,5),an3(5,5),an4(5,5),an5(5,5),
     .          acnl(2,5),acnr(2,5)
      dimension h1brsn1(5,5),h2brsn1(5,5),h3brsn1(5,5),ha1brsn1(5,5),
     .     ha2brsn1(5,5)
      character*30 output
      double precision nmhmixval(3,3),nmamixval(2,3),numixval(9,9),
     .                 numassval(9),imnumixval(9,9)
      double precision snsmixval(9,9),snpmixval(9,9)
      double precision minval(1:20),smval(1:24),massval(1:100),
     .                 nmixval(5,5),umixval(2,2),vmixval(2,2),
     .                 stopmixval(2,2),sbotmixval(2,2),staumixval(2,2),
     .                 hmixval(1:10),gaugeval(1:3),msoftval(1:100),
     .                 auval(3,3),adval(3,3),aeval(3,3),yuval(3,3),
     .                 ydval(3,3),yeval(3,3),qvalue(1:20),extval(0:100),
     .                 m_softval(1:100)
      double precision slhaneut(1:5),slhaxneut(1:5),slhachar(1:2),
     .                 slhau(2,2),slhav(2,2),slhaz(5,5),slhaxchar(1:2),
     .                 slhast(2),slhasb(2),slhasu(2),slhasd(2),
     .                 slhase(2),slhasl(2),slhasn(18),slhasnl(2),
     .                 warning(1:10)
      double precision ghigb(5),ghigt(5),ghigvv(3),ghighpwm(3),
     .     gzaihj(2,3),g3hihjhk(5,5,5),ghigpm(3),dghib(5)
      double precision vckmval(4)
      integer   imod(1:2)
      integer check(1:100)
      double precision mbmsbar,mbl,mbu
      character spinfo1*100,spinfo2*100,mincom(1:20)*20,
     .          extcom(0:100)*20,softcom(1:100)*20,hmixcom(1:10)*20,
     .          m_softcom(1:100)*20
      integer modselval(15)
     
      double precision cj(5)
      COMMON/MASSES_HDEC/ams,amc,amb,amt
      COMMON/STRANGE_HDEC/amsb
      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
      COMMON/CKMPAR_HDEC/VTB,VTS,VTD,VCB,VCS,VCD,VUB,VUS,VUD
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
      COMMON/BREAK_NMSSM/amel,amer,amsq,amur,amdr,al,au,ad,alam,akap,
     .     amu,am2
      COMMON/BREAKGLU_HDEC/amglu
      COMMON/SFER1ST_HDEC/amql1,amur1,amdr1,amel1,amer1
      COMMON/GLUINO_NMSSM/amgluino,xmsb1,xmsb2,sthb,cthb,
     .     xh1bb(2,2),xh2bb(2,2),xh3bb(2,2),xha1bb(2,2),xha2bb(2,2),
     .     xmst1,xmst2,stht,ctht,xh1tt(2,2),xh2tt(2,2),xh3tt(2,2),
     .     xha1tt(2,2),xha2tt(2,2)
      COMMON/WZWDTH_HDEC/gamc0,gamt0,gamt1,gamw,gamz
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/SHORTCOUPL2_NMSSM/ghigb,ghigt,ghigvv,ghighpwm,gzaihj,
     .     g3hihjhk,ghigpm
      COMMON/ALS_HDEC/xlambda,amc0,amb0,amt0,n0
      COMMON/FLAG_HDEC/IHIGGS,NNLO
      COMMON/MODEL_NMSSM/IMODEL
      COMMON/ONSHELL_NMSSM/IONSH,IONWZ,IOFSUSY
      COMMON/OLDFASH_NMSSM/NFGG
      COMMON/WIDTHSCALAR_NMSSM/brhiggg,brhigmm,brhigll,brhigss,
     .     brhigcc,brhigbb,brhigtt,brhigga,brhigzga,
     .     brhigww,brhigzz,brhitohpwm,brhitohjhk,
     .     brhitoajz,brwhigch,brwhigne,brwhigcht,
     .     brwhignet,brwhislt,brwhisq,brwhisqt,
     .     brwhislnl,brwhislnltau,brwhislel,brwhisler,
     .     brwhisqul,brwhisqur,brwhisqdl,brwhisqdr,
     .     brwhist,brwhisb,brwhistau,whitot,brhignn
      COMMON/WIDTHPSEUDOSCALAR_NMSSM/brhagg,brhamm,brhall,brhass,
     .     brhacc,brhabb,brhatt,brhaga,brhazga,
     .     brhaiwphm,braitoajhk,brhjaiz,brwhaich,
     .     brwhaine,brwhaicht,brwhainet,brwhaisl,
     .     brwhaist,brwhaisb,whatot,brhann
      COMMON/WIDTHA1_NMSSM/A1BRB,A1BRL,A1BRM,A1BRS,A1BRC,A1BRT,A1BRG,
     .     A1BRGA,A1BRZGA,A1BRZH1,A1BRZH2,A1BRZH3,A1WDTH
      COMMON/WIDTHA2_NMSSM/A2BRB,A2BRL,A2BRM,A2BRS,A2BRC,A2BRT,A2BRG,
     .     A2BRGA,A2BRZGA,A2BRZH1,A2BRZH2,A2BRZH3,A2BRA1H1,A2BRA1H2,
     .     A2BRA1H3,A2BRHCW,A2WDTH
      COMMON/WIDTHH1_NMSSM/H1BRB,H1BRL,H1BRM,H1BRS,H1BRC,H1BRT,H1BRG,
     .     H1BRGA,H1BRZGA,H1BRW,H1BRZ,H1BRA1A1,H1BRA1Z,H1BRA2Z,H1BRHW,
     .     H1BRA1A2,H1WDTH
      COMMON/WIDTHH2_NMSSM/H2BRB,H2BRL,H2BRM,H2BRS,H2BRC,H2BRT,H2BRG,
     .     H2BRGA,H2BRZGA,H2BRW,H2BRZ,H2BRH1H1,H2BRA1A1,H2BRA1A2,
     .     H2BRA1Z,H2BRA2Z,H2BRHW,H2WDTH
      COMMON/WIDTHH3_NMSSM/H3BRB,H3BRL,H3BRM,H3BRS,H3BRC,H3BRT,H3BRG,
     .     H3BRGA,H3BRZGA,H3BRW,H3BRZ,H3BRH1H1,H3BRH1H2,H3BRH2H2,
     .     H3BRA1A1,H3BRA1A2,H3BRA2A2,H3BRA1Z,H3BRA2Z,H3BRHW,H3WDTH
      COMMON/WIDTHHC_NMSSM/HCBRB,HCBRL,HCBRM,HCBRBU,HCBRS,HCBRC,HCBRT,
     .               HCBRWH1,HCBRWH2,HCBRWH3,HCBRA1,HCBRA2,hcbrd,hcbrdt,
     .               hcbrst,HCWDTH
      COMMON/WISUSY_NMSSM/H1BRSC,H1BRSN,H2BRSC,H2BRSN,H3BRSC,H3BRSN,
     .     HA1BRSC,HA1BRSN,HA2BRSC,HA2BRSN,HCBRSU,H1BRCHT,H2BRCHT,
     .     H3BRCHT,HA1BRCHT,HA2BRCHT,H1BRNET,H2BRNET,H3BRNET,HA1BRNET,
     .     HA2BRNET,HCBRCNT,H1BRSL,H2BRSL,H3BRSL,HCBRSL,HA1BRSL,HA2BRSL,
     .     HA1BRST,HA2BRST,HA1BRSB,HA2BRSB,H2BRSQ,H2BRST,H2BRSB,H2BRSQT,
     .     H3BRSQ,H3BRST,H3BRSB,H3BRSQT,HCBRSQ,HCBRSTB,HCBRSQT,H1BRSQ,
     .     H1BRSQT
      COMMON/WISFER_NMSSM/BH1SLNL,BH1SLEL,BH1SLER,BH1SQUL,BH1SQUR,
     .     BH1SQDL,BH1SQDR,BH1ST(2,2),BH1SB(2,2),BH1STAU(2,2),BH2SLNL,
     .     BH2SLEL,BH2SLER,BH2SQUL,BH2SQUR,BH2SQDL,BH2SQDR,BH2ST(2,2),
     .     BH2SB(2,2),BH2STAU(2,2),BH3SLNL,BH3SLEL,BH3SLER,BH3SQUL,
     .     BH3SQUR,BH3SQDL,BH3SQDR,BH3ST(2,2),BH3SB(2,2),BH3STAU(2,2),
     .     BHA1STAU,BHA1SB,BHA1ST,BHA2STAU,BHA2SB,BHA2ST,BHCSL00,
     .     BHCSL11,BHCSL21,BHCSQ,BHCSTB(2,2),BH1SLNLTAU,BH2SLNLTAU,
     .     BH3SLNLTAU
      COMMON/SMASS_NMSSM/GMN,XMN,GMC,GMST,GMSB,GMSL,GMSU,GMSD,GMSE,
     .     GMSN,GMSNL,GMNU
c      COMMON/FLAGS_HDEC/INDIDEC
c -------------- common block given by read_leshouches ------------ c
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval
      COMMON/SLHA_leshouches1_NMSSM/extcom,mincom,softcom,hmixcom
      COMMON/SLHA_leshouches2_NMSSM/minval,extval,smval,massval,
     .                   nmixval,umixval,vmixval,stopmixval,sbotmixval,
     .                   staumixval,hmixval,gaugeval,msoftval,auval,
     .                   adval,aeval,yuval,ydval,yeval,qvalue,imod,numixval
     .                   ,numassval,imnumixval
      COMMON/SLHA_leshouches3_NMSSM/nmhmixval,nmamixval
      COMMON/SLHA_leshouches5_NMSSM/snsmixval,snpmixval
      COMMON/SLHA_checkval_CNMSSM/check
      COMMON/SLHA_leshouches3_HDEC/vckmval
c -------------------------------------------------------------------- c
      COMMON/SD_scaleofewsb/scaleofewsb
c -------------- common blocks needed in HDECAY subroutines ---------- c
      COMMON/SLHA_m1_HDEC/am1
      COMMON/SLHA_gaug_NMSSM/slhaneut,slhaxneut,slhachar,slhau,slhav,
     .                      slhaz,slhaxchar
      COMMON/SLHA_sfer_HDEC/slhast,slhasb,slhasu,slhasd,slhase,slhasl,
     .                 slhasn,slhasnl,slhacot,slhasit,slhacob,slhasib,
     .                 slhacol,slhasil
      COMMON/SLHA_hmass_NMSSM/slhamh1,slhamh2,slhamh3,slhama1,slhama2,
     .     slhamc
      COMMON/GAUGINOMIX_NMSSM/ZZ(5,5),UU(2,2),VV(2,2)
      COMMON/TAUMIX_HDEC/CL,SL
      COMMON/SLHAVAL_HDEC/g1ew,g2ew
      COMMON/M1andM2val/am1val,am2val
      COMMON/GLUONCOUP/cj
c ND added 7/2021
c--------------------connect to calcmasses ISS------------------------c
      logical isstrue
      common/issswitch/isstrue
      logical isscheck(10)
      common/iss_check/isscheck
      double precision para_S,para_T,para_U,brmuega,brtauega,brtaumuga
      common/lowen_observables/para_S,para_T,para_U,brmuega,brtauega,
     -                 brtaumuga
      double precision brsum(3),brcpn(3,6,2)
      double complex Yvout(3,3)
      double precision mv9R(9),UVR(9,9)
      COMMON /neumassesout/ mv9R,UVR
c--------------------connect to calcmasses ISS------------------------c
c end ND added 7/2021
c nhung added for the output of Higgs WFR factor 6/2021
      double complex ZHfac(5,5)
      common /Zfactor/ ZHfac 
c end nhung added for the output of Higgs WFR factor 6/2021

      PI = 4*DATAN(1D0)

      open(nout,file=output)

c      open(nout2,file='exclh1.dat'
c      write(nout2,108) amh1,dabs(gh1vv*h1brb),h1brb,gh1vv

      id =1
      idb=-1
      iu =2
      iub=-2
      is =3
      isb=-3
      ic =4
      icb=-4
      ib =5
      ibb=-5
      it =6
      itb=-6
      
      ie   =11
      ine  =12
      imu  =13
      inmu =14
      itau =15
      intau=16
      
      ih1=25
      ih2=35
      ih3=45
      iha1=36
      iha2=46
      ihc=37
      igl=21
      iga=22
      iz =23
      iwc=24
      
      isdl=1000001
      isdr=2000001
      isul=1000002
      isur=2000002
      issl=1000003
      issr=2000003
      iscl=1000004
      iscr=2000004
      isb1=1000005
      isb2=2000005
      ist1=1000006
      ist2=2000006
      
      iglo=1000021
      in1 =1000022
      in2 =1000023
      in3 =1000025
      in4 =1000035
      in5 =1000045
      ic1 =1000024
      ic2 =1000037
      
      intau1=1000016 
      intau2=2000016 
      inel  =1000012
      iner  =2000012
      inmul =1000014
      inmur =2000014
      
      isell =1000011
      iselr =2000011
      ismul =1000013
      ismur =2000013
      istau1=1000015
      istau2=2000015

c ND added 7/2021
      if(isstrue)then
      insn4=1000118
      insn5=1000120
      insn6=1000122
      insn7=1000124
      insn8=1000126
      insn9=1000128
      insn10=2000012
      insn11=2000014
      insn12=2000016
      insn13=2000118
      insn14=2000120
      insn15=2000122
      insn16=2000124
      insn17=2000126
      insn18=2000128
      endif
c end ND added 7/2021
c ----------------------------------- c
c Information about the program       c
c ----------------------------------- c

      write(nout,105)
      write(nout,51) 'PRINFO','Program information'
      write(nout,61) 1,'NMSSMCALC-nuSS  # program name'
      write(nout,61) 2,'2.00            # version number'      
c ----------------------------------- c
c Information about the decay program c
c ----------------------------------- c

      write(nout,105)
      write(nout,51) 'DCINFO','Decay Program information'
      write(nout,61) 1,'BHDECAY    # decay calculator'
      write(nout,61) 2,'2.00       # version number'

c ----------------------------------------------------------------- c
c The program information: Which spectrum calculator has been used. c
c ----------------------------------------------------------------- c

      if(check(22).eq.1) then
         write(nout,105)
         write(nout,51) 'SPINFO','Spectrum calculator information'
         write(nout,61) 1,spinfo1(1:50)
         write(nout,61) 2,spinfo2(1:50)
      endif

c ------------------------------------------------ c
c Information on the model which has been selected c
c ------------------------------------------------ c

      write(nout,105)
      write(nout,51) 'MODSEL','Model selection'
c ND changed 7/2021
      if(isstrue)then
      write(nout,62) 3,2,'# NMSSM with ISS'
      else
      write(nout,62) 3,1,'# NMSSM'
      endif
c end ND changed 7/2021
      write(nout,62) 5,0,'# CP-conserving'
      write(nout,62) 6,modselval(6),'# loop level 1: one 2: two O(alpha_t 
     .alpha_s) 3: two O(alpha_t alpha_s + alpha_t^2)'
      write(nout,62) 7,modselval(7),'# for top/stop sector: 1: DRbar sche
     .me no gauge running; 2: DRbar w/ gauge running 3: OS scheme'
      write(nout,62) 8,modselval(8),'# 0: MHpm as Input,
     . 1: Alambda as Input'
      write(nout,62) 11,modselval(11),'# 0: no AMMs calculated (default),
     . 1: AMMs calculated, 2: detailed output' 
c ----------------------- c
c The SM input parameters c
c ----------------------- c

c -- calculation of mb(mb)_MSbar from the mb pole mass --
      del = 1.d-8
      rmb0 = amb
444   rmb = rmb0
      rmb0 = runm_hdec(rmb,5)
      if(dabs(rmb0/rmb-1).gt.del)goto 444
      rmb = rmb0
      alsmz = alphas_hdec(amz,3)
      write(nout,105)
      write(nout,51) 'SMINPUTS','Standard Model inputs'
      write(nout,52) 1,smval(1),'1/alpha_em'
      write(nout,52) 2,gf,'G_F [GeV^-2]'
      write(nout,52) 3,alsmz,'alpha_S(M_Z)^MSbar'
      write(nout,52) 4,amz,'M_Z pole mass'
      write(nout,52) 5,rmb,'mb(mb)^MSbar'
      write(nout,52) 6,amt,'mt pole mass'
      write(nout,52) 7,amtau,'mtau pole mass'
c ND added 7/2021
      if(isstrue)then
      write(nout,52) ine,mv9R(1),'nu_1'
      write(nout,52) inmu,mv9R(2),'nu_2'
      write(nout,52) intau,mv9R(3),'nu_3'
      endif
c end ND added 7/2021
c ------------------------------------------------ c
c Input parameters for minimal/default SUSY models c
c ------------------------------------------------ c

      if(check(3).eq.1) then
         write(nout,105)
         write(nout,51) 'MINPAR','Input parameters - minimal models'
         unlikely = -123456789D0
         do ii=1,20,1
            if(minval(ii).ne.unlikely) then
               write(nout,52) ii,minval(ii),mincom(ii)
            endif
         end do
      endif

c ------------------------------------------------------------------- c
c Optional input parameters for non-minimal/non-universal SUSY models c
c ------------------------------------------------------------------- c

      if(check(4).eq.1) then
         write(nout,105)
         write(nout,51) 'EXTPAR','Input parameters - non-minimal models'
         unlikely = -123456789D0
         do ii=1,100,1
            if(extval(ii-1).ne.unlikely) then
               write(nout,52) ii-1,extval(ii-1),extcom(ii-1)
            endif
         end do
      endif

c ----------------- c
c The mass spectrum c
c ----------------- c

      write(nout,105)
      write(nout,51) 'MASS','Mass Spectrum'
      write(nout,50) 'PDG code           mass       particle'
      write(nout,52) iwc,amw,'W+'
      write(nout,52) ih1,amh1,'H1'
      write(nout,52) ih2,amh2,'H2'
      write(nout,52) ih3,amh3,'H3'
      write(nout,52) iha1,ama1,'A1'
      write(nout,52) iha2,ama2,'A2'
      write(nout,52) ihc,amch,'H+'
      write(nout,52) ib,amb,
     .'b-quark pole mass calculated from mb(mb)_Msbar'
      write(nout,52) isdl,gmsd(1),'~d_L'
      write(nout,52) isdr,gmsd(2),'~d_R'
      write(nout,52) isul,gmsu(1),'~u_L'
      write(nout,52) isur,gmsu(2),'~u_R'
      write(nout,52) issl,gmsd(1),'~s_L'
      write(nout,52) issr,gmsd(2),'~s_R'
      write(nout,52) iscl,gmsu(1),'~c_L'
      write(nout,52) iscr,gmsu(2),'~c_R'
      write(nout,52) isb1,GMSB(1),'~b_1'
      write(nout,52) isb2,GMSB(2),'~b_2'
      write(nout,52) ist1,GMST(1),'~t_1'
      write(nout,52) ist2,GMST(2),'~t_2'
      write(nout,52) isell,GMSE(1),'~e_L'
      write(nout,52) iselr,GMSE(2),'~e_R'
      write(nout,52) inel,GMSN(1),'~nu_eL'
      write(nout,52) ismul,GMSE(1),'~mu_L'
      write(nout,52) ismur,GMSE(2),'~mu_R'
      write(nout,52) istau1,GMSL(1),'~tau_1'
      write(nout,52) istau2,GMSL(2),'~tau_2'
c ND changed 7/2021
      if(.not.isstrue)then 
      write(nout,52) inel,GMSN(1),'~nu_eL'
      write(nout,52) intau1,GMSNL(1),'~nu_tauL'
      endif
c end ND changed 7/2021
      write(nout,52) iglo,amgluino,'~g'
      write(nout,52) in1,xmn(1),'~chi_10'
      write(nout,52) in2,xmn(2),'~chi_20'
      write(nout,52) in3,xmn(3),'~chi_30'
      write(nout,52) in4,xmn(4),'~chi_40'
      write(nout,52) in5,xmn(5),'~chi_50'
      write(nout,52) ic1,gmc(1),'~chi_1+'
      write(nout,52) ic2,gmc(2),'~chi_2+'
      
c ND changed 7/2021
      if(isstrue)then 
      write(nout,52) inel,GMSN(1),'~nu_1'
      write(nout,52) inmul,GMSN(2),'~nu_2'
      write(nout,52) intau1,GMSN(3),'~nu_3'
      write(nout,52) insn4,GMSN(4),'~nu_4'
      write(nout,52) insn5,GMSN(5),'~nu_5'
      write(nout,52) insn6,GMSN(6),'~nu_6'
      write(nout,52) insn7,GMSN(7),'~nu_7'
      write(nout,52) insn8,GMSN(8),'~nu_8'
      write(nout,52) insn9,GMSN(9),'~nu_9'
      write(nout,52) insn10,GMSN(10),'~nu_10'
      write(nout,52) insn11,GMSN(11),'~nu_11'
      write(nout,52) insn12,GMSN(12),'~nu_12'
      write(nout,52) insn13,GMSN(13),'~nu_13'
      write(nout,52) insn14,GMSN(14),'~nu_14'
      write(nout,52) insn15,GMSN(15),'~nu_15'
      write(nout,52) insn16,GMSN(16),'~nu_16'
      write(nout,52) insn17,GMSN(17),'~nu_17'
      write(nout,52) insn18,GMSN(18),'~nu_18'
      endif
c end ND changed 7/2021
      write(nout,105) 
      write(nout,51) 'DMASS','delta M_H_i theoretical error'
      write(nout,52) ih1,3d0,'H1'
      write(nout,52) ih2,3d0,'H2'
      write(nout,52) ih3,1d0,'H3'
      write(nout,52) iha1,3d0,'A1'
      write(nout,52) iha2,1d0,'A2'
      write(nout,52) ihc,1d0,'H+'

c ------------------------------------------------------------------- c
c The neutralino mixing matrix N and the chargino mixing matrices U,V c
c ------------------------------------------------------------------- c

      write(nout,105)
      write(nout,51) 'NMNMIX','Neutralino mixing matrix'
      write(nout,53) 1,1,zz(1,1),'N_11'
      write(nout,53) 1,2,zz(1,2),'N_12'
      write(nout,53) 1,3,zz(1,3),'N_13'
      write(nout,53) 1,4,zz(1,4),'N_14'
      write(nout,53) 1,5,zz(1,5),'N_15'
      write(nout,53) 2,1,zz(2,1),'N_21'
      write(nout,53) 2,2,zz(2,2),'N_22'
      write(nout,53) 2,3,zz(2,3),'N_23'
      write(nout,53) 2,4,zz(2,4),'N_24'
      write(nout,53) 2,5,zz(2,5),'N_25'
      write(nout,53) 3,1,zz(3,1),'N_31'
      write(nout,53) 3,2,zz(3,2),'N_32'
      write(nout,53) 3,3,zz(3,3),'N_33'
      write(nout,53) 3,4,zz(3,4),'N_34'
      write(nout,53) 3,5,zz(3,5),'N_35'
      write(nout,53) 4,1,zz(4,1),'N_41'
      write(nout,53) 4,2,zz(4,2),'N_42'
      write(nout,53) 4,3,zz(4,3),'N_43'
      write(nout,53) 4,4,zz(4,4),'N_44'
      write(nout,53) 4,5,zz(4,5),'N_45'
      write(nout,53) 5,1,zz(5,1),'N_51'
      write(nout,53) 5,2,zz(5,2),'N_52'
      write(nout,53) 5,3,zz(5,3),'N_53'
      write(nout,53) 5,4,zz(5,4),'N_54'
      write(nout,53) 5,5,zz(5,5),'N_55'

      write(nout,105)
      write(nout,51) 'UMIX','Chargino mixing matrix U'
      write(nout,53) 1,1,uu(1,1),'U_11'
      write(nout,53) 1,2,uu(1,2),'U_12'
      write(nout,53) 2,1,uu(2,1),'U_21'
      write(nout,53) 2,2,uu(2,2),'U_22'

      write(nout,105)
      write(nout,51) 'VMIX','Chargino mixing matrix V'
      write(nout,53) 1,1,vv(1,1),'V_11'
      write(nout,53) 1,2,vv(1,2),'V_12'
      write(nout,53) 2,1,vv(2,1),'V_21'
      write(nout,53) 2,2,vv(2,2),'V_22'

c ------------------------------------------ c
c The stop, sbottom and stau mixing matrices c
c ------------------------------------------ c

      write(nout,105)
      write(nout,51) 'STOPMIX','Stop mixing matrix'
      write(nout,53) 1,1,ctht,'cos(theta_t)'
      write(nout,53) 1,2,stht,'sin(theta_t)'
      write(nout,53) 2,1,-stht,'-sin(theta_t)'
      write(nout,53) 2,2,ctht,'cos(theta_t)'

      write(nout,105)
      write(nout,51) 'SBOTMIX','Sbottom mixing matrix'
      write(nout,53) 1,1,cthb,'cos(theta_b)'
      write(nout,53) 1,2,sthb,'sin(theta_b)'
      write(nout,53) 2,1,-sthb,'-sin(theta_b)'
      write(nout,53) 2,2,cthb,'cos(theta_b)'

      write(nout,105)
      write(nout,51) 'STAUMIX','Stau mixing matrix'
      write(nout,53) 1,1,cl,'cos(theta_tau)'
      write(nout,53) 1,2,sl,'sin(theta_tau)'
      write(nout,53) 2,1,-sl,'-sin(theta_tau)'
      write(nout,53) 2,2,cl,'cos(theta_tau)'
c ------------------------------------------------------------------- c
c Neutrino and sneutrino mixing matrices                              c
c ------------------------------------------------------------------- c
c ND changed 7/2021
      if(isstrue)then 
      write(nout,105)
      write(nout,51) 'NUMASS', 'Neutrino masses'
      write(nout,52) 12,mv9R(1),'nu_1'
      write(nout,52) 14,mv9R(2),'nu_2'
      write(nout,52) 16,mv9R(3),'nu_3'
      write(nout,52) 118,mv9R(4),'nu_4'
      write(nout,52) 120,mv9R(5),'nu_5'
      write(nout,52) 122,mv9R(6),'nu_6'
      write(nout,52) 124,mv9R(7),'nu_7'
      write(nout,52) 126,mv9R(8),'nu_8'
      write(nout,52) 128,mv9R(9),'nu_9'

      write(nout,105)
      write(nout,51) 'NUMIX','Real part of the neutrino mixing matrix'
      do i=1,9
      do j=1,9
            write(nout,115) i,j,numixval(i,j),'Re(ZFv)',i,j
      end do
      end do
      write(nout,105)


      write(nout,51) 'SNSMIX','Mixing matrix of CP-even sneutrinos'
        do i=1,9
        do j=1,9
           write(nout,115) i,j,snsmixval(i,j),'ZSvS',i,j
        end do
        end do
      write(nout,105)

      write(nout,51) 'SNAMIX','Mixing matrix of CP-odd sneutrinos'
        do i=1,9
        do j=1,9
           write(nout,115) i,j,snpmixval(i,j),'ZSvA',i,j
        end do
        end do
      write(nout,105)

      call Yvoutput(Yvout,Yvmax,iimax,jjmax)


      write(nout,51) 'ReYv','Neutrino Yukawa matrix'
      do i=1,3
      do j=1,3
            write(nout,115) i,j,DBLE(Yvout(i,j)),'Re(Yv)',i,j
      end do
      end do
      
      endif
c end ND changed 7/2021
c ------------------------------------------------------------------- c
c The angle alpha in the Higgs sector and the Higgs mixing parameters c
c ------------------------------------------------------------------- c

      amudrbar = AMU/(1.D0+g1ew**2/16.D0/pi**2*3.D0/5.D0+
     .                 g2ew**2/16.D0/pi**2*3.D0/4.D0)
      if(qvalue(1).ne.0.d0)then
       qq = qvalue(1)
      else
       qq = amt
      endif
      write(nout,105)
      write(nout,54) 'HMIX Q= ',qq,'DRbar Higgs parameters'
      write(nout,52) 1,amudrbar,'mu(Q)'
      write(nout,52) 2,tgbet,'tanbeta(Q)'

      write(nout,105)
      write(nout,51) 'NMHMIX','Scalar Higgs mixing matrix R_H'
      write(nout,53) 1,1,nmhmixval(1,1),'R_H_11'
      write(nout,53) 1,2,nmhmixval(1,2),'R_H_12'
      write(nout,53) 1,3,nmhmixval(1,3),'R_H_13'
      write(nout,53) 2,1,nmhmixval(2,1),'R_H_21'
      write(nout,53) 2,2,nmhmixval(2,2),'R_H_22'
      write(nout,53) 2,3,nmhmixval(2,3),'R_H_23'
      write(nout,53) 3,1,nmhmixval(3,1),'R_H_31'
      write(nout,53) 3,2,nmhmixval(3,2),'R_H_32'
      write(nout,53) 3,3,nmhmixval(3,3),'R_H_33'

      write(nout,105)
      write(nout,51) 'NMAMIX','Pseudoscalar Higgs mixing matrix R_H'
      write(nout,53) 1,1,nmamixval(1,1),'R_A_11'
      write(nout,53) 1,2,nmamixval(1,2),'R_A_12'
      write(nout,53) 1,3,nmamixval(1,3),'R_A_13'
      write(nout,53) 2,1,nmamixval(2,1),'R_A_21'
      write(nout,53) 2,2,nmamixval(2,2),'R_A_22'
      write(nout,53) 2,3,nmamixval(2,3),'R_A_23'

c nhung added for the output of Higgs WFR factor 6/2021
      write(nout,105)
      write(nout,51) 'ReZFacA','Real part of CP-odd Higgs Zfactor'
      do i=1,2
      do j=1,2
      write(nout,123) i,j,DBLE(ZHfac(i,j)),'Re_ZFA',i,j
      end do
      end do
      write(nout,105)
      write(nout,51) 'ImZFacA','Imaginary part of CP-odd Higgs Zfactor'
      do i=1,2
      do j=1,2
      write(nout,123) i,j,DIMAG(ZHfac(i,j)),'Im_ZFA',i,j
      end do
      end do
      write(nout,105)
      write(nout,51) 'ReZFacH','Real part of CP-even Higgs Zfactor'
      do i=1,3
      do j=1,3
      write(nout,123) i,j,DBLE(ZHfac(i+2,j+2)),'Re_ZFH',i,j
      end do
      end do
      write(nout,105)
      write(nout,51) 'ImZFacH','Imaginary part of CP-even Higgs Zfactor'
      do i=1,3
      do j=1,3
      write(nout,123) i,j,DIMAG(ZHfac(i+2,j+2)),'Im_ZFH',i,j
      end do
      end do
c end nhung added for the output of Higgs WFR factor 6/2021

c --------------------- c
c The CKM mixing matrix c
c --------------------- c

      unlikely = -123456789D0
      r0 = 0.132d0
      e0 = 0.341d0
      fac1 = r0/dsqrt(r0**2+e0**2)
      fac2 = e0/dsqrt(r0**2+e0**2)
      if(vckmval(1).eq.unlikely)vckmval(1) = vus
      if(vckmval(2).eq.unlikely)vckmval(2) = vcb/vus**2
      if(vckmval(3).eq.unlikely)vckmval(3) = fac1*vub/vus/vcb
      if(vckmval(4).eq.unlikely)vckmval(4) = fac2*vub/vus/vcb
      write(nout,105)
      write(nout,51) 'VCKMIN','CKM mixing'
      write(nout,52) 1,vckmval(1),'lambda'
      write(nout,52) 2,vckmval(2),'A'
      write(nout,52) 3,vckmval(3),'rhobar'
      write(nout,52) 4,vckmval(4),'etabar'

c ------------------- c
c The gauge couplings c
c ------------------- c
 
      if(qvalue(2).ne.0.d0)then
       write(nout,105)
       write(nout,54) 'GAUGE Q=',qvalue(2),'The gauge couplings'
       if(gaugeval(1).ne.0.D0) then
          write(nout,55) 1,gaugeval(1),'gprime(Q) DRbar'
       endif
       if(gaugeval(2).ne.0.D0) then
          write(nout,55) 2,gaugeval(2),'g(Q) DRbar'
       endif
      endif

c ------------------------------------- c
c The trilinear couplings Au, Ad and Ae c
c ------------------------------------- c

      qq = amt
      if(qvalue(4).ne.0.d0)qq = qvalue(4)
      write(nout,105)
      if(auval(3,3).eq.unlikely.and.extval(11).ne.unlikely) then
         write(nout,54) 'AU Q=',extval(0),'The trilinear couplings'
      else
         write(nout,54) 'AU Q=',qq,'The trilinear couplings'
      endif
      unlikely = -123456789D0
      if(auval(1,1).ne.unlikely) then
         write(nout,53) 1,1,auval(1,1),'A_u(Q) DRbar'
      endif
      if(auval(2,2).ne.unlikely) then
         write(nout,53) 2,2,auval(2,2),'A_c(Q) DRbar'
      endif
      if(auval(3,3).ne.unlikely) then
         write(nout,53) 3,3,auval(3,3),'A_t(Q) DRbar'
      else
         write(nout,53) 3,3,au,'A_t(Q) DRbar'
      endif
      qq = amt
      if(qvalue(5).ne.0.d0)qq = qvalue(5)
      write(nout,105)
      if(adval(3,3).eq.unlikely.and.extval(12).ne.unlikely) then
         write(nout,54) 'AD Q=',extval(0),'The trilinear couplings'
      else
         write(nout,54) 'AD Q=',qq,'The trilinear couplings'
      endif
      if(adval(1,1).ne.unlikely) then
         write(nout,53) 1,1,adval(1,1),'A_d(Q) DRbar'
      endif
      if(adval(2,2).ne.unlikely) then
         write(nout,53) 2,2,adval(2,2),'A_s(Q) DRbar'
      endif
      if(adval(3,3).ne.unlikely) then
         write(nout,53) 3,3,adval(3,3),'A_b(Q) DRbar'
      else
         write(nout,53) 3,3,ad,'A_b(Q) DRbar'
      endif

      qq = amt
      if(qvalue(6).ne.0.d0)qq = qvalue(6)
      write(nout,105)
      if(aeval(3,3).eq.unlikely.and.extval(13).ne.unlikely) then
         write(nout,54) 'AE Q=',extval(0),'The trilinear couplings'
      else
         write(nout,54) 'AE Q=',qq,'The trilinear couplings'
      endif
      if(aeval(1,1).ne.unlikely) then
         write(nout,53) 1,1,aeval(1,1),'A_e(Q) DRbar'
      endif
      if(aeval(2,2).ne.unlikely) then
         write(nout,53) 2,2,aeval(2,2),'A_mu(Q) DRbar'
      endif
      if(aeval(3,3).ne.unlikely) then
         write(nout,53) 3,3,aeval(3,3),'A_tau(Q) DRbar'
      else
         write(nout,53) 3,3,al,'A_tau(Q) DRbar'
      endif

c ----------------------------- c
c The soft SUSY breaking masses c
c ----------------------------- c

      if(check(15).eq.1) then
         write(nout,105)
         write(nout,54) 'MSOFT Q=',scaleofewsb,'The soft SUSY breaking m
     .asses at the scale Q'
         unlikely = -123456789D0
         do ii=1,99,1
            if(msoftval(ii).ne.unlikely) then
               if(ii.ne.11.and.ii.ne.12.and.ii.ne.13.and.ii.ne.23.and.
     .            ii.ne.24.and.ii.ne.25.and.ii.ne.26) then
                  write(nout,52) ii,msoftval(ii),softcom(ii)
               endif
            endif
         end do
      else
         write(nout,105)
         write(nout,54) 'MSOFT Q=',scaleofewsb,'The soft SUSY breaking m
     .asses at the scale Q'
         cw=amw/amz
         sw=dsqrt(1-cw**2)
         tw=sw/cw
         am1=5.D0/3.D0*tw**2*am2
         am2=am2/(1.D0+g2ew**2/16.D0/pi**2*2.D0)
         write(nout,52) 1,am1val,'M_1(Q)'
         write(nout,52) 2,am2val,'M_2(Q)'
         write(nout,52) 31,amel1,'AMEL1'
         write(nout,52) 33,amel,'AMEL'
         write(nout,52) 34,amer1,'AMER1'
         write(nout,52) 36,amer,'AMER'
         write(nout,52) 41,amql1,'AMQL1'
         write(nout,52) 43,amsq,'AMSQ'
         write(nout,52) 44,amur1,'AMUR1'
         write(nout,52) 46,amur,'AMUR'
         write(nout,52) 47,amdr1,'AMDR1'
         write(nout,52) 49,amdr,'AMDR'
      endif

c ND changed 7/2021
      if(isstrue)then  
      write(nout,105)
      write(nout,51) 'LOWEN', 'Low-energy observables'  
      write(nout,52) 1, para_S, "BSM contribution to oblique parameter S"
      write(nout,52) 2, para_T, "BSM contribution to oblique parameter T"
      write(nout,52) 3, para_U, "BSM contribution to oblique parameter U"
      write(nout,52) 4, brmuega, "Br(mu -> e + gamma)"
      write(nout,52) 5, brtauega, "Br(tau -> e + gamma)"
      write(nout,52) 6, brtaumuga, "Br(tau -> mu + gamma)"

      write(nout,105)
      write(nout,51) 'ISSCONSTRAINTS' , 'Check constraints for NMSSM-ISS'
      if(isscheck(1))then
      write(nout,58) 1,0,'satisfying neutrino oscillation data'
      else
      write(nout,58) 1,1,'violating neutrino oscillation data'
      endif
      if(isscheck(2))then
      write(nout,58) 2,0,'satisfying sum of neutrino masses'
      else
      write(nout,58) 2,1,'violating sum of neutrino masses'
      endif

      if(isscheck(3))then
      write(nout,58) 3,0,'satisfying constraints of S,T,U parameters'
      else
      write(nout,58) 3,1,'violating constraints of S,T,U parameters'
      endif

      if(isscheck(4))then
      write(nout,58) 4,0,'satisfying charged lepton flavour violating 
     . process l_i->l_j + gamma'
      else
      write(nout,58) 4,1,'violating charged lepton flavour violating 
     . process l_i->l_j + gamma'
      endif

      if(isscheck(5))then
      write(nout,58) 5,0,'satisfying constraints on the violation of
     . the unitary 3x3 neutrino mixing matrix'
      else
      write(nout,58) 5,1,'violating constraints on the violation of 
     . the unitary 3x3 neutrino mixing matrix'
      endif  
      endif

c end ND changed 7/2021

c -------------------- c
c Neutral Higgs decays c
c -------------------- c

c --- H1 decays --- c

      write(nout,105)

      if(whitot(1).ne.0.D0) then

      write(nout,99)
      write(nout,100) 25,whitot(1),'H1 decays'

      write(nout,101)
      if(brhigbb(1).ne.0.D0) then
      write(nout,102) brhigbb(1),2,ib,ibb   ,'BR(H1-> b       bb     )'
      endif
      if(brhigll(1).ne.0.D0) then
      write(nout,102) brhigll(1),2,-itau,itau,'BR(H1-> tau+    tau-   )'
      endif
      if(brhigmm(1).ne.0.D0) then
      write(nout,102) brhigmm(1),2,-imu,imu ,'BR(H1-> mu+     mu-    )'
      endif
c ND added 7/2021
      if(isstrue)then
      if(brhignn(1).ne.0D0) then
      write(nout,102) brhignn(1),2,ine,ine  ,'BR(H1-> nu_i     nu_j  )'
      endif
      endif
c end ND added 7/2021
      if(brhigss(1).ne.0.D0) then
      write(nout,102) brhigss(1),2,is,isb   ,'BR(H1-> s       sb     )'
      endif
      if(brhigcc(1).ne.0.D0) then
      write(nout,102) brhigcc(1),2,ic,icb   ,'BR(H1-> c       cb     )'
      endif
      if(brhigtt(1).ne.0.D0) then
      write(nout,102) brhigtt(1),2,it,itb   ,'BR(H1-> t       tb     )' 
      endif
      if(brhiggg(1).ne.0.D0) then
      write(nout,102) brhiggg(1),2,igl,igl  ,'BR(H1-> g       g      )' 
      endif
      if(brhigga(1).ne.0.D0) then
      write(nout,102) brhigga(1),2,iga,iga  ,'BR(H1-> gam     gam    )' 
      endif
      if(brhigzga(1).ne.0.D0) then
      write(nout,102) brhigzga(1),2,iga,iz  ,'BR(H1-> Z       gam    )' 
      endif
      if(brhigww(1).ne.0.D0) then
      write(nout,102) brhigww(1),2,iwc,-iwc ,'BR(H1-> W+      W-     )' 
      endif
      if(brhigzz(1).ne.0.D0) then
      write(nout,102) brhigzz(1),2,iz,iz    ,'BR(H1-> Z       Z      )' 
      endif
      if(brhitohjhk(1,4,4).ne.0.D0) then
      write(nout,102) brhitohjhk(1,4,4),2,iha1,iha1,
     .        'BR(H1-> A1      A1     )' 
      endif
      if(brhitohjhk(1,4,5).ne.0.D0) then
      write(nout,102) brhitohjhk(1,4,5),2,iha1,iha2  ,
     .        'BR(H1-> A1      A2     )'
      endif
      if(brhitoajz(1,4).ne.0.D0) then
      write(nout,102) brhitoajz(1,4),2,iz,iha1,
     .        'BR(H1-> Z       A1     )' 
      endif
      if(brhitoajz(1,5).ne.0.D0) then
      write(nout,102) brhitoajz(1,5),2,iz,iha2,
     .        'BR(H1-> Z       A2     )' 
      endif
      if(brhitohpwm(1).ne.0.D0) then
      write(nout,102) brhitohpwm(1)/2.D0,2,iwc,-ihc,
     .        'BR(H1-> W+      H-     )'
      write(nout,102) brhitohpwm(1)/2.D0,2,-iwc,ihc,
     .     'BR(H1-> W-      H+     )'
      endif
      if(brwhigch(1,1,1).ne.0.D0) then
      write(nout,102) brwhigch(1,1,1),2,ic1,-ic1,
     .        'BR(H1-> ~chi_1+ ~chi_1-)' 
      endif
      if(brwhigch(1,2,2).ne.0.D0) then
      write(nout,102) brwhigch(1,2,2),2,ic2,-ic2,
     .        'BR(H1-> ~chi_2+ ~chi_2-)' 
      endif
      if(brwhigch(1,1,2).ne.0.D0) then
      write(nout,102) brwhigch(1,1,2),2,ic1,-ic2,
     .        'BR(H1-> ~chi_1+ ~chi_2-)' 
      endif
      if(brwhigch(1,2,1).ne.0.D0) then
      write(nout,102) brwhigch(1,2,1),2,ic2,-ic1,
     .        'BR(H1-> ~chi_2+ ~chi_1-)' 
      endif
      brwhigne1(1,1,2) = 2.D0*brwhigne(1,1,2) 
      brwhigne1(1,1,3) = 2.D0*brwhigne(1,1,3) 
      brwhigne1(1,1,4) = 2.D0*brwhigne(1,1,4)
      brwhigne1(1,1,5) = 2.D0*brwhigne(1,1,5)
      brwhigne1(1,2,3) = 2.D0*brwhigne(1,2,3) 
      brwhigne1(1,2,4) = 2.D0*brwhigne(1,2,4)  
      brwhigne1(1,2,5) = 2.D0*brwhigne(1,2,5)  
      brwhigne1(1,3,4) = 2.D0*brwhigne(1,3,4)  
      brwhigne1(1,3,5) = 2.D0*brwhigne(1,3,5)  
      brwhigne1(1,4,5) = 2.D0*brwhigne(1,4,5)  
      if(brwhigne(1,1,1).ne.0.D0) then
      write(nout,102) brwhigne(1,1,1),2,in1,in1 ,
     .        'BR(H1-> ~chi_10 ~chi_10)' 
      endif
      if(brwhigne(1,2,2).ne.0.D0) then
      write(nout,102) brwhigne(1,2,2),2,in2,in2 ,
     .        'BR(H1-> ~chi_20 ~chi_20)' 
      endif
      if(brwhigne(1,3,3).ne.0.D0) then
      write(nout,102) brwhigne(1,3,3),2,in3,in3 ,
     .        'BR(H1-> ~chi_30 ~chi_30)' 
      endif
      if(brwhigne(1,4,4).ne.0.D0) then
      write(nout,102) brwhigne(1,4,4),2,in4,in4 ,
     .        'BR(H1-> ~chi_40 ~chi_40)' 
      endif
      if(brwhigne(1,5,5).ne.0.D0) then
      write(nout,102) brwhigne(1,5,5),2,in5,in5 ,
     .        'BR(H1-> ~chi_50 ~chi_50)' 
      endif
      if(brwhigne(1,1,2).ne.0.D0) then
      write(nout,102) brwhigne1(1,1,2),2,in1,in2,
     .        'BR(H1-> ~chi_10 ~chi_20)' 
      endif
      if(brwhigne(1,1,3).ne.0.D0) then
      write(nout,102) brwhigne1(1,1,3),2,in1,in3,
     .        'BR(H1-> ~chi_10 ~chi_30)' 
      endif
      if(brwhigne(1,1,4).ne.0.D0) then
      write(nout,102) brwhigne1(1,1,4),2,in1,in4,
     .        'BR(H1-> ~chi_10 ~chi_40)' 
      endif
      if(brwhigne(1,1,5).ne.0.D0) then
      write(nout,102) brwhigne1(1,1,5),2,in1,in5,
     .        'BR(H1-> ~chi_10 ~chi_50)' 
      endif
      if(brwhigne(1,2,3).ne.0.D0) then
      write(nout,102) brwhigne1(1,2,3),2,in2,in3,
     .        'BR(H1-> ~chi_20 ~chi_30)' 
      endif
      if(brwhigne(1,2,4).ne.0.D0) then
      write(nout,102) brwhigne1(1,2,4),2,in2,in4,
     .        'BR(H1-> ~chi_20 ~chi_40)' 
      endif
      if(brwhigne(1,2,5).ne.0.D0) then
      write(nout,102) brwhigne1(1,2,5),2,in2,in5,
     .        'BR(H1-> ~chi_20 ~chi_50)' 
      endif
      if(brwhigne(1,3,4).ne.0.D0) then
      write(nout,102) brwhigne1(1,3,4),2,in3,in4,
     .        'BR(H1-> ~chi_30 ~chi_40)' 
      endif
      if(brwhigne(1,4,5).ne.0.D0) then
      write(nout,102) brwhigne1(1,4,5),2,in4,in5,
     .        'BR(H1-> ~chi_40 ~chi_50)' 
      endif

      bh1slel1 = brwhislel(1)/2.D0
      bh1sler1 = brwhisler(1)/2.D0
      bh1squl1 = brwhisqul(1)/2.d0
      bh1squr1 = brwhisqur(1)/2.d0
      bh1sqdl1 = brwhisqdl(1)/2.d0
      bh1sqdr1 = brwhisqdr(1)/2.d0
      if(bh1squl1.ne.0.D0) then
      write(nout,102) bh1squl1,2,isul,-isul  ,'BR(H1-> ~u_L    ~u_L*  )'
      endif
      if(bh1squr1.ne.0.D0) then
      write(nout,102) bh1squr1,2,isur,-isur  ,'BR(H1-> ~u_R    ~u_R*  )'
      endif
      if(bh1squl1.ne.0.D0) then
      write(nout,102) bh1squl1,2,iscl,-iscl  ,'BR(H1-> ~c_L    ~c_L*  )'
      endif
      if(bh1squr1.ne.0.D0) then
      write(nout,102) bh1squr1,2,iscr,-iscr  ,'BR(H1-> ~c_R    ~c_R*  )'
      endif
      if(brwhist(1,1,1).ne.0.D0) then
      write(nout,102) brwhist(1,1,1),2,ist1,-ist1,
     -        'BR(H1-> ~t_1    ~t_1*  )'
      endif
      if(brwhist(1,2,2).ne.0.D0) then
      write(nout,102) brwhist(1,2,2),2,ist2,-ist2,
     .        'BR(H1-> ~t_2    ~t_2*  )'
      endif
      if(brwhist(1,1,2).ne.0.D0) then
      write(nout,102) brwhist(1,1,2),2,ist1,-ist2,
     .        'BR(H1-> ~t_1    ~t_2*  )'
      endif
      if(brwhist(1,2,1).ne.0.D0) then
      write(nout,102) brwhist(1,2,1),2,ist2,-ist1,
     .        'BR(H1-> ~t_2    ~t_1*  )'
      endif
      if(bh1sqdl1.ne.0.D0) then
      write(nout,102) bh1sqdl1,2,isdl,-isdl  ,'BR(H1-> ~d_L    ~d_L*  )'
      endif
      if(bh1sqdr1.ne.0.D0) then
      write(nout,102) bh1sqdr1,2,isdr,-isdr  ,'BR(H1-> ~d_R    ~d_R*  )'
      endif
      if(bh1sqdl1.ne.0.D0) then
      write(nout,102) bh1sqdl1,2,issl,-issl  ,'BR(H1-> ~s_L    ~s_L*  )'
      endif
      if(bh1sqdr1.ne.0.D0) then
      write(nout,102) bh1sqdr1,2,issr,-issr  ,'BR(H1-> ~s_R    ~s_R*  )'
      endif
      if(brwhisb(1,1,1).ne.0.D0) then
      write(nout,102) brwhisb(1,1,1),2,isb1,-isb1,
     .        'BR(H1-> ~b_1    ~b_1*  )'
      endif
      if(brwhisb(1,2,2).ne.0.D0) then
      write(nout,102) brwhisb(1,2,2),2,isb2,-isb2,
     .        'BR(H1-> ~b_2    ~b_2*  )'
      endif
      if(brwhisb(1,1,2).ne.0.D0) then
      write(nout,102) brwhisb(1,1,2),2,isb1,-isb2,
     .        'BR(H1-> ~b_1    ~b_2*  )'
      endif
      if(brwhisb(1,2,1).ne.0.D0) then
      write(nout,102) brwhisb(1,2,1),2,isb2,-isb1,
     .        'BR(H1-> ~b_2    ~b_1*  )'
      endif
      if(bh1slel1.ne.0.D0) then
      write(nout,102) bh1slel1,2,isell,-isell,
     .        'BR(H1-> ~e_L-   ~e_L+  )'
      endif
      if(bh1sler1.ne.0.D0) then
      write(nout,102) bh1sler1,2,iselr,-iselr,
     .        'BR(H1-> ~e_R-   ~e_R+  )'
      endif
      if(bh1slel1.ne.0.D0) then
      write(nout,102) bh1slel1,2,ismul,-ismul,
     .        'BR(H1-> ~mu_L-  ~mu_L+ )'
      endif
      if(bh1sler1.ne.0.D0) then
      write(nout,102) bh1sler1,2,ismur,-ismur,
     .        'BR(H1-> ~mu_R-  ~mu_R+ )'
      endif
      if(brwhistau(1,1,1).ne.0.D0) then
      write(nout,102) brwhistau(1,1,1),2,istau1,-istau1,
     .        'BR(H1-> ~tau_1- ~tau_1+)'
      endif
      if(brwhistau(1,2,2).ne.0.D0) then
      write(nout,102) brwhistau(1,2,2),2,istau2,-istau2,
     -        'BR(H1-> ~tau_2- ~tau_2+)'
      endif
      if(brwhistau(1,1,2).ne.0.D0) then
      write(nout,102) brwhistau(1,1,2),2,istau1,-istau2,
     -        'BR(H1-> ~tau_1- ~tau_2+)'
      endif
      if(brwhistau(1,2,1).ne.0.D0) then
      write(nout,102) brwhistau(1,2,1),2,istau2,-istau1,
     -        'BR(H1-> ~tau_2- ~tau_1+)'
      endif
c ND changed 7/2021
      if (.not.isstrue)then
      bh1slnl1 = brwhislnl(1)/2.D0
      bh1slnl3 = brwhislnltau(1)
      if(bh1slnl1.ne.0.D0) then
      write(nout,102) bh1slnl1,2,inel,-inel  ,'BR(H1-> ~nu_eL  ~nu_eL*  
     . )'
      write(nout,102) bh1slnl1,2,inmul,-inmul,'BR(H1-> ~nu_muL ~nu_muL* 
     . )'
      endif
      if(bh1slnl3.ne.0.D0) then
      write(nout,102) bh1slnl3,2,intau1,-intau1,'BR(H1-> ~nu_tauL ~nu_ta
     .uL*)'
      endif
      else !!!
      bh1slnl1 = brwhislnl(1)
      if(bh1slnl1.ne.0D0) then
      write(nout,102) bh1slnl1,2,inel,-inel  ,'BR(H1-> ~nu_i  ~nu_j  
     . )'
      endif
      endif
c end ND changed 7/2021
      elseif(whitot(1).eq.0.D0) then
      write(nout,99)
c --- changed 21/1/2014
      write(nout,100) 25,0.000000000E+00,'H1 decays'
c --- end changed 21/1/2014

      endif

c --- H2 decays --- c

      write(nout,105)
      if(whitot(2).ne.0.D0) then
      write(nout,99)
      write(nout,100) 35,whitot(2),'H2 decays'

      write(nout,101)
      if(brhigbb(2).ne.0.D0) then
      write(nout,102) brhigbb(2),2,ib,ibb   ,'BR(H2-> b       bb     )'
      endif
      if(brhigll(2).ne.0.D0) then
      write(nout,102) brhigll(2),2,-itau,itau,'BR(H2-> tau+    tau-   )'
      endif
      if(brhigmm(2).ne.0.D0) then
      write(nout,102) brhigmm(2),2,-imu,imu ,'BR(H2-> mu+     mu-    )'
      endif

c ND added 7/2021
      if(isstrue)then
      if(brhignn(2).ne.0D0) then
      write(nout,102) brhignn(2),2,ine,ine  ,'BR(H2-> nu_i     nu_j  )'
      endif
      endif
c end ND added 7/2021

      if(brhigss(2).ne.0.D0) then
      write(nout,102) brhigss(2),2,is,isb   ,'BR(H2-> s       sb     )'
      endif
      if(brhigcc(2).ne.0.D0) then
      write(nout,102) brhigcc(2),2,ic,icb   ,'BR(H2-> c       cb     )'
      endif
      if(brhigtt(2).ne.0.D0) then
      write(nout,102) brhigtt(2),2,it,itb   ,'BR(H2-> t       tb     )' 
      endif
      if(brhiggg(2).ne.0.D0) then
      write(nout,102) brhiggg(2),2,igl,igl  ,'BR(H2-> g       g      )' 
      endif
      if(brhigga(2).ne.0.D0) then
      write(nout,102) brhigga(2),2,iga,iga  ,'BR(H2-> gam     gam    )' 
      endif
      if(brhigzga(2).ne.0.D0) then
      write(nout,102) brhigzga(2),2,iz,iga  ,'BR(H2-> Z       gam    )' 
      endif
      if(brhigww(2).ne.0.D0) then
      write(nout,102) brhigww(2),2,iwc,-iwc ,'BR(H2-> W+      W-     )' 
      endif
      if(brhigzz(2).ne.0.D0) then
      write(nout,102) brhigzz(2),2,iz,iz    ,'BR(H2-> Z       Z      )' 
      endif
      if(brhitohjhk(2,1,1).ne.0.D0) then
      write(nout,102) brhitohjhk(2,1,1),2,ih1,ih1
     .        ,'BR(H2-> H1      H1     )' 
      endif
      if(brhitohjhk(2,4,4).ne.0.D0) then
      write(nout,102) brhitohjhk(2,4,4),2,iha1,iha1  ,
     .        'BR(H2-> A1      A1     )' 
      endif
      if(brhitohjhk(2,4,5).ne.0.D0) then
      write(nout,102) brhitohjhk(2,4,5),2,iha1,iha2  ,
     .        'BR(H2-> A1      A2     )' 
      endif
      if(brhitohjhk(2,5,5).ne.0.D0) then
      write(nout,102) brhitohjhk(2,5,5),2,iha2,iha2  ,
     .        'BR(H2-> A2      A2     )' 
      endif
      if(brhitoajz(2,4).ne.0.D0) then
      write(nout,102) brhitoajz(2,4),2,iz,iha1,
     .        'BR(H2-> Z       A1     )' 
      endif
      if(brhitoajz(2,5).ne.0.D0) then
      write(nout,102) brhitoajz(2,5),2,iz,iha2,
     .        'BR(H2-> Z       A2     )' 
      endif
      if(brhitohpwm(2).ne.0.D0) then
      write(nout,102) brhitohpwm(2)/2.D0,2,iwc,-ihc,
     .        'BR(H2-> W+      H-     )'
      write(nout,102) brhitohpwm(2)/2.D0,2,-iwc,ihc,
     .     'BR(H2-> W-      H+     )'
      endif
      if(brwhigch(2,1,1).ne.0.D0) then
      write(nout,102) brwhigch(2,1,1),2,ic1,-ic1,
     .        'BR(H2-> ~chi_1+ ~chi_1-)' 
      endif
      if(brwhigch(2,2,2).ne.0.D0) then
      write(nout,102) brwhigch(2,2,2),2,ic2,-ic2,
     .        'BR(H2-> ~chi_2+ ~chi_2-)' 
      endif
      if(brwhigch(2,1,2).ne.0.D0) then
      write(nout,102) brwhigch(2,1,2),2,ic1,-ic2,
     .        'BR(H2-> ~chi_1+ ~chi_2-)' 
      endif
      if(brwhigch(2,2,1).ne.0.D0) then
      write(nout,102) brwhigch(2,2,1),2,ic2,-ic1,
     .        'BR(H2-> ~chi_2+ ~chi_1-)' 
      endif
      if(brwhigne(2,1,1).ne.0.D0) then
      write(nout,102) brwhigne(2,1,1),2,in1,in1 ,
     .        'BR(H2-> ~chi_10 ~chi_10)' 
      endif
      if(brwhigne(2,2,2).ne.0.D0) then
      write(nout,102) brwhigne(2,2,2),2,in2,in2 ,
     .        'BR(H2-> ~chi_20 ~chi_20)' 
      endif
      if(brwhigne(2,3,3).ne.0.D0) then
      write(nout,102) brwhigne(2,3,3),2,in3,in3 ,
     .        'BR(H2-> ~chi_30 ~chi_30)' 
      endif
      if(brwhigne(2,4,4).ne.0.D0) then
      write(nout,102) brwhigne(2,4,4),2,in4,in4 ,
     .        'BR(H2-> ~chi_40 ~chi_40)' 
      endif
      if(brwhigne(2,5,5).ne.0.D0) then
      write(nout,102) brwhigne(2,5,5),2,in5,in5 ,
     .        'BR(H2-> ~chi_50 ~chi_50)' 
      endif
      h2brsn1(1,2) = 2.D0*brwhigne(2,1,2) 
      h2brsn1(1,3) = 2.D0*brwhigne(2,1,3) 
      h2brsn1(1,4) = 2.D0*brwhigne(2,1,4)
      h2brsn1(1,5) = 2.D0*brwhigne(2,1,5)
      h2brsn1(2,3) = 2.D0*brwhigne(2,2,3) 
      h2brsn1(2,4) = 2.D0*brwhigne(2,2,4)  
      h2brsn1(2,5) = 2.D0*brwhigne(2,2,5)  
      h2brsn1(3,4) = 2.D0*brwhigne(2,3,4)  
      h2brsn1(3,5) = 2.D0*brwhigne(2,3,5)  
      h2brsn1(4,5) = 2.D0*brwhigne(2,4,5)  
      if(h2brsn1(1,2).ne.0.D0) then
      write(nout,102) h2brsn1(1,2),2,in1,in2,'BR(H2-> ~chi_10 ~chi_20)' 
      endif
      if(h2brsn1(1,3).ne.0.D0) then
      write(nout,102) h2brsn1(1,3),2,in1,in3,'BR(H2-> ~chi_10 ~chi_30)' 
      endif
      if(h2brsn1(1,4).ne.0.D0) then
      write(nout,102) h2brsn1(1,4),2,in1,in4,'BR(H2-> ~chi_10 ~chi_40)' 
      endif
      if(h2brsn1(1,5).ne.0.D0) then
      write(nout,102) h2brsn1(1,5),2,in1,in5,'BR(H2-> ~chi_10 ~chi_50)' 
      endif
      if(h2brsn1(2,3).ne.0.D0) then
      write(nout,102) h2brsn1(2,3),2,in2,in3,'BR(H2-> ~chi_20 ~chi_30)' 
      endif
      if(h2brsn1(2,4).ne.0.D0) then
      write(nout,102) h2brsn1(2,4),2,in2,in4,'BR(H2-> ~chi_20 ~chi_40)' 
      endif
      if(h2brsn1(2,5).ne.0.D0) then
      write(nout,102) h2brsn1(2,5),2,in2,in5,'BR(H2-> ~chi_20 ~chi_50)' 
      endif
      if(h2brsn1(3,4).ne.0.D0) then
      write(nout,102) h2brsn1(3,4),2,in3,in4,'BR(H2-> ~chi_30 ~chi_40)' 
      endif
      if(h2brsn1(3,5).ne.0.D0) then
      write(nout,102) h2brsn1(3,5),2,in3,in5,'BR(H2-> ~chi_30 ~chi_50)' 
      endif
      if(h2brsn1(4,5).ne.0.D0) then
      write(nout,102) h2brsn1(4,5),2,in4,in5,'BR(H2-> ~chi_40 ~chi_50)' 
      endif
      bh2slel1 = brwhislel(2)/2.D0
      bh2sler1 = brwhisler(2)/2.D0
      bh2squl1 = brwhisqul(2)/2.d0
      bh2squr1 = brwhisqur(2)/2.d0
      bh2sqdl1 = brwhisqdl(2)/2.d0
      bh2sqdr1 = brwhisqdr(2)/2.d0
      if(bh2squl1.ne.0.D0) then
      write(nout,102) bh2squl1,2,isul,-isul  ,'BR(H2-> ~u_L    ~u_L*  )'
      endif
      if(bh2squr1.ne.0.D0) then
      write(nout,102) bh2squr1,2,isur,-isur  ,'BR(H2-> ~u_R    ~u_R*  )'
      endif
      if(bh2squl1.ne.0.D0) then
      write(nout,102) bh2squl1,2,iscl,-iscl  ,'BR(H2-> ~c_L    ~c_L*  )'
      endif
      if(bh2squr1.ne.0.D0) then
      write(nout,102) bh2squr1,2,iscr,-iscr  ,'BR(H2-> ~c_R    ~c_R*  )'
      endif
      if(brwhist(2,1,1).ne.0.D0) then
      write(nout,102) brwhist(2,1,1),2,ist1,-ist1,
     .        'BR(H2-> ~t_1    ~t_1*  )'
      endif
      if(brwhist(2,2,2).ne.0.D0) then
      write(nout,102) brwhist(2,2,2),2,ist2,-ist2,
     .        'BR(H2-> ~t_2    ~t_2*  )'
      endif
      if(brwhist(2,1,2).ne.0.D0) then
      write(nout,102) brwhist(2,1,2),2,ist1,-ist2,
     .        'BR(H2-> ~t_1    ~t_2*  )'
      endif
      if(brwhist(2,2,1).ne.0.D0) then
      write(nout,102) brwhist(2,2,1),2,ist2,-ist1,
     .        'BR(H2-> ~t_2    ~t_1*  )'
      endif
      if(bh2sqdl1.ne.0.D0) then
      write(nout,102) bh2sqdl1,2,isdl,-isdl  ,'BR(H2-> ~d_L    ~d_L*  )'
      endif
      if(bh2sqdr1.ne.0.D0) then
      write(nout,102) bh2sqdr1,2,isdr,-isdr  ,'BR(H2-> ~d_R    ~d_R*  )'
      endif
      if(bh2sqdl1.ne.0.D0) then
      write(nout,102) bh2sqdl1,2,issl,-issl  ,'BR(H2-> ~s_L    ~s_L*  )'
      endif
      if(bh2sqdr1.ne.0.D0) then
      write(nout,102) bh2sqdr1,2,issr,-issr  ,'BR(H2-> ~s_R    ~s_R*  )'
      endif
      if(brwhisb(2,1,1).ne.0.D0) then
      write(nout,102) brwhisb(2,1,1),2,isb1,-isb1,
     .        'BR(H2-> ~b_1    ~b_1*  )'
      endif
      if(brwhisb(2,2,2).ne.0.D0) then
      write(nout,102) brwhisb(2,2,2),2,isb2,-isb2,
     .        'BR(H2-> ~b_2    ~b_2*  )'
      endif
      if(brwhisb(2,1,2).ne.0.D0) then
      write(nout,102) brwhisb(2,1,2),2,isb1,-isb2,
     .        'BR(H2-> ~b_1    ~b_2*  )'
      endif
      if(brwhisb(2,2,1).ne.0.D0) then
      write(nout,102) brwhisb(2,2,1),2,isb2,-isb1,
     .        'BR(H2-> ~b_2    ~b_1*  )'
      endif
      if(bh2slel1.ne.0.D0) then
      write(nout,102) bh2slel1,2,isell,-isell,'BR(H2-> ~e_L-   ~e_L+  )'
      endif
      if(bh2sler1.ne.0.D0) then
      write(nout,102) bh2sler1,2,iselr,-iselr,'BR(H2-> ~e_R-   ~e_R+  )'
      endif
      if(bh2slel1.ne.0.D0) then
      write(nout,102) bh2slel1,2,ismul,-ismul,'BR(H2-> ~mu_L-  ~mu_L+ )'
      endif
      if(bh2sler1.ne.0.D0) then
      write(nout,102) bh2sler1,2,ismur,-ismur,'BR(H2-> ~mu_R-  ~mu_R+ )'
      endif
      if(brwhistau(2,1,1).ne.0.D0) then
      write(nout,102) brwhistau(2,1,1),2,istau1,-istau1,
     .        'BR(H2-> ~tau_1- ~tau_1+)'
      endif
      if(brwhistau(2,2,2).ne.0.D0) then
      write(nout,102) brwhistau(2,2,2),2,istau2,-istau2,
     .        'BR(H2-> ~tau_2- ~tau_2+)'
      endif
      if(brwhistau(2,1,2).ne.0.D0) then
      write(nout,102) brwhistau(2,1,2),2,istau1,-istau2,
     .        'BR(H2-> ~tau_1- ~tau_2+)'
      endif
      if(brwhistau(2,2,1).ne.0.D0) then
      write(nout,102) brwhistau(2,2,1),2,istau2,-istau1,
     .        'BR(H2-> ~tau_2- ~tau_1+)'
      endif
c ND changed 7/2021
      if (.not.isstrue)then
      bh2slnl1 = brwhislnl(2)/2.D0
      bh2slnl3 = brwhislnltau(2)
      if(bh2slnl1.ne.0.D0) then
      write(nout,102) bh2slnl1,2,inel,-inel  ,'BR(H2-> ~nu_eL  ~nu_eL*  
     . )'
      write(nout,102) bh2slnl1,2,inmul,-inmul,'BR(H2-> ~nu_muL ~nu_muL* 
     . )'
      endif
      if(bh2slnl3.ne.0.D0) then
      write(nout,102) bh2slnl3,2,intau1,-intau1,'BR(H2-> ~nu_tauL ~nu_ta
     .uL*)'
      endif

      else !!!
      bh2slnl1 = brwhislnl(2)
      if(bh2slnl1.ne.0D0) then
      write(nout,102) bh2slnl1,2,inel,-inel  ,'BR(H2-> ~nu_i  ~nu_j  
     . )'
      endif
      endif
c end ND changed 7/2021

      elseif(whitot(2).eq.0.D0) then
      write(nout,99)
      write(nout,100) 35,0.000000000E+00,'H2 decays'

      endif

c --- H3 decays --- c

      write(nout,105)
      if(whitot(3).ne.0.D0) then
      write(nout,99)
      write(nout,100) 45,whitot(3),'H3 decays'

      write(nout,101)
      if(brhigbb(3).ne.0.D0) then
      write(nout,102) brhigbb(3),2,ib,ibb   ,'BR(H3-> b       bb     )'
      endif
      if(brhigll(3).ne.0.D0) then
      write(nout,102) brhigll(3),2,-itau,itau,'BR(H3-> tau+    tau-   )'
      endif
      if(brhigmm(3).ne.0.D0) then
      write(nout,102) brhigmm(3),2,-imu,imu ,'BR(H3-> mu+     mu-    )'
      endif
c ND added 7/2021
      if(isstrue)then
      if(brhignn(3).ne.0D0) then
      write(nout,102) brhignn(3),2,ine,ine  ,'BR(H3-> nu_i     nu_j  )'
      endif
      endif
c end ND added 7/2021

      if(brhigss(3).ne.0.D0) then
      write(nout,102) brhigss(3),2,is,isb   ,'BR(H3-> s       sb     )'
      endif
      if(brhigcc(3).ne.0.D0) then
      write(nout,102) brhigcc(3),2,ic,icb   ,'BR(H3-> c       cb     )'
      endif
      if(brhigtt(3).ne.0.D0) then
      write(nout,102) brhigtt(3),2,it,itb   ,'BR(H3-> t       tb     )' 
      endif
      if(brhiggg(3).ne.0.D0) then
      write(nout,102) brhiggg(3),2,igl,igl  ,'BR(H3-> g       g      )' 
      endif
      if(brhigga(3).ne.0.D0) then
      write(nout,102) brhigga(3),2,iga,iga  ,'BR(H3-> gam     gam    )' 
      endif
      if(brhigzga(3).ne.0.D0) then
      write(nout,102) brhigzga(3),2,iz,iga  ,'BR(H3-> Z       gam    )' 
      endif
      if(brhigww(3).ne.0.D0) then
      write(nout,102) brhigww(3),2,iwc,-iwc ,'BR(H3-> W+      W-     )' 
      endif
      if(brhigzz(3).ne.0.D0) then
      write(nout,102) brhigzz(3),2,iz,iz    ,'BR(H3-> Z       Z      )' 
      endif
      if(brhitohjhk(3,1,1).ne.0.D0) then
      write(nout,102) brhitohjhk(3,1,1),2,ih1,ih1,
     .        'BR(H3-> H1      H1     )' 
      endif
      if(brhitohjhk(3,1,2).ne.0.D0) then
      write(nout,102) brhitohjhk(3,1,2),2,ih1,ih2    ,
     .        'BR(H3-> H1      H2     )' 
      endif
      if(brhitohjhk(3,2,2).ne.0.D0) then
      write(nout,102) brhitohjhk(3,2,2),2,ih2,ih2    ,
     .        'BR(H3-> H2      H2     )' 
      endif
      if(brhitohjhk(3,4,4).ne.0.D0) then
      write(nout,102) brhitohjhk(3,4,4),2,iha1,iha1  ,
     .        'BR(H3-> A1      A1     )' 
      endif
      if(brhitohjhk(3,4,5).ne.0.D0) then
      write(nout,102) brhitohjhk(3,4,5),2,iha1,iha2  ,
     .        'BR(H3-> A1      A2     )' 
      endif
      if(brhitohjhk(3,5,5).ne.0.D0) then
      write(nout,102) brhitohjhk(3,5,5),2,iha2,iha2  ,
     .        'BR(H3-> A2      A2     )' 
      endif
      if(brhitoajz(3,4).ne.0.D0) then
      write(nout,102) brhitoajz(3,4),2,iz,iha1     ,
     .        'BR(H3-> Z       A1     )' 
      endif
      if(brhitoajz(3,5).ne.0.D0) then
      write(nout,102) brhitoajz(3,5),2,iz,iha2     ,
     .        'BR(H3-> Z       A2     )' 
      endif
      if(brhitohpwm(3).ne.0.D0) then
      write(nout,102) brhitohpwm(3)/2.D0,2,iwc,-ihc,
     .        'BR(H3-> W+      H-     )'
      write(nout,102) brhitohpwm(3)/2.D0,2,-iwc,ihc,
     .        'BR(H3-> W-      H+     )'
      endif
      if(brwhigch(3,1,1).ne.0.D0) then
      write(nout,102) brwhigch(3,1,1),2,ic1,-ic1,
     .        'BR(H3-> ~chi_1+ ~chi_1-)' 
      endif
      if(brwhigch(3,2,2).ne.0.D0) then
      write(nout,102) brwhigch(3,2,2),2,ic2,-ic2,
     .        'BR(H3-> ~chi_2+ ~chi_2-)' 
      endif
      if(brwhigch(3,1,2).ne.0.D0) then
      write(nout,102) brwhigch(3,1,2),2,ic1,-ic2,
     .        'BR(H3-> ~chi_1+ ~chi_2-)' 
      endif
      if(brwhigch(3,2,1).ne.0.D0) then
      write(nout,102) brwhigch(3,2,1),2,ic2,-ic1,
     .        'BR(H3-> ~chi_2+ ~chi_1-)' 
      endif
      if(brwhigne(3,1,1).ne.0.D0) then
      write(nout,102) brwhigne(3,1,1),2,in1,in1 ,
     .        'BR(H3-> ~chi_10 ~chi_10)' 
      endif
      if(brwhigne(3,2,2).ne.0.D0) then
      write(nout,102) brwhigne(3,2,2),2,in2,in2 ,
     .        'BR(H3-> ~chi_20 ~chi_20)' 
      endif
      if(brwhigne(3,3,3).ne.0.D0) then
      write(nout,102) brwhigne(3,3,3),2,in3,in3 ,
     .        'BR(H3-> ~chi_30 ~chi_30)' 
      endif
      if(brwhigne(3,4,4).ne.0.D0) then
      write(nout,102) brwhigne(3,4,4),2,in4,in4 ,
     .        'BR(H3-> ~chi_40 ~chi_40)' 
      endif
      if(brwhigne(3,5,5).ne.0.D0) then
      write(nout,102) brwhigne(3,5,5),2,in5,in5 ,
     .        'BR(H3-> ~chi_50 ~chi_50)' 
      endif
      h3brsn1(1,2) = 2.D0*brwhigne(3,1,2) 
      h3brsn1(1,3) = 2.D0*brwhigne(3,1,3) 
      h3brsn1(1,4) = 2.D0*brwhigne(3,1,4)
      h3brsn1(1,5) = 2.D0*brwhigne(3,1,5)
      h3brsn1(2,3) = 2.D0*brwhigne(3,2,3) 
      h3brsn1(2,4) = 2.D0*brwhigne(3,2,4)  
      h3brsn1(2,5) = 2.D0*brwhigne(3,2,5)  
      h3brsn1(3,4) = 2.D0*brwhigne(3,3,4)  
      h3brsn1(3,5) = 2.D0*brwhigne(3,3,5)  
      h3brsn1(4,5) = 2.D0*brwhigne(3,4,5)  
      if(h3brsn1(1,2).ne.0.D0) then
      write(nout,102) h3brsn1(1,2),2,in1,in2,'BR(H3-> ~chi_10 ~chi_20)' 
      endif
      if(h3brsn1(1,3).ne.0.D0) then
      write(nout,102) h3brsn1(1,3),2,in1,in3,'BR(H3-> ~chi_10 ~chi_30)' 
      endif
      if(h3brsn1(1,4).ne.0.D0) then
      write(nout,102) h3brsn1(1,4),2,in1,in4,'BR(H3-> ~chi_10 ~chi_40)' 
      endif
      if(h3brsn1(1,5).ne.0.D0) then
      write(nout,102) h3brsn1(1,5),2,in1,in5,'BR(H3-> ~chi_10 ~chi_50)' 
      endif
      if(h3brsn1(2,3).ne.0.D0) then
      write(nout,102) h3brsn1(2,3),2,in2,in3,'BR(H3-> ~chi_20 ~chi_30)' 
      endif
      if(h3brsn1(2,4).ne.0.D0) then
      write(nout,102) h3brsn1(2,4),2,in2,in4,'BR(H3-> ~chi_20 ~chi_40)' 
      endif
      if(h3brsn1(2,5).ne.0.D0) then
      write(nout,102) h3brsn1(2,5),2,in2,in5,'BR(H3-> ~chi_20 ~chi_50)' 
      endif
      if(h3brsn1(3,4).ne.0.D0) then
      write(nout,102) h3brsn1(3,4),2,in3,in4,'BR(H3-> ~chi_30 ~chi_40)' 
      endif
      if(h3brsn1(3,5).ne.0.D0) then
      write(nout,102) h3brsn1(3,5),2,in3,in5,'BR(H3-> ~chi_30 ~chi_50)' 
      endif
      if(h3brsn1(4,5).ne.0.D0) then
      write(nout,102) h3brsn1(4,5),2,in4,in5,'BR(H3-> ~chi_40 ~chi_50)' 
      endif

      bh3slel1 = brwhislel(3)/2.D0
      bh3sler1 = brwhisler(3)/2.D0
      bh3squl1 = brwhisqul(3)/2.d0
      bh3squr1 = brwhisqur(3)/2.d0
      bh3sqdl1 = brwhisqdl(3)/2.d0
      bh3sqdr1 = brwhisqdr(3)/2.d0
      if(bh3squl1.ne.0.D0) then
      write(nout,102) bh3squl1,2,isul,-isul  ,'BR(H3-> ~u_L    ~u_L*  )'
      endif
      if(bh3squr1.ne.0.D0) then
      write(nout,102) bh3squr1,2,isur,-isur  ,'BR(H3-> ~u_R    ~u_R*  )'
      endif
      if(bh3squl1.ne.0.D0) then
      write(nout,102) bh3squl1,2,iscl,-iscl  ,'BR(H3-> ~c_L    ~c_L*  )'
      endif
      if(bh3squr1.ne.0.D0) then
      write(nout,102) bh3squr1,2,iscr,-iscr  ,'BR(H3-> ~c_R    ~c_R*  )'
      endif
      if(brwhist(3,1,1).ne.0.D0) then
      write(nout,102) brwhist(3,1,1),2,ist1,-ist1,
     .        'BR(H3-> ~t_1    ~t_1*  )'
      endif
      if(brwhist(3,2,2).ne.0.D0) then
      write(nout,102) brwhist(3,2,2),2,ist2,-ist2,
     .        'BR(H3-> ~t_2    ~t_2*  )'
      endif
      if(brwhist(3,1,2).ne.0.D0) then
      write(nout,102) brwhist(3,1,2),2,ist1,-ist2,
     .        'BR(H3-> ~t_1    ~t_2*  )'
      endif
      if(brwhist(3,2,1).ne.0.D0) then
      write(nout,102) brwhist(3,2,1),2,ist2,-ist1,
     .        'BR(H3-> ~t_2    ~t_1*  )'
      endif
      if(bh3sqdl1.ne.0.D0) then
      write(nout,102) bh3sqdl1,2,isdl,-isdl  ,'BR(H3-> ~d_L    ~d_L*  )'
      endif
      if(bh3sqdr1.ne.0.D0) then
      write(nout,102) bh3sqdr1,2,isdr,-isdr  ,'BR(H3-> ~d_R    ~d_R*  )'
      endif
      if(bh3sqdl1.ne.0.D0) then
      write(nout,102) bh3sqdl1,2,issl,-issl  ,'BR(H3-> ~s_L    ~s_L*  )'
      endif
      if(bh3sqdr1.ne.0.D0) then
      write(nout,102) bh3sqdr1,2,issr,-issr  ,'BR(H3-> ~s_R    ~s_R*  )'
      endif
      if(brwhisb(3,1,1).ne.0.D0) then
      write(nout,102) brwhisb(3,1,1),2,isb1,-isb1,
     .        'BR(H3-> ~b_1    ~b_1*  )'
      endif
      if(brwhisb(3,2,2).ne.0.D0) then
      write(nout,102) brwhisb(3,2,2),2,isb2,-isb2,
     .        'BR(H3-> ~b_2    ~b_2*  )'
      endif
      if(brwhisb(3,1,2).ne.0.D0) then
      write(nout,102) brwhisb(3,1,2),2,isb1,-isb2,
     .        'BR(H3-> ~b_1    ~b_2*  )'
      endif
      if(brwhisb(3,2,1).ne.0.D0) then
      write(nout,102) brwhisb(3,2,1),2,isb2,-isb1,
     .        'BR(H3-> ~b_2    ~b_1*  )'
      endif
      if(bh3slel1.ne.0.D0) then
      write(nout,102) bh3slel1,2,isell,-isell,'BR(H3-> ~e_L-   ~e_L+  )'
      endif
      if(bh3sler1.ne.0.D0) then
      write(nout,102) bh3sler1,2,iselr,-iselr,'BR(H3-> ~e_R-   ~e_R+  )'
      endif
      if(bh3slel1.ne.0.D0) then
      write(nout,102) bh3slel1,2,ismul,-ismul,'BR(H3-> ~mu_L-  ~mu_L+ )'
      endif
      if(bh3sler1.ne.0.D0) then
      write(nout,102) bh3sler1,2,ismur,-ismur,'BR(H3-> ~mu_R-  ~mu_R+ )'
      endif
      if(brwhistau(3,1,1).ne.0.D0) then
      write(nout,102) brwhistau(3,1,1),2,istau1,-istau1,
     .        'BR(H3-> ~tau_1- ~tau_1+)'
      endif
      if(brwhistau(3,2,2).ne.0.D0) then
      write(nout,102) brwhistau(3,2,2),2,istau2,-istau2,
     .        'BR(H3-> ~tau_2- ~tau_2+)'
      endif
      if(brwhistau(3,1,2).ne.0.D0) then
      write(nout,102) brwhistau(3,1,2),2,istau1,-istau2,
     .        'BR(H3-> ~tau_1- ~tau_2+)'
      endif
      if(brwhistau(3,2,1).ne.0.D0) then
      write(nout,102) brwhistau(3,2,1),2,istau2,-istau1,
     .        'BR(H3-> ~tau_2- ~tau_1+)'
      endif
c ND changed 7/2021
      if (.not.isstrue)then
      bh3slnl1 = brwhislnl(3)/2.D0
      bh3slnl3 = brwhislnltau(3)
      if(bh3slnl1.ne.0.D0) then
      write(nout,102) bh3slnl1,2,inel,-inel  ,'BR(H3-> ~nu_eL  ~nu_eL*  
     . )'
      write(nout,102) bh3slnl1,2,inmul,-inmul,'BR(H3-> ~nu_muL ~nu_muL* 
     . )'
      endif
      if(bh3slnl3.ne.0.D0) then
      write(nout,102) bh3slnl3,2,intau1,-intau1,'BR(H3-> ~nu_tauL ~nu_ta
     .uL*)'
      endif

      else !!!
      bh3slnl1 = brwhislnl(3)
      if(bh3slnl1.ne.0D0) then
      write(nout,102) bh3slnl1,2,inel,-inel  ,'BR(H3-> ~nu_i  ~nu_j  
     . )'
      endif
      endif
c end ND changed 7/2021

      elseif(whitot(3).eq.0.D0) then
      write(nout,99)
      write(nout,100) 45,0.000000000E+00,'H3 decays'

      endif

c --- A1 decays --- c

      write(nout,105)

      if(whatot(1).ne.0.D0) then
      write(nout,99)
      write(nout,100) 36,whatot(1),'A1 decays'

      write(nout,101)
      if(brhabb(1).ne.0.D0) then
      write(nout,102) brhabb(1),2,ib,ibb    ,'BR(A1-> b       bb     )'
      endif
      if(brhall(1).ne.0.D0) then
      write(nout,102) brhall(1),2,-itau,itau,'BR(A1-> tau+    tau-   )'
      endif
      if(brhamm(1).ne.0.D0) then
      write(nout,102) brhamm(1),2,-imu,imu  ,'BR(A1-> mu+     mu-    )'
      endif
c ND added 7/2021
      if(isstrue)then
      if(brhann(1).ne.0D0) then
      write(nout,102) brhann(1),2,ine,ine  ,'BR(A1-> nu_i     nu_j  )'
      endif
      endif
c end ND added 7/2021
      if(brhass(1).ne.0.D0) then
      write(nout,102) brhass(1),2,is,isb    ,'BR(A1-> s       sb     )'
      endif
      if(brhacc(1).ne.0.D0) then
      write(nout,102) brhacc(1),2,ic,icb    ,'BR(A1-> c       cb     )'
      endif
      if(brhatt(1).ne.0.D0) then
      write(nout,102) brhatt(1),2,it,itb    ,'BR(A1-> t       tb     )' 
      endif
      if(brhagg(1).ne.0.D0) then
      write(nout,102) brhagg(1),2,igl,igl   ,'BR(A1-> g       g      )' 
      endif
      if(brhaga(1).ne.0.D0) then
      write(nout,102) brhaga(1),2,iga,iga   ,'BR(A1-> gam     gam    )' 
      endif
      if(brhazga(1).ne.0.D0) then
      write(nout,102) brhazga(1),2,iz,iga   ,'BR(A1-> Z       gam    )' 
      endif
      if(brhjaiz(1,1).ne.0.D0) then
      write(nout,102) brhjaiz(1,1),2,iz,ih1 ,'BR(A1-> Z       H1     )' 
      endif
      if(brhjaiz(1,2).ne.0.D0) then
      write(nout,102) brhjaiz(1,2),2,iz,ih2 ,'BR(A1-> Z       H2     )' 
      endif
      if(brhjaiz(1,3).ne.0.D0) then
      write(nout,102) brhjaiz(1,3),2,iz,ih3 ,'BR(A1-> Z       H3     )' 
      endif
      if(brwhaich(1,1,1).ne.0.D0) then
      write(nout,102) brwhaich(1,1,1),2,ic1,-ic1,
     .        'BR(A1-> ~chi_1+ ~chi_1-)'
      endif
      if(brwhaich(1,2,2).ne.0.D0) then
      write(nout,102) brwhaich(1,2,2),2,ic2,-ic2,
     .        'BR(A1-> ~chi_2+ ~chi_2-)'
      endif
      if(brwhaich(1,1,2).ne.0.D0) then
      write(nout,102) brwhaich(1,1,2),2,ic1,-ic2,
     .        'BR(A1-> ~chi_1+ ~chi_2-)'
      endif
      if(brwhaich(1,2,1).ne.0.D0) then
      write(nout,102) brwhaich(1,2,1),2,ic2,-ic1,
     .        'BR(A1-> ~chi_2+ ~chi_1-)'
      endif
      ha1brsn1(1,2) = 2.D0*brwhaine(1,1,2) 
      ha1brsn1(1,3) = 2.D0*brwhaine(1,1,3) 
      ha1brsn1(1,4) = 2.D0*brwhaine(1,1,4)
      ha1brsn1(1,5) = 2.D0*brwhaine(1,1,5)
      ha1brsn1(2,3) = 2.D0*brwhaine(1,2,3) 
      ha1brsn1(2,4) = 2.D0*brwhaine(1,2,4)  
      ha1brsn1(2,5) = 2.D0*brwhaine(1,2,5)  
      ha1brsn1(3,4) = 2.D0*brwhaine(1,3,4)  
      ha1brsn1(3,5) = 2.D0*brwhaine(1,3,5)  
      ha1brsn1(4,5) = 2.D0*brwhaine(1,4,5)  
      if(brwhaine(1,1,1).ne.0.D0) then
      write(nout,102) brwhaine(1,1,1),2,in1,in1 ,
     .        'BR(A1-> ~chi_10 ~chi_10)'
      endif
      if(brwhaine(1,2,2).ne.0.D0) then
      write(nout,102) brwhaine(1,2,2),2,in2,in2 ,
     .        'BR(A1-> ~chi_20 ~chi_20)'
      endif
      if(brwhaine(1,3,3).ne.0.D0) then
      write(nout,102) brwhaine(1,3,3),2,in3,in3 ,
     .        'BR(A1-> ~chi_30 ~chi_30)'
      endif
      if(brwhaine(1,4,4).ne.0.D0) then
      write(nout,102) brwhaine(1,4,4),2,in4,in4 ,
     .        'BR(A1-> ~chi_40 ~chi_40)'
      endif
      if(brwhaine(1,5,5).ne.0.D0) then
      write(nout,102) brwhaine(1,5,5),2,in5,in5 ,
     .        'BR(A1-> ~chi_50 ~chi_50)'
      endif
      if(ha1brsn1(1,2).ne.0.D0) then
      write(nout,102) ha1brsn1(1,2),2,in1,in2,
     .        'BR(A1-> ~chi_10 ~chi_20)'
      endif
      if(ha1brsn1(1,3).ne.0.D0) then
      write(nout,102) ha1brsn1(1,3),2,in1,in3,
     .        'BR(A1-> ~chi_10 ~chi_30)'
      endif
      if(ha1brsn1(1,4).ne.0.D0) then
      write(nout,102) ha1brsn1(1,4),2,in1,in4,
     .        'BR(A1-> ~chi_10 ~chi_40)'
      endif
      if(ha1brsn1(1,5).ne.0.D0) then
      write(nout,102) ha1brsn1(1,5),2,in1,in5,
     .        'BR(A1-> ~chi_10 ~chi_50)'
      endif
      if(ha1brsn1(2,3).ne.0.D0) then
      write(nout,102) ha1brsn1(2,3),2,in2,in3,
     .        'BR(A1-> ~chi_20 ~chi_30)'
      endif
      if(ha1brsn1(2,4).ne.0.D0) then
      write(nout,102) ha1brsn1(2,4),2,in2,in4,
     .        'BR(A1-> ~chi_20 ~chi_40)'
      endif
      if(ha1brsn1(2,5).ne.0.D0) then
      write(nout,102) ha1brsn1(2,5),2,in2,in5,
     .        'BR(A1-> ~chi_20 ~chi_50)'
      endif
      if(ha1brsn1(3,4).ne.0.D0) then
      write(nout,102) ha1brsn1(3,4),2,in3,in4,
     .        'BR(A1-> ~chi_30 ~chi_40)'
      endif
      if(ha1brsn1(3,5).ne.0.D0) then
      write(nout,102) ha1brsn1(3,5),2,in3,in5,
     .        'BR(A1-> ~chi_30 ~chi_50)'
      endif
      if(ha1brsn1(4,5).ne.0.D0) then
      write(nout,102) ha1brsn1(4,5),2,in4,in5,
     .        'BR(A1-> ~chi_40 ~chi_50)'
      endif
      if(brwhaist(1).ne.0.D0) then
      write(nout,102) brwhaist(1)/2.D0,2,ist1,-ist2,
     .      'BR(A1-> ~t_1    ~t_2*)'
      write(nout,102) brwhaist(1)/2.D0,2,-ist1,ist2,
     .     'BR(A1-> ~t_1*   ~t_2 )'
      endif
      if(brwhaisb(1).ne.0.D0) then
      write(nout,102) brwhaisb(1)/2.D0,2,isb1,-isb2,
     .     'BR(A1-> ~b_1    ~b_2*)'
      write(nout,102) brwhaisb(1)/2.D0,2,-isb1,isb2,
     .     'BR(A1-> ~b_1*   ~b_2 )'
      endif
      if(brwhaisl(1).ne.0.D0) then
      write(nout,102) brwhaisl(1)/2.D0,2,istau1,-istau2,
     .     'BR(A1-> ~tau_1- ~tau_2+)'
      write(nout,102) brwhaisl(1)/2.D0,2,-istau1,istau2,
     .     'BR(A1-> ~tau_1+ ~tau_2-)'
      endif

      elseif(whatot(1).eq.0.D0) then
      write(nout,99)
      write(nout,100) 36,0.000000000E+00,'A1 decays'

      endif

c --- A2 decays --- c

      write(nout,105)

      if(whatot(2).ne.0.D0) then
      write(nout,99)
      write(nout,100) 46,whatot(2),'A2 decays'

      write(nout,101)
      if(brhabb(2).ne.0.D0) then
      write(nout,102) brhabb(2),2,ib,ibb    ,'BR(A2-> b       bb     )'
      endif
      if(brhall(2).ne.0.D0) then
      write(nout,102) brhall(2),2,-itau,itau ,'BR(A2-> tau+    tau-   )'
      endif
      if(brhamm(2).ne.0.D0) then
      write(nout,102) brhamm(2),2,-imu,imu  ,'BR(A2-> mu+     mu-    )'
      endif
c ND added 7/2021
      if(isstrue)then
      if(brhann(2).ne.0D0) then
      write(nout,102) brhann(2),2,ine,ine  ,'BR(A2-> nu_i     nu_j  )'
      endif
      endif
c end ND added 7/2021
      if(brhass(2).ne.0.D0) then
      write(nout,102) brhass(2),2,is,isb    ,'BR(A2-> s       sb     )'
      endif
      if(brhacc(2).ne.0.D0) then
      write(nout,102) brhacc(2),2,ic,icb    ,'BR(A2-> c       cb     )'
      endif
      if(brhatt(2).ne.0.D0) then
      write(nout,102) brhatt(2),2,it,itb    ,'BR(A2-> t       tb     )' 
      endif
      if(brhagg(2).ne.0.D0) then
      write(nout,102) brhagg(2),2,igl,igl   ,'BR(A2-> g       g      )' 
      endif
      if(brhaga(2).ne.0.D0) then
      write(nout,102) brhaga(2),2,iga,iga   ,'BR(A2-> gam     gam    )' 
      endif
      if(brhazga(2).ne.0.D0) then
      write(nout,102) brhazga(2),2,iz,iga   ,'BR(A2-> Z       gam    )' 
      endif
      if(brhjaiz(2,1).ne.0.D0) then
      write(nout,102) brhjaiz(2,1),2,iz,ih1 ,'BR(A2-> Z       H1     )' 
      endif
      if(brhjaiz(2,2).ne.0.D0) then
      write(nout,102) brhjaiz(2,2),2,iz,ih2 ,'BR(A2-> Z       H2     )' 
      endif
      if(brhjaiz(2,3).ne.0.D0) then
      write(nout,102) brhjaiz(2,3),2,iz,ih3 ,'BR(A2-> Z       H3     )' 
      endif
      if(brhaiwphm(2).ne.0.D0) then
      write(nout,102) brhaiwphm(2),2,iwc,-ihc,'BR(A2-> W+      H-     )'
      endif
      if(brhaiwphm(2).ne.0.D0) then
      write(nout,102) brhaiwphm(2),2,-iwc,ihc,'BR(A2-> W-      H+     )'
      endif
      if(braitoajhk(2,1,1).ne.0.D0) then
      write(nout,102) braitoajhk(2,1,1),2,iha1,ih1,
     .        'BR(A2-> A1      H1     )' 
      endif
      if(braitoajhk(2,1,2).ne.0.D0) then
      write(nout,102) braitoajhk(2,1,2),2,iha1,ih2,
     .        'BR(A2-> A1      H2     )' 
      endif
      if(braitoajhk(2,1,3).ne.0.D0) then
      write(nout,102) braitoajhk(2,1,3),2,iha1,ih3,
     .        'BR(A2-> A1      H3     )' 
      endif
      if(brwhaich(2,1,1).ne.0.D0) then
      write(nout,102) brwhaich(2,1,1),2,ic1,-ic1,
     .        'BR(A2-> ~chi_1+ ~chi_1-)'
      endif
      if(brwhaich(2,2,2).ne.0.D0) then
      write(nout,102) brwhaich(2,2,2),2,ic2,-ic2,
     .        'BR(A2-> ~chi_2+ ~chi_2-)'
      endif
      if(brwhaich(2,1,2).ne.0.D0) then
      write(nout,102) brwhaich(2,1,2),2,ic1,-ic2,
     .        'BR(A2-> ~chi_1+ ~chi_2-)'
      endif
      if(brwhaich(2,2,1).ne.0.D0) then
      write(nout,102) brwhaich(2,2,1),2,ic2,-ic1,
     .        'BR(A2-> ~chi_2+ ~chi_1-)'
      endif
      ha2brsn1(1,2) = 2.D0*brwhaine(2,1,2) 
      ha2brsn1(1,3) = 2.D0*brwhaine(2,1,3) 
      ha2brsn1(1,4) = 2.D0*brwhaine(2,1,4)
      ha2brsn1(1,5) = 2.D0*brwhaine(2,1,5)
      ha2brsn1(2,3) = 2.D0*brwhaine(2,2,3) 
      ha2brsn1(2,4) = 2.D0*brwhaine(2,2,4)  
      ha2brsn1(2,5) = 2.D0*brwhaine(2,2,5)  
      ha2brsn1(3,4) = 2.D0*brwhaine(2,3,4)  
      ha2brsn1(3,5) = 2.D0*brwhaine(2,3,5)  
      ha2brsn1(4,5) = 2.D0*brwhaine(2,4,5)  
      if(brwhaine(2,1,1).ne.0.D0) then
      write(nout,102) brwhaine(2,1,1),2,in1,in1 ,
     .        'BR(A2-> ~chi_10 ~chi_10)'
      endif
      if(brwhaine(2,2,2).ne.0.D0) then
      write(nout,102) brwhaine(2,2,2),2,in2,in2 ,
     .        'BR(A2-> ~chi_20 ~chi_20)'
      endif
      if(brwhaine(2,3,3).ne.0.D0) then
      write(nout,102) brwhaine(2,3,3),2,in3,in3 ,
     .        'BR(A2-> ~chi_30 ~chi_30)'
      endif
      if(brwhaine(2,4,4).ne.0.D0) then
      write(nout,102) brwhaine(2,4,4),2,in4,in4 ,
     .        'BR(A2-> ~chi_40 ~chi_40)'
      endif
      if(brwhaine(2,5,5).ne.0.D0) then
      write(nout,102) brwhaine(2,5,5),2,in5,in5 ,
     .        'BR(A2-> ~chi_50 ~chi_50)'
      endif
      if(ha2brsn1(1,2).ne.0.D0) then
      write(nout,102) ha2brsn1(1,2),2,in1,in2,'BR(A2-> ~chi_10 ~chi_20)'
      endif
      if(ha2brsn1(1,3).ne.0.D0) then
      write(nout,102) ha2brsn1(1,3),2,in1,in3,'BR(A2-> ~chi_10 ~chi_30)'
      endif
      if(ha2brsn1(1,4).ne.0.D0) then
      write(nout,102) ha2brsn1(1,4),2,in1,in4,'BR(A2-> ~chi_10 ~chi_40)'
      endif
      if(ha2brsn1(1,5).ne.0.D0) then
      write(nout,102) ha2brsn1(1,5),2,in1,in5,'BR(A2-> ~chi_10 ~chi_50)'
      endif
      if(ha2brsn1(2,3).ne.0.D0) then
      write(nout,102) ha2brsn1(2,3),2,in2,in3,'BR(A2-> ~chi_20 ~chi_30)'
      endif
      if(ha2brsn1(2,4).ne.0.D0) then
      write(nout,102) ha2brsn1(2,4),2,in2,in4,'BR(A2-> ~chi_20 ~chi_40)'
      endif
      if(ha2brsn1(2,5).ne.0.D0) then
      write(nout,102) ha2brsn1(2,5),2,in2,in5,'BR(A2-> ~chi_20 ~chi_50)'
      endif
      if(ha2brsn1(3,4).ne.0.D0) then
      write(nout,102) ha2brsn1(3,4),2,in3,in4,'BR(A2-> ~chi_30 ~chi_40)'
      endif
      if(ha2brsn1(3,5).ne.0.D0) then
      write(nout,102) ha2brsn1(3,5),2,in3,in5,'BR(A2-> ~chi_30 ~chi_50)'
      endif
      if(ha2brsn1(4,5).ne.0.D0) then
      write(nout,102) ha2brsn1(4,5),2,in4,in5,'BR(A2-> ~chi_40 ~chi_50)'
      endif
      if(brwhaist(2).ne.0.D0) then
      write(nout,102) brwhaist(2)/2.D0,2,ist1,-ist2,
     .        'BR(A2-> ~t_1    ~t_2*)'
      write(nout,102) brwhaist(2)/2.D0,2,-ist1,ist2,
     .     'BR(A2-> ~t_1*   ~t_2)'
      endif
      if(brwhaisb(2).ne.0.D0) then
      write(nout,102) brwhaisb(2)/2.D0,2,isb1,-isb2,
     .     'BR(A2-> ~b_1    ~b_2*)'
      write(nout,102) brwhaisb(2)/2.D0,2,-isb1,isb2,
     .     'BR(A2-> ~b_1*   ~b_2)'
      endif
      if(brwhaisl(2).ne.0.D0) then
      write(nout,102) brwhaisl(2)/2.D0,2,istau1,-istau2,
     .     'BR(A2-> ~tau_1- ~tau_2+)'
      write(nout,102) brwhaisl(2)/2.D0,2,-istau1,istau2,
     .     'BR(A2-> ~tau_1+ ~tau_2-)'
      endif

      elseif(whatot(2).eq.0.D0) then
      write(nout,99)
      write(nout,100) 46,0.000000000E+00,'A2 decays'

      endif

c --------- c
c H+ decays c
c --------- c

      if(hcwdth.ne.0.D0) then
      write(nout,105)
      write(nout,99)
      write(nout,100) 37,hcwdth,'H+ decays'

      write(nout,101)
      if(hcbrb.ne.0.D0) then
      write(nout,102) hcbrb,2,ic,ibb        ,'BR(H+ -> c       bb     )'
      endif
c ND changed 7/2021
      if(.not.isstrue)then 
      if(hcbrl.ne.0.D0) then
      write(nout,102) hcbrl,2,-itau,intau   ,'BR(H+ -> tau+    nu_tau )'
      endif
      if(hcbrm.ne.0.D0) then
      write(nout,102) hcbrm,2,-imu,inmu     ,'BR(H+ -> mu+     nu_mu  )'
      endif
      else !!!
      if(hcbrl.ne.0D0) then
      write(nout,102) hcbrl,2,-itau,intau   ,'BR(H+ -> tau+    nu_i   )'
      endif
      if(hcbrm.ne.0D0) then
      write(nout,102) hcbrm,2,-imu,inmu     ,'BR(H+ -> mu+     nu_i   )'
      endif
      endif
c end ND changed 7/2021

      if(hcbrbu.ne.0.D0) then
      write(nout,102) hcbrbu,2,iu,ibb       ,'BR(H+ -> u       bb     )'
      endif
      if(hcbrs.ne.0.D0) then
      write(nout,102) hcbrs,2,iu,isb        ,'BR(H+ -> u       sb     )'
      endif
      if(hcbrc.ne.0.D0) then
      write(nout,102) hcbrc,2,ic,isb        ,'BR(H+ -> c       sb     )'
      endif
      if(hcbrd.ne.0.D0) then
      write(nout,102) hcbrd,2,ic,idb        ,'BR(H+ -> c       db     )'
      endif
      if(hcbrdt.ne.0.D0) then
      write(nout,102) hcbrdt,2,it,idb       ,'BR(H+ -> t       db     )'
      endif
      if(hcbrst.ne.0.D0) then
      write(nout,102) hcbrst,2,it,isb       ,'BR(H+ -> t       sb     )'
      endif
      if(hcbrt.ne.0.D0) then
      write(nout,102) hcbrt,2,it,ibb        ,'BR(H+ -> t       bb     )'
      endif
      if(hcbrwh1.ne.0.D0) then
      write(nout,102) hcbrwh1,2,iwc,ih1     ,'BR(H+ -> W+      H1     )'
      endif
      if(hcbrwh2.ne.0.D0) then
      write(nout,102) hcbrwh2,2,iwc,ih2     ,'BR(H+ -> W+      H2     )'
      endif
      if(hcbrwh3.ne.0.D0) then
      write(nout,102) hcbrwh3,2,iwc,ih3     ,'BR(H+ -> W+      H3     )'
      endif
      if(hcbra1.ne.0.D0) then
      write(nout,102) hcbra1,2,iwc,iha1     ,'BR(H+ -> W+      A1     )'
      endif
      if(hcbra2.ne.0.D0) then
      write(nout,102) hcbra2,2,iwc,iha1     ,'BR(H+ -> W+      A2     )'
      endif
      if(hcbrsu(1,1).ne.0.D0) then
      write(nout,102) hcbrsu(1,1),2,ic1,in1 ,'BR(H+ -> ~chi_1+ ~chi_10)'
      endif
      if(hcbrsu(1,2).ne.0.D0) then
      write(nout,102) hcbrsu(1,2),2,ic1,in2 ,'BR(H+ -> ~chi_1+ ~chi_20)'
      endif
      if(hcbrsu(1,3).ne.0.D0) then
      write(nout,102) hcbrsu(1,3),2,ic1,in3 ,'BR(H+ -> ~chi_1+ ~chi_30)'
      endif
      if(hcbrsu(1,4).ne.0.D0) then
      write(nout,102) hcbrsu(1,4),2,ic1,in4 ,'BR(H+ -> ~chi_1+ ~chi_40)'
      endif
      if(hcbrsu(1,5).ne.0.D0) then
      write(nout,102) hcbrsu(1,5),2,ic1,in5 ,'BR(H+ -> ~chi_1+ ~chi_50)'
      endif
      if(hcbrsu(2,1).ne.0.D0) then
      write(nout,102) hcbrsu(2,1),2,ic2,in1 ,'BR(H+ -> ~chi_2+ ~chi_10)'
      endif
      if(hcbrsu(2,2).ne.0.D0) then
      write(nout,102) hcbrsu(2,2),2,ic2,in2 ,'BR(H+ -> ~chi_2+ ~chi_20)'
      endif
      if(hcbrsu(2,3).ne.0.D0) then
      write(nout,102) hcbrsu(2,3),2,ic2,in3 ,'BR(H+ -> ~chi_2+ ~chi_30)'
      endif
      if(hcbrsu(2,4).ne.0.D0) then
      write(nout,102) hcbrsu(2,4),2,ic2,in4 ,'BR(H+ -> ~chi_2+ ~chi_40)'
      endif
      if(hcbrsu(2,5).ne.0.D0) then
      write(nout,102) hcbrsu(2,5),2,ic2,in5 ,'BR(H+ -> ~chi_2+ ~chi_50)'
      endif

c ND changed 7/2021
      if(.not.isstrue)then 

      bhcsl02=bhcsl00/2.D0
      if(bhcsl02.ne.0.D0) then
      write(nout,102) bhcsl02,2,-isell,inel ,'BR(H+ -> ~e_L+   ~nu_eL )'
      write(nout,102) bhcsl02,2,-ismul,inmul,'BR(H+ -> ~mu_L+  ~nu_muL)'
      endif
      if(bhcsl11.ne.0.D0) then
      write(nout,102) bhcsl11,2,-istau1,intau1,'BR(H+ -> ~tau_1+ ~nu_tau
     .L)'
      endif
      if(bhcsl21.ne.0.D0) then
      write(nout,102) bhcsl21,2,-istau2,intau1,'BR(H+ -> ~tau_2+ ~nu_tau
     .L)'
      endif
      else !!!
      bhcsl02=bhcsl00/2.D0
      if(bhcsl02.ne.0D0) then
      write(nout,102) bhcsl02,2,-isell,inel ,'BR(H+ -> ~e_L+   ~nu_i )'
      write(nout,102) bhcsl02,2,-ismul,inmul,'BR(H+ -> ~mu_L+  ~nu_i)'
      endif
      if(bhcsl11.ne.0D0) then
      write(nout,102) bhcsl11,2,-istau1,intau1,'BR(H+ -> ~tau_1+ ~nu_i
     .L)'
      endif
      if(bhcsl21.ne.0.D0) then
      write(nout,102) bhcsl21,2,-istau2,intau1,'BR(H+ -> ~tau_2+ ~nu_i
     .L)'
      endif
      endif
c end ND changed 7/2021

      hcbrsq1=hcbrsq/2.D0
      if(hcbrsq1.ne.0.D0) then
      write(nout,102) hcbrsq1,2,isul,-isdl  ,'BR(H+ -> ~u_L    ~d_L*  )'
      write(nout,102) hcbrsq1,2,iscl,-issl  ,'BR(H+ -> ~c_L    ~s_L*  )'
      endif
      if(hcbrstb(1,1).ne.0.D0) then
      write(nout,102) hcbrstb(1,1),2,ist1,-isb1,'BR(H+ -> ~t_1    ~b_1* 
     . )'
      endif
      if(hcbrstb(2,2).ne.0.D0) then
      write(nout,102) hcbrstb(2,2),2,ist2,-isb2,'BR(H+ -> ~t_2    ~b_2* 
     . )'
      endif
      if(hcbrstb(1,2).ne.0.D0) then
      write(nout,102) hcbrstb(1,2),2,ist1,-isb2,'BR(H+ -> ~t_1    ~b_2* 
     . )'
      endif
      if(hcbrstb(2,1).ne.0.D0) then
      write(nout,102) hcbrstb(2,1),2,ist2,-isb1,'BR(H+ -> ~t_2    ~b_1* 
     . )'
      endif

      elseif(hcwdth.eq.0.D0) then
      write(nout,99)
      write(nout,100) 37,0.000000000E+00,'H+ decays'

      endif

c -------------------------------------------------------------------- c
c ---------- c
c Top decays c
c ---------- c

      write(nout,105)

      if(gamt1.ne.0.D0) then
      write(nout,99)
      write(nout,100) 6,gamt1,'top decays'

      write(nout,101)
      if(gamt0.ne.0.D0) then
       write(nout,102) gamt0/gamt1,2,ib,iwc,'BR(t -> b       W+    )'
      endif
      if(gamt1-gamt0.ne.0.D0) then
       write(nout,102) (gamt1-gamt0)/gamt1,2,ib,ihc,
     .                                      'BR(t -> b       H+    )'
      endif

      elseif(gamt1.eq.0.D0) then
      write(nout,99)
      write(nout,100) 6,0.000000000E+00,'top decays'

      endif
      
      
C     HiggsBounds Blocks
      write(nout,105)
      write(nout,105)
      write(nout,51) 'HiggsCouplingsFermions', 
     .               'extra Block for HiggsBounds'
      write(nout,111)
      write(nout,112) gh1b,0d0,3,25,5,5,
     .                'H1-b-b coupling normalized to SM'
      write(nout,112) gh2b,0d0,3,35,5,5,
     .                'H2-b-b coupling normalized to SM'
      write(nout,112) 0d0,ga1b,3,36,5,5,
     .                'A1-b-b coupling normalized to SM'
      write(nout,112) gh3b,0d0,3,45,5,5,
     .                'H3-b-b coupling normalized to SM'
      write(nout,112) 0d0,ga2b,3,46,5,5,
     .                'A2-b-b coupling normalized to SM'

      write(nout,105)
      write(nout,112) gh1t,0d0,3,25,6,6,
     .                'H1-t-t coupling normalized to SM'
      write(nout,112) gh2t,0d0,3,35,6,6,
     .                'H2-t-t coupling normalized to SM'
      write(nout,112) 0d0,ga1t,3,36,6,6,
     .                'A1-t-t coupling normalized to SM'
      write(nout,112) gh3t,0d0,3,45,6,6,
     .                'H3-t-t coupling normalized to SM'
      write(nout,112) 0d0,ga2t,3,46,6,6,
     .                'A2-t-t coupling normalized to SM'

      write(nout,105)
      write(nout,112) gh1b,0d0,3,25,15,15,
     .                'H1-tau-tau coupling normalized to SM'
      write(nout,112) gh2b,0d0,3,35,15,15,
     .                'H2-tau-tau coupling normalized to SM'
      write(nout,112) 0d0,ga1b,3,36,15,15,
     .                'A1-tau-tau coupling normalized to SM'
      write(nout,112) gh3b,0d0,3,45,15,15,
     .                'H3-tau-tau coupling normalized to SM'
      write(nout,112) 0d0,ga2b,3,46,15,15,
     .                'A2-tau-tau coupling normalized to SM'

      write(nout,105)
      write(nout,105)
      write(nout,51) 'HiggsCouplingsBosons', 
     .               'extra Block for HiggsBounds'
      write(nout,113) gh1vv,3,25,24,24,
     .                'H1-W-W coupling normalized to SM'
      write(nout,113) gh2vv,3,35,24,24,
     .                'H2-W-W coupling normalized to SM'
      write(nout,113) 0d0,3,36,24,24,
     .                'A1-W-W coupling normalized to SM'
      write(nout,113) gh3vv,3,45,24,24,
     .                'H3-W-W coupling normalized to SM'
      write(nout,113) 0d0,3,46,24,24,
     .                'A2-W-W coupling normalized to SM'
      write(nout,105)
      write(nout,113) gh1vv,3,25,23,23,
     .                'H1-Z-Z coupling normalized to SM'
      write(nout,113) gh2vv,3,35,23,23,
     .                'H2-Z-Z coupling normalized to SM'
      write(nout,113) 0d0,3,36,23,23,
     .                'A1-Z-Z coupling normalized to SM'
      write(nout,113) gh3vv,3,45,23,23,
     .                'H3-Z-Z coupling normalized to SM'
      write(nout,113) 0d0,3,46,23,23,
     .                'A2-Z-Z coupling normalized to SM'
      write(nout,105)
      write(nout,113) cj(1),3,25,21,21,
     .                'H1-gluon-gluon coupling normalized to SM'
      write(nout,113) cj(2),3,35,21,21,
     .                'H2-gluon-gluon coupling normalized to SM'
      write(nout,113) cj(4),3,36,21,21,
     .                'A1-gluon-gluon coupling normalized to SM'
      write(nout,113) cj(3),3,45,21,21,
     .                'H3-gluon-gluon coupling normalized to SM'
      write(nout,113) cj(5),3,46,21,21,
     .                'A2-gluon-gluon coupling normalized to SM'
      write(nout,105)
      write(nout,113) 0d0,3,25,25,23,
     .                'H1-H1-Z coupling normalized to SM'
      write(nout,113) 0d0,3,25,35,23,
     .                'H1-H2-Z coupling normalized to SM'
      write(nout,113) gzaihj(1,1),3,25,36,23,
     .                'H1-A1-Z coupling normalized to SM'
      write(nout,113) 0d0,3,25,45,23,
     .                'H1-H3-Z coupling normalized to SM'
      write(nout,113) gzaihj(2,1),3,25,46,23,
     .                'H1-A2-Z coupling normalized to SM'
      write(nout,113) 0d0,3,35,35,23,
     .                'H2-H2-Z coupling normalized to SM'
      write(nout,113) gzaihj(1,2),3,35,36,23,
     .                'H2-A1-Z coupling normalized to SM'
      write(nout,113) 0d0,3,35,45,23,
     .                'H2-H3-Z coupling normalized to SM'
      write(nout,113) gzaihj(2,2),3,35,46,23,
     .                'H2-A2-Z coupling normalized to SM'
      write(nout,113) 0d0,3,36,36,23,
     .                'A1-A1-Z coupling normalized to SM'
      write(nout,113) gzaihj(1,3),3,36,45,23,
     .                'A1-H3-Z coupling normalized to SM'
      write(nout,113) 0d0,3,36,46,23,
     .                'A1-A2-Z coupling normalized to SM'
      write(nout,113) 0d0,3,45,45,23,
     .                'H3-H3-Z coupling normalized to SM'
      write(nout,113) gzaihj(2,3),3,45,46,23,
     .                'H3-A2-Z coupling normalized to SM'
      write(nout,113) 0d0,3,46,46,23,
     .                'A2-A2-Z coupling normalized to SM'
      write(nout,105) 
      write(nout,114) 0d0,4,25,21,21,23,
     .                'H1-gluon-gluon-Z coupling'
      write(nout,114) 0d0,4,35,21,21,23,
     .                'H2-gluon-gluon-Z coupling'
      write(nout,114) 0d0,4,36,21,21,23,
     .                'A1-gluon-gluon-Z coupling'
      write(nout,114) 0d0,4,45,21,21,23,
     .                'H3-gluon-gluon-Z coupling'
      write(nout,114) 0d0,4,46,21,21,23,
     .                'A2-gluon-gluon-Z coupling'

      write(nout,105)
      write(nout,105)
c -------------------------------------------------------------------- c

 49   format('#',1x,A,E16.8)
 50   format('#',1x,A)
 51   format('BLOCK',1x,A,2x,'#',1x,A)
 551  format(1x,A,2x,'#',1x,A)
 52   format(1x,I9,3x,1P,E16.8,0P,3x,'#',1x,A)
 552  format(2x,E16.8,0P,3x,A)
 53   format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A)
 54   format('BLOCK',1x,A,1P,E16.8,2x,'#',1x,A)
 554  format(2x,A,1P,E16.8,2x,1x,A)
 55   format(1x,I5,3x,1P,E16.8,0P,3x,'#',1x,A)
 56   format(1x,I4,3x,'#',1x,A,E16.8)
 57   format(1x,I5,3x,1P,E16.8,0P,3x,'#',1x,A,E16.8)
 58   format(1x,I2,1x,I2,3x,'#',1x,A)
 59   format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A,E16.8)
 60   format(9x,1P,E16.8,0P,3x,'#',1x,A)
 61   format(1x,I5,3x,A)
 661  format(2x,A)
 62   format(1x,I5,1x,I5,3x,A)
 662  format(2x,A)
 63   format(1x,I5,3x,A,1x,'#',1x,A)

 99   format('#',9x,'PDG',12x,'Width')
 100  format('DECAY',1x,I9,3x,1P,E16.8,0P,3x,'#',1x,A)
 101  format('#',10x,'BR',9x,'NDA',6x,'ID1',7x,'ID2')
 102  format(3x,1P,E16.8,0P,3x,I2,3x,(I9,1x),(I9,1x),2x,'#',1x,A)
 103  format('#',11x,'BR',9x,'NDA',6x,'ID1',7x,'ID2',7x,'ID3')
 107  format('#',11x,'BR',9x,'NDA',6x,'ID1',7x,'ID2',7x,'ID3',7x,'ID4')
 104  format(3x,1P,E16.8,0P,3x,I2,3x,(I9,1x),(I9,1x),(I9,1x),2x,'#',
     .1x,A)
 106  format(3x,1P,E16.8,0P,3x,I2,3x,(I9,1x),(I9,1x),(I9,1x),(I9,1x),
     .2x,'#',1x,A)
 108  format(E16.8,2x,E16.8,2x,E16.8,2x,E16.8)
 105  format('#') 
 111  format('#',2x,'ScalarNormEffCoupSq',2x,'PseudoSNormEffCoupSq',6x,
     .       'NP',3x,'IP1',3x,'IP2',3x,'IP3')
 112  format(1x,E16.8,6x,E16.8,10x,I2,4x,I2,4x,I2,4x,I2,4x,'#',1x,A)
 113  format(1x,E16.8,10x,I2,4x,I2,4x,I2,4x,I2,4x,'#',1x,A) 
 114  format(1x,E16.8,10x,I2,4x,I2,4x,I2,4x,I2,4x,I2,4x,'#',1x,A) 
 115   format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A,'(',I2,',',I2,')')
 123  format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A,'(',I1,',',I1,')')
      close(nout)
      close(nout2)

      return
      end

c ==================================================================== c
c =========== subroutine for the NMSSM Higgs boson decays ============ c
c ==================================================================== c

      subroutine hdec_nmssm(tgbet)

      implicit double precision (a-h,o-z)
      double precision xfac
      integer ivegas(4)
      character outputfile*100
      double precision lamb_hdec
      double precision amhia(2),ghiat(2),ghiab(2),xghiam(2),xghiab(2),
     .     xghias(2)
      double precision hagg(2),hamm(2),hall(2),hass(2),hacc(2),habb(2),
     .     hatt(2),haga(2),hazga(2),hjaiz(2,3),whaich(2,2,2),
     .     whaine(2,5,5),whainet(2),whaisl(2),whaist(2),whaisb(2),
     .     whaicht(2),haiwphm(2),aitoajhk(2,2,3),atoah(2),
     .     aitoajhko(2,2,3),hann(2),whaslnl(2)
      double precision amhi(5),xghigm(5),xghigb(5),xghigs(5)
      double precision ghigb(5),ghigt(5),ghigvv(3),ghighpwm(3),
     .     gzaihj(2,3),g3hihjhk(5,5,5),ghigpm(3),dghib(5)
      double precision ghigtt(5,2,2),ghigbb(5,2,2),ghigee(5,2,2),
     .     ghiguu(3,2,2),ghigdd(3,2,2),ghigelel(3,2,2),ghignene(3,2,2),
     .     yhitt(3,2,2),yhaitt(2),yhibb(3,2,2),yhaibb(2)
      double precision achig(5,2,2),anhig(5,5,5)
      double precision higgg(5),higmm(5),higll(5),higss(5),higcc(5),
     .     higbb(5),higtt(5),higga(5),higzga(5),higww(5),higzz(5),hignn(5)
      double precision htoh(5),hitohjhk(3,5,5),hitoajz(3,5),htoaz(3),
     .     hitohpwm(3),hitohhoff(5),hitohjhko(5,5,5),errhitohjhko(5,5,5)
      double precision brhiggg(3),brhigmm(3),brhigll(3),brhigss(3),
     .     brhigcc(3),brhigbb(3),brhigtt(3),brhigga(3),brhigzga(3),
     .     brhigww(3),brhigzz(3),brhitohpwm(3),brhitohjhk(3,5,5),
     .     brhitoajz(3,5),brwhigch(3,2,2),brwhigne(3,5,5),brwhigcht(3),
     .     brwhignet(3),brwhislt(3),brwhisq(3),brwhisqt(3),
     .     brwhislnl(3),brwhislnltau(3),brwhislel(3),brwhisler(3),
     .     brwhisqul(3),brwhisqur(3),brwhisqdl(3),brwhisqdr(3),
     .     brwhist(3,2,2),brwhisb(3,2,2),brwhistau(3,2,2),brhignn(3)
      double precision brhagg(2),brhamm(2),brhall(2),brhass(2),
     .     brhacc(2),brhabb(2),brhatt(2),brhaga(2),brhazga(2),
     .     brhaiwphm(2),braitoajhk(2,2,3),brhjaiz(2,3),brwhaich(2,2,2),
     .     brwhaine(2,5,5),brwhaicht(2),brwhainet(2),brwhaisl(2),
     .     brwhaist(2),brwhaisb(2),brhann(2)
      complex*16 cfacq_hdec,cfacsq_hdec
      dimension xx(4),yy(4)
      dimension amchar(2),amneut(5),xmneut(5),ach1(2,2),ach2(2,2),
     .     ach3(2,2),anh1(5,5),anh2(5,5),anh3(5,5),acnl(2,5),acnr(2,5),
     .     aca1(2,2),aca2(2,2),ana1(5,5),ana2(5,5)
      dimension amst(2),amsb(2),amsl(2),amsu(2),amsd(2),amse(2),amsn(18),
     .     ghcen(2,2),ghctb(2,2),ghcelne(2,2),ghcud(2,2),gh1elel(2,2),
     .     gh1nene(2,2),gh1uu(2,2),gh1dd(2,2),gh1ee(2,2),gh1tt(2,2),
     .     gh1bb(2,2),gh2elel(2,2),gh2nene(2,2),gh2uu(2,2),gh2dd(2,2),
     .     gh2ee(2,2),gh2tt(2,2),gh2bb(2,2),gh3elel(2,2),gh3nene(2,2),
     .     gh3uu(2,2),gh3dd(2,2),gh3ee(2,2),gh3tt(3,3),gh3bb(3,3),
     .     amsnl(2),amnu(9)
      dimension h1brsc(2,2),h1brsn(5,5),h2brsc(2,2),h2brsn(5,5),
     .     h3brsc(2,2),h3brsn(5,5),ha1brsc(2,2),ha1brsn(5,5),
     .     ha2brsc(2,2),ha2brsn(5,5),hcbrsu(2,5),h2brst(2,2),
     .     h2brsb(2,2),h3brst(2,2),h3brsb(2,2),hcbrstb(2,2) 
      dimension wh1ch(2,2),wh1ne(5,5),wh2ch(2,2),wh2ne(5,5),
     .     wh3ch(2,2),wh3ne(5,5),wha1ch(2,2),wha1ne(5,5),wha2ch(2,2),
     .     wha2ne(5,5),whccn(2,5),wh2st(2,2),wh2sb(2,2),wh2stau(2,2),
     .     whcstb(2,2),wh1st(2,2),wh1sb(2,2),wh1stau(2,2),wh3st(2,2),
     .     wh3sb(2,2),wh3stau(2,2)
      dimension whigch(5,2,2),whigne(5,5,5),whigcht(5),whignet(5),
     .     whislel(5),whisler(5),whislnl(5),whislnltau(5),
     .     whistau(5,2,2),whislt(5),whisqul(5),whisqur(5),whisqdl(5),
     .     whisqdr(5),whisq(5),whist(5,2,2),whisb(5,2,2),whistt(5),
     .     whisbb(5),whisqt(5),whitot(5),whatot(2)
      dimension slhaneut(5),slhaxneut(5),slhachar(2),slhau(2,2),
     .          slhav(2,2),slhaz(5,5),xmchar(2)
      complex*16 cf,cg,ci1,ci2,ca,cb,ctt,ctb,ctc,ctw,clt,clb,clw,clc,
     .           cat,cab,cac,caw,cah,cth,clh,cx1,cx2,cax1,cax2,ctl,cal,
     .           csl,csq,csb1,csb2,cst1,cst2,csl1,csl2,
     .           cxl,cxq,cxb1,cxb2,cxt1,cxt2,cxl1,cxl2,cle
c maggie changed 25/7/2012
      complex*16 catsm,cabsm,cacsm,cawsm,calsm
c end maggie changed 25/7/2012
      complex*16 csel,cser,csul,csur,csdl,csdr,
     .           cxel,cxer,cxul,cxur,cxdl,cxdr
      complex*16 cat0,cab0,cac0,cxul0,cxur0,cxdl0,cxdr0,cxb10,cxb20,
     .           cxt10,cxt20
      complex*16 cttp,ctbp,ctep,cltp,clbp,clep,catp,cabp,caep
      complex*16 catp0,cabp0
      logical mainrun,calcmasseserror
      complex*16 catc,cabc,cacc
      complex*16 detau      
      double precision cj(5),ci(5)

      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
      COMMON/HMASSR_NMSSM/AMLR,AMHR
      COMMON/CHIMASS_HDEC/AMCHI
      COMMON/MASSES_HDEC/AMS,AMC,AMB,AMT
      COMMON/ALS_HDEC/XLAMBDA,AMC0,AMB0,AMT0,N0
      COMMON/PARAM_HDEC/GF,ALPH,AMTAU,AMMUON,AMZ,AMW
      COMMON/CKMPAR_HDEC/VTB,VTS,VTD,VCB,VCS,VCD,VUB,VUS,VUD
      COMMON/BREAK_NMSSM/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AT,AB,Alam,Akap,
     .     AMU,AM2
      COMMON/BREAKGLU_HDEC/AMGLU
      COMMON/WZWDTH_HDEC/GAMC0,GAMT0,GAMT1,GAMW,GAMZ
      COMMON/ONSHELL_NMSSM/IONSH,IONWZ,IOFSUSY
      COMMON/OLDFASH_NMSSM/NFGG
      COMMON/FLAG_HDEC/IHIGGS,NNLO
      COMMON/FERMIOPHOBIC_NMSSM/IFERMPHOB
      COMMON/WIDTHSCALAR_NMSSM/brhiggg,brhigmm,brhigll,brhigss,
     .     brhigcc,brhigbb,brhigtt,brhigga,brhigzga,
     .     brhigww,brhigzz,brhitohpwm,brhitohjhk,
     .     brhitoajz,brwhigch,brwhigne,brwhigcht,
     .     brwhignet,brwhislt,brwhisq,brwhisqt,
     .     brwhislnl,brwhislnltau,brwhislel,brwhisler,
     .     brwhisqul,brwhisqur,brwhisqdl,brwhisqdr,
     .     brwhist,brwhisb,brwhistau,whitot,brhignn
      COMMON/WIDTHPSEUDOSCALAR_NMSSM/brhagg,brhamm,brhall,brhass,
     .     brhacc,brhabb,brhatt,brhaga,brhazga,
     .     brhaiwphm,braitoajhk,brhjaiz,brwhaich,
     .     brwhaine,brwhaicht,brwhainet,brwhaisl,
     .     brwhaist,brwhaisb,whatot,brhann
      COMMON/WIDTHA1_NMSSM/A1BRB,A1BRL,A1BRM,A1BRS,A1BRC,A1BRT,A1BRG,
     .     A1BRGA,A1BRZGA,A1BRZH1,A1BRZH2,A1BRZH3,A1WDTH
      COMMON/WIDTHA2_NMSSM/A2BRB,A2BRL,A2BRM,A2BRS,A2BRC,A2BRT,A2BRG,
     .     A2BRGA,A2BRZGA,A2BRZH1,A2BRZH2,A2BRZH3,A2BRA1H1,A2BRA1H2,
     .     A2BRA1H3,A2BRHCW,A2WDTH
      COMMON/WIDTHH1_NMSSM/H1BRB,H1BRL,H1BRM,H1BRS,H1BRC,H1BRT,H1BRG,
     .     H1BRGA,H1BRZGA,H1BRW,H1BRZ,H1BRA1A1,H1BRA1Z,H1BRA2Z,H1BRHW,
     .     H1BRA1A2,H1WDTH
      COMMON/WIDTHH2_NMSSM/H2BRB,H2BRL,H2BRM,H2BRS,H2BRC,H2BRT,H2BRG,
     .     H2BRGA,H2BRZGA,H2BRW,H2BRZ,H2BRH1H1,H2BRA1A1,H2BRA1A2,
     .     H2BRA1Z,H2BRA2Z,H2BRHW,H2WDTH
      COMMON/WIDTHH3_NMSSM/H3BRB,H3BRL,H3BRM,H3BRS,H3BRC,H3BRT,H3BRG,
     .     H3BRGA,H3BRZGA,H3BRW,H3BRZ,H3BRH1H1,H3BRH1H2,H3BRH2H2,
     .     H3BRA1A1,H3BRA1A2,H3BRA2A2,H3BRA1Z,H3BRA2Z,H3BRHW,H3WDTH
      COMMON/WIDTHHC_NMSSM/HCBRB,HCBRL,HCBRM,HCBRBU,HCBRS,HCBRC,HCBRT,
     .               HCBRWH1,HCBRWH2,HCBRWH3,HCBRA1,HCBRA2,hcbrd,hcbrdt,
     .               hcbrst,HCWDTH
      COMMON/WISUSY_NMSSM/H1BRSC,H1BRSN,H2BRSC,H2BRSN,H3BRSC,H3BRSN,
     .     HA1BRSC,HA1BRSN,HA2BRSC,HA2BRSN,HCBRSU,H1BRCHT,H2BRCHT,
     .     H3BRCHT,HA1BRCHT,HA2BRCHT,H1BRNET,H2BRNET,H3BRNET,HA1BRNET,
     .     HA2BRNET,HCBRCNT,H1BRSL,H2BRSL,H3BRSL,HCBRSL,HA1BRSL,HA2BRSL,
     .     HA1BRST,HA2BRST,HA1BRSB,HA2BRSB,H2BRSQ,H2BRST,H2BRSB,H2BRSQT,
     .     H3BRSQ,H3BRST,H3BRSB,H3BRSQT,HCBRSQ,HCBRSTB,HCBRSQT,H1BRSQ,
     .     H1BRSQT
      COMMON/WISFER_NMSSM/BH1SLNL,BH1SLEL,BH1SLER,BH1SQUL,BH1SQUR,
     .     BH1SQDL,BH1SQDR,BH1ST(2,2),BH1SB(2,2),BH1STAU(2,2),BH2SLNL,
     .     BH2SLEL,BH2SLER,BH2SQUL,BH2SQUR,BH2SQDL,BH2SQDR,BH2ST(2,2),
     .     BH2SB(2,2),BH2STAU(2,2),BH3SLNL,BH3SLEL,BH3SLER,BH3SQUL,
     .     BH3SQUR,BH3SQDL,BH3SQDR,BH3ST(2,2),BH3SB(2,2),BH3STAU(2,2),
     .     BHA1STAU,BHA1SB,BHA1ST,BHA2STAU,BHA2SB,BHA2ST,BHCSL00,
     .     BHCSL11,BHCSL21,BHCSQ,BHCSTB(2,2),BH1SLNLTAU,BH2SLNLTAU,
     .     BH3SLNLTAU
      COMMON/SMASS_NMSSM/AMNEUT,XMNEUT,AMCHAR,AMST,AMSB,AMSL,
     .              AMSU,AMSD,AMSE,AMSN,AMSNL,AMNU
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/SLHA_gaug_NMSSM/slhaneut,slhaxneut,slhachar,slhau,slhav,
     .                      slhaz,xmchar
      COMMON/calcerror/mainrun,calcmasseserror
c -------------------------------------------------------------------- c
      COMMON/DAVID/QSUSY1,QSUSY2,LOOP
      COMMON/SQNLO_NMSSM/ymsb(2),ysthb,ycthb,yh1bb(2,2),yh2bb(2,2),
     .     yh3bb(2,2),yha1bb,yha2bb,ymst(2),ystht,yctht,yh1tt(2,2),
     .     yh2tt(2,2),yh3tt(2,2),yha1tt,yha2tt
c -------------------------------------------------------------------- c
c common blocks needed for the solution of the RGEs
      COMMON/GUTSCALE_NMSSM/egut
c -------------------------------------------------------------------- c
      COMMON/RUNCOUPTEST/iruncoup
      COMMON/SHORTCOUPL1_NMSSM/ghigtt,ghigbb,ghigee,ghiguu,ghigdd,
     .     ghigelel,ghignene
      COMMON/SHORTCOUPL2_NMSSM/ghigb,ghigt,ghigvv,ghighpwm,gzaihj,
     .     g3hihjhk,ghigpm
      COMMON/SHORTCOUPL3_NMSSM/achig,anhig
      COMMON/SHORTCOUPL4_NMSSM/yhitt,yhaitt,yhibb,yhaibb
      COMMON/M1andM2val/am1val,am2val
      COMMON/GLUONCOUP/cj
c -------------------------------------------------------------------- c
      logical isstrue
      common/issswitch/isstrue


      hvv(x,y)= gf/(4.d0*pi*dsqrt(2.d0))*x**3/2.d0*beta_hdec(y)
     .            *(1.d0-4.d0*y+12.d0*y**2)
      aff(x,y)= gf/(4*pi*dsqrt(2.d0))*x**3*y*(beta_hdec(y))
      hff(x,y)= gf/(4*pi*dsqrt(2.d0))*x**3*y*(beta_hdec(y))**3
      cff(z,tb,x,y)= gf/(4*pi*dsqrt(2.d0))*z**3*lamb_hdec(x,y)
     .              *((1.d0-x-y)*(x*tb**2+y/tb**2)-4.d0*x*y)
      hv(v)=3.d0*(1.d0-8.d0*v+20.d0*v**2)/dsqrt((4.d0*v-1.d0))
     .      *dacos((3.d0*v-1.d0)/2.d0/dsqrt(v**3))
     .      -(1.d0-v)*(47.d0/2.d0*v-13.d0/2.d0+1.d0/v)
     .      -3.d0/2.d0*(1.d0-6.d0*v+4.d0*v**2)*dlog(v)
      HVH(X,Y)=0.25D0*( (1-X)*(-2+4*X-2*X**2+9*Y+9*X*Y-6*Y**2)
     .        /(3*Y)-2*(1-X-X**2+X**3-3*Y-2*X*Y-3*X**2*Y+3*Y**2
     .        +3*X*Y**2-Y**3)*(-PI/2- DATAN((1-2*X+X**2-Y-X*Y)/
     .         ((1-X)*DSQRT(-1.D0+2*X+2*Y-(X-Y)**2))))/DSQRT(-1.D0
     .         +2*X-(X-Y)**2+2*Y)-(1+X**2-2*Y-2*X*Y+Y**2)*DLOG(X))
      qcd0(x) = (1+x**2)*(4*sp_hdec((1-x)/(1+x))+2*sp_hdec((x-1)/(x+1))
     .        - 3*dlog((1+x)/(1-x))*dlog(2/(1+x))
     .        - 2*dlog((1+x)/(1-x))*dlog(x))
     .        - 3*x*dlog(4/(1-x**2)) - 4*x*dlog(x)
      hqcdm(x)=qcd0(x)/x+(3+34*x**2-13*x**4)/16/x**3*dlog((1+x)/(1-x))
     .        + 3.d0/8/x**2*(7*x**2-1)
      aqcdm(x)=qcd0(x)/x + (19+2*x**2+3*x**4)/16/x*dlog((1+x)/(1-x))
     .        + 3.d0/8*(7-x**2)
      hqcd(x)=(4.d0/3*hqcdm(beta_hdec(x))
     .        +2*(4.d0/3-dlog(x))*(1-10*x)/(1-4*x))*ash/pi
     .       + (29.14671d0 + x*(-93.72459d0+12)
     .         +ratcoup*(1.570d0 - 2*dlog(higtop)/3
     .                                     + dlog(x)**2/9))*(ash/pi)**2
     .       + (164.14d0 - 25.77d0*5 + 0.259d0*5**2)*(ash/pi)**3
     .       +(39.34d0-220.9d0*5+9.685d0*5**2-0.0205d0*5**3)*(ash/pi)**4
      aqcd(x)=(4.d0/3*aqcdm(beta_hdec(x))
     .       + 2*(4.d0/3-dlog(x))*(1-6*x)/(1-4*x))*ash/pi
     .       + (29.14671d0 +ratcoup*(23/6.d0 - dlog(higtop)
     .                               +dlog(x)**2/6))*(ash/pi)**2
     .       + (164.14d0 - 25.77d0*5 + 0.259d0*5**2)*(ash/pi)**3
     .    +(39.34d0-220.9d0*5+9.685d0*5**2-0.0205d0*5**3)*(ash/pi)**4
      qcdh(x)=1.d0+hqcd(x)
      qcda(x)=1.d0+aqcd(x)
      tqcdh(x)=1.d0+4.d0/3*hqcdm(beta_hdec(x))*ash/pi
      tqcda(x)=1.d0+4.d0/3*aqcdm(beta_hdec(x))*ash/pi
      qcdc(x,y)=1.d0+4/3.d0*ash/pi*(9/4.d0 + (3-2*x+2*y)/4*dlog(x/y)
     .         +((1.5d0-x-y)*lamb_hdec(x,y)**2+5*x*y)/2/lamb_hdec(x,y)
     .         /(1-x-y)*dlog(xi_hdec(x,y)*xi_hdec(y,x))
     .         + bij_hdec(x,y))
     .         + ash/pi*(2*(4/3.d0-dlog(x))
     .         - (x*2*(4/3.d0-dlog(x)) + y*2*(4/3.d0-dlog(y)))/(1-x-y)
     .         - (x*2*(4/3.d0-dlog(x))*(1-x+y)
     .           +y*2*(4/3.d0-dlog(y))*(1+x-y))/lamb_hdec(x,y)**2)
      qcdci(x,y)=1.d0+4/3.d0*ash/pi*(3 + (y-x)/2*dlog(x/y)
     .         +(2*(1-x-y)+lamb_hdec(x,y)**2)/2/lamb_hdec(x,y)
     .         *dlog(xi_hdec(x,y)*xi_hdec(y,x))
     .         + bij_hdec(x,y))
     .         + ash/pi*(2*(4/3.d0-dlog(x)) + 2*(4/3.d0-dlog(y))
     .         - (x*2*(4/3.d0-dlog(x))*(1-x+y)
     .           +y*2*(4/3.d0-dlog(y))*(1+x-y))/lamb_hdec(x,y)**2)
      qcdcm(x,y)=1.d0+4/3.d0*ash/pi*(9/4.d0 + (3-2*x+2*y)/4*dlog(x/y)
     .         +((1.5d0-x-y)*lamb_hdec(x,y)**2+5*x*y)/2/lamb_hdec(x,y)
     .         /(1-x-y)*dlog(4*x*y/(1-x-y+lamb_hdec(x,y))**2)
     .         + bij_hdec(x,y))
      qcdcmi(x,y)=1.d0+4/3.d0*ash/pi*(3 + (y-x)/2*dlog(x/y)
     .         +(2*(1-x-y)+lamb_hdec(x,y)**2)/2/lamb_hdec(x,y)
     .         *dlog(4*x*y/(1-x-y+lamb_hdec(x,y))**2)
     .         + bij_hdec(x,y))
c----------------------------------------------------------------------
c--running x Yukawa coupling
      QCDCM1(X,Y)=1.D0+4/3.D0*ASH/PI*(9/4.D0 + (3-2*X+2*Y)/4*DLOG(X/Y)
     .         +((1.5D0-X-Y)*LAMB_HDEC(X,Y)**2+5*X*Y)/2/LAMB_HDEC(X,Y)
     .         /(1-X-Y)*DLOG(4*X*Y/(1-X-Y+LAMB_HDEC(X,Y))**2)
     .         + BIJ_HDEC(X,Y))
     .         + ASH/PI*(2*(4/3.D0-DLOG(X)))
      QCDCMI1(X,Y)=1.D0+4/3.D0*ASH/PI*(3 + (Y-X)/2*DLOG(X/Y)
     .         +(2*(1-X-Y)+LAMB_HDEC(X,Y)**2)/2/LAMB_HDEC(X,Y)
     .         *DLOG(4*X*Y/(1-X-Y+LAMB_HDEC(X,Y))**2)
     .         + BIJ_HDEC(X,Y))
     .         + ASH/PI*(2*(4/3.D0-DLOG(X)))
c----------------------------------------------------------------------
      cqcd(z,tb,x,y,r)= gf/(4*pi*dsqrt(2.d0))*z**3*lamb_hdec(x,y)
     .              *((1.d0-x-y)*(x*tb**2*r**2*qcdc(x,y)
     .                           +y/tb**2*qcdc(y,x))
     .               -4.d0*x*y*qcdci(x,y))
      CQCDM(Z,TB,X,Y,R)= GF/(4*PI*DSQRT(2.D0))*Z**3*LAMB_HDEC(X,Y)
     .              *((1.D0-X-Y)*(X*TB**2*R**2*QCDCM1(X,Y)
     .                           +Y/TB**2*QCDCM(Y,X))
     .               -4.D0*X*Y*R*QCDCMI1(X,Y))
      elw(amh,amf,qf,acf)=alph/pi*3.d0/2*qf**2
     .                              *(3.d0/2-dlog(amh**2/amf**2))
     .      +gf/8/dsqrt(2.d0)/pi**2*(acf*amt**2
     .        +amw**2*(3*dlog(cs)/ss-5)+amz**2*(0.5d0
     .          -3*(1-4*ss*dabs(qf))**2))
      cf(ca) = -cdlog(-(1+cdsqrt(1-ca))/(1-cdsqrt(1-ca)))**2/4
      cg(ca) = cdsqrt(1-ca)/2*cdlog(-(1+cdsqrt(1-ca))/(1-cdsqrt(1-ca)))
      ci1(ca,cb) = ca*cb/2/(ca-cb)
     .           + ca**2*cb**2/2/(ca-cb)**2*(cf(ca)-cf(cb))
     .           + ca**2*cb/(ca-cb)**2*(cg(ca)-cg(cb))
      ci2(ca,cb) = -ca*cb/2/(ca-cb)*(cf(ca)-cf(cb))
      hggqcd(asg,nf)=1.d0+asg/pi*(95.d0/4.d0-nf*7.d0/6.d0)
      hggqcd2(asg,nf,amh,amt)=1.D0+ASG/PI*(95.D0/4.D0-NF*7.D0/6.D0)
     . +(ASG/PI)**2*(149533/288.D0-363/8.D0*ZETA2-495/8.D0*ZETA3
     .              +19/8.D0*DLOG(AMH**2/AMT**2)
     . +NF*(-4157/72.D0+11/2.D0*ZETA2+5/4.D0*ZETA3
     . +2/3.D0*DLOG(AMH**2/AMT**2))
     . +NF**2*(127/108.D0-1/6.D0*ZETA2))
     . +(ASG/PI)**3*(467.683620788D0+122.440972222D0*DLOG(AMH**2/AMT**2)
     .              +10.9409722222D0*DLOG(AMH**2/AMT**2)**2)
      sggqcd(asg)=asg/pi*7.D0/2.D0
      aggqcd(asg,nf)=1.d0+asg/pi*(97.d0/4.d0-nf*7.d0/6.d0)
      aggqcd2(asg,nf,ama,amt)=1.d0+asg/pi*(97.d0/4.d0-nf*7.d0/6.d0)
     . +(asg/pi)**2*(237311/864.d0-529/24.d0*zeta2-445/8.d0*zeta3
     . +5*dlog(ama**2/amt**2))
      hffself(amh)=1.d0+gf*amh**2/16.d0/pi**2/dsqrt(2.d0)*2.117203d0
     .            -(gf*amh**2/16.d0/pi**2/dsqrt(2.d0))**2*32.6567d0
      hvvself(amh)=1.d0+gf*amh**2/16.d0/pi**2/dsqrt(2.d0)*2.800952d0
     .            +(gf*amh**2/16.d0/pi**2/dsqrt(2.d0))**2*62.0308d0
      WTOP(X,ASG,NF)=ASG/PI*(-2/3.D0*(PI**2+2*SP_HDEC(X)-2*SP_HDEC(1-X)
     .               +(4*X*(1-X-2*X**2)*DLOG(X)
     .                +2*(1-X)**2*(5+4*X)*DLOG(1-X)
     .                -(1-X)*(5+9*X-6*X**2))
     .               /2/(1-X)**2/(1+2*X)))
     .       + (ASG/2/PI)**2*(-110.4176D0+7.978D0*NF)
      chtop(x)=8/3.d0*(sp_hdec(1-x)-x/2/(1-x)*dlog(x)
     .        + dlog(x)*dlog(1-x)/2+(1-2.5d0*x)/2/x*dlog(1-x)-pi**2/3
     .        +9/8.d0)
      chtop1(x)=8/3.d0*(sp_hdec(1-x)-x/2/(1-x)*dlog(x)
     .        + dlog(x)*dlog(1-x)/2+(1-2.5d0*x)/2/x*dlog(1-x)-pi**2/3
     .        +17/8.d0)

c --- some parameter settings --- c

      pi=4d0*datan(1d0)
      ss=1.d0-(amw/amz)**2
      cs=1.d0-ss

      zeta2 = pi**2/6
      zeta3 = 1.202056903159594d0
      zeta4 = pi**4/90
      zeta5 = 1.03692775514337d0

      if(ihiggs.ne.0) then
         call susycp_nmssm(tgbet)
      endif

c--decoupling the top quark from alphas
      amt0=3.D8

c--top quark decay width
      GAMT0 = GF*AMT**3/8/DSQRT(2D0)/PI*VTB**2
     .      * LAMB_HDEC(AMB**2/AMT**2,AMW**2/AMT**2)
     .      * ((1-AMW**2/AMT**2)*(1+2*AMW**2/AMT**2)
     .         -AMB**2/AMT**2*(2-AMW**2/AMT**2-AMB**2/AMT**2))
     .      * (1+WTOP(AMW**2/AMT**2,ALPHAS_HDEC(AMT,3),5))

      if(ihiggs.ne.0.and.amt.gt.amch+amb)then
       tsc = (amsq+amur+amdr)/3
       bsc = (amsq+amur+amdr)/3

       rmt = amt
       rmb = amb

       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,rmt,rmb)

       loop  = 1
       qsusy = 1
       qsusy1 = qsusy
       qsusy2 = qsusy
       call botsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,xgh2b,
     .     xgh3b,xga1b,xga2b,xghpm,tgbet,qsusy,loop)
       ymb = runm_hdec(amt,5)
       gamt1 = gf*amt**3/8/dsqrt(2d0)/pi*VTB**2
     .      * lamb_hdec(amb**2/amt**2,amch**2/amt**2)
     .      * (1.d0/tgbet**2*(1+amb**2/amt**2-amch**2/amt**2)
     .      * (1+alphas_hdec(amt,3)/pi*chtop(amch**2/amt**2))
     .        +(ymb/amt)**2*xghpm**2*(1+amb**2/amt**2-amch**2/amt**2)
     .      * (1+alphas_hdec(amt,3)/pi*chtop1(amch**2/amt**2))
     .      + 4*ymb**2/amt**2*xghpm*1.D0/tgbet)
      else
       gamt1 = 0
      endif
      gamt1 = gamt0+gamt1

c      print*,''
c      print*,'top quark width',gamt1
c      print*,''

c ========================= NMSSM Higgs bosons ======================= c

      call gaugino_nmssm(amu,am2,b,amchar,amneut,xmneut,ach1,ach2,ach3,
     .     aca1,aca2,anh1,anh2,anh3,ana1,ana2,acnl,acnr)    

      tsc = (amsq+amur+amdr)/3
      bsc = (amsq+amur+amdr)/3

      rmt = amt
      rmb = amb

       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,rmt,rmb)

      QSUSY = 1
      LOOP = 1

      QSUSY1 = QSUSY
      QSUSY2 = QSUSY

      amhi(1) = amh1
      amhi(2) = amh2
      amhi(3) = amh3
      amhi(4) = ama1
      amhi(5) = ama2

c ==================================================================== c
c                        NMSSM Higgs Boson Decays
c ==================================================================== c

c ==================================================================== c
c ------------------------ Charged Higgs decays ---------------------- c
c ==================================================================== c

      tb=tgbet
c --- running masses --- c

      rms = runm_hdec(amch,3)
      rmc = runm_hdec(amch,4)
      rmb = runm_hdec(amch,5)
      rmt = runm_hdec(amch,6)
      ash=alphas_hdec(amch,3)

c --- partial widths --- c

c H+ ---> mu nmu
      xgh1m = gh1b
      xgh2m = gh2b
      xgh3m = gh3b
      xga1m = ga1b
      xga2m = ga2b
      xghpm = tgbet
c ND added 7/2021
      if(.not. isstrue)then

      if(iofsusy.eq.0) then
       call staususy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1m,
     .     xgh2m,xgh3m,xga1m,xga2m,xghpm,tgbet,qsusy,0)
      endif

      if(amch.le.ammuon) then
       hmn = 0
      else
         hmn=cff(amch,xghpm,(ammuon/amch)**2,0.d0)
      endif
      else !!!
        hmn = 0
        call dmtau_EWnmssm(detau,tgbet,2)
        do i1=1,9
         if (amch.gt.ammuon+amnu(i1)) then
         call Delta_CHlnu(awidth,detau,0,2,i1)
        hmn=hmn+awidth
         endif
        enddo       
      endif !!!
c end ND added 7/2021

c      print*,''
c      print*,'H+- decays'
c      print*,'hmn',hmn,xghpm

c  H+ ---> tau ntau
      xgh1m = gh1b
      xgh2m = gh2b
      xgh3m = gh3b
      xga1m = ga1b
      xga2m = ga2b
      xghpm = tgbet
c ND added 7/2021
      if(.not. isstrue)then
      if(iofsusy.eq.0) then
       call staususy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1m,
     .     xgh2m,xgh3m,xga1m,xga2m,xghpm,tgbet,qsusy,1)
      endif

      if(amch.le.amtau) then
         hln = 0
      else
         hln=cff(amch,xghpm,(amtau/amch)**2,0.d0)
      endif
      else !!!
        hln = 0
        call dmtau_EWnmssm(detau,tgbet,3)
        do i1=1,9
         if (amch.gt.amtau+amnu(i1)) then
         call Delta_CHlnu(awidth,detau,0,3,i1)
        hln=hln+awidth
         endif
        enddo       
      endif !!!
c end ND added 7/2021
c      print*,'hln',hln,xghpm

c  H+ ---> su
      eps = 1.d-12
      ratx = 1
      xgh1s = gh1b
      xgh2s = gh2b
      xgh3s = gh3b
      xga1s = ga1b
      xga2s = ga2b
      xghpm = tgbet
      if(iofsusy.eq.0) then
       call strsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1s,xgh2s,
     .        xgh3s,xga1s,xga2s,xghpm,tgbet,qsusy,loop)
       ratx=xghpm/tgbet
      endif
      if(amch.le.ams+eps) then
       hsu = 0
      else
       hsu1=3.d0*vus**2*cqcdm(amch,tb,(ams/amch)**2,eps,ratx)
       hsu2=3.d0*vus**2*cqcd(amch,tb,(rms/amch)**2,eps,ratx)
       if(hsu2.lt.0.d0) hsu2 = 0
       rat = ams/amch
       hsu = qqint_hdec(rat,hsu1,hsu2)
      endif

c      print*,'hsu',hsu

c  H+ ---> cs
      RATX = RMS/AMS
      RATY = 1
      IF(IOFSUSY.EQ.0) THEN
       call strsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1s,xgh2s,
     .        xgh3s,xga1s,xga2s,xghpm,tgbet,qsusy,loop)
       ratx = rms/ams*xghpm/tgbet
       raty = xghpm/tgbet
      ENDIF
      if(amch.le.ams+amc) then
       hsc = 0
      else
       hsc1=3.d0*cqcdm(amch,tb,(ams/amch)**2,(amc/amch)**2,ratx)*vcs**2
       hsc2=3.d0*cqcd(amch,tb,(rms/amch)**2,(rmc/amch)**2,raty)*vcs**2
       if(hsc2.lt.0.d0) hsc2 = 0
       rat = (ams+amc)/amch
       hsc = qqint_hdec(rat,hsc1,hsc2)
      endif

c      print*,'hsc',hsc

c  H+ ---> cd
      ratx = 1
      eps = 1d-12
      if(amch.le.amc) then
       hcd = 0
      else
       hcd1=3.d0*cqcdm(amch,tb,eps,(amc/amch)**2,ratx)*vcd**2
       hcd2=3.d0*cqcd(amch,tb,eps,(rmc/amch)**2,ratx)*vcd**2
       if(hcd2.lt.0.d0) hcd2 = 0
       rat = (eps+amc)/amch
       hcd = qqint_hdec(rat,hcd1,hcd2)
      endif

c      print*,'hcd',hcd

c  H+ ---> cb
      ratx = 1
      qq = amb
      susy = 0
      xghpm = tgbet
      ssusy = (amsb(1)+amsb(2)+amglu)/3*qsusy
      as0 = alphas_hdec(ssusy,3)
      if(iofsusy.eq.0) then
       i0 = 1
       CALL DMBAPP_NMSSM_HDEC(I0,dgh1b,dgh2b,dgh3b,dga1b,dga2b,dghpm,
     .      QSUSY,LOOP)
       i0 = 1
       bsc = (amsq+amur+amdr)/3
       xmb = amb
       CALL BOTSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,xgh2b,
     .     xgh3b,xga1b,xga2b,xghpm,tgbet,QSUSY,LOOP)
      endif
      ratx = xghpm/tgbet
      if(amch.le.amb+amc) then
       hbc = 0
      else
       hbc1=3.d0*vcb**2*cqcdm(amch,tb,(amb/amch)**2,(amc/amch)**2,ratx)
       hbc2=3.d0*vcb**2*cqcd(amch,tb,(rmb/amch)**2,(rmc/amch)**2,ratx)
       if(hbc2.lt.0.d0) hbc2 = 0
       rat = (amb+amc)/amch
       hbc = qqint_hdec(rat,hbc1,hbc2)
      endif

c      print*,'hbc',hbc

c  H+ ---> bu
      eps = 1.d-12
      if(amch.le.amb+eps) then
       hbu = 0
      else
       hbu1=3.d0*vub**2*cqcdm(amch,tb,(amb/amch)**2,eps,ratx)
       hbu2=3.d0*vub**2*cqcd(amch,tb,(rmb/amch)**2,eps,ratx)
       if(hbu2.lt.0.d0) hbu2 = 0
       rat = amb/amch
       hbu = qqint_hdec(rat,hbu1,hbu2)
      endif

c      print*,'hbu',hbu

c  H+ --> td 
      eps = 1.d-12
      if(ionsh.eq.0)then
       dld=2d0
       dlu=2d0
       xm1 = amt-dld
       xm2 = amt+dlu
       if (amch.le.amw) then
        hdt=0.d0
       elseif (amch.le.xm1) then
        factb=3.d0*gf**2*amch*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(amch,amt,eps,amw,tgbet,ctt0)
        hdt=vtd**2*factb*ctt0
       elseif (amch.le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        factb=3.d0*gf**2*xx(1)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(1),amt,eps,amw,tgbet,ctt0)
        yy(1)=vtd**2*factb*ctt0
        factb=3.d0*gf**2*xx(2)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(2),amt,eps,amw,tgbet,ctt0)
        yy(2)=vtd**2*factb*ctt0
        xmb = runm_hdec(xx(3),5)
        xmt = runm_hdec(xx(3),6)
        xyz2 = 3.d0*cqcd(xx(3),tb,(eps/xx(3))**2,(xmt/xx(3))**2,ratx)
        if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(3),tb,(eps/xx(3))**2,(amt/xx(3))**2,ratx)
        rat = (amt)/xx(3)
        yy(3) = vtd**2*qqint_hdec(rat,xyz1,xyz2)
        xmb = runm_hdec(xx(4),5)
        xmt = runm_hdec(xx(4),6)
        xyz2 = 3.d0*cqcd(xx(4),tb,(eps/xx(4))**2,(xmt/xx(4))**2,ratx)
       if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(4),tb,(eps/xx(4))**2,(amt/xx(4))**2,ratx)
        rat = (amt)/xx(4)
        yy(4) = vtd**2*qqint_hdec(rat,xyz1,xyz2)
        hdt = fint_hdec(amch,xx,yy)
       else
        hdt2=3.d0*vtd**2*cqcd(amch,tb,(eps/amch)**2,(rmt/amch)**2,ratx)
        if(hdt2.lt.0.d0) hdt2 = 0
        hdt1=3.d0*vtd**2*cqcdm(amch,tb,(eps/amch)**2,(amt/amch)**2,ratx)
        rat = (amt)/amch
        hdt = qqint_hdec(rat,hdt1,hdt2)
       endif
      else
       if (amch.le.amt) then
        hdt=0.d0
       else
        hdt2=3.d0*vtd**2*cqcd(amch,tb,eps,(rmt/amch)**2,ratx)
        if(hdt2.lt.0.d0) hdt2 = 0
        hdt1=3.d0*vtd**2*cqcdm(amch,tb,eps,(amt/amch)**2,ratx)
        rat = (amt)/amch
        hdt = qqint_hdec(rat,hdt1,hdt2)
       endif
      endif

c      print*,'hdt',hdt

c  H+ --> ts
      eps = 1.d-12
      ratx = rms/ams
      raty = 1
      if(iofsusy.eq.0) then
       call strsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1s,xgh2s,
     .        xgh3s,xga1s,xga2s,xghpm,tgbet,qsusy,loop)
       ratx = rms/ams*xghpm/tgbet
       raty = xghpm/tgbet
      endif
      if(ionsh.eq.0)then
       dld=2d0
       dlu=2d0
       xm1 = amt+ams-dld
       xm2 = amt+ams+dlu
       if (amch.le.amw+2*ams) then
        hst=0.d0
       elseif (amch.le.xm1) then
        factb=3.d0*gf**2*amch*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(amch,amt,eps,amw,tgbet,ctt0)
        hst=vts**2*factb*ctt0
       elseif (amch.le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        factb=3.d0*gf**2*xx(1)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(1),amt,eps,amw,tgbet,ctt0)
        yy(1)=vts**2*factb*ctt0
        factb=3.d0*gf**2*xx(2)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(2),amt,eps,amw,tgbet,ctt0)
        yy(2)=vts**2*factb*ctt0
        xms = runm_hdec(xx(3),3)
        xmt = runm_hdec(xx(3),6)
        xyz2 = 3.d0*cqcd(xx(3),tb,(xms/xx(3))**2,(xmt/xx(3))**2,raty)
        if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(3),tb,(ams/xx(3))**2,(amt/xx(3))**2,ratx)
        rat = (ams+amt)/xx(3)
        yy(3) = vts**2*qqint_hdec(rat,xyz1,xyz2)
        xms = runm_hdec(xx(4),3)
        xmt = runm_hdec(xx(4),6)
        xyz2 = 3.d0*cqcd(xx(4),tb,(xms/xx(4))**2,(xmt/xx(4))**2,raty)
        if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(4),tb,(ams/xx(4))**2,(amt/xx(4))**2,ratx)
        rat = (ams+amt)/xx(4)
        yy(4) = vts**2*qqint_hdec(rat,xyz1,xyz2)
        hst = fint_hdec(amch,xx,yy)
       else
        hst2=3.d0*vts**2*cqcd(amch,tb,(rms/amch)**2,(rmt/amch)**2,raty)
        if(hst2.lt.0.d0) hst2 = 0
        hst1=3.d0*vts**2*cqcdm(amch,tb,(ams/amch)**2,(amt/amch)**2,ratx)
        rat = (ams+amt)/amch
        hst = qqint_hdec(rat,hst1,hst2)
       endif
      else
       if (amch.le.amt+ams) then
        hst=0.d0
       else
        hst2=3.d0*vts**2*cqcd(amch,tb,(rms/amch)**2,(rmt/amch)**2,raty)
        if(hst2.lt.0.d0) hst2 = 0
        hst1=3.d0*vts**2*cqcdm(amch,tb,(ams/amch)**2,(amt/amch)**2,ratx)
        rat = (ams+amt)/amch
        hst = qqint_hdec(rat,hst1,hst2)
       endif
      endif

c      print*,'hst',hst

c  H+ ---> tb
      ratx = rmb/amb
      raty = 1
      xghpm = tgbet
      if(iofsusy.eq.0) then
       call BOTSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,xgh2b,
     .     xgh3b,xga1b,xga2b,xghpm,tgbet,QSUSY,LOOP)
       ratx = rmb/amb*xghpm/tgbet
       raty = xghpm/tgbet
      endif
      if(ionsh.eq.0)then
       dld=2d0
       dlu=2d0
       xm1 = amt+amb-dld
       xm2 = amt+amb+dlu
       if (amch.le.amw+2*amb) then
        hbt=0.d0
       elseif (amch.le.xm1) then
        factb=3.d0*gf**2*amch*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(amch,amt,amb,amw,tgbet,ctt0)
        hbt=vtb**2*factb*ctt0
       elseif (amch.le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        factb=3.d0*gf**2*xx(1)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(1),amt,amb,amw,tgbet,ctt0)
        yy(1)=VTB**2*factb*ctt0
        factb=3.d0*gf**2*xx(2)*amt**4/32.d0/pi**3/tb**2
        call ctott_hdec(xx(2),amt,amb,amw,tgbet,ctt0)
        yy(2)=VTB**2*factb*ctt0
        xmb = runm_hdec(xx(3),5)
        xmt = runm_hdec(xx(3),6)
        xyz2 = 3.d0*cqcd(xx(3),tb,(xmb/xx(3))**2,(xmt/xx(3))**2,raty)
        if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(3),tb,(amb/xx(3))**2,(amt/xx(3))**2,ratx)
        rat = (amb+amt)/xx(3)
        yy(3) = VTB**2*qqint_hdec(rat,xyz1,xyz2)
        xmb = runm_hdec(xx(4),5)
        xmt = runm_hdec(xx(4),6)
        xyz2 = 3.d0*cqcd(xx(4),tb,(xmb/xx(4))**2,(xmt/xx(4))**2,raty)
        if(xyz2.lt.0.d0) xyz2 = 0
        xyz1 = 3.d0*cqcdm(xx(4),tb,(amb/xx(4))**2,(amt/xx(4))**2,ratx)
        rat = (amb+amt)/xx(4)
        yy(4) = VTB**2*qqint_hdec(rat,xyz1,xyz2)
        hbt = fint_hdec(amch,xx,yy)
       else
        hbt2=3.d0*VTB**2*cqcd(amch,tb,(rmb/amch)**2,(rmt/amch)**2,raty)
        if(hbt2.lt.0.d0) hbt2 = 0
        hbt1=3.d0*VTB**2*cqcdm(amch,tb,(amb/amch)**2,(amt/amch)**2,ratx)
        rat = (amb+amt)/amch
        hbt = qqint_hdec(rat,hbt1,hbt2)
       endif
      else
       if (amch.le.amt+amb) then
        hbt=0.d0
       else
        hbt2=3.d0*VTB**2*cqcd(amch,tb,(rmb/amch)**2,(rmt/amch)**2,raty)
        if(hbt2.lt.0.d0) hbt2 = 0
        hbt1=3.d0*VTB**2*cqcdm(amch,tb,(amb/amch)**2,(amt/amch)**2,ratx)
        rat = (amb+amt)/amch
        hbt = qqint_hdec(rat,hbt1,hbt2)
       endif
      endif

c      print*,'hbt',hbt

c  H+ ---> W H1
      if(ionsh.eq.0)then
       dld=3d0
       dlu=5d0
       xm1 = amw+amh1-dld
       xm2 = amw+amh1+dlu
       if (amch.lt.amh1) then
        hwh=0
       elseif (amch.le.xm1) then
        if(amch.le.dabs(amw-amh1))then
         hwh=0
        else
         hwh=9.d0*gf**2/16.d0/pi**3*amw**4*amch*gh1hpwm**2	
     .      *hvh((amh1/amch)**2,(amw/amch)**2)
        endif
       elseif (amch.lt.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        yy(1) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)
     .         *hvh((amh1/xx(1))**2,(amw/xx(1))**2)
        yy(2) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)
     .         *hvh((amh1/xx(2))**2,(amw/xx(2))**2)
        cwh=lamb_hdec(amh1**2/xx(3)**2,amw**2/xx(3)**2)
     .     *lamb_hdec(xx(3)**2/amw**2,amh1**2/amw**2)**2
        yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwh
        cwh=lamb_hdec(amh1**2/xx(4)**2,amw**2/xx(4)**2)
     .     *lamb_hdec(xx(4)**2/amw**2,amh1**2/amw**2)**2
        yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwh
        hwh = fint_hdec(amch,xx,yy)*gh1hpwm**2
       else
        cwh=lamb_hdec(amh1**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh1**2/amw**2)**2
        hwh=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh1hpwm**2*cwh
       endif
      else
       if (amch.lt.amw+amh1) then
        hwh=0
       else
        cwh=lamb_hdec(amh1**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh1**2/amw**2)**2
        hwh=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh1hpwm**2*cwh
       endif
      endif

c      print*,'hwh1',hwh

c  H+ ---> W H2

      if(ionsh.eq.0)then
       dld=3d0
       dlu=5d0
       xm1 = amw+amh2-dld
       xm2 = amw+amh2+dlu
       if (amch.lt.amh2) then
        hwh2=0
       elseif (amch.le.xm1) then
        if(amch.le.dabs(amw-amh2))then
         hwh2=0
        else
         hwh2=9.d0*gf**2/16.d0/pi**3*amw**4*amch*gh2hpwm**2	
     .      *hvh((amh2/amch)**2,(amw/amch)**2)
        endif
       elseif (amch.lt.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        yy(1) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)
     .         *hvh((amh2/xx(1))**2,(amw/xx(1))**2)
        yy(2) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)
     .         *hvh((amh2/xx(2))**2,(amw/xx(2))**2)
        cwh=lamb_hdec(amh2**2/xx(3)**2,amw**2/xx(3)**2)
     .     *lamb_hdec(xx(3)**2/amw**2,amh2**2/amw**2)**2
        yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwh
        cwh=lamb_hdec(amh2**2/xx(4)**2,amw**2/xx(4)**2)
     .     *lamb_hdec(xx(4)**2/amw**2,amh2**2/amw**2)**2
        yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwh
        hwh2 = fint_hdec(amch,xx,yy)*gh2hpwm**2
       else
        cwh=lamb_hdec(amh2**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh2**2/amw**2)**2
        hwh2=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh2hpwm**2*cwh
       endif
      else
       if (amch.lt.amw+amh2) then
        hwh2=0
       else
        cwh=lamb_hdec(amh2**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh2**2/amw**2)**2
        hwh2=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh2hpwm**2*cwh
       endif
      endif

c      print*,'hwh2',hwh2

c  H+ ---> W H3

      if(ionsh.eq.0)then
       dld=3d0
       dlu=5d0
       xm1 = amw+amh3-dld
       xm2 = amw+amh3+dlu
       if (amch.lt.amh3) then
        hwh3=0
       elseif (amch.le.xm1) then
        if(amch.le.dabs(amw-amh3))then
         hwh3=0
        else
         hwh3=9.d0*gf**2/16.d0/pi**3*amw**4*amch*gh3hpwm**2	
     .      *hvh((amh3/amch)**2,(amw/amch)**2)
        endif
       elseif (amch.lt.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        yy(1) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)
     .         *hvh((amh3/xx(1))**2,(amw/xx(1))**2)
        yy(2) = 9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)
     .         *hvh((amh3/xx(2))**2,(amw/xx(2))**2)
        cwh=lamb_hdec(amh3**2/xx(3)**2,amw**2/xx(3)**2)
     .     *lamb_hdec(xx(3)**2/amw**2,amh3**2/amw**2)**2
        yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwh
        cwh=lamb_hdec(amh3**2/xx(4)**2,amw**2/xx(4)**2)
     .     *lamb_hdec(xx(4)**2/amw**2,amh3**2/amw**2)**2
        yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwh
        hwh3 = fint_hdec(amch,xx,yy)*gh3hpwm**2
       else
        cwh=lamb_hdec(amh3**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh3**2/amw**2)**2
        hwh3=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh3hpwm**2*cwh
       endif
      else
       if (amch.lt.amw+amh3) then
        hwh3=0
       else
        cwh=lamb_hdec(amh3**2/amch**2,amw**2/amch**2)
     .     *lamb_hdec(amch**2/amw**2,amh3**2/amw**2)**2
        hwh3=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*gh3hpwm**2*cwh
       endif
      endif

c      print*,'hwh3',hwh3

c  H+ ---> W A1

      deltalow = 3.D0
      deltahigh = 5.D0

      if(ionsh.eq.0) then
         xm1 = ama1+amw - deltalow
         xm2 = ama1+amw + deltahigh

         if (amch.lt.ama1.or.ama1.eq.0.D0) then
            hwa=0
         elseif(amch.lt.xm1) then
            if(amch.le.dabs(amw-ama1)) then
               hwa = 0.D0
            else
               hwa=9.d0*gf**2/16.d0/pi**3*amw**4*amch	
     .              *hvh((ama1/amch)**2,(amw/amch)**2)*(ga1b/tgbet)**2  
            endif
         elseif(amch.lt.xm2) then
            xx(1) = xm1-1.D0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1.D0

            yy(1)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)	
     .        *hvh((ama1/xx(1))**2,(amw/xx(1))**2)
            yy(2)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)	
     .        *hvh((ama1/xx(2))**2,(amw/xx(2))**2)

            cwpa1=lamb_hdec(ama1**2/xx(3)**2,amw**2/xx(3)**2)
     .           *lamb_hdec(xx(3)**2/amw**2,ama1**2/amw**2)**2
            yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwpa1
            cwpa1=lamb_hdec(ama1**2/xx(4)**2,amw**2/xx(4)**2)
     .           *lamb_hdec(xx(4)**2/amw**2,ama1**2/amw**2)**2
            yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwpa1

            hwa = fint_hdec(amch,xx,yy)*(ga1b/tgbet)**2
         else
            cwpa1=lamb_hdec(ama1**2/amch**2,amw**2/amch**2)
     .           *lamb_hdec(amch**2/amw**2,ama1**2/amw**2)**2
            hwa=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*cwpa1*(ga1b/tgbet)**2
         endif
      else
         if (amch.lt.amw+ama1.or.ama1.eq.0.D0) then
            hwa=0
         else
            cwpa1=lamb_hdec(ama1**2/amch**2,amw**2/amch**2)
     .           *lamb_hdec(amch**2/amw**2,ama1**2/amw**2)**2
            hwa=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*cwpa1*(ga1b/tgbet)**2
         endif
      endif

c      print*,'hwa',hwa

c  H+ ---> W A2

      deltalow = 3.D0
      deltahigh = 5.D0

      if(ionsh.eq.0) then
         xm1 = ama2+amw - deltalow
         xm2 = ama2+amw + deltahigh

       if (amch.lt.ama2) then
        hwa2=0
      elseif(amch.lt.xm1) then
         if(amch.le.dabs(amw-ama2)) then
            hwa2 = 0.D0
         else
            hwa2=9.d0*gf**2/16.d0/pi**3*amw**4*amch	
     .           *hvh((ama2/amch)**2,(amw/amch)**2)*(ga2b/tgbet)**2  
         endif
      elseif(amch.lt.xm2) then
            xx(1) = xm1-1.D0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1.D0

            yy(1)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)	
     .        *hvh((ama2/xx(1))**2,(amw/xx(1))**2)
            yy(2)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)	
     .        *hvh((ama2/xx(2))**2,(amw/xx(2))**2)

            cwpa2=lamb_hdec(ama2**2/xx(3)**2,amw**2/xx(3)**2)
     .           *lamb_hdec(xx(3)**2/amw**2,ama2**2/amw**2)**2
            yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwpa2
            cwpa2=lamb_hdec(ama2**2/xx(4)**2,amw**2/xx(4)**2)
     .           *lamb_hdec(xx(4)**2/amw**2,ama2**2/amw**2)**2
            yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwpa2

            hwa2 = fint_hdec(amch,xx,yy)*(ga2b/tgbet)**2
        else
           cwpa2=lamb_hdec(ama2**2/amch**2,amw**2/amch**2)
     .          *lamb_hdec(amch**2/amw**2,ama2**2/amw**2)**2
           hwa2=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*cwpa2*(ga2b/tgbet)**2
        endif
      else
         if (amch.lt.amw+ama2) then
            hwa2=0
         else
            cwpa2=lamb_hdec(ama2**2/amch**2,amw**2/amch**2)
     .           *lamb_hdec(amch**2/amw**2,ama2**2/amw**2)**2
            hwa2=gf/8.d0/dsqrt(2d0)/pi*amw**4/amch*cwpa2*(ga2b/tgbet)**2
         endif
      endif

c      print*,'hwa2',hwa2

c ============================ SUSY decays =========================== c
      if(iofsusy.eq.0) then

c  H+ ---> charginos+neutralinos

      do ii=1,2
      do jj=1,5
      if (amch.gt.amchar(ii)+amneut(jj)) then
         whccn(ii,jj)=gf*amw**2/(2*pi*dsqrt(2.d0))/amch
     .        *lamb_hdec(amchar(ii)**2/amch**2,amneut(jj)**2/amch**2)*(
     .       (acnl(ii,jj)**2+acnr(ii,jj)**2)*(amch**2-amchar(ii)**2
     .     -xmneut(jj)**2)-4.d0*acnl(ii,jj)*acnr(ii,jj)*xmchar(ii)*
     .        xmneut(jj) )
      else
      whccn(ii,jj)=0.d0
      endif
      enddo
      enddo

      whccnt=whccn(1,1)+whccn(1,2)+whccn(1,3)+whccn(1,4)+whccn(1,5)
     .      +whccn(2,1)+whccn(2,2)+whccn(2,3)+whccn(2,4)+whccn(2,5)

c      do ii=1,2,1
c         do jj=1,5,1
c            print*,'H+chari+neutj',whccn(ii,jj),ii,jj
c         end do
c      end do
c      print*,'whccnt',whccnt

c  H+ ---> sleptons 
      if(.not.isstrue)then
      if (amch.gt.amse(1)+amsn(1)) then
      whcsl00=2*gf/2.d0/dsqrt(2d0)/pi*amw**4/amch*ghcelne(1,1)**2
     .     *lamb_hdec(amse(1)**2/amch**2,amsn(1)**2/amch**2)
      else 
      whcsl00=0.d0
      endif

      if (amch.gt.amsl(1)+amsnl(1)) then
      whcsl11=gf/2.d0/dsqrt(2d0)/pi*amw**4/amch*ghcen(1,1)**2
     .     *lamb_hdec(amsl(1)**2/amch**2,amsnl(1)**2/amch**2)
      else 
      whcsl11=0.d0
      endif

      if (amch.gt.amsl(2)+amsnl(1)) then
      whcsl21=gf/2.d0/dsqrt(2d0)/pi*amw**4/amch*ghcen(1,2)**2
     .     *lamb_hdec(amsl(2)**2/amch**2,amsnl(1)**2/amch**2)
      else 
      whcsl21=0.d0
      endif

      whcslt=whcsl00+whcsl11+whcsl21
      else
      whcsl00=0.d0
      whcsl11=0.d0
      whcsl21=0.d0
      do i1=1,18
      if (amch.gt.amse(1)+amsn(i1)) then
      call Delta_CHstausneu(amch,amse(1),amsn(i1),1,i1,awidth,1,0)
      whcsl00=whcsl00+awidth
      endif
      if (amch.gt.amse(2)+amsn(i1)) then
      call Delta_CHstausneu(amch,amse(2),amsn(i1),2,i1,awidth,1,0)
      whcsl00=whcsl00+awidth
      endif

      if (amch.gt.amse(1)+amsn(i1)) then
      call Delta_CHstausneu(amch,amse(1),amsn(i1),3,i1,awidth,1,0)
      whcsl00=whcsl00+awidth
      endif
      if (amch.gt.amse(2)+amsn(i1)) then
      call Delta_CHstausneu(amch,amse(2),amsn(i1),4,i1,awidth,1,0)
      whcsl00=whcsl00+awidth
      endif

      if (amch.gt.amsl(1)+amsn(i1)) then
      call Delta_CHstausneu(amch,amsl(1),amsn(i1),5,i1,awidth,1,0)
      whcsl11=whcsl11+awidth
      endif

      if (amch.gt.amsl(2)+amsn(i1)) then
      call Delta_CHstausneu(amch,amsl(2),amsn(i1),6,i1,awidth,1,0)
      whcsl21=whcsl21+awidth
      endif
      enddo

      whcslt=whcsl00+whcsl11+whcsl21
      endif

c end ND changed 7/2021

c      print*,'H+ esn_e',whcsl00
c      print*,'H+ stau1 sn_tau',whcsl11
c      print*,'H+ stau2 sn_tau',whcsl21
c      print*,'whcslt',whcslt

c  H+ ---> squarks 

      if (amch.gt.amsu(1)+amsd(1)) then
      whcsq=6*gf*amw**4/2.d0/dsqrt(2d0)/pi*ghcud(1,1)**2
     .     *lamb_hdec(amsu(1)**2/amch**2,amsd(1)**2/amch**2)/amch
      else 
      whcsq=0.d0
      endif

      do ii=1,2
      do jj=1,2
      if(amch.gt.amst(ii)+amsb(jj)) then
      whcstb(ii,jj)=3*gf*amw**4/2.d0/dsqrt(2.d0)/pi*ghctb(ii,jj)**2
     .      *lamb_hdec(amst(ii)**2/amch**2,amsb(jj)**2/amch**2)/amch
      else
      whcstb(ii,jj)=0.d0
      endif
      enddo
      enddo

      whcsqt=whcsq+whcstb(1,1)+whcstb(1,2)+whcstb(2,1)+whcstb(2,2) 

c      do ii=1,2,1
c         do jj=1,2,1
c            print*,'whcstb,i,j',whcstb(ii,jj),ii,jj
c         end do
c      end do
c      print*,'whcsq',whcsq
c      print*,'whcsqt',whcsqt
      
      else 
      whccnt=0.d0
      whcslt=0.d0
      whcsqt=0.d0
c -- change thanks to Elzbieta Richter-Was -- c
      do ii=1,2
       do jj=1,2
        whcstb(ii,jj)=0.d0
       enddo
      enddo
      do ii=1,2
       do jj=1,4
        whccn(ii,jj)=0.d0
       enddo
      enddo
      endif

c  =================  total width and branching ratios =============== c

      wtot=hln+hmn+hsu+hbu+hsc+hcd+hbc+hbt+hwh+hwh2+hwh3+hwa+hwa2+whccnt
     .     +whcslt+whcsqt+hdt+hst

      hcbrl=hln/wtot
      hcbrm=hmn/wtot
      hcbrs=hsu/wtot
      hcbrbu=hbu/wtot
      hcbrc=hsc/wtot
      hcbrd= hcd/wtot
      hcbrb=hbc/wtot
      hcbrdt = hdt/wtot
      hcbrst = hst/wtot
      hcbrt=hbt/wtot
      hcbrwh1=hwh/wtot
      hcbrwh2=hwh2/wtot
      hcbrwh3=hwh3/wtot
      hcbra1=hwa/wtot
      hcbra2=hwa2/wtot
      do  ii=1,2
      do  jj=1,5
      hcbrsu(ii,jj)=whccn(ii,jj)/wtot
      enddo
      enddo
      hcbrcnt=whccnt/wtot
      hcbrsl=whcslt/wtot 
      hcbrsq=whcsq/wtot 
      hcbrsqt=whcsqt/wtot 
      do  ii=1,2
      do  jj=1,2
      hcbrstb(ii,jj)=whcstb(ii,jj)/wtot
      enddo
      enddo
      hcwdth=wtot

      bhcsl00 = whcsl00/wtot
      bhcsl11 = whcsl11/wtot
      bhcsl21 = whcsl21/wtot
      bhcsq = whcsq/wtot
      do i = 1,2
       do j = 1,2
        bhcstb(i,j) = whcstb(i,j)/wtot
       enddo
      enddo

c      print*,'total width',hcwdth

      gamc0 = wtot

c ==================================================================== c
c ------------------ Neutral NMSSM Higgs boson decays  --------------  c
c ==================================================================== c

c ==================================================================== c
c --------------------- Scalar Higgs boson decays -------------------- c
c ==================================================================== c

c      print*,'Hi decays'
c --- running masses ---
         do i=1,3,1
            rms = runm_hdec(amhi(i),3)
            rmc = runm_hdec(amhi(i),4)
            rmb = runm_hdec(amhi(i),5)
            rmt = runm_hdec(amhi(i),6)

      if(iruncoup.eq.1) then
       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,rmt,rmb)
      elseif(iruncoup.eq.0) then
       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,amt,amb)
      endif

      ratcoup = ghigt(i)/ghigb(i)
      higtop = amhi(i)**2/amt**2

      ash=alphas_hdec(amhi(i),3)
      amc0=1.d8
      amb0=2.d8
      as3=alphas_hdec(amhi(i),3)
      amc0=amc
      as4=alphas_hdec(amhi(i),3)
      amb0=amb

c --- partial widths --- c
c Hi ---> g g    i=1,2,3
       eps=1.d-8
       nfext = 3
       asg = as3
       ctt = 4*amt**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctb = 4*amb**2/amhi(i)**2*dcmplx(1d0,-eps)
       cat = 2*ctt*(1+(1-ctt)*cf(ctt))*ghigt(i)
       cab = 2*ctb*(1+(1-ctb)*cf(ctb))*ghigb(i)
       ctc = 4*amc**2/amhi(i)**2*dcmplx(1d0,-eps)
       cac = 2*ctc*(1+(1-ctc)*cf(ctc))*ghigt(i)
       
       catc= 2*ctt*(1+(1-ctt)*cf(ctt))
       cabc= 2*ctb*(1+(1-ctb)*cf(ctb))
       cacc= 2*ctc*(1+(1-ctc)*cf(ctc))

C
       if(iofsusy.eq.0) then 
       csb1= 4*amsb(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
       csb2= 4*amsb(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
       cst1= 4*amst(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
       cst2= 4*amst(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
       cxb1=-amz**2/amsb(1)**2*csb1*(1-csb1*cf(csb1))*ghigbb(i,1,1)
       cxb2=-amz**2/amsb(2)**2*csb2*(1-csb2*cf(csb2))*ghigbb(i,2,2)
       cxt1=-amz**2/amst(1)**2*cst1*(1-cst1*cf(cst1))*ghigtt(i,1,1)
       cxt2=-amz**2/amst(2)**2*cst2*(1-cst2*cf(cst2))*ghigtt(i,2,2)

       csul = 4*amsu(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
       csur = 4*amsu(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
       csdl = 4*amsd(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
       csdr = 4*amsd(2)**2/amhi(i)**2*dcmplx(1d0,-eps)

       cxul=-2*amz**2/amsu(1)**2*csul*(1-csul*cf(csul))*ghiguu(i,1,1)
       cxur=-2*amz**2/amsu(2)**2*csur*(1-csur*cf(csur))*ghiguu(i,2,2)
       cxdl=-2*amz**2/amsd(1)**2*csdl*(1-csdl*cf(csdl))*ghigdd(i,1,1)
       cxdr=-2*amz**2/amsd(2)**2*csdr*(1-csdr*cf(csdr))*ghigdd(i,2,2) 

       else
       cxb1=0.d0 
       cxb2=0.d0 
       cxt1=0.d0 
       cxt2=0.d0 

       cxul=0.d0 
       cxur=0.d0 
       cxdl=0.d0 
       cxdr=0.d0 
       endif

       fqcd=hggqcd(asg,nfext)
       sqcd=sggqcd(asg)
       xfac = cdabs(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .             +cxul+cxur+cxdl+cxdr)**2*fqcd
     .      + dreal(dconjg(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .                    +cxul+cxur+cxdl+cxdr)
     .             *(cxb1+cxb2+cxt1+cxt2+cxul+cxur+cxdl+cxdr))*sqcd
       hgg=hvv(amhi(i),0.D0)*(asg/pi)**2*xfac/8
       

      cj(i)=sqrt(cdabs(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .             +cxul+cxur+cxdl+cxdr)**2+
     .          dreal(dconjg(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .                    +cxul+cxur+cxdl+cxdr)
     .             *(cxb1+cxb2+cxt1+cxt2+cxul+cxur+cxdl+cxdr))*sqcd/fqcd)/
     .      abs(catc+cacc+cabc)
!       write(*,*) 'cj',i,cj(i)

c       print*,''
c       print*,'Scalar Higgs decays'

c  Hi ---> g g* ---> g cc  to be added to H1 ---> cc
       nfext = 4
       asg = as4
       fqcd=hggqcd(asg,nfext)
       sqcd=sggqcd(asg)
       xfac = cdabs(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .             +cxul+cxur+cxdl+cxdr)**2*fqcd
     .      + dreal(dconjg(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .                    +cxul+cxur+cxdl+cxdr)
     .             *(cxb1+cxb2+cxt1+cxt2+cxul+cxur+cxdl+cxdr))*sqcd
       dcc=hvv(amhi(i),0.D0)*(asg/pi)**2*xfac/8 - hgg

c       print*,'dcc,i',dcc,i

c  Hi ---> g g* ---> g bb  to be added to H1 ---> bb
       nfext = 5
       asg = ash
       fqcd=hggqcd(asg,nfext)
       sqcd=sggqcd(asg)
       xfac = cdabs(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .             +cxul+cxur+cxdl+cxdr)**2*fqcd
     .      + dreal(dconjg(cat+cab+cac+cxb1+cxb2+cxt1+cxt2
     .                    +cxul+cxur+cxdl+cxdr)
     .             *(cxb1+cxb2+cxt1+cxt2+cxul+cxur+cxdl+cxdr))*sqcd
       dbb=hvv(amhi(i),0.D0)*(asg/pi)**2*xfac/8 - hgg - dcc
       hgg=hvv(amhi(i),0.D0)*(asg/pi)**2*xfac/8

c       print*,'hgg 2,i',hgg,i

c  H1 ---> g g: full NNNLO corrections to top loops for nf=5
       fqcd0=hggqcd(asg,5)
       fqcd=hggqcd2(asg,5,amhi(i),amt)
       xfac = cdabs(cat+cab)**2*(fqcd-fqcd0)
       hgg=hgg+hvv(amhi(i),0.D0)*(asg/pi)**2*xfac/8

      if(nfgg.eq.3)then
       hgg = hgg - dbb - dcc
      elseif(nfgg.eq.4)then
       hgg = hgg - dbb
       dcc = 0
      else
       dcc = 0
       dbb = 0
      endif

      higgg(i) = hgg

c      print*,'hgg',hgg,i
c ND added 7/2021
c  Hi ---> nu nu
      hnn=0
      if (isstrue)then
      do i1=1,9
        do i2=1,9
         if (amhi(i) .gt. amnu(i1)+amnu(i2)) then
           call Delta_Hnunu(amhi(i),i+2,i1,i2,awidth,0)
           hnn=hnn+awidth
         endif
        enddo
      enddo
      endif
      hignn(i) = hnn
c end ND added 7/2021
c  Hi ---> mu mu
      xgh1m = gh1b
      xgh2m = gh2b
      xgh3m = gh3b
      xga1m = ga1b
      xga2m = ga2b
      xghpm = tgbet
      IF(IOFSUSY.EQ.0) THEN
       CALL STAUSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1m,
     .     xgh2m,xgh3m,xga1m,xga2m,xghpm,tgbet,qsusy,0)
      ENDIF
      xghigm(1) = xgh1m
      xghigm(2) = xgh2m
      xghigm(3) = xgh3m
      xghigm(4) = xga1m
      xghigm(5) = xga2m

      if(amhi(i).le.2*ammuon) then
       hmm = 0
      else
       hmm=hff(amhi(i),(ammuon/amhi(i))**2)*xghigm(i)**2
      endif

      higmm(i) = hmm

c      print*,'hmm',hmm,i

c  Hi ---> tau tau
      xgh1b = gh1b
      xgh2b = gh2b
      xgh3b = gh3b
      xga1b = ga1b
      xga2b = ga2b
      xghpm = tgbet
      IF(IOFSUSY.EQ.0) THEN
       CALL STAUSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,
     .     xgh2b,xgh3b,xga1b,xga2b,xghpm,tgbet,qsusy,1)
      ENDIF
      xghigb(1) = xgh1b
      xghigb(2) = xgh2b
      xghigb(3) = xgh3b
      xghigb(4) = xga1b
      xghigb(5) = xga2b

      if(amhi(i).le.2*amtau) then
       hll = 0
      else
      hll=hff(amhi(i),(amtau/amhi(i))**2)*xghigb(i)**2
      endif

      higll(i) = hll

c      print*,'hll',hll,i

c  Hi ---> ss
      xgh1s = gh1b
      xgh2s = gh2b
      xgh3s = gh3b
      xga1s = ga1b
      xga2s = ga2b
      if(iofsusy.eq.0) then
       call STRSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1s,xgh2s,
     .        xgh3s,xga1s,xga2s,xghpm,tgbet,qsusy,loop)
      endif
      xghigs(1) = xgh1s
      xghigs(2) = xgh2s
      xghigs(3) = xgh3s
      xghigs(4) = xga1s
      xghigs(5) = xga2s

      if(amhi(i).le.2*ams) then
       hss = 0
      else
       hs1=3.d0*hff(amhi(i),(ams/amhi(i))**2)
     .    *xghigs(i)**2
     .    *tqcdh(ams**2/amhi(i)**2)
       hs2=3.d0*hff(amhi(i),(rms/amhi(i))**2)*xghigs(i)**2
     .    *qcdh(rms**2/amhi(i)**2)
       if(hs2.lt.0.d0) hs2 = 0
       rat = 2*ams/amhi(i)
       hss = qqint_hdec(rat,hs1,hs2)
      endif

      higss(i) = hss

c      print*,'hss',hss,i

c  Hi --> cc
      ratcoup = 1
      if(amhi(i).le.2*amc) then
       hcc = 0
      else
       hc1=3.d0*hff(amhi(i),(amc/amhi(i))**2)
     .    *ghigt(i)**2
     .    *tqcdh(amc**2/amhi(i)**2)
       hc2=3.d0*hff(amhi(i),(rmc/amhi(i))**2)*ghigt(i)**2
     .    *qcdh(rmc**2/amhi(i)**2)
     .   + dcc
       if(hc2.lt.0.d0) hc2 = 0
       rat = 2*amc/amhi(i)
       hcc = qqint_hdec(rat,hc1,hc2)
      endif

      higcc(i) = hcc

c      print*,'hcc',hcc,i

c  Hi ---> bb
      qq = amb
      susy = 0
      xgh1b = gh1b
      xgh2b = gh2b
      xgh3b = gh3b
      xghigb(1) = xgh1b
      xghigb(2) = xgh2b
      xghigb(3) = xgh3b
      ssusy = (amsb(1)+amsb(2)+amglu)/3*qsusy
      as0 = alphas_hdec(ssusy,3)
      if(iofsusy.eq.0) then
       I0 = 1
       CALL DMBAPP_NMSSM_HDEC(I0,dgh1b,dgh2b,dgh3b,dga1b,dga2b,dghpm,
     .      QSUSY,LOOP)
       i0 = i
       bsc = (amsq+amur+amdr)/3
       xmb = amb
       dghib(1) = dgh1b
       dghib(2) = dgh2b
       dghib(3) = dgh3b
       susy = cofsusy_nmssm(i0,amb,xmb,qq)*as0/pi - 2*dghib(i)
c       print*,'SUSY',susy
       CALL BOTSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,xgh2b,
     .     xgh3b,xga1b,xga2b,xghpm,tgbet,QSUSY,LOOP)
       xghigb(1) = xgh1b
       xghigb(2) = xgh2b
       xghigb(3) = xgh3b
      ENDIF
      ratcoup = ghigt(i)/xghigb(i)
      if(amhi(i).le.2*amb) then
       hbb = 0
      else
c         susy = 0.D0
       hb1=3.d0*hff(amhi(i),(amb/amhi(i))**2)
     .    *(xghigb(i)**2+xghigb(i)*ghigb(i)*susy)
     .    *tqcdh(amb**2/amhi(i)**2)
       hb2=3.d0*hff(amhi(i),(rmb/amhi(i))**2)
     .    *(xghigb(i)**2+xghigb(i)*ghigb(i)*susy)
     .    *qcdh(rmb**2/amhi(i)**2)
     .   + dbb
c       print*,'susy',susy,gh2b,xghigb(i)
       if(hb2.lt.0.d0) hb2 = 0
       rat = 2*amb/amhi(i)
       hbb = qqint_hdec(rat,hb1,hb2)
      endif

      higbb(i) = hbb

c      print*,'hbb',hbb,i

c  Hi ---> tt
      ratcoup = 0
      if(ionsh.eq.0)then
         dld=3d0
         dlu=5d0
         xm1 = 2d0*amt-dld
         xm2 = 2d0*amt+dlu
         if (amhi(i).le.amt+amw+amb) then
            htt=0.d0
         elseif (amhi(i).le.xm1) then
            factt=6.d0*gf**2*amhi(i)**3*amt**2/2.d0/128.d0/pi**3
            CALL HTOTT_NMSSM(AMHI(I),AMT,AMB,AMW,AMCH,TB,GHIGT(I),
     .           GHIGB(I),1.D0/tb,ghigvv(i),ghighpwm(i),HTT0)
            htt=factt*htt0
         elseif (amhi(i).le.xm2) then
            xx(1) = xm1-1d0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1d0
            
            outputfile = 'interpolhitt_1.dat'
            call MHpmfromMHi(xx(1),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            factt=6.d0*gf**2*xx(1)**3*amt**2/2.d0/128.d0/pi**3
            CALL HTOTT_NMSSM(xx(1),amt,amb,amw,amhichar,tb,ghit,ghict,
     .           1.d0/tb,ghivv,ghihpwm,htt0)
            yy(1)=factt*htt0

            outputfile = 'interpolhitt_2.dat'
            call MHpmfromMHi(xx(2),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            factt=6.d0*gf**2*xx(2)**3*amt**2/2.d0/128.d0/pi**3
            CALL HTOTT_NMSSM(xx(2),amt,amb,amw,amhichar,tb,ghit,ghict,
     .           1.d0/tb,ghivv,ghihpwm,htt0)
            yy(2)=factt*htt0

            outputfile = 'interpolhitt_3.dat'
            call MHpmfromMHi(xx(3),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            xmt = runm_hdec(xx(3),6)
            ht1=3.d0*hff(xx(3),(amt/xx(3))**2)*ghit**2
     .           *tqcdh(amt**2/xx(3)**2)
            ht2=3.d0*hff(xx(3),(xmt/xx(3))**2)*ghit**2
     .           *qcdh(xmt**2/xx(3)**2)
            if(ht2.lt.0.d0) ht2 = 0
            rat = 2*amt/xx(3)
            yy(3) = qqint_hdec(rat,ht1,ht2)

            outputfile = 'interpolhitt_4.dat'
            call MHpmfromMHi(xx(4),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            xmt = runm_hdec(xx(4),6)
            ht1=3.d0*hff(xx(4),(amt/xx(4))**2)*ghit**2
     .           *tqcdh(amt**2/xx(4)**2)
            ht2=3.d0*hff(xx(4),(xmt/xx(4))**2)*ghit**2
     .           *qcdh(xmt**2/xx(4)**2)
            if(ht2.lt.0.d0) ht2 = 0
            rat = 2*amt/xx(4)
            yy(4) = qqint_hdec(rat,ht1,ht2)

            htt=fint_hdec(amhi(i),xx,yy)
            end if
            end if
            end if
            end if
         else
            ht1=3.d0*hff(amhi(i),(amt/amhi(i))**2)*ghigt(i)**2
     .           *tqcdh(amt**2/amhi(i)**2)
            ht2=3.d0*hff(amhi(i),(rmt/amhi(i))**2)*ghigt(i)**2
     .           *qcdh(rmt**2/amhi(i)**2)
            if(ht2.lt.0.d0) ht2 = 0
            rat = 2*amt/amhi(i)
            htt = qqint_hdec(rat,ht1,ht2)
         endif
         
         if(calcmasseserror) then
           calcmasseserror=.FALSE.
           print*, 'Error in off-shell decay Hi -> tt. i=',i
           print*, 'Off-shell decay turned off'
           if (amhi(i).le.2.d0*amt) then
              htt=0.d0
           else
              ht1=3.d0*hff(amhi(i),(amt/amhi(i))**2)*ghigt(i)**2
     .             *tqcdh(amt**2/amhi(i)**2)
              ht2=3.d0*hff(amhi(i),(rmt/amhi(i))**2)*ghigt(i)**2
     .             *qcdh(rmt**2/amhi(i)**2)
              if(ht2.lt.0.d0) ht2 = 0
              rat = 2*amt/amhi(i)
              htt = qqint_hdec(rat,ht1,ht2)
           endif
         end if
         
      else
         if (amhi(i).le.2.d0*amt) then
            htt=0.d0
         else
            ht1=3.d0*hff(amhi(i),(amt/amhi(i))**2)*ghigt(i)**2
     .           *tqcdh(amt**2/amhi(i)**2)
            ht2=3.d0*hff(amhi(i),(rmt/amhi(i))**2)*ghigt(i)**2
     .           *qcdh(rmt**2/amhi(i)**2)
            if(ht2.lt.0.d0) ht2 = 0
            rat = 2*amt/amhi(i)
            htt = qqint_hdec(rat,ht1,ht2)
         endif
      endif

      higtt(i) = htt

c      print*,'htt',higtt(i),i

c  Hi ---> gamma gamma
       eps=1.d-8
       xrmc = runm_hdec(amhi(i)/2,4)*amc/runm_hdec(amc,4)
       xrmb = runm_hdec(amhi(i)/2,5)*amb/runm_hdec(amb,5)
       xrmt = runm_hdec(amhi(i)/2,6)*amt/runm_hdec(amt,6)
       ctt = 4*xrmt**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctb = 4*xrmb**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctc = 4*xrmc**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctl = 4*amtau**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctw = 4*amw**2/amhi(i)**2*dcmplx(1d0,-eps)
       cth = 4*amch**2/amhi(i)**2*dcmplx(1d0,-eps)
       cat = 4/3d0 * 2*ctt*(1+(1-ctt)*cf(ctt))*ghigt(i)
     .     * cfacq_hdec(0,amhi(i),xrmt)
       cab = 1/3d0 * 2*ctb*(1+(1-ctb)*cf(ctb))*ghigb(i)
     .     * cfacq_hdec(0,amhi(i),xrmb)
       cac = 4/3d0 * 2*ctc*(1+(1-ctc)*cf(ctc))*ghigt(i)
     .     * cfacq_hdec(0,amhi(i),xrmc)
       cal = 1.d0  * 2*ctl*(1+(1-ctl)*cf(ctl))*ghigb(i)
       caw = -(2+3*ctw+3*ctw*(2-ctw)*cf(ctw))*ghigvv(i)
       cah = -amz**2/2/amch**2*cth*(1-cth*cf(cth))*ghigpm(i)

c maggie changed 25/7/2012
       catsm = 4/3d0 * 2*ctt*(1+(1-ctt)*cf(ctt))
     .     * cfacq_hdec(0,amhi(i),xrmt)
       cabsm = 1/3d0 * 2*ctb*(1+(1-ctb)*cf(ctb))
     .     * cfacq_hdec(0,amhi(i),xrmb)
       cacsm = 4/3d0 * 2*ctc*(1+(1-ctc)*cf(ctc))
     .     * cfacq_hdec(0,amhi(i),xrmc)
       calsm = 1.d0  * 2*ctl*(1+(1-ctl)*cf(ctl))
       cawsm = -(2+3*ctw+3*ctw*(2-ctw)*cf(ctw))
c end maggie changed 25/7/2012

       if(iofsusy.eq.0) then 
        rmsu1 = runms_hdec(amhi(i)/2,amsu(1))
        rmsu2 = runms_hdec(amhi(i)/2,amsu(2))
        rmsd1 = runms_hdec(amhi(i)/2,amsd(1))
        rmsd2 = runms_hdec(amhi(i)/2,amsd(2))
        rmsb1 = runms_hdec(amhi(i)/2,amsb(1))
        rmsb2 = runms_hdec(amhi(i)/2,amsb(2))
        rmst1 = runms_hdec(amhi(i)/2,amst(1))
        rmst2 = runms_hdec(amhi(i)/2,amst(2))
        cx1 = 4*amchar(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
        cx2 = 4*amchar(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
        csb1= 4*rmsb1**2/amhi(i)**2*dcmplx(1d0,-eps)
        csb2= 4*rmsb2**2/amhi(i)**2*dcmplx(1d0,-eps)
        cst1= 4*rmst1**2/amhi(i)**2*dcmplx(1d0,-eps)
        cst2= 4*rmst2**2/amhi(i)**2*dcmplx(1d0,-eps)
        csl1= 4*amsl(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
        csl2= 4*amsl(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
        cax1= amw/xmchar(1) * 2*cx1*(1+(1-cx1)*cf(cx1))*2*achig(i,1,1) 
        cax2= amw/xmchar(2) * 2*cx2*(1+(1-cx2)*cf(cx2))*2*achig(i,2,2) 

        csel = 4*amse(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
        cser = 4*amse(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
        csul = 4*rmsu1**2/amhi(i)**2*dcmplx(1d0,-eps)
        csur = 4*rmsu2**2/amhi(i)**2*dcmplx(1d0,-eps)
        csdl = 4*rmsd1**2/amhi(i)**2*dcmplx(1d0,-eps)
        csdr = 4*rmsd2**2/amhi(i)**2*dcmplx(1d0,-eps)

        cxel=-2*amz**2/amse(1)**2*csel*(1-csel*cf(csel))*ghigelel(i,1,1)
        cxer=-2*amz**2/amse(2)**2*cser*(1-cser*cf(cser))*ghigelel(i,2,2)
        cxul=-2*4.D0/3.D0*amz**2/amsu(1)**2*csul*(1-csul*cf(csul))*
     .       ghiguu(i,1,1)* cfacsq_hdec(amhi(i),rmsu1)
        cxur=-2*4.D0/3.D0*amz**2/amsu(2)**2*csur*(1-csur*cf(csur))*
     .       ghiguu(i,2,2)* cfacsq_hdec(amhi(i),rmsu2)
        cxdl=-2/3.d0*amz**2/amsd(1)**2*csdl*(1-csdl*cf(csdl))*
     .       ghigdd(i,1,1)* cfacsq_hdec(amhi(i),rmsd1)
        cxdr=-2/3.d0*amz**2/amsd(2)**2*csdr*(1-csdr*cf(csdr))*
     .       ghigdd(i,2,2)* cfacsq_hdec(amhi(i),rmsd2)

        cxb1=-1/3d0*amz**2/amsb(1)**2*csb1*(1-csb1*cf(csb1))*
     .       ghigbb(i,1,1)* cfacsq_hdec(amhi(i),rmsb1)
        cxb2=-1/3d0*amz**2/amsb(2)**2*csb2*(1-csb2*cf(csb2))*
     .       ghigbb(i,2,2)* cfacsq_hdec(amhi(i),rmsb2)
        cxt1=-4/3d0*amz**2/amst(1)**2*cst1*(1-cst1*cf(cst1))*
     .       ghigtt(i,1,1)* cfacsq_hdec(amhi(i),rmst1)
        cxt2=-4/3d0*amz**2/amst(2)**2*cst2*(1-cst2*cf(cst2))*
     .       ghigtt(i,2,2)* cfacsq_hdec(amhi(i),rmst2)

        csl1= 4*amsl(1)**2/amhi(i)**2*dcmplx(1d0,-eps)
        csl2= 4*amsl(2)**2/amhi(i)**2*dcmplx(1d0,-eps)
        cxl1=   -amz**2/amsl(1)**2*csl1*(1-csl1*cf(csl1))*ghigee(i,1,1)
        cxl2=   -amz**2/amsl(2)**2*csl2*(1-csl2*cf(csl2))*ghigee(i,2,2)

        xfac = cdabs(cat+cab+cac+cal+caw+cah+cax1+cax2
     .      +  cxel+cxer+cxul+cxur+cxdl+cxdr
     .      +  cxb1+cxb2+cxt1+cxt2+cxl1+cxl2)**2
       else 
        xfac = cdabs(cat+cab+cac+cal+caw+cah)**2
       endif
       hga=hvv(amhi(i),0.D0)*(alph/pi)**2/16.D0*xfac

       higga(i) = hga

c       print*,'hga',hga,i

c  Hi ---> Z gamma
      if(amhi(i).le.amz)then
       hzga=0
      else
       ts = ss/cs
       ft = -3*2D0/3*(1-4*2D0/3*ss)/dsqrt(ss*cs)*ghigt(i)
       fb = 3*1D0/3*(-1+4*1D0/3*ss)/dsqrt(ss*cs)*ghigb(i)
       fc = -3*2D0/3*(1-4*2D0/3*ss)/dsqrt(ss*cs)*ghigt(i)
       fl = (-1.D0+4.D0*ss)/dsqrt(ss*cs)*ghigb(i)
       eps=1.D-8
       ctt = 4*amt**2/amhi(i)**2*dcmplx(1D0,-eps)
       ctb = 4*amb**2/amhi(i)**2*dcmplx(1D0,-eps)
       ctc = 4*amc**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctl = 4*amtau**2/amhi(i)**2*dcmplx(1d0,-eps)
       ctw = 4*amw**2/amhi(i)**2*dcmplx(1D0,-eps)
       cth = 4*amch**2/amhi(i)**2*dcmplx(1D0,-eps)
       clt = 4*amt**2/amz**2*dcmplx(1D0,-eps)
       clb = 4*amb**2/amz**2*dcmplx(1D0,-eps)
       clc = 4*amc**2/amz**2*dcmplx(1d0,-eps)
       cle = 4*amtau**2/amz**2*dcmplx(1d0,-eps)
       clw = 4*amw**2/amz**2*dcmplx(1D0,-eps)
       clh = 4*amch**2/amz**2*dcmplx(1D0,-eps)
       cat = ft*(ci1(ctt,clt) - ci2(ctt,clt))
       cab = fb*(ci1(ctb,clb) - ci2(ctb,clb))
       cac = fc*(ci1(ctc,clc) - ci2(ctc,clc))
       cal = fl*(ci1(ctl,cle) - ci2(ctl,cle))
       caw = -1/dsqrt(ts)*(4*(3-ts)*ci2(ctw,clw)
     .     + ((1+2/ctw)*ts - (5+2/ctw))*ci1(ctw,clw))*ghigvv(i)
       cah = (1-2*ss)/dsqrt(ss*cs)*amz**2/2/amch**2*ci1(cth,clh)*
     ,      ghigpm(i)
       xfac = cdabs(cat+cab+cac+cal+caw+cah)**2
       acoup = dsqrt(2d0)*gf*amz**2*ss*cs/pi**2
       hzga = gf/(4.d0*pi*dsqrt(2.d0))*amhi(i)**3*(alph/pi)*acoup/16.d0
     .        *xfac*(1-amz**2/amhi(i)**2)**3
      endif

      higzga(i) = hzga

c      print*,'hzga',hzga,i

C  Hi ---> W W
c      print*,'ionwz',ionwz

      if(ionwz.eq.0)then
       call htovv_hdec(0,amhi(i),amw,gamw,htww)
       hww = 3D0/2D0*gf*amw**4/dsqrt(2d0)/pi/amhi(i)**3*htww*
     .      ghigvv(i)**2
      elseif(ionwz.eq.-1)then
       dld=2D0
       dlu=2D0
       xm1 = 2D0*amw-dld
       xm2 = 2D0*amw+dlu
       if (amhi(i).le.xm1) then
        call htovv_hdec(0,amhi(i),amw,gamw,htww)
        hww = 3D0/2D0*gf*amw**4/dsqrt(2D0)/pi/amhi(i)**3*htww*
     .       ghigvv(i)**2
       elseif (amhi(i).le.xm2) then
        xx(1) = xm1-1D0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1D0
        call htovv_hdec(0,xx(1),amw,gamw,htww)
        yy(1)=3d0/2d0*gf*amw**4/dsqrt(2d0)/pi/xx(1)**3*htww
        call htovv_hdec(0,xx(2),amw,gamw,htww)
        yy(2)=3d0/2d0*gf*amw**4/dsqrt(2d0)/pi/xx(2)**3*htww
        yy(3)=hvv(xx(3),amw**2/xx(3)**2)
        yy(4)=hvv(xx(4),amw**2/xx(4)**2)
        hww = fint_hdec(amhi(i),xx,yy)*ghigvv(i)**2
       else
        hww=hvv(amhi(i),amw**2/amhi(i)**2)*ghigvv(i)**2
       endif
      else
      dld=2D0
      dlu=2D0
      xm1 = 2D0*amw-dld
      xm2 = 2D0*amw+dlu
      if (amhi(i).le.amw) then
       hww=0
      else if (amhi(i).le.xm1) then
       cww=3.d0*gf**2*amw**4/16.d0/pi**3
       hww=hv(amw**2/amhi(i)**2)*cww*amhi(i)*ghigvv(i)**2
      else if (amhi(i).lt.xm2) then
       cww=3.d0*gf**2*amw**4/16.d0/pi**3
       xx(1) = xm1-1d0
       xx(2) = xm1
       xx(3) = xm2
       xx(4) = xm2+1d0
       yy(1)=hv(amw**2/xx(1)**2)*cww*xx(1)
       yy(2)=hv(amw**2/xx(2)**2)*cww*xx(2)
       yy(3)=hvv(xx(3),amw**2/xx(3)**2)
       yy(4)=hvv(xx(4),amw**2/xx(4)**2)
       hww = fint_hdec(amhi(i),xx,yy)*ghigvv(i)**2
      else
       hww=hvv(amhi(i),amw**2/amhi(i)**2)*ghigvv(i)**2
      endif
      endif

      higww(i) = hww

c      print*,'hww',hww

c  Hi ---> Z Z

      if(ionwz.eq.0)then
       call htovv_hdec(0,amhi(i),amz,gamz,htzz)
       hzz = 3d0/4d0*gf*amz**4/dsqrt(2d0)/pi/amhi(i)**3*htzz*
     .      ghigvv(i)**2
      elseif(ionwz.eq.-1)then
       dld=2d0
       dlu=2d0
       xm1 = 2d0*amz-dld
       xm2 = 2d0*amz+dlu
       if (amhi(i).le.xm1) then
        call htovv_hdec(0,amhi(i),amz,gamz,htzz)
        hzz = 3d0/4d0*gf*amz**4/dsqrt(2d0)/pi/amhi(i)**3*htzz*
     .       ghigvv(i)**2
       elseif (amhi(i).le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        call htovv_hdec(0,xx(1),amz,gamz,htzz)
        yy(1)=3d0/4d0*gf*amz**4/dsqrt(2d0)/pi/xx(1)**3*htzz
        call htovv_hdec(0,xx(2),amz,gamz,htzz)
        yy(2)=3d0/4d0*gf*amz**4/dsqrt(2d0)/pi/xx(2)**3*htzz
        yy(3)=hvv(xx(3),amz**2/xx(3)**2)/2
        yy(4)=hvv(xx(4),amz**2/xx(4)**2)/2
        hzz = fint_hdec(amhi(i),xx,yy)*ghigvv(i)**2
       else
        hzz=hvv(amhi(i),amz**2/amhi(i)**2)/2.d0*ghigvv(i)**2
       endif
      else
      dld=2d0
      dlu=2d0
      xm1 = 2d0*amz-dld
      xm2 = 2d0*amz+dlu
      if (amhi(i).le.amz) then
       hzz=0
      else if (amhi(i).le.xm1) then
       czz=3.d0*gf**2*amz**4/192.d0/pi**3*(7-40/3.d0*ss+160/9.d0*ss**2)
       hzz=hv(amz**2/amhi(i)**2)*czz*amhi(i)*ghigvv(i)**2
      else if (amhi(i).lt.xm2) then
       czz=3.d0*gf**2*amz**4/192.d0/pi**3*(7-40/3.d0*ss+160/9.d0*ss**2)
       xx(1) = xm1-1d0
       xx(2) = xm1
       xx(3) = xm2
       xx(4) = xm2+1d0
       yy(1)=hv(amz**2/xx(1)**2)*czz*xx(1)
       yy(2)=hv(amz**2/xx(2)**2)*czz*xx(2)
       yy(3)=hvv(xx(3),amz**2/xx(3)**2)/2d0
       yy(4)=hvv(xx(4),amz**2/xx(4)**2)/2d0
       hzz = fint_hdec(amhi(i),xx,yy)*ghigvv(i)**2
      else
       hzz=hvv(amhi(i),amz**2/amhi(i)**2)/2.d0*ghigvv(i)**2
      endif
      endif

      higzz(i) = hzz

c      print*,'hzz',hzz,i

c =========================== SUSY decays ============================ c

      if(iofsusy.eq.0) then

c Hi ---> charginos

      do ii=1,2,1
      do jj=1,2,1
      if (amhi(i).gt.amchar(ii)+amchar(jj)) then
      whigch(i,ii,jj)=gf*amw**2/(2*pi*dsqrt(2.d0))/amhi(i)
     .     *lamb_hdec(amchar(ii)**2/amhi(i)**2,amchar(jj)**2/amhi(i)**2)
     .     *( (achig(i,ii,jj)**2+achig(i,jj,ii)**2)*
     .        (amhi(i)**2-amchar(ii)**2-amchar(jj)**2) 
     .      -4.d0*achig(i,ii,jj)*achig(i,jj,ii)*xmchar(ii)*xmchar(jj))
 
      else
      whigch(i,ii,jj)=0.d0
      endif

      end do 
      end do

      whigcht(i)=whigch(i,1,1)+whigch(i,1,2)+whigch(i,2,1)+whigch(i,2,2)

c      do ii=1,2,1
c         do jj=1,2,1
c            print*,'Hi->charjk, jk',whigch(i,ii,jj),i,ii,jj
c         end do 
c      end do

c      print*,'whigcht',whigcht(i),i

c  Hi ---> neutralinos 

      do ii=1,5
      do jj=1,5
      if (amhi(i).gt.amneut(ii)+amneut(jj)) then
      whigne(i,ii,jj)=gf*amw**2/(2*pi*dsqrt(2.d0))/amhi(i) 
     .     *anhig(i,ii,jj)**2*(amhi(i)**2-(xmneut(ii)+xmneut(jj))**2)
     .     *lamb_hdec(amneut(ii)**2/amhi(i)**2,amneut(jj)**2/amhi(i)**2)
      else 
      whigne(i,ii,jj)=0.d0
      endif
      end do
      end do

      whignet(i)= whigne(i,1,1)+whigne(i,1,2)+whigne(i,1,3)
     .     +whigne(i,1,4)+whigne(i,1,5)
     .     +whigne(i,2,1)+whigne(i,2,2)+whigne(i,2,3)+whigne(i,2,4)
     .     +whigne(i,2,5)
     .     +whigne(i,3,1)+whigne(i,3,2)+whigne(i,3,3)+whigne(i,3,4)
     .     +whigne(i,3,5)
     .     +whigne(i,4,1)+whigne(i,4,2)+whigne(i,4,3)+whigne(i,4,4)
     .     +whigne(i,4,5)
     .     +whigne(i,5,1)+whigne(i,5,2)+whigne(i,5,3)+whigne(i,5,4)
     .     +whigne(i,5,5)

c      do ii=1,5,1
c         do jj=1,5,1
c            print*,'Hi->neutraljk,j,k',whigne(i,ii,jj),i,ii,jj
c         end do
c      end do
c      print*,'whignet',whignet(i),i

c  Hi ---> sleptons 

      if(amhi(i).gt.2.d0*amse(1)) then
         whislel(i)=2.d0*gf*amz**4/2.d0/dsqrt(2.d0)/pi*
     .        ghigelel(i,1,1)**2*
     .        beta_hdec(amse(1)**2/amhi(i)**2)/amhi(i)
      else
         whislel(i)=0.d0
      endif
      if(amhi(i).gt.2.d0*amse(2)) then
         whisler(i)=2.d0*gf*amz**4/2.d0/dsqrt(2.d0)/pi*
     .        ghigelel(i,2,2)**2*
     .        beta_hdec(amse(2)**2/amhi(i)**2)/amhi(i)
      else
         whisler(i)=0.d0
      endif

      do ii=1,2
      do jj=1,2
      if(amhi(i).gt.amsl(ii)+amsl(jj)) then
      whistau(i,ii,jj)=gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghigee(i,ii,jj)**2*
     .  lamb_hdec(amsl(ii)**2/amhi(i)**2,amsl(jj)**2/amhi(i)**2)/amhi(i)
      else
      whistau(i,ii,jj)=0.d0
      endif
      end do
      end do

c ND changed 7/2021
c  Hi ---> sneutrinos
      hsnsn=0D0
      if(.not.isstrue)then
      if (amhi(i).gt.2.d0*amsn(1)) then
         whislnl(i)=2*gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghignene(i,1,1)**2*
     .        beta_hdec(amsn(1)**2/amhi(i)**2)/amhi(i)
      else
         whislnl(i)=0.d0
      endif

      if (amhi(i).gt.2.d0*amsnl(1)) then
         whislnltau(i)=gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghignene(i,1,1)**2*
     .        beta_hdec(amsnl(1)**2/amhi(i)**2)/amhi(i)
      else
         whislnltau(i)=0.d0
      endif
      else
        do i1=1,18
          do i2=1,18
           if (amhi(i) .gt. amsn(i1) +amsn(i2) )then
            call Delta_Hsneusneu(amhi(i),amsn(i1),amsn(i2),i+2,i1,i2,awidth,1,0)
            hsnsn=hsnsn+awidth          
           endif
          enddo
        enddo
       whislnl(i)=hsnsn
       whislnltau(i)=0.d0
      endif
c end ND changed 7/2021


      whislt(i)=whistau(i,1,1)+whistau(i,2,1)+whistau(i,1,2)
     .     +whistau(i,2,2)+whislel(i)+whisler(i)+whislnl(i)
     .     +whislnltau(i)

c      do ii=1,2,1
c         do jj=1,2,1
c            print*,'Hi->staus,j,k',whistau(i,ii,jj),i,ii,jj
c         end do
c      end do
c      print*,'Hi->slel',whislel(i)
c      print*,'Hi->sler',whisler(i)
c      print*,'Hi->snel',whislnl(i)
c      print*,'Hi->sntaul',whislnltau(i)

c      print*,'whislt',whislt(i)

c  H1 ---> squarks 

      if (amhi(i).gt.2.D0*amsu(1)) then
         whisqul(i)=6*gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghiguu(i,1,1)**2*
     .        beta_hdec(amsu(1)**2/amhi(i)**2)/amhi(i)
      else
         whisqul(i)=0.D0
      endif

      if (amhi(i).gt.2.d0*amsu(2)) then
         whisqur(i)=6*gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghiguu(i,2,2)**2*
     .        beta_hdec(amsu(2)**2/amhi(i)**2)/amhi(i)
      else
         whisqur(i)=0.D0
      endif

      if (amhi(i).gt.2.d0*amsd(1)) then
         whisqdl(i)=6*gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghigdd(i,1,1)**2*
     .        beta_hdec(amsd(1)**2/amhi(i)**2)/amhi(i)
      else
         whisqdl(i)=0.D0
      endif

      if (amhi(i).gt.2.d0*amsd(2)) then
         whisqdr(i)=6*gf*amz**4/2.d0/dsqrt(2.d0)/pi*ghigdd(i,2,2)**2*
     .        beta_hdec(amsd(2)**2/amhi(i)**2)/amhi(i)
      else
         whisqdr(i)=0.D0
      endif

      whisq(i)=whisqul(i)+whisqur(i)+whisqdl(i)+whisqdr(i)

c      print*,'Hi -> supL',whisqul(i),i
c      print*,'Hi -> supR',whisqur(i),i
c      print*,'Hi -> sdowL',whisqdl(i),i
c      print*,'Hi -> sdowR',whisqdr(i),i
c      print*,'whisq',whisq(i)

c  Hi ---> stops 

      susy = 1
      do ii=1,2
      do jj=1,2
      qsq = amhi(i)
      susy = 1
      istoperror=0
      if(amhi(i).gt.ymst(ii)+ymst(jj)) then
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       CALL SQMBAPP_NMSSM_HDEC(QSQ)
       SUSY = 1+SQSUSY_NMSSM_HDEC(i,1,II,JJ,QSQ,0,1)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       whist(i,ii,jj)=3*gf*amz**4/2.d0/dsqrt(2.d0)/pi*yhitt(i,ii,jj)**2*
     . lamb_hdec(ymst(ii)**2/amhi(i)**2,ymst(jj)**2/amhi(i)**2)/amhi(i)
     .          *susy

      if(whist(i,ii,jj).ne.whist(i,ii,jj)) then
         istoperror=1
       whist(i,ii,jj)=3*gf*amz**4/2.d0/dsqrt(2.d0)/pi*
     .        ghigtt(i,ii,jj)**2*
     . lamb_hdec(amst(ii)**2/amhi(i)**2,amst(jj)**2/amhi(i)**2)/amhi(i)
      endif

      else
      whist(i,ii,jj)=0.d0
      endif
      end do
      end do

      if(istoperror.eq.1) then
         print*,'There is a problem in the loop-corrected stop decays. T
     .he tree-level decay is taken instead for Hi, i=',i
         print*,''
      endif
      istoperror=0

c  Hi ---> sbottoms

      susy = 1
      isboterror=0
      do ii=1,2
      do jj=1,2

      qsq = amhi(i)
      if(amhi(i).gt.ymsb(ii)+ymsb(jj)) then
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       call sqmbapp_NMSSM_hdec(qsq)
       susy = 1+sqsusy_nmssm_hdec(i,2,ii,jj,qsq,0,1)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       whisb(i,ii,jj)=3*gf*amz**4/2.d0/dsqrt(2.d0)/pi*yhibb(i,ii,jj)**2*
     . lamb_hdec(ymsb(ii)**2/amhi(i)**2,ymsb(jj)**2/amhi(i)**2)/amhi(i)
     .      *susy

      if(whisb(i,ii,jj).ne.whisb(i,ii,jj)) then
         isboterror=1
       whisb(i,ii,jj)=3*gf*amz**4/2.d0/dsqrt(2.d0)/pi*
     .        ghigbb(i,ii,jj)**2*
     . lamb_hdec(amsb(ii)**2/amhi(i)**2,amsb(jj)**2/amhi(i)**2)/amhi(i)
      endif

      else
      whisb(i,ii,jj)=0.d0
      endif
      end do
      end do
c

      if(isboterror.eq.1) then
         print*,'There is a problem in the loop-corrected sbot decays. T
     .he tree-level decay is taken instead for Hi, i=',i
         print*,''
      endif
      isboterror=0

c      do ii=1,2,1
c         do jj=1,2,1
c            print*,'Hi->sbottoms,j,k',whisb(i,ii,jj),ii,jj
c         end do
c      end do

      whistt(i)=whist(i,1,1)+whist(i,1,2)+whist(i,2,1)+whist(i,2,2) 
      whisbb(i)=whisb(i,1,1)+whisb(i,1,2)+whisb(i,2,1)+whisb(i,2,2) 
      whisqt(i)=whistt(i)+whisbb(i)+whisq(i)

c      print*,'sum-stops',whistt(i),i
c      print*,'sum-sbottoms',whisbb(i),i
c      print*,'sum-all squarks',whisqt(i),i

      else 
      whigcht(i)=0.d0
      whignet(i)=0.d0
      whislt(i)=0.d0
      whisqt(i)=0.d0

      do ii=1,2
       do jj=1,2
        whigch(i,ii,jj)=0.d0
        whist(i,ii,jj)=0.d0
        whisb(i,ii,jj)=0.d0
        whistau(i,ii,jj)=0.d0
       enddo
      enddo
      do ii=1,5
       do jj=1,5
        whigne(i,ii,jj)=0.d0
       enddo
      enddo
      endif

c -- total width w/o Higgs-to-Higgs and Higgs-to-gauge-Higgs decays -- c
c ND changed 7/2021
      whitot(i)=higll(i)+higmm(i)+higss(i)+higcc(i)+higbb(i)+higtt(i)+
     .     higgg(i)+higga(i)+higzga(i)+higww(i)+higzz(i)
     .     +whigcht(i)+whignet(i)+whislt(i)+whisqt(i)+hignn(i)
c end ND changed 7/2021
c      tmp = higll(i)+higmm(i)+higss(i)+higcc(i)+higtt(i)+
c     .     higgg(i)+higbb(i)
c
c      print*,'tmp',tmp,i

c      print*,'whitot,1',whitot(i),i

c -------------------------------------------------------------------- c

c Hi ---> Hj Hk

      do j=1,5,1
         do k=1,5,1
            hitohjhk(i,j,k) = 0.D0
         end do
      end do

c --- on-shell decays --- c

      htoh(i) = 0.D0

      do j=1,5,1
         do k=1,5,1
            if(amhi(i).ge.amhi(j)+amhi(k)) then
               hitohjhk(i,j,k)=gf/16.D0/dsqrt(2D0)/pi*amz**4/amhi(i)
     .          *lamb_hdec(amhi(j)**2/amhi(i)**2,amhi(k)**2/amhi(i)**2)*
     .           g3hihjhk(i,j,k)**2
               htoh(i) = htoh(i) + hitohjhk(i,j,k)
c               if(hitohjhk(i,j,k).ne.0.D0) then
c                  print*,'hitohjhk on',hitohjhk(i,j,k),i,j,k
c               endif
            endif
         end do
      end do

c Hi ---> Aj Z on- and off-shell decays
      
      do j=4,5,1
         hitoajz(i,j) = 0.D0
      end do

      htoaz(i) = 0.D0

      deltalow = 3.D0
      deltahigh = 5.D0

      do j=4,5,1
      if(ionsh.eq.0) then
         xm1 = amhi(j)+amz - deltalow
         xm2 = amhi(j)+amz + deltahigh

         if(amhi(i).le.amhi(j).or.amhi(j).eq.0.D0) then
            hitoajz(i,j) = 0.D0
         elseif(amhi(i).le.xm1) then
            if(amhi(i).le.dabs(amz-amhi(j))) then
               hitoajz(i,j) = 0.D0
            else
               hitoajz(i,j)=9.d0*gf**2/16.d0/pi**3*amz**4*amhi(i)*
     .              gzaihj(j-3,i)**2*
     .              (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .              *hvh((amhi(j)/amhi(i))**2,(amz/amhi(i))**2)
            endif
         elseif(amhi(i).le.xm2) then
            xx(1) = xm1-1.D0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1.D0

            outputfile = 'interpolhiajz_1.dat'
            call MHpmfromMHi(xx(1),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,j-3,1,ghjb,ghkb,gajb,
     .           gakb,ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,
     .           amhiggsk,amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.(amhaj+amz)) then
               caz=lamb_hdec(amhaj**2/xx(1)**2,amz**2/xx(1)**2)
     .         *lamb_hdec(xx(1)**2/amz**2,amhaj**2/amz**2)**2
               yy(1)=gf/8.d0/dsqrt(2d0)/pi*amz**4/xx(1)*caz
            elseif(amhiggsi.le.amhaj) then
               yy(1) = 0
            else
               yy(1)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(1)*
     .              (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .              *hvh((amhaj/xx(1))**2,(amz/xx(1))**2)
            endif

            outputfile = 'interpolhiajz_2.dat'
            call MHpmfromMHi(xx(2),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,j-3,1,ghjb,ghkb,gajb,
     .           gakb,ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,
     .           amhiggsk,amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.(amhaj+amz)) then
               caz=lamb_hdec(amhaj**2/xx(2)**2,amz**2/xx(2)**2)
     .         *lamb_hdec(xx(2)**2/amz**2,amhaj**2/amz**2)**2
               yy(2)=gf/8.d0/dsqrt(2d0)/pi*amz**4/xx(2)*caz
            elseif(amhiggsi.le.amhaj) then
               yy(2) = 0
            else
               yy(2)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(2)*
     .              (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .              *hvh((amhaj/xx(2))**2,(amz/xx(2))**2)
            endif

            outputfile = 'interpolhiajz_3.dat'
            call MHpmfromMHi(xx(3),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,j-3,1,ghjb,ghkb,gajb,
     .           gakb,ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,
     .           amhiggsk,amhai,amhaj,amhak,amhichar) 
            if(amhiggsi.ge.(amhaj+amz)) then
               caz=lamb_hdec(amhaj**2/xx(3)**2,amz**2/xx(3)**2)
     .         *lamb_hdec(xx(3)**2/amz**2,amhaj**2/amz**2)**2
               yy(3)=gf/8.d0/dsqrt(2d0)/pi*amz**4/xx(3)*caz
            elseif(amhiggsi.le.amhaj) then
               yy(3) = 0
            else
               yy(3)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(3)*
     .              (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .              *hvh((amhaj/xx(3))**2,(amz/xx(3))**2)
            endif

            outputfile = 'interpolhiajz_4.dat'
            call MHpmfromMHi(xx(4),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,j-3,1,ghjb,ghkb,gajb,
     .           gakb,ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,
     .           amhiggsk,amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.(amhaj+amz)) then
               caz=lamb_hdec(amhaj**2/xx(4)**2,amz**2/xx(4)**2)
     .         *lamb_hdec(xx(4)**2/amz**2,amhaj**2/amz**2)**2
               yy(4)=gf/8.d0/dsqrt(2d0)/pi*amz**4/xx(4)*caz
            elseif(amhiggsi.le.amhaj) then
               yy(4) = 0
            else
               yy(4)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(4)*
     .              (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .              *hvh((amhaj/xx(4))**2,(amz/xx(4))**2)
            endif
            
            hitoajz(i,j) = fint_hdec(amhi(i),xx,yy)*gzaihj(j-3,i)**2
            end if
            end if
            end if
            end if
         else
            caz=lamb_hdec(amhi(j)**2/amhi(i)**2,amz**2/amhi(i)**2)
     .           *lamb_hdec(amhi(i)**2/amz**2,amhi(j)**2/amz**2)**2
            hitoajz(i,j)=gf/8.d0/dsqrt(2d0)/pi*amz**4/amhi(i)*caz*
     .           gzaihj(j-3,i)**2
         endif
         
         if(calcmasseserror) then
            calcmasseserror=.FALSE.
            print*, 'Error in off-shell decay Hi -> Aj Z. i,j=',i,j 
            print*, 'Off-shell decay turned off'
            if (amhi(i).lt.amz+amhi(j).or.amhi(j).eq.0.D0) then
               hitoajz(i,j)=0.D0
            else
               caz=lamb_hdec(amhi(j)**2/amhi(i)**2,amz**2/amhi(i)**2)
     .              *lamb_hdec(amhi(i)**2/amz**2,amhi(j)**2/amz**2)**2
               hitoajz(i,j)=gf/8.d0/dsqrt(2d0)/pi*amz**4/amhi(i)*caz*
     .              gzaihj(j-3,i)**2
            endif  
         endif
      else
         if (amhi(i).lt.amz+amhi(j).or.amhi(j).eq.0.D0) then
            hitoajz(i,j)=0.D0
         else
            caz=lamb_hdec(amhi(j)**2/amhi(i)**2,amz**2/amhi(i)**2)
     .           *lamb_hdec(amhi(i)**2/amz**2,amhi(j)**2/amz**2)**2
            hitoajz(i,j)=gf/8.d0/dsqrt(2d0)/pi*amz**4/amhi(i)*caz*
     .           gzaihj(j-3,i)**2
         endif
      endif

      htoaz(i) = htoaz(i) + hitoajz(i,j)

c      if(hitoajz(i,j).ne.0.D0) then
c         print*,'hitoajz,A',hitoajz(i,j),i,j,htoaz(i),amhi(4)
c      endif
      
      end do

c      print*,'hitoajz check',hitoajz(1,4),hitoajz(2,4)

c Hi ---> W+ H- on- and off-shell decays

      deltalow = 3.D0
      deltahigh = 5.D0

      if(ionsh.eq.0) then
         xm1 = amch+amw - deltalow
         xm2 = amch+amw + deltahigh

         if(amhi(i).le.amch) then
            hitohpwm(i) = 0.D0
         elseif(amhi(i).le.xm1) then
            if(amhi(i).le.dabs(amw-amch)) then
               hitohpwm(i) = 0.D0
            else
               hitohpwm(i) = 9.D0*gf**2/16.d0/pi**3*amw**4*amhi(i)*
     .              ghighpwm(i)**2*2
     .              *hvh((amch/amhi(i))**2,(amw/amhi(i))**2)
            endif
         elseif(amhi(i).le.xm2) then
            xx(1) = xm1-1.D0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1.D0

            outputfile = 'interpolhihcw_1.dat'
            call MHpmfromMHi(xx(1),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.amhichar+amw) then
               chw=lamb_hdec(amhichar**2/xx(1)**2,amw**2/xx(1)**2)
     .              *lamb_hdec(xx(1)**2/amw**2,amhichar**2/amw**2)**2
               yy(1)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(1)*chw
            elseif(amhiggsi.le.amhichar) then
               yy(1) = 0
            else
               yy(1)=9.D0*gf**2/16.d0/pi**3*amw**4*xx(1)*2
     .              *hvh((amhichar/xx(1))**2,(amw/xx(1))**2)
            endif

            outputfile = 'interpolhihcw_2.dat'
            call MHpmfromMHi(xx(2),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)   
            if(amhiggsi.ge.(amhichar+amw)) then
               chw=lamb_hdec(amhichar**2/xx(2)**2,amw**2/xx(2)**2)
     .              *lamb_hdec(xx(2)**2/amw**2,amhichar**2/amw**2)**2
               yy(2)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(2)*chw
            elseif(amhiggsi.le.amhichar) then
               yy(2) = 0
            else
               yy(2)=9.D0*gf**2/16.d0/pi**3*amw**4*xx(2)*2
     .              *hvh((amhichar/xx(2))**2,(amw/xx(2))**2)
            endif

            outputfile = 'interpolhihcw_3.dat'
            call MHpmfromMHi(xx(3),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.(amhichar+amw)) then
               chw=lamb_hdec(amhichar**2/xx(3)**2,amw**2/xx(3)**2)
     .              *lamb_hdec(xx(3)**2/amw**2,amhichar**2/amw**2)**2
               yy(3)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*chw
            elseif(amhiggsi.le.amhichar) then
               yy(3) = 0
            else
               yy(3)=9.D0*gf**2/16.d0/pi**3*amw**4*xx(3)*2
     .              *hvh((amhichar/xx(3))**2,(amw/xx(3))**2)
            endif

            outputfile = 'interpolhihcw_4.dat'
            call MHpmfromMHi(xx(4),i+2,1d-9,outputfile)
            if(.not. calcmasseserror) then
            call DecInterpolation(outputfile,i,1,1,ghjb,ghkb,gajb,gakb,
     .           ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,
     .           amhai,amhaj,amhak,amhichar)
            if(amhiggsi.ge.(amhichar+amw)) then
               chw=lamb_hdec(amhichar**2/xx(4)**2,amw**2/xx(4)**2)
     .              *lamb_hdec(xx(4)**2/amw**2,amhichar**2/amw**2)**2
               yy(4)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*chw
            elseif(amhiggsi.le.amhichar) then
               yy(4) = 0
            else
               yy(4)=9.D0*gf**2/16.d0/pi**3*amw**4*xx(4)*2
     .              *hvh((amhichar/xx(4))**2,(amw/xx(4))**2)
            endif

            hitohpwm(i)=fint_hdec(amhi(i),xx,yy)*ghighpwm(i)**2
            end if
            end if
            end if
            end if
         else
            chw=lamb_hdec(amch**2/amhi(i)**2,amw**2/amhi(i)**2)
     .           *lamb_hdec(amhi(i)**2/amw**2,amch**2/amw**2)**2
            hitohpwm(i)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/amhi(i)*chw*
     .           ghighpwm(i)**2
         endif
         
         if(calcmasseserror) then
           calcmasseserror=.FALSE.
           print*, 'Error in off-shell decay Hi -> H+ W- + H- W+. i=',i
           print*, 'Off-shell decay turned off'
           if(amhi(i).lt.amw+amch) then
              hitohpwm(i)=0.d0
           else
              chw=lamb_hdec(amch**2/amhi(i)**2,amw**2/amhi(i)**2)
     .             *lamb_hdec(amhi(i)**2/amw**2,amch**2/amw**2)**2
              hitohpwm(i)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/amhi(i)*
     .             chw*ghighpwm(i)**2
         endif
         end if
         
      else
         if(amhi(i).lt.amw+amch) then
            hitohpwm(i)=0.d0
         else
            chw=lamb_hdec(amch**2/amhi(i)**2,amw**2/amhi(i)**2)
     .           *lamb_hdec(amhi(i)**2/amw**2,amch**2/amw**2)**2
            hitohpwm(i)=2.D0*gf/8.d0/dsqrt(2d0)/pi*amw**4/amhi(i)*chw*
     .           ghighpwm(i)**2
         endif
      endif

c      print*,'Hi->H+W-+H-W+',hitohpwm(i),i

c -------------------------------------------------------------------- c

      whitot(i) = whitot(i) + htoh(i) + htoaz(i) + hitohpwm(i)

c      print*,'whitot 2',whitot(i),i

      end do

c ==================================================================== c
c -------- Pseudoscalar Higgs decays w/o off-shell H-to-H decays ----- c
c ==================================================================== c

      amhia(1) = amhi(4)
      amhia(2) = amhi(5)
      ghiat(1) = ghigt(4)
      ghiat(2) = ghigt(5)
      ghiab(1) = ghigb(4)
      ghiab(2) = ghigb(5)

      do i=1,2,1

c --- running masses --- 
      rms = runm_hdec(amhia(i),3)
      rmc = runm_hdec(amhia(i),4)
      rmb = runm_hdec(amhia(i),5)
      rmt = runm_hdec(amhia(i),6)
      ratcoup = ghiat(i)/ghiab(i)
      higtop = amhia(i)**2/amt**2

      ash=alphas_hdec(amhia(i),3)
      amc0=1.d8
      amb0=2.d8
      as3=alphas_hdec(amhia(i),3)
      amc0=amc
      as4=alphas_hdec(amhia(i),3)
      amb0=amb

c maggie changed 14/3/12
      if(iruncoup.eq.1) then
       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,rmt,rmb)
      elseif(iruncoup.eq.0) then
       call sfermion_nmssm(amst,amsb,amsl,amsu,amsd,amse,amsn,amsnl,
     .      gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .      gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .      gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .      gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,amt,amb)
      endif
c end maggie changed 14/3/12

c --- partial widths --- c
c  Ai ---> g g
       eps=1.d-8
       nfext = 3
       asg = as3
       ctt = 4*amt**2/amhia(i)**2*dcmplx(1d0,-eps)
       ctb = 4*amb**2/amhia(i)**2*dcmplx(1d0,-eps)
       cat = ctt*cf(ctt)*ghiat(i)
       cab = ctb*cf(ctb)*ghiab(i)
       ctc = 4*amc**2/amhia(i)**2*dcmplx(1d0,-eps)
       cac = ctc*cf(ctc)*ghiat(i)
       
       catc= 2*ctt*(1+(1-ctt)*cf(ctt))
       cabc= 2*ctb*(1+(1-ctb)*cf(ctb))
       cacc= 2*ctc*(1+(1-ctc)*cf(ctc))
       
       fqcd=aggqcd(asg,nfext)
       xfac = cdabs(cat+cab+cac)**2*fqcd
       hgg=gf/(16.d0*pi*dsqrt(2.d0))*amhia(i)**3*(asg/pi)**2*xfac
       
       cj(i+3)=2d0*cdabs(cat+cab+cac)/cdabs(catc+cabc+cacc)
!        write(*,*)'hgg',hgg
!        write(*,*) 'cj',cj(i+3)


c       print*,'hgg,ai',hgg,i

c Ai ---> g g* ---> g cc   to be added to Ai ---> cc
       nfext = 4
       asg = as4
       fqcd=aggqcd(asg,nfext)
       xfac = cdabs(cat+cab+cac)**2*fqcd
       dcc=gf/(16.d0*pi*dsqrt(2.d0))*amhia(i)**3*(asg/pi)**2*xfac
     .     - hgg

c Ai ---> g g* ---> g bb   to be added to Ai ---> bb
       nfext = 5
       asg = ash
       fqcd=aggqcd(asg,nfext)
       xfac = cdabs(cat+cab+cac)**2*fqcd
       dbb=gf/(16.d0*pi*dsqrt(2.d0))*amhia(i)**3*(asg/pi)**2*xfac
     .     - hgg - dcc
       hgg=gf/(16.d0*pi*dsqrt(2.d0))*amhia(i)**3*(asg/pi)**2*xfac

c Ai ---> g g: full NNLO corrections to top loops for nf=5
       fqcd0=aggqcd(asg,5)
       fqcd=aggqcd2(asg,5,amhia(i),amt)
       xfac = cdabs(cat+cab+cac)**2*(fqcd-fqcd0)
       hgg=hgg+gf/(16.d0*pi*dsqrt(2.d0))*amhia(i)**3*(asg/pi)**2*xfac

      if(nfgg.eq.3)then
       hgg = hgg - dbb - dcc
      elseif(nfgg.eq.4)then
       hgg = hgg - dbb
       dcc = 0
      else
       dcc = 0
       dbb = 0
      endif

      hagg(i) = hgg

c      print*,''
c      print*,'Ai decays'
c      print*,'hgg',hagg(i),i

c ND added 7/2021
c  Ai ---> nu nu
      ann=0
      if (isstrue)then
      do i1=1,9
        do i2=1,9
         if (amhia(i) .gt. amnu(i1)+amnu(i2)) then
           call Delta_Hnunu(amhia(i),i,i1,i2,awidth,0)
           ann=ann+awidth
         endif
        enddo
      enddo
      endif
      hann(i) = hnn
c end ND added 7/2021


c Ai ---> mu mu
      xgh1m = gh1b
      xgh2m = gh2b
      xgh3m = gh3b
      xga1m = ga1b
      xga2m = ga2b
      xghpm = tgbet
      IF(IOFSUSY.EQ.0) THEN
         CALL STAUSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1m,
     .        xgh2m,xgh3m,xga1m,xga2m,xghpm,tgbet,qsusy,0)
      ENDIF
      xghigm(1) = xgh1m
      xghigm(2) = xgh2m
      xghigm(3) = xgh3m
      xghigm(4) = xga1m
      xghigm(5) = xga2m

      if(amhia(i).le.2*ammuon) then
         hmm = 0
      else
         hmm=aff(amhia(i),(ammuon/amhia(i))**2)*xghigm(i+3)**2
      endif

      hamm(i) = hmm

c      print*,'hmm',hamm(i),i

c Ai ---> tau tau
      xgh1b = gh1b
      xgh2b = gh2b
      xgh3b = gh3b
      xga1b = ga1b
      xga2b = ga2b
      xghpm = tgbet
      IF(IOFSUSY.EQ.0) THEN
       CALL STAUSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,
     .     xgh2b,xgh3b,xga1b,xga2b,xghpm,tgbet,qsusy,1)
      ENDIF
      xghigb(1) = xgh1b
      xghigb(2) = xgh2b
      xghigb(3) = xgh3b
      xghigb(4) = xga1b
      xghigb(5) = xga2b

      if(amhia(i).le.2.D0*amtau) then
         hll = 0
      else
         hll=aff(amhia(i),(amtau/amhia(i))**2)*xghigb(i+3)**2

      endif
      hall(i) = hll

c      print*,'hll',hall(i),i

c Ai ---> ss
      xgh1s = gh1b
      xgh2s = gh2b
      xgh3s = gh3b
      xga1s = ga1b
      xga2s = ga2b
      if(iofsusy.eq.0) then
       call strsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1s,xgh2s,
     .        xgh3s,xga1s,xga2s,xghpm,tgbet,qsusy,loop)
      endif
      xghigs(1) = xgh1s
      xghigs(2) = xgh2s
      xghigs(3) = xgh3s
      xghigs(4) = xga1s
      xghigs(5) = xga2s

      if(amhia(i).le.2*ams) then
         hss = 0
      else
         hs1=3.d0*aff(amhia(i),(ams/amhia(i))**2)
     .        *xghigs(i+3)**2
     .        *tqcda(ams**2/amhia(i)**2)
         hs2=3.d0*aff(amhia(i),(rms/amhia(i))**2)
     .        *xghigs(i+3)**2
     .        *qcda(rms**2/amhia(i)**2)
         if(hs2.lt.0.d0) hs2 = 0
         rat = 2*ams/amhia(i)
         hss = qqint_hdec(rat,hs1,hs2)
      endif

      hass(i) = hss

c      print*,'hss',hass(i),i

c  Ai ---> cc
      ratcoup = 1
      if(amhia(i).le.2*amc) then
       hcc = 0
      else
       hc1=3.d0*aff(amhia(i),(amc/amhia(i))**2)
     .    *ghiat(i)**2
     .    *tqcda(amc**2/amhia(i)**2)
       hc2=3.d0*aff(amhia(i),(rmc/amhia(i))**2)
     .    *ghiat(i)**2
     .    *qcda(rmc**2/amhia(i)**2)
     .   + dcc
       if(hc2.lt.0.d0) hc2 = 0
       rat = 2*amc/amhia(i)
       hcc = qqint_hdec(rat,hc1,hc2)
      endif

      hacc(i) = hcc

c      print*,'hcc',hacc(i),i

c  Ai ---> bb
      qq = amb
      susy = 0
      xgh1b = gh1b
      xgh2b = gh2b
      xgh3b = gh3b
      xga1b = ga1b
      xga2b = ga2b
      xghigb(1) = xgh1b
      xghigb(2) = xgh2b
      xghigb(3) = xgh3b
      xghigb(4) = xga1b
      xghigb(5) = xga2b
      ssusy = (amsb(1)+amsb(2)+amglu)/3*qsusy
      as0 = alphas_hdec(ssusy,3)
      if(iofsusy.eq.0) then
       i0 = 1
       CALL DMBAPP_NMSSM_HDEC(I0,dgh1b,dgh2b,dgh3b,dga1b,dga2b,dghpm,
     .      QSUSY,LOOP)
       i0 = i+3
       bsc = (amsq+amur+amdr)/3
       xmb = amb
       dghib(1) = dgh1b
       dghib(2) = dgh2b
       dghib(3) = dgh3b
       dghib(4) = dga1b
       dghib(5) = dga2b
       susy = cofsusy_nmssm(i0,amb,xmb,qq)*as0/pi - 2*dghib(i+3)
       call botsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,xgh2b,
     .     xgh3b,xga1b,xga2b,xghpm,tgbet,qsusy,loop)
      endif
      xghigb(1)=xgh1b
      xghigb(2)=xgh2b
      xghigb(3)=xgh3b
      xghigb(4)=xga1b
      xghigb(5)=xga2b
      ratcoup = ghiat(i)/xghigb(i+3)
      if(amhia(i).le.2*amb) then
       hbb = 0
      else
c         susy = 0.D0
       hb1=3.d0*aff(amhia(i),(amb/amhia(i))**2)
     .    *(xghigb(i+3)**2+xghigb(i+3)*ghiab(i)*susy)
     .    *tqcda(amb**2/amhia(i)**2)
       hb2=3.d0*aff(amhia(i),(rmb/amhia(i))**2)
     .    *(xghigb(i+3)**2+xghigb(i+3)*ghiab(i)*susy)
     .    *qcda(rmb**2/amhia(i)**2)
     .   + dbb
       if(hb2.lt.0.d0) hb2 = 0
       rat = 2*amb/amhia(i)
       hbb = qqint_hdec(rat,hb1,hb2)
      endif

      habb(i) = hbb

c      print*,'hbb',habb(i),i

c  Ai ---> tt 
      ratcoup = 0
      if(ionsh.eq.0)then
       dld=3d0
       dlu=4d0
       xm1 = 2d0*amt-dld
       xm2 = 2d0*amt+dlu
       if (amhia(i).le.amt+amw+amb) then
        htt=0.d0
       elseif (amhia(i).le.xm1) then
        factt=6.d0*gf**2*amhia(i)**3*amt**2/2.d0/128.d0/pi**3*
     .         ghiat(i)**2
        call atott_NMSSM(amhia(i),amt,amb,amw,amch,att0,tgbet,ghiab(i),
     .       ghiat(i))
        htt=factt*att0
       elseif (amhia(i).le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        factt=6.d0*gf**2*xx(1)**3*amt**2/2.d0/128.d0/pi**3
        call atott_NMSSM(xx(1),amt,amb,amw,amch,att0,tgbet,ghiab(i),
     .       ghiat(i))
        yy(1)=factt*att0
        factt=6.d0*gf**2*xx(2)**3*amt**2/2.d0/128.d0/pi**3
        call atott_NMSSM(xx(2),amt,amb,amw,amch,att0,tgbet,ghiab(i),
     .       ghiat(i))
        yy(2)=factt*att0
        xmt = runm_hdec(xx(3),6)
        xyz1 =3.d0*aff(xx(3),(amt/xx(3))**2)
     .    *tqcda(amt**2/xx(3)**2)
        xyz2 =3.d0*aff(xx(3),(xmt/xx(3))**2)
     .    *qcda(xmt**2/xx(3)**2)
        if(xyz2.lt.0.d0) xyz2 = 0
        rat = 2*amt/xx(3)
        yy(3) = qqint_hdec(rat,xyz1,xyz2)
        xmt = runm_hdec(xx(4),6)
        xyz1 =3.d0*aff(xx(4),(amt/xx(4))**2)
     .    *tqcda(amt**2/xx(4)**2)
        xyz2 =3.d0*aff(xx(4),(xmt/xx(4))**2)
     .    *qcda(xmt**2/xx(4)**2)
        if(xyz2.lt.0.d0) xyz2 = 0
        rat = 2*amt/xx(4)
        yy(4) = qqint_hdec(rat,xyz1,xyz2)
        htt = fint_hdec(amhia(i),xx,yy)*ghiat(i)**2
       else
        ht1=3.d0*aff(amhia(i),(amt/amhia(i))**2)*ghiat(i)**2
     .    *tqcda(amt**2/amhia(i)**2)
        ht2=3.d0*aff(amhia(i),(rmt/amhia(i))**2)*ghiat(i)**2
     .    *qcda(rmt**2/amhia(i)**2)
        if(ht2.lt.0.d0) ht2 = 0
        rat = 2*amt/amhia(i)
        htt = qqint_hdec(rat,ht1,ht2)
       endif
      else
       if (amhia(i).le.2.d0*amt) then
        htt=0.d0
       else
        ht1=3.d0*aff(amhia(i),(amt/amhia(i))**2)*ghiat(i)**2
     .    *tqcda(amt**2/amhia(i)**2)
        ht2=3.d0*aff(amhia(i),(rmt/amhia(i))**2)*ghiat(i)**2
     .    *qcda(rmt**2/amhia(i)**2)
        if(ht2.lt.0.d0) ht2 = 0
        rat = 2*amt/amhia(i)
        htt = qqint_hdec(rat,ht1,ht2)
       endif
      endif

      hatt(i) = htt

c      print*,'htt',hatt(i),i

c  Ai ---> gamma gamma
       eps=1.d-8
       xrmc = runm_hdec(amhia(i)/2,4)*amc/runm_hdec(amc,4)
       xrmb = runm_hdec(amhia(i)/2,5)*amb/runm_hdec(amb,5)
       xrmt = runm_hdec(amhia(i)/2,6)*amt/runm_hdec(amt,6)
       ctt = 4*xrmt**2/amhia(i)**2*dcmplx(1d0,-eps)
       ctb = 4*xrmb**2/amhia(i)**2*dcmplx(1d0,-eps)
       cat = 4/3d0 * ctt*cf(ctt)*ghiat(i)
     .     * cfacq_hdec(1,amhia(i),xrmt)
       cab = 1/3d0 * ctb*cf(ctb)*ghiab(i)
     .     * cfacq_hdec(1,amhia(i),xrmb)
       ctc = 4*xrmc**2/amhia(i)**2*dcmplx(1d0,-eps)
       cac = 4/3d0 * ctc*cf(ctc)*ghiat(i)
     .     * cfacq_hdec(1,amhia(i),xrmc)
       ctl = 4*amtau**2/amhia(i)**2*dcmplx(1d0,-eps)
       cal = 1.d0  * ctl*cf(ctl)*ghiab(i)
       if(iofsusy.eq.0) then 
        cx1 = 4*amchar(1)**2/amhia(i)**2*dcmplx(1d0,-eps)
        cx2 = 4*amchar(2)**2/amhia(i)**2*dcmplx(1d0,-eps)
        cax1= amw/xmchar(1) * cx1*cf(cx1) * 2*achig(i+3,1,1)
        cax2= amw/xmchar(2) * cx2*cf(cx2) * 2*achig(i+3,2,2)
        xfac = cdabs(cat+cab+cac+cal+cax1+cax2)**2
       else 
        xfac = cdabs(cat+cab+cac+cal)**2
       endif
       hga=gf/(32.d0*pi*dsqrt(2.d0))*amhia(i)**3*(alph/pi)**2*xfac

       haga(i) = hga

c       print*,'hga',haga(i),i

c  Ai ---> Z gamma
      if(amhia(i).le.amz)then
       hzga=0
      else
       ts = ss/cs
       ft = -3*2d0/3*(1-4*2d0/3*ss)/dsqrt(ss*cs)*ghiat(i)
       fb = 3*1d0/3*(-1+4*1d0/3*ss)/dsqrt(ss*cs)*ghiab(i)
       fc = -3*2d0/3*(1-4*2d0/3*ss)/dsqrt(ss*cs)*ghiat(i)
       fl = (-1+4*ss)/dsqrt(ss*cs)*ghiab(i)
       eps=1.d-8
       ctt = 4*amt**2/amhia(i)**2*dcmplx(1d0,-eps)
       ctb = 4*amb**2/amhia(i)**2*dcmplx(1d0,-eps)
       ctc = 4*amc**2/amhia(i)**2*dcmplx(1d0,-eps)
       ctl = 4*amtau**2/amhia(i)**2*dcmplx(1d0,-eps)
       clt = 4*amt**2/amz**2*dcmplx(1d0,-eps)
       clb = 4*amb**2/amz**2*dcmplx(1d0,-eps)
       clc = 4*amc**2/amz**2*dcmplx(1d0,-eps)
       cle = 4*amtau**2/amz**2*dcmplx(1d0,-eps)
       cat = ft*(- ci2(ctt,clt))
       cab = fb*(- ci2(ctb,clb))
       cac = fc*(- ci2(ctc,clc))
       cal = fl*(- ci2(ctl,cle))
       xfac = cdabs(cat+cab+cac+cal)**2
       acoup = dsqrt(2d0)*gf*amz**2*ss*cs/pi**2
       hzga = gf/(4.d0*pi*dsqrt(2.d0))*amhia(i)**3*(alph/pi)*acoup/16.d0
     .        *xfac*(1-amz**2/amhia(i)**2)**3
      endif

      hazga(i) = hzga

c      print*,'hzga',hazga(i),i

c  Ai ---> Hj Z* ---> HFF

      do j=1,3,1

      if(ionsh.eq.0)then
       dld=3d0
       dlu=5d0
       xm1 = amhi(j)+amz-dld
       xm2 = amhi(j)+amz+dlu
       if (amhia(i).le.amhi(j)) then
        hjaiz(i,j)=0
       elseif (amhia(i).le.xm1) then
        if (amhia(i).le.dabs(amz-amhi(j))) then
         hjaiz(i,j)=0
        else
         hjaiz(i,j)=9.d0*gf**2/16.d0/pi**3*amz**4*amhia(i)*
     .          gzaihj(i,j)**2*
     .      (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .      *hvh((amhi(j)/amhia(i))**2,(amz/amhia(i))**2)
        endif
       elseif (amhia(i).le.xm2) then
        xx(1) = xm1-1d0
        xx(2) = xm1
        xx(3) = xm2
        xx(4) = xm2+1d0
        yy(1)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(1)*
     .      (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .      *hvh((amhi(j)/xx(1))**2,(amz/xx(1))**2)
        yy(2)=9.d0*gf**2/16.d0/pi**3*amz**4*xx(2)*
     .      (7.d0/12.d0-10.d0/9.d0*ss+40.d0/27.d0*ss**2)
     .      *hvh((amhi(j)/xx(2))**2,(amz/xx(2))**2)
        caz=lamb_hdec(amhi(j)**2/xx(3)**2,amz**2/xx(3)**2)
     .     *lamb_hdec(xx(3)**2/amz**2,amhi(j)**2/amz**2)**2
        yy(3)=gf/8d0/dsqrt(2d0)/pi*amz**4/xx(3)*caz
        caz=lamb_hdec(amhi(j)**2/xx(4)**2,amz**2/xx(4)**2)
     .     *lamb_hdec(xx(4)**2/amz**2,amhi(j)**2/amz**2)**2
        yy(4)=gf/8d0/dsqrt(2d0)/pi*amz**4/xx(4)*caz
        hjaiz(i,j) = fint_hdec(amhia(i),xx,yy)*gzaihj(i,j)**2
       else
        caz=lamb_hdec(amhi(j)**2/amhia(i)**2,amz**2/amhia(i)**2)
     .     *lamb_hdec(amhia(i)**2/amz**2,amhi(j)**2/amz**2)**2
        hjaiz(i,j)=gf/8d0/dsqrt(2d0)/pi*amz**4/amhia(i)*gzaihj(i,j)**2*
     .       caz
       endif
      else
       if (amhia(i).le.amz+amhi(j)) then
        hjaiz(i,j)=0
       else
        caz=lamb_hdec(amhi(j)**2/amhia(i)**2,amz**2/amhia(i)**2)
     .     *lamb_hdec(amhia(i)**2/amz**2,amhi(j)**2/amz**2)**2
        hjaiz(i,j)=gf/8d0/dsqrt(2d0)/pi*amz**4/amhia(i)*gzaihj(i,j)**2*
     .       caz
       endif
      endif

c      print*,'hjaiz',hjaiz(i,j),i,j

      end do

c  Ai ---> W+ H-

      deltalow = 3.D0
      deltahigh = 5.D0

      if(ionsh.eq.0) then
         xm1 = amch+amw - deltalow
         xm2 = amch+amw + deltahigh

         if (amhia(i).lt.amch) then
            haiwphm(i)=0
         elseif(amhia(i).lt.xm1) then
            if(amhia(i).le.dabs(amw-amch)) then
               haiwphm(i) = 0.D0
            else
               haiwphm(i)=9.d0*gf**2/16.d0/pi**3*amw**4*amhia(i)	
     .              *hvh((amch/amhia(i))**2,(amw/amhia(i))**2)*
     .              (ghiab(i)/tgbet)**2  
         endif
      elseif(amhia(i).lt.xm2) then
            xx(1) = xm1-1.D0
            xx(2) = xm1
            xx(3) = xm2
            xx(4) = xm2+1.D0

            yy(1)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(1)	
     .        *hvh((amch/xx(1))**2,(amw/xx(1))**2)
            yy(2)=9.d0*gf**2/16.d0/pi**3*amw**4*xx(2)	
     .        *hvh((amch/xx(2))**2,(amw/xx(2))**2)

            cwpa2=lamb_hdec(amch**2/xx(3)**2,amw**2/xx(3)**2)
     .           *lamb_hdec(xx(3)**2/amw**2,amch**2/amw**2)**2
            yy(3)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(3)*cwpa2
            cwpa2=lamb_hdec(amch**2/xx(4)**2,amw**2/xx(4)**2)
     .           *lamb_hdec(xx(4)**2/amw**2,amch**2/amw**2)**2
            yy(4)=gf/8.d0/dsqrt(2d0)/pi*amw**4/xx(4)*cwpa2

            haiwphm(i) = fint_hdec(amhia(i),xx,yy)*(ghiab(i)/tgbet)**2
        else
           cwpa2=lamb_hdec(amch**2/amhia(i)**2,amw**2/amhia(i)**2)
     .          *lamb_hdec(amhia(i)**2/amw**2,amch**2/amw**2)**2
           haiwphm(i)=gf/8.d0/dsqrt(2d0)/pi*amw**4/amhia(i)*cwpa2*
     .          (ghiab(i)/tgbet)**2
        endif
      else
         if (amhia(i).lt.amw+amch) then
            haiwphm(i)=0
         else
            cwpa2=lamb_hdec(amch**2/amhia(i)**2,amw**2/amhia(i)**2)
     .           *lamb_hdec(amhia(i)**2/amw**2,amch**2/amw**2)**2
            haiwphm(i)=gf/8.d0/dsqrt(2d0)/pi*amw**4/amhia(i)*cwpa2*
     .           (ghiab(i)/tgbet)**2
         endif
      endif

c      print*,'haiwphm',haiwphm(i)*2.D0,i

c Ai ---> Aj Hk

      do j=1,2,1
         do k=1,3,1
            aitoajhk(i,j,k) = 0.D0
         end do
      end do

c --- on-shell decays --- c
      deltalow = 3.D0
      deltahigh = 5.D0

      atoah(i) = 0.D0

      do j=1,2,1
         do k=1,3,1
            if(amhia(i).ge.amhia(j)+amhi(k)) then
               aitoajhk(i,j,k)=gf/8d0/dsqrt(2d0)/pi*amz**4/amhia(i)*
     .         lamb_hdec(amhia(j)**2/amhia(i)**2,amhi(k)**2/amhia(i)**2)
     .           *g3hihjhk(i+3,j+3,k)**2
               atoah(i) = atoah(i) + aitoajhk(i,j,k)
c               print*,'atoah on',aitoajhk(i,j,k),i,j,k
            endif
         end do
      end do

c =========================== SUSY decays ============================ c

      if(iofsusy.eq.0) then 

c  Ai ---> charginos
      do ii=1,2
      do jj=1,2
      if (amhia(i).gt.amchar(ii)+amchar(jj)) then
      whaich(i,ii,jj)=gf*amw**2/(2*pi*dsqrt(2.d0))/amhia(i)
     .   *lamb_hdec(amchar(ii)**2/amhia(i)**2,amchar(jj)**2/amhia(i)**2)
     .   *( (achig(i+3,ii,jj)**2+achig(i+3,jj,ii)**2)*(amhia(i)**2
     .        -amchar(ii)**2-amchar(jj)**2)+4.d0*achig(i+3,ii,jj)*
     .        achig(i+3,jj,ii)*xmchar(ii)*xmchar(jj) ) 
      else 
      whaich(i,ii,jj)=0.d0
      endif
      end do
      end do
      whaicht(i)=whaich(i,1,1)+whaich(i,1,2)+whaich(i,2,1)+
     .     whaich(i,2,2)

c      do ii=1,2,1
c         do jj=1,2,1
c            print*,'Ai->charjk,j,k',whaich(i,ii,jj),ii,jj
c         end do
c      end do

c      print*,'sum char',whaicht(i),i

c  Ai ---> neutralinos 

      do ii=1,5
      do jj=1,5
      if (amhia(i).gt.amneut(ii)+amneut(jj)) then
         whaine(i,ii,jj)=gf*amw**2/(2*pi*dsqrt(2.d0))/amhia(i)
     .        *anhig(i+3,ii,jj)**2*(amhia(i)**2-(xmneut(ii)
     .        -xmneut(jj))**2)
     .        *lamb_hdec(amneut(ii)**2/amhia(i)**2,amneut(jj)**2
     .        /amhia(i)**2)
      else 
      whaine(i,ii,jj)=0.d0
      endif
      end do
      end do

      whainet(i)= whaine(i,1,1)+whaine(i,1,2)+whaine(i,1,3)
     .     +whaine(i,1,4)+whaine(i,1,5)
     .     +whaine(i,2,1)+whaine(i,2,2)+whaine(i,2,3)+whaine(i,2,4)
     .     +whaine(i,2,5)
     .     +whaine(i,3,1)+whaine(i,3,2)+whaine(i,3,3)+whaine(i,3,4)
     .     +whaine(i,3,5)
     .     +whaine(i,4,1)+whaine(i,4,2)+whaine(i,4,3)+whaine(i,4,4)
     .     +whaine(i,4,5)
     .     +whaine(i,5,1)+whaine(i,5,2)+whaine(i,5,3)+whaine(i,5,4)
     .     +whaine(i,5,5)

c      do ii=1,5,1
c         do jj=1,5,1
c            print*,'Ai->neutraljk',whaine(i,ii,jj),ii,jj
c         end do
c      end do
c      print*,'sum neut',whainet(i)

c  Ai ---> stau's 

      if(amhia(i).gt.amsl(1)+amsl(2)) then
      whaisl(i)=gf*amz**4/dsqrt(2.d0)/pi*ghigee(i+3,1,2)**2*
     .      lamb_hdec(amsl(1)**2/amhia(i)**2,amsl(2)**2/amhia(i)**2)
     .        /amhia(i)
      else
      whaisl(i)=0.d0
      endif

c      print*,'Ai->stau1 stau2',whaisl(i),i

c  Ai ---> stops 

      susy = 1
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      qsq = amhia(i)
      if(amhia(i).gt.ymst(1)+ymst(2)) then
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
c         print*,'vals0',yhaitt(1),yhaitt(2)
       call sqmbapp_NMSSM_hdec(qsq)
       susy = 1+sqsusy_nmssm_hdec(i+3,1,1,2,qsq,0,1)
       yhaitt(1) = yha1tt
       yhaitt(2) = yha2tt
c         print*,'vals1',yhaitt(1),yhaitt(2)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       whaist(i)=3*gf*amz**4/dsqrt(2.d0)/pi*yhaitt(i)**2*
     .      lamb_hdec(ymst(1)**2/amhia(i)**2,ymst(2)**2/amhia(i)**2)
     .      /amhia(i)*susy
      if(whaist(i).ne.whaist(i)) then
         print*,'There is a problem in the loop-corrected stop decays. T
     .he tree-level decay is taken instead, for Ai, i=',i
         print*,''
       whaist(i)=3*gf*amz**4/dsqrt(2.d0)/pi*ghigtt(i+3,1,2)**2*
     .      lamb_hdec(amst(1)**2/amhia(i)**2,amst(2)**2/amhia(i)**2)
     .      /amhia(i)
      endif
      else
      whaist(i)=0.d0
      endif

c      print*,'Ai->stop1 stop2',whaist(i),i

c  Ai ---> sbottoms 

      susy = 1
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      qsq = amhia(i)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      if(amhia(i).gt.ymsb(1)+ymsb(2)) then
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       call sqmbapp_NMSSM_hdec(qsq)
       susy = 1+sqsusy_nmssm_hdec(i+3,2,1,2,qsq,0,1)
       yhaibb(1) = yha1bb
       yhaibb(2) = yha2bb
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
       whaisb(i)=3*gf*amz**4/dsqrt(2.d0)/pi*yhaibb(i)**2*
     .       lamb_hdec(ymsb(1)**2/amhia(i)**2,ymsb(2)**2/amhia(i)**2)
     .      /amhia(i)*susy
      if(whaisb(i).ne.whaisb(i)) then
         print*,'There is a problem in the loop-corrected sbot decays. T
     .he tree-level decay is taken instead, for Ai, i=',i
         print*,''
         whaisb(i)=3*gf*amz**4/dsqrt(2.d0)/pi*ghigbb(i+3,1,2)**2*
     .       lamb_hdec(amsb(1)**2/amhia(i)**2,amsb(2)**2/amhia(i)**2)
     .      /amhia(i)
      endif
      else
         whaisb(i)=0.d0
      endif

c      print*,'Ai->sbot1 sbot2',whaisb(i),i

      else 
         whaicht(i) = 0.D0
         whainet(i) = 0.D0
         whaisl(i)  = 0.D0
         whaist(i)  = 0.D0
         whaisb(i)  = 0.D0

c --- change thanks to Elzbieta Richter-Was
      do ii=1,2
       do jj=1,2
          whaich(i,ii,jj) = 0.D0
       enddo
      enddo
      do ii=1,5
       do jj=1,5
          whaine(i,ii,jj) = 0.D0
       enddo
      enddo

      endif

c ---------- total width w/o Higgs-to-Higgs off-shell decays --------- c

      whitot(i+3) = hall(i)+hamm(i)+hass(i)+hacc(i)+habb(i)+hagg(i)+
     .     haga(i)+hazga(i)+hjaiz(i,1)+hjaiz(i,2)+hjaiz(i,3)+hatt(i)+
     .     2.D0*haiwphm(i)+whaicht(i)+whainet(i)+whaisl(i)+whaist(i)+
     .     whaisb(i)+atoah(i) + hann(i) !ND changed 7/2021


      end do

c ==================================================================== c
c ---- Off-shell decays Hi -> Hj Hk (for scalar and pseudoscalar) ---- c
c ==================================================================== c

      do i=1,5,1
         do j=1,5,1
            do k=1,5,1
               hitohjhko(i,j,k) = 0.D0
            end do
         end do
      end do

      epsi = 1.D0

      do i=1,5,1 
         do j=1,5,1
            do k=1,5,1
               if(amhi(i).lt.(amhi(j)+amhi(k))) then
                  if(ionsh.eq.0) then
            if(amhi(i).gt.amhi(j)+epsi.and.amhi(i).gt.amhi(k)+epsi) then
                        if(g3hihjhk(i,j,k).ne.0.D0) then
                           iicase=0
      call htohhoffshell_nmssm(iicase,amhi(i),amhi(j),amhi(k),whitot(j),
     .     whitot(k),htohhoff,errhtohhoff)
               hitohjhko(i,j,k)=gf/16.D0/dsqrt(2D0)/pi*amz**4/amhi(i)*
     .          g3hihjhk(i,j,k)**2*htohhoff
            errhitohjhko(i,j,k)=gf/16.D0/dsqrt(2D0)/pi*amz**4/amhi(i)*
     .          g3hihjhk(i,j,k)**2*errhtohhoff
            else
               hitohjhko(i,j,k)=0.D0
            endif
         else
            hitohjhko(i,j,k)=0.D0
            htohhoff = 0.D0
         endif
      else
         hitohjhko(i,j,k) = 0.D0
      endif
      endif

c      if(hitohjhko(i,j,k).ne.0.D0) then
c         print*,'hitohjhk off',hitohjhko(i,j,k),i,j,k
c      endif

c      if(hitohjhko(i,j,k).eq.NAN.and.hitohjhko(i,j,k).ne.0.D0) then
c         print*,'There is a problem in the off-shell decays.'
c      endif

      end do
      end do

      hitohhoff(i)=0d0
      do j=1,5,1
         do k=1,5,1
            if(hitohjhko(i,j,k).ne.hitohjhko(i,j,k)) then
               hitohjhko(i,j,k) = hitohjhko(i,k,j)
            endif

            if(hitohjhko(i,j,k).ne.hitohjhko(i,j,k)) then
             print*,'There is a problem in the off-shell decays.',i,j,k,
     .              amhi(i),amhi(j),amhi(k)
      if(g3hihjhk(i,j,k).ne.0.D0) then
      iicase=1
      call htohhoffshell_nmssm(iicase,amhi(i),amhi(j),amhi(k),whitot(j),
     .     whitot(k),htohhoff,errhtohhoff)
           hitohjhko(i,j,k)=gf/16.D0/dsqrt(2D0)/pi*amz**4/amhi(i)*
     .          g3hihjhk(i,j,k)**2*htohhoff
           errhitohjhko(i,j,k)=gf/16.D0/dsqrt(2D0)/pi*amz**4/amhi(i)*
     .          g3hihjhk(i,j,k)**2*errhtohhoff
      iicase=0
      else
      hitohjhko(i,j,k)=0.D0
      endif
            endif

            hitohhoff(i) = hitohhoff(i) + hitohjhko(i,j,k)
         enddo
      enddo

      end do

      do i=1,2,1
         do j=1,2,1
            do k=1,3,1
               aitoajhko(i,j,k) = 2.D0*hitohjhko(i+3,j+3,k)
c               print*,'Ai->Aj Hk off',aitoajhko(i,j,k),i,j,k
            end do
         end do
      end do

c ------------ total widths for the neutral Higgs bosons ------------- c

      do i=1,5,1
         whitot(i) = whitot(i) + hitohhoff(i)
      end do

      whatot(1) = whitot(4)
      whatot(2) = whitot(5)

c      print*,'Hi total width',whitot(1),whitot(2),whitot(3)
c      print*,'Ai total width',whatot(1),whatot(2)

c -------------------------- branching ratios ------------------------ c
c ------------------------- scalar Higgs bosons ---------------------- c

c      do i=1,5,1
c         do j=1,5,1
c            do k=1,5,1
c               hitohjhko(i,j,k) = 0.D0
c            end do
c         end do
c      end do

      do i=1,3,1
         brhiggg(i) = higgg(i)/whitot(i)
         brhigmm(i) = higmm(i)/whitot(i)
         brhigll(i) = higll(i)/whitot(i)
         brhigss(i) = higss(i)/whitot(i)
         brhigcc(i) = higcc(i)/whitot(i)
         brhigbb(i) = higbb(i)/whitot(i)
         brhigtt(i) = higtt(i)/whitot(i)
         brhigga(i) = higga(i)/whitot(i)
         brhigzga(i) = higzga(i)/whitot(i)
         brhigww(i) = higww(i)/whitot(i)
         brhigzz(i) = higzz(i)/whitot(i)
         brhitohpwm(i) = hitohpwm(i)/whitot(i)
c ND added 7/2021
         brhignn(i) = hignn(i)/whitot(i)
c end ND added 7/2021
         do j=1,5,1
            do k=1,5,1
               if(hitohjhk(i,j,k).ne.0.D0) then
                  brhitohjhk(i,j,k) = hitohjhk(i,j,k)/whitot(i)
               endif
               if(hitohjhko(i,j,k).ne.0.D0) then
                  brhitohjhk(i,j,k) = hitohjhko(i,j,k)/whitot(i)
               endif
               if(j.ne.k) then
                  brhitohjhk(i,j,k) = 2.D0*brhitohjhk(i,j,k)
               endif
            end do
         end do

         do j=4,5,1
            brhitoajz(i,j) = hitoajz(i,j)/whitot(i)
         end do

         do j=1,2,1
            do k=1,2,1
               brwhigch(i,j,k) = whigch(i,j,k)/whitot(i)
            end do
         end do

         do j=1,5,1
            do k=1,5,1
               brwhigne(i,j,k) = whigne(i,j,k)/whitot(i)
            end do
         end do

      brwhigcht(i)=whigcht(i)/whitot(i)
      brwhignet(i)=whignet(i)/whitot(i)
      brwhislt(i) =whislt(i)/whitot(i)
      brwhisq(i)  =whisq(i)/whitot(i)
      brwhisqt(i) =whisqt(i)/whitot(i)

      brwhislnl(i)   =whislnl(i)/whitot(i)
      brwhislnltau(i)=whislnltau(i)/whitot(i)
      brwhislel(i)   =whislel(i)/whitot(i)
      brwhisler(i)   =whisler(i)/whitot(i)
      brwhisqul(i)   =whisqul(i)/whitot(i)
      brwhisqur(i)   =whisqur(i)/whitot(i)
      brwhisqdl(i)   =whisqdl(i)/whitot(i)
      brwhisqdr(i)   =whisqdr(i)/whitot(i)

      do j=1,2,1
         do k=1,2,1
            brwhist(i,j,k)   = whist(i,j,k)/whitot(i)
            brwhisb(i,j,k)   = whisb(i,j,k)/whitot(i)
            brwhistau(i,j,k) = whistau(i,j,k)/whitot(i)
         end do
      end do

      end do

c ---------------------- pseudoscalar Higgs bosons ------------------- c
         
      do i=1,2,1
         brhagg(i) = hagg(i)/whatot(i)
         brhamm(i) = hamm(i)/whatot(i)
         brhall(i) = hall(i)/whatot(i)
         brhass(i) = hass(i)/whatot(i)
         brhacc(i) = hacc(i)/whatot(i)
         brhabb(i) = habb(i)/whatot(i)
         brhatt(i) = hatt(i)/whatot(i)
         brhaga(i) = haga(i)/whatot(i)
         brhazga(i) = hazga(i)/whatot(i)
         brhaiwphm(i) = haiwphm(i)/whatot(i)
c ND added 7/2021
         brhann(i) = hann(i)/whatot(i)
c end ND added 7/2021
     
         do j=1,2,1
            do k=1,3,1
               if(aitoajhk(i,j,k).ne.0.D0) then
                  braitoajhk(i,j,k) = aitoajhk(i,j,k)/whatot(i)
               endif
               if(aitoajhko(i,j,k).ne.0.D0) then
                  braitoajhk(i,j,k) = aitoajhko(i,j,k)/whatot(i)
               endif
            end do
         end do

         do j=1,3,1
            brhjaiz(i,j) = hjaiz(i,j)/whatot(i)
         end do

         do j=1,2,1
            do k=1,2,1
               brwhaich(i,j,k) = whaich(i,j,k)/whatot(i)
            end do
         end do

         do j=1,5,1
            do k=1,5,1
               brwhaine(i,j,k) = whaine(i,j,k)/whatot(i)
            end do
         end do

         brwhaicht(i)=whaicht(i)/whatot(i)
         brwhainet(i)=whainet(i)/whatot(i)
         
         brwhaisl(i) =whaisl(i)/whatot(i)
         brwhaist(i) =whaist(i)/whatot(i)
         brwhaisb(i) =whaisb(i)/whatot(i)

      end do

c ==================================================================== c

      RETURN
      END

c ==================================================================== c
c ==================================================================== c
 
      double precision function bij_hdec(x,y)
      implicit double precision (a-h,o-z)
      double precision lamb_hdec
      bij_hdec = (1-x-y)/lamb_hdec(x,y)*(
     .          4*sp_hdec(xi_hdec(x,y)*xi_hdec(y,x))
     .        - 2*sp_hdec(-xi_hdec(x,y)) - 2*sp_hdec(-xi_hdec(y,x))
     .        + 2*dlog(xi_hdec(x,y)*xi_hdec(y,x))
     .           *dlog(1-xi_hdec(x,y)*xi_hdec(y,x))
     .        - dlog(xi_hdec(x,y))*dlog(1+xi_hdec(x,y))
     .        - dlog(xi_hdec(y,x))*dlog(1+xi_hdec(y,x))
     .          )
     .        -4*(dlog(1-xi_hdec(x,y)*xi_hdec(y,x))
     .        +xi_hdec(x,y)*xi_hdec(y,x)/(1-xi_hdec(x,y)*xi_hdec(y,x))
     .          *dlog(xi_hdec(x,y)*xi_hdec(y,x)))
     .        +(lamb_hdec(x,y)+x-y)/lamb_hdec(x,y)*(dlog(1+xi_hdec(x,y))
     .              - xi_hdec(x,y)/(1+xi_hdec(x,y))*dlog(xi_hdec(x,y)))
     .        +(lamb_hdec(x,y)-x+y)/lamb_hdec(x,y)*(dlog(1+xi_hdec(y,x))
     .              - xi_hdec(y,x)/(1+xi_hdec(y,x))*dlog(xi_hdec(y,x)))
      return
      end

c -------------------------------------------------------------------- c

      double precision function beta_hdec(x)
      implicit double precision (a-h,o-z)
      beta_hdec=dsqrt(1.d0-4.d0*x)
      return
      end

c -------------------------------------------------------------------- c

      double precision function lamb_hdec(x,y)
      implicit double precision (a-h,o-z)
      lamb_hdec=dsqrt((1.d0-x-y)**2-4.d0*x*y)
      return
      end

c -------------------------------------------------------------------- c

      double precision function xi_hdec(x,y)
      implicit double precision (a-h,o-z)
      double precision lamb_hdec
      xi_hdec = 2*x/(1-x-y+lamb_hdec(x,y))
      return
      end

c -------------------------------------------------------------------- c

      double precision function ghself(i,j,k,sb,cb)

      implicit double precision (a-h,m,o-z)
      double precision slhahmix(3,3),slhaamix(2,2)

      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix

      v=1.d0/dsqrt(dsqrt(2.d0)*gf)
      vs = xlamvs/xlambdapar*dsqrt(2.D0)

      ghself = 6.D0*v*xkappa**2*vs/amz**2*
     .         slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,3)
c
     .     +(slhahmix(i,1)*slhahmix(j,3)*slhahmix(k,1)+slhahmix(i,1)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*
     .     slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,1)+
     .     slhahmix(i,2)*slhahmix(j,2)*slhahmix(k,3)+slhahmix(i,3)*
     .     slhahmix(j,2)*slhahmix(k,2))*xlambdapar**2*v*vs/amz**2
c
     .     +xlambdapar**2*v**2/amz**2*(slhahmix(i,3)*slhahmix(j,3)*
     .     slhahmix(k,2)*sb+slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,1)*
     .     cb+slhahmix(i,1)*slhahmix(j,2)*sb*slhahmix(k,1)+
     .     slhahmix(i,1)*slhahmix(j,2)*cb*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,3)*cb+slhahmix(i,2)*slhahmix(j,2)*
     .     cb*slhahmix(k,1)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,3)*
     .     sb+slhahmix(i,2)*slhahmix(j,1)*sb*slhahmix(k,1)+
     .     slhahmix(i,2)*slhahmix(j,1)*cb*slhahmix(k,2)+slhahmix(i,3)*
     .     slhahmix(j,2)*slhahmix(k,3)*sb+slhahmix(i,1)*slhahmix(j,1)*
     .     sb*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,3)*
     .     cb)
c
     .     +(-1.D0/amz**2)*xlambdapar*xkappa*v*vs*(slhahmix(i,2)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,1)*slhahmix(j,2)*
     .     slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,1)+
     .     slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,2)*
     .     slhahmix(k,1))
c
     .     +(-1.D0/amz**2)*xlambdapar*xkappa*v**2*(slhahmix(i,3)*
     .     slhahmix(j,3)*slhahmix(k,1)*sb+slhahmix(i,3)*slhahmix(j,1)*
     .     slhahmix(k,3)*sb+slhahmix(i,1)*slhahmix(j,3)*slhahmix(k,3)*
     .     sb+slhahmix(i,3)*slhahmix(j,2)*slhahmix(k,3)*cb+
     .     slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,2)*cb+slhahmix(i,2)*
     .     slhahmix(j,3)*slhahmix(k,3)*cb)
c
     .     +dsqrt(2.D0)/amz**2*xkappa*Akap*v*slhahmix(i,3)*
     .     slhahmix(j,3)*slhahmix(k,3)
c
     .     +(-1.D0)/amz**2/dsqrt(2.D0)*xlambdapar*Alam*v*(slhahmix(i,2)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,1)*slhahmix(j,2)*
     .     slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,1)+
     .     slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,2)*
     .     slhahmix(k,1))
c
     .     -slhahmix(i,1)*slhahmix(j,2)*sb*slhahmix(k,1)-slhahmix(i,1)*
     .     slhahmix(j,1)*sb*slhahmix(k,2)-slhahmix(i,1)*slhahmix(j,2)*
     .     cb*slhahmix(k,2)-slhahmix(i,2)*slhahmix(j,2)*cb*
     .     slhahmix(k,1)+3.D0*slhahmix(i,2)*slhahmix(j,2)*sb*
     .     slhahmix(k,2)-slhahmix(i,2)*slhahmix(j,1)*sb*slhahmix(k,1)
     .     -slhahmix(i,2)*slhahmix(j,1)*cb*slhahmix(k,2)+
     .     3.D0*slhahmix(i,1)*slhahmix(j,1)*cb*slhahmix(k,1)

      return 
      end

c ==================================================================== c
c subroutine for the interpolation of off-shell Higgs-to-Higgs decays  c
c ==================================================================== c

      subroutine DecInterpolation(inputfile,i,j,k,ghjb,ghkb,gajb,gakb,
     .     ghivv,ghihpwm,ghit,ghict,amhiggsi,amhiggsj,amhiggsk,amhai,
     .     amhaj,amhak,amhichar)

      implicit double precision (a-h,m,o-z)
      parameter(nintmp=111)
      double precision slhahmix(3,3),slhaamix(2,2)
      character inputfile*100

      character spinfo1*100,spinfo2*100,mincom(1:20)*20,
     .          extcom(0:100)*20,softcom(1:100)*20,hmixcom(1:10)*20
      character ospinfo1*100,ospinfo2*100,
     .          omincom(1:20)*20,oextcom(0:100)*20,osoftcom(1:100)*20,
     .          ohmixcom(1:10)*20
      integer modselval(15),omodselval(15)

      double precision minval(1:20),smval(1:24),massval(1:100),
     .                 nmixval(5,5),umixval(2,2),vmixval(2,2),
     .                 stopmixval(2,2),sbotmixval(2,2),staumixval(2,2),
     .                 hmixval(1:10),gaugeval(1:3),msoftval(1:100),
     .                 auval(3,3),adval(3,3),aeval(3,3),yuval(3,3),
     .                 ydval(3,3),yeval(3,3),qvalue(1:20),extval(0:100),
     .                 vckmval(4)
      double precision ominval(1:20),osmval(1:24),omassval(1:100),
     .                 onmixval(5,5),oumixval(2,2),ovmixval(2,2),
     .                 ostopmixval(2,2),osbotmixval(2,2),
     .                 ostaumixval(2,2),ohmixval(1:10),ogaugeval(1:3),
     .                 omsoftval(1:100),oauval(3,3),oadval(3,3),
     .                 oaeval(3,3),oyuval(3,3),oydval(3,3),oyeval(3,3),
     .                 oqvalue(1:20),oextval(0:100),ovckmval(4)

      double precision nmhmixval(3,3),nmamixval(2,3)
      double precision snsmixval(9,9),snpmixval(9,9)
      double precision onmhmixval(3,3),onmamixval(2,3),numixval(9,9)
     .                  ,numassval(9),imnumixval(9,9)

      integer check(1:100),ocheck(1:34),imod(1:2),ioimod(1:2)

      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
c -------------- common block given by read_leshouches -------------- c
      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval
      COMMON/SLHA_leshouches1_NMSSM/extcom,mincom,softcom,hmixcom
      COMMON/SLHA_leshouches2_NMSSM/minval,extval,smval,massval,
     .                   nmixval,umixval,vmixval,stopmixval,sbotmixval,
     .                   staumixval,hmixval,gaugeval,msoftval,auval,
     .                   adval,aeval,yuval,ydval,yeval,qvalue,imod,numixval
     .                   ,numassval,imnumixval
      COMMON/SLHA_leshouches3_NMSSM/nmhmixval,nmamixval
      COMMON/SLHA_leshouches5_NMSSM/snsmixval,snpmixval
      COMMON/SLHA_checkval_CNMSSM/check
      COMMON/SLHA_leshouches3_HDEC/vckmval
c -------------------------------------------------------------------- c

c -- save the previous parameters -- c

      ospinfo1 = spinfo1
      ospinfo2 = spinfo2
      omodselval = modselval
      
      do ii=1,2,1
         ioimod(ii) = imod(ii)
      end do

      do ii=1,20,1
         ominval(ii) = minval(ii)
         omincom(ii) = mincom(ii)
         oqvalue(ii) = qvalue(ii)
      end do
      do ii=1,24,1
         osmval(ii) = smval(ii)
      end do
      do ii=1,100,1
         omassval(ii) = massval(ii)
      end do
      do ii=1,5,1
         do jj=1,5,1
            onmixval(ii,jj) = nmixval(ii,jj)
         end do
      end do
      do ii=1,2,1
         do jj=1,2,1
            oumixval(ii,jj) = umixval(ii,jj)
            ovmixval(ii,jj) = vmixval(ii,jj)
            ostopmixval(ii,jj) = stopmixval(ii,jj)
            osbotmixval(ii,jj) = sbotmixval(ii,jj)
            ostaumixval(ii,jj) = staumixval(ii,jj)
         end do
      end do
      do ii=1,10,1
         ohmixval(ii) = hmixval(ii)
         ohmixcom(ii) = hmixcom(ii)
      end do
      do ii=1,3,1
         ogaugeval(ii) = gaugeval(ii)
      end do
      do ii=1,100,1
         omsoftval(ii) = msoftval(ii)
         osoftcom(ii)  = softcom(ii)
      end do
      do ii=1,3,1
         do jj=1,3,1
            oauval(ii,jj) = auval(ii,jj)
            oadval(ii,jj) = adval(ii,jj)
            oaeval(ii,jj) = aeval(ii,jj)
            oyuval(ii,jj) = yuval(ii,jj)
            oydval(ii,jj) = ydval(ii,jj)
            oyeval(ii,jj) = yeval(ii,jj)
         end do
      end do
      do ii=0,100,1
         oextval(ii) = extval(ii)
         oextcom(ii) = extcom(ii)
      end do
      do ii=1,3,1
         do jj=1,3,1
            onmhmixval(ii,jj) = nmhmixval(ii,jj)
         end do
      end do
      do ii=1,2,1
         do jj=1,3,1
            onmamixval(ii,jj) = nmamixval(ii,jj)
         end do
      end do
      do ii=1,4,1
         ovckmval(ii) = vckmval(ii)
      end do

      do ii=1,34,1
         ocheck(ii) = check(ii)
      end do

c ---------------------------------- c

      open(nintmp,file=inputfile)

      call SLHA_read_leshouches_NMSSM(nintmp)

c -- setting of parameters -- c

      unlikely = -123456789D0

      v=1.d0/dsqrt(dsqrt(2.d0)*gf)

      if(hmixval(2).ne.unlikely) then
         TGBET = hmixval(2)
      else
         tgbet = minval(3)
      endif
      b = datan(tgbet)

      sb = dsin(b)
      cb = dcos(b)

      if(extval(61).ne.unlikely) then
         xlambdapar = extval(61)
      endif
      if(extval(62).ne.unlikely) then
         xkappa = extval(62)
      endif
      if(extval(63).ne.unlikely) then
         Alam = extval(63)
      endif
      if(extval(64).ne.unlikely) then
         Akap = extval(64)
      endif
      if(extval(65).ne.unlikely) then
         xlamvs = extval(65)
      endif
      do ii=1,3,1
         do jj=1,3,1
            slhahmix(ii,jj)=nmhmixval(ii,jj)
c            print*,'slhahmix',slhahmix(i,j),i,j
         end do
      end do
      slhaamix(1,1)=nmamixval(1,1)/dsin(b)
      slhaamix(1,2)=nmamixval(1,3)
      slhaamix(2,2)=nmamixval(2,3)
      slhaamix(2,1)=nmamixval(2,1)/dsin(b)

      vs = xlamvs/xlambdapar*dsqrt(2.D0)

c --- the couplings --- c

      ghself = 6.D0*v*xkappa**2*vs/amz**2*
     .         slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,3)
c
     .     +(slhahmix(i,1)*slhahmix(j,3)*slhahmix(k,1)+slhahmix(i,1)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*
     .     slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,1)+
     .     slhahmix(i,2)*slhahmix(j,2)*slhahmix(k,3)+slhahmix(i,3)*
     .     slhahmix(j,2)*slhahmix(k,2))*xlambdapar**2*v*vs/amz**2
c
     .     +xlambdapar**2*v**2/amz**2*(slhahmix(i,3)*slhahmix(j,3)*
     .     slhahmix(k,2)*sb+slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,1)*
     .     cb+slhahmix(i,1)*slhahmix(j,2)*sb*slhahmix(k,1)+
     .     slhahmix(i,1)*slhahmix(j,2)*cb*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,3)*cb+slhahmix(i,2)*slhahmix(j,2)*
     .     cb*slhahmix(k,1)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,3)*
     .     sb+slhahmix(i,2)*slhahmix(j,1)*sb*slhahmix(k,1)+
     .     slhahmix(i,2)*slhahmix(j,1)*cb*slhahmix(k,2)+slhahmix(i,3)*
     .     slhahmix(j,2)*slhahmix(k,3)*sb+slhahmix(i,1)*slhahmix(j,1)*
     .     sb*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,3)*
     .     cb)
c
     .     +(-1.D0/amz**2)*xlambdapar*xkappa*v*vs*(slhahmix(i,2)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,1)*slhahmix(j,2)*
     .     slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,1)+
     .     slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,2)*
     .     slhahmix(k,1))
c
     .     +(-1.D0/amz**2)*xlambdapar*xkappa*v**2*(slhahmix(i,3)*
     .     slhahmix(j,3)*slhahmix(k,1)*sb+slhahmix(i,3)*slhahmix(j,1)*
     .     slhahmix(k,3)*sb+slhahmix(i,1)*slhahmix(j,3)*slhahmix(k,3)*
     .     sb+slhahmix(i,3)*slhahmix(j,2)*slhahmix(k,3)*cb+
     .     slhahmix(i,3)*slhahmix(j,3)*slhahmix(k,2)*cb+slhahmix(i,2)*
     .     slhahmix(j,3)*slhahmix(k,3)*cb)
c
     .     +dsqrt(2.D0)/amz**2*xkappa*Akap*v*slhahmix(i,3)*
     .     slhahmix(j,3)*slhahmix(k,3)
c
     .     +(-1.D0)/amz**2/dsqrt(2.D0)*xlambdapar*Alam*v*(slhahmix(i,2)*
     .     slhahmix(j,1)*slhahmix(k,3)+slhahmix(i,1)*slhahmix(j,2)*
     .     slhahmix(k,3)+slhahmix(i,2)*slhahmix(j,3)*slhahmix(k,1)+
     .     slhahmix(i,3)*slhahmix(j,1)*slhahmix(k,2)+slhahmix(i,1)*
     .     slhahmix(j,3)*slhahmix(k,2)+slhahmix(i,3)*slhahmix(j,2)*
     .     slhahmix(k,1))
c
     .     -slhahmix(i,1)*slhahmix(j,2)*sb*slhahmix(k,1)-slhahmix(i,1)*
     .     slhahmix(j,1)*sb*slhahmix(k,2)-slhahmix(i,1)*slhahmix(j,2)*
     .     cb*slhahmix(k,2)-slhahmix(i,2)*slhahmix(j,2)*cb*
     .     slhahmix(k,1)+3.D0*slhahmix(i,2)*slhahmix(j,2)*sb*
     .     slhahmix(k,2)-slhahmix(i,2)*slhahmix(j,1)*sb*slhahmix(k,1)
     .     -slhahmix(i,2)*slhahmix(j,1)*cb*slhahmix(k,2)+
     .     3.D0*slhahmix(i,1)*slhahmix(j,1)*cb*slhahmix(k,1)

      ghihjhk = ghself

      kv = i
      iv = j
      jv = k
      ghiajak = slhaamix(iv,1)*slhaamix(jv,1)*((-dcos(b)*slhahmix(kv,1)
     .          +dsin(b)*slhahmix(kv,2))*dcos(2.D0*b)+xlambdapar**2*v**2
     .          /amz**2*(dcos(b)**3*slhahmix(kv,1)
     .          +dsin(b)**3*slhahmix(kv,2))+xlambdapar*v/amz**2*
     .          (dsqrt(2.D0)*Alam*dcos(b)*dsin(b)+(xlamvs+xlamvs
     .          /xlambdapar*xkappa*dsin(2.D0*b)))*slhahmix(kv,3) )
     .       +(slhaamix(iv,1)*slhaamix(jv,2)+slhaamix(iv,2)*
     .        slhaamix(jv,1))*
     .        xlambdapar*v/2.D0/amz**2*((dsqrt(2.D0)*Alam-2.D0*xkappa*
     .        xlamvs/xlambdapar)*(dcos(b)*slhahmix(kv,1)+dsin(b)*
     .        slhahmix(kv,2))-2.D0*xkappa*v*slhahmix(kv,3))
     .       +slhaamix(iv,2)*slhaamix(jv,2)*(xlambdapar*v**2/amz**2*
     .       ((xlambdapar*dcos(b)+xkappa*dsin(b))*slhahmix(kv,1)
     .       +(xlambdapar*dsin(b)*xkappa*dcos(b))*slhahmix(kv,2))
     .       +xkappa*v/amz**2*(-dsqrt(2.D0)*Akap+2.D0*xkappa*xlamvs
     .       /xlambdapar)*slhahmix(kv,3))

      ghjb = slhahmix(j,1)/dcos(b)

      ghkb = slhahmix(k,1)/dcos(b)

      gajb = slhaamix(j,1)*tgbet

      gakb = slhaamix(k,1)*tgbet

      ghivv = slhahmix(i,1)*dcos(b)+slhahmix(i,2)*dsin(b) 
      
      ghihpwm = -(-slhahmix(i,1)*dsin(b)+slhahmix(i,2)*dcos(b))

      ghit = slhahmix(i,2)/dsin(b)

      ghict = 1.D0/tgbet

c --- the masses --- c

      if(i.eq.1) then
         amhiggsi = massval(2)
         amhai = massval(3)
      elseif(i.eq.2) then
         amhiggsi = massval(34)
         amhai = massval(4)
      elseif(i.eq.3) then
         amhiggsi = massval(35)
         amhai = 0.D0
      endif
      if(j.eq.1) then
         amhiggsj = massval(2)
         amhaj = massval(3)
      elseif(j.eq.2) then
         amhiggsj = massval(34)
         amhaj = massval(4)
      elseif(j.eq.3) then
         amhiggsj = massval(35)
         amhaj = 0.D0
      endif
      if(k.eq.1) then
         amhiggsk = massval(2)
         amhak = massval(3)
      elseif(k.eq.2) then
         amhiggsk = massval(34)
         amhak = massval(4)
      elseif(k.eq.3) then
         amhiggsk = massval(35)
         amhak = 0.D0
      endif
      amhichar = massval(5)

c -- put back original parameters -- c

      spinfo1 = ospinfo1
      spinfo2 = ospinfo2
      modselval = omodselval

      do ii=1,2,1      
         imod(ii) = ioimod(ii)
      end do

      do ii=1,20,1
         minval(ii) = ominval(ii)
         mincom(ii) = omincom(ii)
         qvalue(ii) = oqvalue(ii)
      end do
      do ii=1,24,1
         smval(ii) = osmval(ii)
      end do
      do ii=1,100,1
         massval(ii) = omassval(ii)
      end do
      do ii=1,5,1
         do jj=1,5,1
            nmixval(ii,jj) = onmixval(ii,jj)
         end do
      end do
      do ii=1,2,1
         do jj=1,2,1
            umixval(ii,jj) = oumixval(ii,jj)
            vmixval(ii,jj) = ovmixval(ii,jj)
            stopmixval(ii,jj) = ostopmixval(ii,jj)
            sbotmixval(ii,jj) = osbotmixval(ii,jj)
            staumixval(ii,jj) = ostaumixval(ii,jj)
         end do
      end do
      do ii=1,10,1
         hmixval(ii) = ohmixval(ii)
         hmixcom(ii) = ohmixcom(ii)
      end do
      do ii=1,3,1
         gaugeval(ii) = ogaugeval(ii)
      end do
      do ii=1,100,1
         msoftval(ii) = omsoftval(ii)
         softcom(ii) = osoftcom(ii)
      end do
      do ii=1,3,1
         do jj=1,3,1
            auval(ii,jj) = oauval(ii,jj)
            adval(ii,jj) = oadval(ii,jj)
            aeval(ii,jj) = oaeval(ii,jj)
            yuval(ii,jj) = oyuval(ii,jj)
            ydval(ii,jj) = oydval(ii,jj)
            yeval(ii,jj) = oyeval(ii,jj)
         end do
      end do
      do ii=0,100,1
         extval(ii) = oextval(ii)
         extcom(ii) = oextcom(ii)
      end do
      do ii=1,3,1
         do jj=1,3,1
            nmhmixval(ii,jj) = onmhmixval(ii,jj)
         end do
      end do
      do ii=1,2,1
         do jj=1,3,1
            nmamixval(ii,jj) = onmamixval(ii,jj)
         end do
      end do
      do ii=1,4,1
         vckmval(ii) = ovckmval(ii)
      end do

      do ii=1,34,1
         check(ii) = ocheck(ii)
      end do

c ---------------------------------- c

      return

      end

c ==================================================================== c
c ================ subroutine for the susy couplings ================= c
c ==================================================================== c

      subroutine susycp_nmssm(tgbet)

      implicit double precision (a-h,m,o-z)
      double precision slhahmix(3,3),slhaamix(2,2)
      double precision ghigb(5),ghigt(5),ghigvv(3),ghighpwm(3),
     .     gzaihj(2,3),g3hihjhk(5,5,5),ghigpm(3)
      dimension ghkaiaj(3,2,2)

      COMMON/HMASS_NMSSM/amha1,amha2,amh1,amh2,amh3,amch,amar
      COMMON/HMASSR_NMSSM/amlr,amhr
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/SLHA_hmass_NMSSM/slhamh1,slhamh2,slhamh3,slhama1,slhama2,
     .     slhamc
c      COMMON/NMSSMCOUPL1/GHS3(5,5,5),GHS3PM(5),Gt(5),Gb(5),Ghivv(3),
c     .     ghiva(3,2),ghipwm(3)
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
      COMMON/SHORTCOUPL2_NMSSM/ghigb,ghigt,ghigvv,ghighpwm,gzaihj,
     .     g3hihjhk,ghigpm

c --- some parameter settings --- c

      pi=4*datan(1d0)
      v=1.d0/dsqrt(dsqrt(2.d0)*gf)
      bet=datan(tgbet)
      sb = dsin(bet)
      cb = dcos(bet)
      amar = ama2

c --- setting of the Higgs masses --- c

      amlr = slhamh1
      amhr = slhamh3
      amh1  = slhamh1
      amh2  = slhamh2
      amh3  = slhamh3
      amch = slhamc

c --- setting of the Higgs couplings --- c

      b = bet
      
      gh1t = slhahmix(1,2)/dsin(b)
      gh2t = slhahmix(2,2)/dsin(b)
      gh3t = slhahmix(3,2)/dsin(b)
      ga1t = slhaamix(1,1)/tgbet
      ga2t = slhaamix(2,1)/tgbet

      gh1b = slhahmix(1,1)/dcos(b)
      gh2b = slhahmix(2,1)/dcos(b)
      gh3b = slhahmix(3,1)/dcos(b)
      ga1b = slhaamix(1,1)*tgbet
      ga2b = slhaamix(2,1)*tgbet

      gh1vv = slhahmix(1,1)*dcos(b)+slhahmix(1,2)*dsin(b)
      gh2vv = slhahmix(2,1)*dcos(b)+slhahmix(2,2)*dsin(b)
      gh3vv = slhahmix(3,1)*dcos(b)+slhahmix(3,2)*dsin(b)

      ghigt(1) = gh1t
      ghigt(2) = gh2t
      ghigt(3) = gh3t
      ghigt(4) = ga1t
      ghigt(5) = ga2t

      ghigb(1) = gh1b
      ghigb(2) = gh2b
      ghigb(3) = gh3b
      ghigb(4) = ga1b
      ghigb(5) = ga2b

      ghigvv(1) = gh1vv
      ghigvv(2) = gh2vv
      ghigvv(3) = gh3vv

      gh1hpwm = -(-slhahmix(1,1)*dsin(b)+slhahmix(1,2)*dcos(b))
      gh2hpwm = -(-slhahmix(2,1)*dsin(b)+slhahmix(2,2)*dcos(b))
      gh3hpwm = -(-slhahmix(3,1)*dsin(b)+slhahmix(3,2)*dcos(b))

      ghighpwm(1) = gh1hpwm
      ghighpwm(2) = gh2hpwm
      ghighpwm(3) = gh3hpwm

      gza1h1 = (slhahmix(1,1)*dsin(b)-slhahmix(1,2)*dcos(b))*
     .     slhaamix(1,1)
      gza1h2 = (slhahmix(2,1)*dsin(b)-slhahmix(2,2)*dcos(b))*
     .     slhaamix(1,1)
      gza1h3 = (slhahmix(3,1)*dsin(b)-slhahmix(3,2)*dcos(b))*
     .     slhaamix(1,1)

      gza2h1 = (slhahmix(1,1)*dsin(b)-slhahmix(1,2)*dcos(b))*
     .     slhaamix(2,1)
      gza2h2 = (slhahmix(2,1)*dsin(b)-slhahmix(2,2)*dcos(b))*
     .     slhaamix(2,1)
      gza2h3 = (slhahmix(3,1)*dsin(b)-slhahmix(3,2)*dcos(b))*
     .     slhaamix(2,1)

      gzaihj(1,1) = gza1h1
      gzaihj(1,2) = gza1h2
      gzaihj(1,3) = gza1h3
      gzaihj(2,1) = gza2h1
      gzaihj(2,2) = gza2h2
      gzaihj(2,3) = gza2h3

c -- H_i H_j H_k -- c

      do i=1,5,1
         do j=1,5,1
            do k=1,5,1
               g3hihjhk(i,j,k) = 123456.D0
            end do
         end do
      end do

      gh1h1h1 = ghself(1,1,1,sb,cb)
      gh1h2h2 = ghself(1,2,2,sb,cb)
      gh2h1h1 = ghself(2,1,1,sb,cb)
      gh1h3h3 = ghself(1,3,3,sb,cb)
      gh3h1h1 = ghself(3,1,1,sb,cb)
      gh2h3h3 = ghself(2,3,3,sb,cb)
      gh3h2h2 = ghself(3,2,2,sb,cb)
      gh2h2h2 = ghself(2,2,2,sb,cb)
      gh3h3h3 = ghself(3,3,3,sb,cb)

      gh1h2h3 = ghself(1,2,3,sb,cb)

      g3hihjhk(1,1,1) = gh1h1h1
      g3hihjhk(1,2,2) = gh1h2h2
      g3hihjhk(2,1,1) = gh2h1h1
      g3hihjhk(1,3,3) = gh1h3h3
      g3hihjhk(3,1,1) = gh3h1h1
      g3hihjhk(2,3,3) = gh2h3h3
      g3hihjhk(3,2,2) = gh3h2h2
      g3hihjhk(2,2,2) = gh2h2h2
      g3hihjhk(3,3,3) = gh3h3h3
      g3hihjhk(1,2,3) = gh1h2h3

      g3hihjhk(1,1,2) = gh2h1h1
      g3hihjhk(1,2,1) = gh2h1h1
      g3hihjhk(1,1,3) = gh3h1h1
      g3hihjhk(1,3,1) = gh3h1h1
      g3hihjhk(1,3,2) = gh1h2h3

      g3hihjhk(2,1,2) = gh1h2h2
      g3hihjhk(2,2,1) = gh1h2h2
      g3hihjhk(2,2,3) = gh3h2h2
      g3hihjhk(2,3,2) = gh3h2h2
      g3hihjhk(2,1,3) = gh1h2h3
      g3hihjhk(2,3,1) = gh1h2h3

      g3hihjhk(3,1,3) = gh1h3h3
      g3hihjhk(3,3,1) = gh1h3h3
      g3hihjhk(3,2,3) = gh2h3h3
      g3hihjhk(3,3,2) = gh2h3h3
      g3hihjhk(3,1,2) = gh1h2h3
      g3hihjhk(3,2,1) = gh1h2h3

      g3hihjhk(1,1,4) = 0.D0
      g3hihjhk(1,4,1) = 0.D0
      g3hihjhk(1,2,4) = 0.D0
      g3hihjhk(1,4,2) = 0.D0
      g3hihjhk(1,3,4) = 0.D0
      g3hihjhk(1,4,3) = 0.D0

      g3hihjhk(1,1,5) = 0.D0
      g3hihjhk(1,5,1) = 0.D0
      g3hihjhk(1,2,5) = 0.D0
      g3hihjhk(1,5,2) = 0.D0
      g3hihjhk(1,3,5) = 0.D0
      g3hihjhk(1,5,3) = 0.D0

      g3hihjhk(2,1,4) = 0.D0
      g3hihjhk(2,4,1) = 0.D0
      g3hihjhk(2,2,4) = 0.D0
      g3hihjhk(2,4,2) = 0.D0
      g3hihjhk(2,3,4) = 0.D0
      g3hihjhk(2,4,3) = 0.D0

      g3hihjhk(2,1,5) = 0.D0
      g3hihjhk(2,5,1) = 0.D0
      g3hihjhk(2,2,5) = 0.D0
      g3hihjhk(2,5,2) = 0.D0
      g3hihjhk(2,3,5) = 0.D0
      g3hihjhk(2,5,3) = 0.D0

      g3hihjhk(3,1,4) = 0.D0
      g3hihjhk(3,4,1) = 0.D0
      g3hihjhk(3,2,4) = 0.D0
      g3hihjhk(3,4,2) = 0.D0
      g3hihjhk(3,3,4) = 0.D0
      g3hihjhk(3,4,3) = 0.D0

      g3hihjhk(3,1,5) = 0.D0
      g3hihjhk(3,5,1) = 0.D0
      g3hihjhk(3,2,5) = 0.D0
      g3hihjhk(3,5,2) = 0.D0
      g3hihjhk(3,3,5) = 0.D0
      g3hihjhk(3,5,3) = 0.D0

c -- H_k A_i A_j -- c

      do k=1,3,1
         do i=1,2,1
            do j=1,2,1
               ghkaiaj(k,i,j) = 
     .          slhaamix(i,1)*slhaamix(j,1)*((-dcos(b)*slhahmix(k,1)
     .          +dsin(b)*slhahmix(k,2))*dcos(2.D0*b)+xlambdapar**2*v**2
     .          /amz**2*(dcos(b)**3*slhahmix(k,1)
     .          +dsin(b)**3*slhahmix(k,2))
     .          +xlambdapar*v/amz**2*(dsqrt(2.D0)*Alam*dcos(b)*dsin(b)
     .          +(xlamvs+xlamvs/xlambdapar*xkappa*dsin(2.D0*b))
     .          *dsqrt(2.D0))*slhahmix(k,3) )
     .       +(slhaamix(i,1)*slhaamix(j,2)+slhaamix(i,2)*slhaamix(j,1))*
     .        xlambdapar*v/2.D0/amz**2*((dsqrt(2.D0)*Alam-2.D0*xkappa*
     .        xlamvs*dsqrt(2.D0)
     .              /xlambdapar)*(dcos(b)*slhahmix(k,1)+dsin(b)*
     .        slhahmix(k,2))-2.D0*xkappa*v*slhahmix(k,3))
     .       +slhaamix(i,2)*slhaamix(j,2)*(xlambdapar*v**2/amz**2*
     .       ((xlambdapar*dcos(b)+xkappa*dsin(b))*slhahmix(k,1)
     .       +(xlambdapar*dsin(b)+xkappa*dcos(b))*slhahmix(k,2))
     .       +xkappa*v/amz**2*(-dsqrt(2.D0)*Akap
     .       +2.D0*xkappa*xlamvs*dsqrt(2.D0)/xlambdapar)*slhahmix(k,3))
            end do
         end do
      end do

      gh1a1a1 = ghkaiaj(1,1,1)
      gh1a2a2 = ghkaiaj(1,2,2)
      gh1a1a2 = ghkaiaj(1,1,2)
      gh2a1a1 = ghkaiaj(2,1,1)
      gh2a2a2 = ghkaiaj(2,2,2)
      gh2a1a2 = ghkaiaj(2,1,2)
      gh3a1a1 = ghkaiaj(3,1,1)
      gh3a2a2 = ghkaiaj(3,2,2)
      gh3a1a2 = ghkaiaj(3,1,2)

      g3hihjhk(1,4,4) = gh1a1a1
      g3hihjhk(1,5,5) = gh1a2a2
      g3hihjhk(1,4,5) = gh1a1a2
      g3hihjhk(1,5,4) = gh1a1a2
      g3hihjhk(2,4,4) = gh2a1a1
      g3hihjhk(2,5,5) = gh2a2a2
      g3hihjhk(2,4,5) = gh2a1a2
      g3hihjhk(2,5,4) = gh2a1a2
      g3hihjhk(3,4,4) = gh3a1a1
      g3hihjhk(3,5,5) = gh3a2a2
      g3hihjhk(3,4,5) = gh3a1a2
      g3hihjhk(3,5,4) = gh3a1a2

      g3hihjhk(4,1,1) = 0.D0
      g3hihjhk(4,1,2) = 0.D0
      g3hihjhk(4,2,1) = 0.D0
      g3hihjhk(4,2,2) = 0.D0
      g3hihjhk(4,3,3) = 0.D0
      g3hihjhk(4,1,3) = 0.D0
      g3hihjhk(4,3,1) = 0.D0
      g3hihjhk(4,2,3) = 0.D0
      g3hihjhk(4,3,2) = 0.D0
      g3hihjhk(4,1,4) = gh1a1a1
      g3hihjhk(4,4,1) = gh1a1a1
      g3hihjhk(4,2,4) = gh2a1a1
      g3hihjhk(4,4,2) = gh2a1a1
      g3hihjhk(4,3,4) = gh3a1a1
      g3hihjhk(4,4,3) = gh3a1a1
      g3hihjhk(4,1,5) = gh1a1a2
      g3hihjhk(4,5,1) = gh1a1a2
      g3hihjhk(4,2,5) = gh2a1a2
      g3hihjhk(4,5,2) = gh2a1a2
      g3hihjhk(4,3,5) = gh3a1a2
      g3hihjhk(4,5,3) = gh3a1a2
      g3hihjhk(4,4,5) = 0.D0
      g3hihjhk(4,5,4) = 0.D0
      g3hihjhk(4,5,5) = 0.D0
      g3hihjhk(4,4,4) = 0.D0
      g3hihjhk(5,5,5) = 0.D0

      g3hihjhk(5,1,1) = 0.D0
      g3hihjhk(5,1,2) = 0.D0
      g3hihjhk(5,2,1) = 0.D0
      g3hihjhk(5,2,2) = 0.D0
      g3hihjhk(5,3,3) = 0.D0
      g3hihjhk(5,1,3) = 0.D0
      g3hihjhk(5,3,1) = 0.D0
      g3hihjhk(5,2,3) = 0.D0
      g3hihjhk(5,3,2) = 0.D0
      g3hihjhk(5,1,4) = gh1a1a2
      g3hihjhk(5,4,1) = gh1a1a2
      g3hihjhk(5,2,4) = gh2a1a2
      g3hihjhk(5,4,2) = gh2a1a2
      g3hihjhk(5,3,4) = gh3a1a2
      g3hihjhk(5,4,3) = gh3a1a2
      g3hihjhk(5,1,5) = gh1a2a2
      g3hihjhk(5,5,1) = gh1a2a2
      g3hihjhk(5,2,5) = gh2a2a2
      g3hihjhk(5,5,2) = gh2a2a2
      g3hihjhk(5,3,5) = gh3a2a2
      g3hihjhk(5,5,3) = gh3a2a2
      g3hihjhk(5,4,5) = 0.D0
      g3hihjhk(5,5,4) = 0.D0
      g3hihjhk(5,4,4) = 0.D0

c -- H_k H+ H- -- c

      gh1pm   = (1.D0-xlambdapar**2*v**2/2.D0/amz**2)*dcos(2.D0*b)*
     .     (-dcos(b)*slhahmix(1,1)+dsin(b)*slhahmix(1,2))
     .     +(2.D0*amw**2/amz**2-xlambdapar**2*v**2/2.D0/amz**2)*
     .     (dcos(b)*slhahmix(1,1)+dsin(b)*slhahmix(1,2))
     .     +(xlambdapar*v/amz**2)*(dsqrt(2.D0)*Alam*dsin(b)*dcos(b)
     .     +xlamvs*dsqrt(2.D0)
     .     +xlamvs/xlambdapar*xkappa*dsin(2.D0*b)*dsqrt(2.D0))*
     .     slhahmix(1,3)

      gh2pm   =  (1.D0-xlambdapar**2*v**2/2.D0/amz**2)*dcos(2.D0*b)*
     .     (-dcos(b)*slhahmix(2,1)+dsin(b)*slhahmix(2,2))
     .     +(2.D0*amw**2/amz**2-xlambdapar**2*v**2/2.D0/amz**2)*
     .     (dcos(b)*slhahmix(2,1)+dsin(b)*slhahmix(2,2))
     .     +(xlambdapar*v/amz**2)*(dsqrt(2.D0)*Alam*dsin(b)*dcos(b)
     .     +xlamvs*dsqrt(2.D0)
     .     +xlamvs/xlambdapar*xkappa*dsin(2.D0*b)*dsqrt(2.D0))*
     .     slhahmix(2,3)

      gh3pm   =  (1.D0-xlambdapar**2*v**2/2.D0/amz**2)*dcos(2.D0*b)*
     .     (-dcos(b)*slhahmix(3,1)+dsin(b)*slhahmix(3,2))
     .     +(2.D0*amw**2/amz**2-xlambdapar**2*v**2/2.D0/amz**2)*
     .     (dcos(b)*slhahmix(3,1)+dsin(b)*slhahmix(3,2))
     .     +(xlambdapar*v/amz**2)*(dsqrt(2.D0)*Alam*dsin(b)*dcos(b)
     .     +xlamvs*dsqrt(2.D0)
     .     +xlamvs/xlambdapar*xkappa*dsin(2.D0*b)*dsqrt(2.D0))*
     .     slhahmix(3,3)

      ghigpm(1) = gh1pm
      ghigpm(2) = gh2pm
      ghigpm(3) = gh3pm


      return
111   stop
      end

c -------------------------------------------------------------------- c

c ===================== the function f0 ===============
      complex*16 function f0_hdec(m1,m2,qsq)
      implicit real*8 (a-h,m,o-z)
      complex*16 cd,cr,cq2,ieps,cbet,cxx
      m1sq = m1*m1
      m2sq = m2*m2
      aqsq = dabs(qsq)
      ieps = dcmplx(1.d0,1.d-12)
      cq2 = qsq*ieps
      cd = (m1sq-m2sq)/cq2
      cr = cdsqrt((1+cd)**2 - 4*m1sq/cq2)
      if(qsq.eq.0.d0) then
       f0_hdec = 0.d0
      else
       if(m1.eq.m2) then
        f0_hdec = -2.d0 + cr*cdlog(-(1+cr)/(1-cr))
       else
        cbet = cdsqrt(1-4*m1*m2/(cq2 - (m1-m2)**2))
        cxx = (cbet-1)/(cbet+1)
        f0_hdec = -1 + ((qsq+m2sq-m1sq)/2/qsq - m2sq/(m2sq-m1sq))
     .                                           *dlog(m2sq/m1sq)
     .     - (qsq-(m1-m2)**2)/qsq*cbet*cdlog(cxx)
       endif
      endif
      RETURN
      END

c -------------------------------------------------------------------- c
c     ************************************************************
c     subroutine for Hsm ---> V*V* ---> 4f
c     ************************************************************
      subroutine htovv_hdec(iv,amh,amv,gamv,htvv)
      implicit double precision (a-h,o-z)
      common/vvoff_hdec/amh1,amv1,gamv1
      common/vvoffflag_hdec/iv1
      common/prec_hdec/ip
      external ftovv1_hdec
c     ip=20
      ip=50
      amh1=amh
      amv1=amv
      gamv1=gamv
      iv1 = iv
      dlt=1d0/ip
      sum=0d0
      do 1 i=1,ip
       uu=dlt*i
       dd=uu-dlt
       call qgaus1_hdec(ftovv1_hdec,dd,uu,res)
       sum=sum+res
1     continue
      htvv=sum
      return
      end

c -------------------------------------------------------------------- c

      double precision function ftovv1_hdec(xx)
      implicit double precision (a-h,o-z)
      common/first_hdec/x1
      common/prec_hdec/ip
      external ftovv2_hdec
      x1=xx
      dlt=1d0/ip
      sum=0d0
      do 1 i=1,ip
       uu=dlt*i
       dd=uu-dlt
       call qgaus2_hdec(ftovv2_hdec,dd,uu,res)
       sum=sum+res
1     continue
      ftovv1_hdec=sum
      return
      end

c -------------------------------------------------------------------- c

      double precision function ftovv2_hdec(xx)
      implicit double precision (a-h,o-z)
      dimension yy(2)
      common/first_hdec/x1
      yy(1)=x1
      yy(2)=xx
      ftovv2_hdec=ftovv_hdec(yy)
      return
      end

c -------------------------------------------------------------------- c

      double precision function ftovv_hdec(xx)
      implicit double precision (a-h,o-z)
      double precision lamb
      dimension xx(2)
      common/vvoff_hdec/amh,amw,gamw
      lamb(x,y)=dsqrt((1.d0-x-y)**2-4.d0*x*y)
      pi=4d0*datan(1d0)
      if(amh.lt.2*amw)then
       icase = 1
      else
       icase = 0
      endif
      if(icase.eq.0)then
       yy = amh**2
       y1 = datan((yy-amw**2)/amw/gamw)
       y2 = -datan((amw**2)/amw/gamw)
       djac = y1-y2
       t1 = tan(y1*xx(1)+y2*(1.d0-xx(1)))
       sp = amw**2 + amw*gamw*t1
       yy = (amh-dsqrt(sp))**2
       y1 = datan((yy-amw**2)/amw/gamw)
       y2 = -datan((amw**2)/amw/gamw)
       djac = djac*(y1-y2)
       t2 = tan(y1*xx(2)+y2*(1.d0-xx(2)))
       sm = amw**2 + amw*gamw*t2
       am2=amh**2
       gam = am2*lamb(sp/am2,sm/am2)*(1+lamb(sp/am2,sm/am2)**2*amh**4
     .                               /sp/sm/12)
       pro1 = sp/amw**2
       pro2 = sm/amw**2
       ftovv_hdec = pro1*pro2*gam*djac/pi**2
     .            * radvv_hdec(sp,sm)
      else
       sp = amh**2*xx(1)
       sm = (amh-dsqrt(sp))**2*xx(2)
       djac = amh**2*(amh-dsqrt(sp))**2/pi**2
       am2=amh**2
       gam = am2*lamb(sp/am2,sm/am2)*(1+lamb(sp/am2,sm/am2)**2*amh**4
     .                               /sp/sm/12)
       pro1 = sp*gamw/amw/((sp-amw**2)**2+amw**2*gamw**2)
       pro2 = sm*gamw/amw/((sm-amw**2)**2+amw**2*gamw**2)
       ftovv_hdec = pro1*pro2*gam*djac
     .            * radvv_hdec(sp,sm)
      endif
      return
      end

c -------------------------------------------------------------------- c

      double precision function radvv_hdec(q12,q22)
      implicit double precision (a-b,d-h,o-z), complex*16 (c)
      common/vvoffflag_hdec/iv
      q1 = dsqrt(q12)
      q2 = dsqrt(q22)
      if(iv.eq.0)then
       radvv_hdec=1
      elseif(iv.eq.1)then
       radvv_hdec=radww_hdec(q1,q2)
      else
       radvv_hdec=radzz_hdec(q1,q2)
      endif
      return
      end

c -------------------------------------------------------------------- c

      double precision function radww_hdec(q1w,q2w)
C     ************************************************************
C     ELECTROWEAK CORRECTIONS TO HSM ---> W*W* ---> 4F (APPROX.)
C     (A. BREDENSTEIN ET AL., PHYS. REV. D74 (2006) 013004
C                             [ARXIV:HEP-PH/0604011])
C     ************************************************************
      implicit double precision (a-b,d-h,o-z), complex*16 (c)
      double precision lamb
      common/vvoff_hdec/amh,amw,gamw
      common/param_hdec/gf,alph,amtau,ammuon,amz,amw0
      common/masses_hdec/ams0,amc0,amb0,amt
      common/wzwdth_hdec/gamc0,gamt,gamt1,gamw0,gamz

      lamb(x,y)=dsqrt((1.d0-x-y)**2-4.d0*x*y)
      pi=4d0*datan(1d0)
      cmut2 = dcmplx(amt**2,-amt*gamt)
      cmuw2 = dcmplx(amw**2,-amw*gamw)
      cmuz2 = dcmplx(amz**2,-amz*gamz*3)
      cbett = cdsqrt(1-4*cmut2/amh**2)
      cbetw = cdsqrt(1-4*cmuw2/amh**2)
      cbetz = cdsqrt(1-4*cmuz2/amh**2)
      cxt = (cbett-1)/(cbett+1)
      cthww = 8+12*cbett**2+3*cbett*(3*cbett**2-1)*cdlog(cxt)
     .      + 3/2.d0*(1-cbett**2)**2*cdlog(cxt)**2
      cbetwb = cdsqrt(dcmplx(amh**4+q1w**4+q2w**4-2*amh**2*q1w**2
     .             -2*amh**2*q2w**2-2*q1w**2*q2w**2))/amh**2
      cgw = (1-cbetwb**2)**2
      dmw = dabs(q1w**2-q2w**2)/amh**2
      cdcoul = alph/cbetwb*dimag(cdlog((cbetw-cbetwb+dmw)
     .                                /(cbetw+cbetwb+dmw)))
c     cgz = 1
      cgz = 0.7d0
      cdcoulz = alph/2/cbetz*dimag(cdlog((cbetz-cbetwb)
     .                                  /(cbetz+cbetwb)))
c     bias = 0.04d0*(1+(100-amh)/500)
      bias = 0.05d0*(1+(100-amh)/500)
      radww_hdec=dreal(1.d0
     .           +gf*cmut2/8.d0/pi**2/dsqrt(2.d0)*(-5+cthww)
     .           +gf*amh**2/16.d0/pi**2/dsqrt(2.d0)*2.800952d0
     .           +(gf*amh**2/16.d0/pi**2/dsqrt(2.d0))**2*62.0308d0
     .           +cgw*cdcoul + cgz*cdcoulz + bias)
      return
      end

c -------------------------------------------------------------------- c

      double precision function radzz_hdec(q1z,q2z)
C     ************************************************************
C     ELECTROWEAK CORRECTIONS TO HSM ---> Z*Z* ---> 4F (APPROX.)
C     (A. BREDENSTEIN ET AL., PHYS. REV. D74 (2006) 013004
C                             [ARXIV:HEP-PH/0604011])
C     ************************************************************
      implicit double precision (a-b,d-h,o-z), complex*16 (c)
      double precision lamb
      common/vvoff_hdec/amh,amz,gamz
      common/param_hdec/gf,alph,amtau,ammuon,amz0,amw
      common/masses_hdec/ams0,amc0,amb0,amt
      common/wzwdth_hdec/gamc0,gamt,gamt1,gamw,gamz0

      lamb(x,y)=dsqrt((1.d0-x-y)**2-4.d0*x*y)
      pi=4.d0*datan(1.d0)
      cmut2 = dcmplx(amt**2,-amt*gamt)
      cmuw2 = dcmplx(amw**2,-amw*gamw)
      cmuz2 = dcmplx(amz**2,-amz*gamz*2)
      cbett = cdsqrt(1-4*cmut2/amh**2)
      cbetw = cdsqrt(1-4*cmuw2/amh**2)
      cbetz = cdsqrt(1-4*cmuz2/amh**2)
      cxt = (cbett-1)/(cbett+1)
      cthzz = 20+6*cbett**2+3*cbett*(cbett**2+1)*cdlog(cxt)
     .      + 3*(1-cbett**2)*cdlog(cxt)**2
      cbetzb = cdsqrt(dcmplx(amh**4+q1z**4+q2z**4-2*amh**2*q1z**2
     .             -2*amh**2*q2z**2-2*q1z**2*q2z**2))/amh**2
c     cgw =-0.15d0 * (1-cbetw**2)**2
      cgw =-0.10d0 * (1-cbetw**2)**2
      cdcoulw = alph/2/cbetw*dimag(cdlog((cbetw-cbetzb)
     .                                  /(cbetw+cbetzb)))
      cgz = 1
      cdcoulz = alph/2/cbetz*dimag(cdlog((cbetz-cbetzb)
     .                                  /(cbetz+cbetzb)))
c     bias = 0.02d0*(1+(100-amh)/350)
      bias = 0.02d0*(1+(100-amh)/300)
      radzz_hdec=dreal(1.d0
     .           +gf*cmut2/8.d0/pi**2/dsqrt(2.d0)*(1+cthzz)
     .           +gf*amh**2/16.d0/pi**2/dsqrt(2.d0)*2.800952d0
     .           +(gf*amh**2/16.d0/pi**2/dsqrt(2.d0))**2*62.0308d0
     .           +cgw*cdcoulw + cgz*cdcoulz + bias)
      return
      end

c -------------------------------------------------------------------- c

c **************************************************
c SUBROUTINE FOR A -> TT* -> TBW
c **************************************************

      subroutine atott_NMSSM(ama,amt,amb,amw,amch,att0,tanb,gab,gat)
      implicit real*8(a-z)
      integer ip,k
      common/prec1_hdec/ip
      external funatt1_hdec
      common/iksy1_hdec/x1,x2,m1,m2,m3,ecm,s
      common/top1_hdec/ama1,amt1,amb1,amw1,amch1
      common/top3_NMSSM/tanb1,gab1,gat1

      ama1=ama
      amt1=amt
      amb1=amb
      amw1=amw
      amch1=amch
      tanb1 = tanb
      gab1 = gab
      gat1 = gat
      ip=5
      m1=amb
      m2=amt
      m3=amw
c        first integrate over x2, i.e. (1+3) system
c        check whether enough phase space
      mastot=m1+m2+m3
      if(mastot.ge.ama) goto 12
      ecm=ama
      s=ecm**2
      u1=(ecm-m2)**2
      d1=(m1+m3)**2
      u=(s-d1+m2**2)/s
      d=(s-u1+m2**2)/s
      del=(u-d)/ip
      u=d+del
      xsec=0.d0
      do k=1,ip
      call qgaus1_hdec(funatt1_hdec,d,u,ss)
      d=u
      u=d+del
      xsec=xsec+ss
      enddo
      att0=xsec
 12   continue
      return
      end

c -------------------------------------------------------------------- c

      double precision function funatt1_hdec(xl)
      implicit real*8(a-z)
      integer ip,i
      common/iksy1_hdec/x1,x2,m1,m2,m3,ecm,s
      common/prec1_hdec/ip
      external funatt2_hdec

      x2=xl
      s13=s-s*x2+m2**2
      tem=2.d0*dsqrt(s13)
      e2s=(s-s13-m2**2)/tem
      e3s=(s13+m3**2-m1**2)/tem
c     second integral over x1, i.e. (2+3) system
      u1=(e2s+e3s)**2-(dsqrt(e2s**2-m2**2)-dsqrt(e3s**2-m3**2))**2
      d1=(e2s+e3s)**2-(dsqrt(e2s**2-m2**2)+dsqrt(e3s**2-m3**2))**2
      u=(s-d1+m1**2)/s
      d=(s-u1+m1**2)/s
      del=(u-d)/ip
      funatt1_hdec=0.d0
      u=d+del
      do i=1,ip
      call qgaus2_hdec(funatt2_hdec,d,u,ss)
      funatt1_hdec=funatt1_hdec+ss
      d=u
      u=d+del
      enddo
      return
      end

c -------------------------------------------------------------------- c

      double precision function funatt2_hdec(xk)
      implicit real*8(a-z)
      common/iksy1_hdec/x1,x2,m1,m2,m3,ecm,s
      x1=xk
      call elematt_NMSSM(ss)
      funatt2_hdec=ss
      return
      end

c -------------------------------------------------------------------- c

      subroutine elematt_NMSSM(res)
      implicit real*8(a-z)
      common/iksy1_hdec/x1,x2,m1,m2,m3,ecm,s
      common/top1_hdec/ama,amt,amb,amw,amch
      common/top3_NMSSM/tanb,gab,gat
      common/wzwdth_hdec/gamc0,gamt0,gamt1,gamw,gamz

      gamt=gamt1**2*amt**2/ama**4
      gamc=gamc0**2*amch**2/ama**4
      ch=amch**2/ama**2
      w=amw**2/ama**2
      t=amt**2/ama**2
      b=amb**2/ama**2
      y1=1-x1
      y2=1-x2
      x0=2.d0-x1-x2
      w1=(1.d0-x2)
      w2=(1.d0-x0+w-ch)
      w22=1.d0/((1.d0-x0+w-ch)**2+gamc)
      w11=1.d0/((1.d0-x2)**2+gamt)
      w12=w1*w2*w11*w22
      w3=(1.d0-x1)
      w33=1.d0/w3**2
      w13=w1*w11/w3
      w23=w2*w22/w3

      r11= 4.d0*b**2 - 8.d0*b*t + 4.d0*t**2 + 4.d0*b*w + 4.d0*t*w - 
     .     8.d0*w**2 - 8.d0*b*y2 + 8.d0*t*y2 + 4.d0*b*y1*y2 - 
     .     4.d0*t*y1*y2 + 8.d0*w*y1*y2 + 4.d0*y2**2 + 4.d0*b*y2**2 -
     .     4.d0*t*y2**2 + 4.d0*w*y2**2 - 4.d0*y1*y2**2 - 4.d0*y2**3
      r22= 4.d0/t*((4.d0*w - (y1 + y2)**2)*(t**2 + b*tanb**4*(-1.d0 +
     .     b - w + y1 + y2) + t*(-1.d0 + b*(1.d0 + 4.d0*tanb**2 +
     .     tanb**4) - w + y1 + y2)) )
      r12= 2.d0*(8.d0*w - 8.d0*b*w - 8.d0*t*w - 16.d0*b*tanb**2*w +
     .     8.d0*w**2 + 4.d0*b*y1 - 4.d0*t*y1 - 8.d0*w*y1 +
     .     4.d0*t*y1**2 + 4.d0*b*tanb**2*y1**2 + 4.d0*b*y2 -
     .     4.d0*t*y2 - 8.d0*w*y2 - 4.d0*y1*y2 + 8.d0*t*y1*y2 +
     .     8.d0*b*tanb**2*y1*y2 - 4.d0*w*y1*y2 + 4.d0*y1**2*y2 -
     .     4.d0*y2**2 + 4.d0*t*y2**2 + 4.d0*b*tanb**2*y2**2 - 
     .     4.d0*w*y2**2 + 8.d0*y1*y2**2 + 4.d0*y2**3)
      r33= 4.d0*b**2 - 8.d0*b*t + 4.d0*t**2 + 4.d0*b*w + 4.d0*t*w -
     .     8.d0*w**2 + 8.d0*b*y1 - 8.d0*t*y1 + 4.d0*y1**2 - 
     .     4.d0*b*y1**2 + 4.d0*t*y1**2 + 4.d0*w*y1**2 - 4.d0*y1**3 - 
     .     4.d0*b*y1*y2 + 4.d0*t*y1*y2 + 8.d0*w*y1*y2 - 4.d0*y1**2*y2
      r13= -8.d0*b*(y1**2+2.d0*y1*y2+y2**2+2.d0*w)
      r23= 2.d0*(-16.d0*t*w + 8.d0*tanb**2*w - 8.d0*b*tanb**2*w - 
     .     8.d0*t*tanb**2*w + 8.d0*tanb**2*w**2 - 4.d0*b*tanb**2*y1 + 
     .     4.d0*t*tanb**2*y1 - 8.d0*tanb**2*w*y1 + 4.d0*t*y1**2 - 
     .     4.d0*tanb**2*y1**2 + 4.d0*b*tanb**2*y1**2 - 
     .     4.d0*tanb**2*w*y1**2 + 4.d0*tanb**2*y1**3 - 
     .     4.d0*b*tanb**2*y2 + 4.d0*t*tanb**2*y2 - 8.d0*tanb**2*w*y2 + 
     .     8.d0*t*y1*y2 - 4.d0*tanb**2*y1*y2 + 8.d0*b*tanb**2*y1*y2 - 
     .     4.d0*tanb**2*w*y1*y2 + 8.d0*tanb**2*y1**2*y2 + 4.d0*t*y2**2 
     .     + 4.d0*b*tanb**2*y2**2 + 4.d0*tanb**2*y1*y2**2)

c --- take off charged Higgs exchange ---
      w22 = 0.D0
      w12 = 0.D0
      w23 = 0.D0
c ----------------------------------------

      res= r11*w11+r22*w22+r12*w12+
     .     r33*w33*b/t*(gab/gat)**2+
     .     r23*w23*gab/gat*b/t+
     .     r13*w13*gab/gat

      return

      end

c -------------------------------------------------------------------- c
C     ************************************************************
C     SUBROUTINE FOR H ---> TT* ---> TBW
C     ************************************************************
      SUBROUTINE HTOTT_NMSSM(AMH,AMT,AMB,AMW,AMCH,TB,GHT,GHB,GAT,GHVV,
     .     GHIHPWM,HTT0)
      IMPLICIT REAL*8(A-Z)
      INTEGER IP,K
      COMMON/PREC1_HDEC/IP
      EXTERNAL FUNHTT1_HDEC
      COMMON/IKSY2_HDEC/X1,X2,M1,M2,M3,ECM,S
      COMMON/TOP2_HDEC/AMH2,AMT2,AMB2,AMW2,AMCH2,TB2,GHT2,GHB2,
     .     GAT2,GHVV2
      COMMON/TOP2_NMSSM/GLVV2
      AMH2=AMH
      AMT2=AMT
      AMB2=AMB
      AMW2=AMW
      AMCH2=AMCH
      TB2=TB
      GHT2=GHT
      GHB2=GHB
      GAT2=GAT
      GHVV2=GHVV
      GLVV2=GHIHPWM
      IP=5
      M1=AMB
      M2=AMT
      M3=AMW
C     FIRST INTEGRATE OVER X2, i.e. (1+3) SYSTEM
C        CHECK WHETHER ENOUGH PHASE SPACE
      MASTOT=M1+M2+M3
      IF(MASTOT.GE.AMH) GOTO 12
      ECM=AMH
      S=ECM**2
      U1=(ECM-M2)**2
      D1=(M1+M3)**2
      U=(S-D1+M2**2)/s
      D=(S-U1+M2**2)/s
      DEL=(U-D)/IP
      U=D+DEL
      XSEC=0.D0
      DO K=1,IP
      CALL QGAUS1_HDEC(FUNHTT1_HDEC,D,U,SS)
      D=U
      U=D+DEL
      XSEC=XSEC+SS
      ENDDO
      HTT0=XSEC
 12   CONTINUE
      RETURN
      END

c -------------------------------------------------------------------- c

      DOUBLE PRECISION FUNCTION FUNHTT1_HDEC(XL)
      IMPLICIT REAL*8(A-Z)
      INTEGER IP,I
      COMMON/IKSY2_HDEC/X1,X2,M1,M2,M3,ECM,S
      COMMON/PREC1_HDEC/IP
      EXTERNAL FUNHTT2_HDEC
      X2=XL
      S13=S-S*X2+M2**2
      TEM=2.D0*DSQRT(S13)
      E2S=(S-S13-M2**2)/TEM
      E3S=(S13+M3**2-M1**2)/TEM
C     SECOND INTEGRAL OVER X1, i.e. (2+3) SYSTEM
      U1=(E2S+E3S)**2-(DSQRT(E2S**2-M2**2)-DSQRT(E3S**2-M3**2))**2
      D1=(E2S+E3S)**2-(DSQRT(E2S**2-M2**2)+DSQRT(E3S**2-M3**2))**2
      U=(S-D1+M1**2)/s
      D=(S-U1+M1**2)/s
      DEL=(U-D)/IP
      FUNHTT1_HDEC=0.d0
      U=D+DEL
      DO I=1,IP
      CALL QGAUS2_HDEC(FUNHTT2_HDEC,D,U,SS)
      FUNHTT1_HDEC=FUNHTT1_HDEC+SS
      D=U
      U=D+DEL
      END DO
      RETURN
      END

c -------------------------------------------------------------------- c

      DOUBLE PRECISION FUNCTION FUNHTT2_HDEC(XK)
      IMPLICIT REAL*8(A-Z)
      COMMON/IKSY2_HDEC/X1,X2,M1,M2,M3,ECM,S
      X1=XK
      CALL ELEMHTT_NMSSM(SS)
      FUNHTT2_HDEC=SS
      RETURN
      END

c -------------------------------------------------------------------- c

      SUBROUTINE ELEMHTT_NMSSM(RES)
      IMPLICIT REAL*8(A-Z)
      COMMON/IKSY2_HDEC/X1,X2,M1,M2,M3,ECM,S
      COMMON/TOP2_HDEC/AMH,AMT,AMB,AMW,AMCH,TB,GHT,GHB,GAT,GHVV
      COMMON/WZWDTH_HDEC/GAMC0,GAMT0,GAMT1,GAMW0,GAMZ0
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,bet
      COMMON/TOP2_NMSSM/GLVV
      GAMT=GAMT1**2*AMT**2/AMH**4
      GAMC=GAMC0**2*AMCH**2/AMH**4
      GAMW=GAMW0**2*AMW**2/AMH**4
      CH=AMCH**2/AMH**2
      W=AMW**2/AMH**2
      T=AMT**2/AMH**2
      B=AMB**2/AMH**2
      Y1=1-X2
      Y2=1-X1
      X0=2.D0-X1-X2
      W1=(1.D0-X2)
      W2=(1.D0-X0+W-CH)
      W3 = (1.D0-X0)
      W22=1.D0/(W2**2+GAMC)
      W11=1.D0/(W1**2+GAMT)
      W33=1.D0/(W3**2+GAMW)
      W12=W1*W2*W11*W22
      W13=W1*W3*W11*W33
      W23=W2*W3*W22*W33
      W4=(1.d0-X1)
      W44=1.d0/W4**2
      W14=W1*W11/W4
      W24=W2*W22/W4
      W34=W3*W33/W4

      R11= 4.d0*B**2-8.d0*B*T-16.d0*B**2*T+4.d0*T**2+32.d0*B*T**2-
     .     16.d0*T**3+4.d0*B*W+4.d0*T*W-16.d0*B*T*W-16.d0*T**2*W-
     .     8.d0*W**2+32.d0*T*W**2-8.d0*B*Y1+8.d0*T*Y1+32.d0*B*T*Y1-
     .     32.d0*T**2*Y1-16.d0*T*W*Y1+4.d0*Y1**2+4.d0*B*Y1**2-
     .     20.d0*T*Y1**2+4.d0*W*Y1**2-4.d0*Y1**3+4.d0*B*Y1*Y2 - 
     .     4.d0*T*Y1*Y2+8.d0*W*Y1*Y2-4.d0*Y1**2*Y2
      R22= 4.d0*(4.d0*W - (Y1+Y2)**2)*
     .     (T**2+B*TB**4*(-1.d0+B-W+Y1+Y2)+ 
     .     T*(-1.d0+B*(1.d0+4.d0*TB**2+TB**4)-W+Y1+Y2))/T
      R33=-4.d0*B*W+4.d0*B**2*W-4.d0*T*W-8.d0*B*T*W+4.d0*T**2*W+
     .     4.d0*B*W**2-4.d0*B**2*W**2+4.d0*T*W**2+8.d0*B*T*W**2-
     .     4.d0*T**2*W**2+4.d0*W**3-4.d0*B*W**3-4.d0*T*W**3+8.d0*W**4+
     .     4.d0*B*W*Y1-4.d0*B**2*W*Y1+4.d0*T*W*Y1+8.d0*B*T*W*Y1-
     .     4.d0*T**2*W*Y1-8.d0*W**3*Y1+B*Y1**2-B**2*Y1**2+T*Y1**2+
     .     2.d0*B*T*Y1**2-T**2*Y1**2+B*W*Y1**2-3.d0*T*W*Y1**2-B*Y1**3-
     .     T*Y1**3+4.d0*B*W*Y2-4.d0*B**2*W*Y2+4.d0*T*W*Y2+
     .     8.d0*B*T*W*Y2-4.d0*T**2*W*Y2-8.d0*W**3*Y2+2.d0*B*Y1*Y2-
     .     2.d0*B**2*Y1*Y2+2.d0*T*Y1*Y2+4.d0*B*T*Y1*Y2-2.d0*T**2*Y1*Y2-
     .     2.d0*B*W*Y1*Y2-2.d0*T*W*Y1*Y2+4.d0*W**2*Y1*Y2-
     .     3.d0*B*Y1**2*Y2-3.d0*T*Y1**2*Y2+B*Y2**2-B**2*Y2**2+T*Y2**2+
     .     2.d0*B*T*Y2**2-T**2*Y2**2-3.d0*B*W*Y2**2+T*W*Y2**2-
     .     3.d0*B*Y1*Y2**2-3.d0*T*Y1*Y2**2-B*Y2**3-T*Y2**3
      R12= 8.d0*W-8.d0*B*W-24.d0*T*W+8.d0*W**2+4.d0*B*Y1-4.d0*T*Y1-
     .     8.d0*B*T*Y1+8.d0*T**2*Y1-8.d0*B**2*TB**2*Y1+
     .     8.d0*B*T*TB**2*Y1-8.d0*W*Y1-8.d0*T*W*Y1-
     .     8.d0*B*TB**2*W*Y1-4.d0*Y1**2+12.d0*T*Y1**2+
     .     4.d0*B*TB**2*Y1**2-4.d0*W*Y1**2+4.d0*Y1**3+4.d0*B*Y2-
     .     4.d0*T*Y2-8.d0*B*T*Y2+8.d0*T**2*Y2-8.d0*B**2*TB**2*Y2+
     .     8.d0*B*T*TB**2*Y2-8.d0*W*Y2+8.d0*T*W*Y2+
     .     8.d0*B*TB**2*W*Y2-4.d0*Y1*Y2+16.d0*T*Y1*Y2-
     .     4.d0*W*Y1*Y2+8.d0*Y1**2*Y2+4.d0*T*Y2**2-
     .     4.d0*B*TB**2*Y2**2+4.d0*Y1*Y2**2 
      R13=(8.d0*W-8*B*W+16.d0*B**2*W-24.d0*T*W-32.d0*B*T*W+16.d0*T**2*W+
     .     16.d0*B*W**2+16.d0*T*W**2-32.d0*W**3+4.d0*B*Y1+8.d0*B**2*Y1-
     .     4.d0*T*Y1-16.d0*B*T*Y1+8.d0*T**2*Y1-8.d0*W*Y1-16.d0*B*W*Y1+
     .     16.d0*T*W*Y1+24.d0*W**2*Y1-4.d0*Y1**2-4.d0*B*Y1**2+
     .     12.d0*T*Y1**2+4.d0*W*Y1**2+4.d0*Y1**3+4.d0*B*Y2+8.d0*B**2*Y2-
     .     4.d0*T*Y2-16.d0*B*T*Y2+8.d0*T**2*Y2-8.d0*W*Y2+16.d0*W**2*Y2-
     .     4.d0*Y1*Y2+16.d0*T*Y1*Y2-12.d0*W*Y1*Y2+8.d0*Y1**2*Y2+
     .     4.d0*B*Y2**2+4.d0*T*Y2**2+4.d0*Y1*Y2**2)*(-1.d0) 
      R23= 4.d0*(T**2*(2.d0*W*(-2.d0+Y1+Y2)+(Y1+Y2)**2)- 
     .     B*TB**2*(2.d0*W**2*(Y1-Y2)+(Y1+Y2)**2*(-1.d0+B+Y1+Y2)+
     .     W*(-(Y1*(4.d0+Y1))+(-2.d0+Y2)**2+2.d0*B*(-2.d0+Y1+Y2)))+
     .     T*(2.d0*W**2*(-Y1+Y2)+(Y1+Y2)**2*(-1.d0+B*(-1.d0+TB**2)+
     .     Y1+Y2)+W*(4.d0+(-4.d0+Y1)*Y1-Y2*(4.d0+Y2)+
     .     2.d0*B*(-1.d0+TB**2)*(-2.d0+Y1+Y2))))

      R44= 4.d0*B**2-16.d0*B**3-8.d0*B*T+32.d0*B**2*T+4.d0*T**2-
     .     16.d0*B*T**2+4.d0*B*W-16.d0*B**2*W+4.d0*T*W-16.d0*B*T*W-
     .     8.d0*W**2+32.d0*B*W**2+8.d0*B*Y2-32.d0*B**2*Y2-8.d0*T*Y2+
     .     32.d0*B*T*Y2-16.d0*B*W*Y2-4.d0*B*Y1*Y2+4.d0*T*Y1*Y2+
     .     8.d0*W*Y1*Y2+4.d0*Y2**2-20.d0*B*Y2**2+4.d0*T*Y2**2+
     .     4.d0*W*Y2**2-4.d0*Y1*Y2**2-4.d0*Y2**3
      R14=-16.d0*B**2+32.d0*B*T-16.d0*T**2+8.d0*W-16.d0*B*W-16.d0*T*W+
     .     32.d0*W**2-16.d0*B*Y2+16.d0*T*Y2-8.d0*W*Y2-4.d0*Y2**2+
     .     16.d0*B*Y1-16.d0*T*Y1-8.d0*W*Y1+8.d0*Y2*Y1-4.d0*Y1**2
      R24=-8.d0*TB**2*W+24.d0*B*TB**2*W+8.d0*T*TB**2*W-8.d0*TB**2*W**2-
     .     8.d0*B*T*Y1+8.d0*T**2*Y1+4.d0*B*TB**2*Y1-8.d0*B**2*TB**2*Y1-
     .     4.d0*T*TB**2*Y1+8.d0*B*T*TB**2*Y1-8.d0*T*W*Y1+
     .     8.d0*TB**2*W*Y1-8.d0*B*TB**2*W*Y1+4.d0*T*Y1**2-
     .     4.d0*B*TB**2*Y1**2-8.d0*B*T*Y2+8.d0*T**2*Y2+4.d0*B*TB**2*Y2-
     .     8.d0*B**2*TB**2*Y2-4.d0*T*TB**2*Y2+8.d0*B*T*TB**2*Y2+
     .     8.d0*T*W*Y2+8.d0*TB**2*W*Y2+8.d0*B*TB**2*W*Y2+
     .     4.d0*TB**2*Y1*Y2-16.d0*B*TB**2*Y1*Y2+4.d0*TB**2*W*Y1*Y2-
     .     4.d0*TB**2*Y1**2*Y2-4.d0*T*Y2**2+4.d0*TB**2*Y2**2-
     .     12.d0*B*TB**2*Y2**2+4.d0*TB**2*W*Y2**2-8.d0*TB**2*Y1*Y2**2-
     .     4.d0*TB**2*Y2**3
      R34=-8.d0*W+24.d0*B*W-16.d0*B**2*W+8*T*W+32.d0*B*T*W-16.d0*T**2*W-
     .     16.d0*B*W**2-16.d0*T*W**2+32.d0*W**3+4.d0*B*Y1-8.d0*B**2*Y1-
     .     4.d0*T*Y1+16.d0*B*T*Y1-8.d0*T**2*Y1+8.d0*W*Y1-16.d0*W**2*Y1-
     .     4.d0*B*Y1**2-4.d0*T*Y1**2+4.d0*B*Y2-8.d0*B**2*Y2-4.d0*T*Y2+
     .     16.d0*B*T*Y2-8.d0*T**2*Y2+8.d0*W*Y2-16.d0*B*W*Y2+
     .     16.d0*T*W*Y2-24.d0*W**2*Y2+4.d0*Y1*Y2-16.d0*B*Y1*Y2+
     .     12.d0*W*Y1*Y2-4.d0*Y1**2*Y2+4.d0*Y2**2-12.d0*B*Y2**2+
     .     4.d0*T*Y2**2-4.d0*W*Y2**2-8.d0*Y1*Y2**2-4.d0*Y2**3

c - take off the charged H+ and W+ exchange -
      W22 = 0.D0
      W33 = 0.D0
      W12 = 0.D0
      W23 = 0.D0
      W13 = 0.D0
      W24 = 0.D0
      W34 = 0.D0
c -------------------------------------------

      RES=GHT**2*R11*W11+GLVV**2*GAT**2*R22*W22+
     .     4.D0*GHVV**2*R33*W33/T+2.D0*GHT*GLVV*GAT*R12*W12+
     .     2.D0*GHT*GHVV*R13*W13+2.D0*GHVV*GLVV*GAT*R23*W23/T+
     .     GHB**2*R44*W44*B/T+2.D0*B*GHB*GHT*R14*W14+
     .     2.D0*GHB*GLVV*GAT*R24*W24*B/T+
     .     2.d0*GHB*GHVV*R34*W34*B/T
      RETURN
      END

c -------------------------------------------------------------------- c

      subroutine htohhoffshell_nmssm(iicase,amhi,amhj,amhk,gamtotj,
     .     gamtotk,htohhoff,errhtohhoff)

      implicit double precision (a-h,o-z)
      integer ivegas(4)
      common/DECPARAMETERS/ijcase,amhi1,amhj1,amhk1,gamtotj1,gamtotk1

      external hitohjhk_nmssm

      amhi1 = amhi
      amhj1 = amhj
      amhk1 = amhk
      gamtotj1 = gamtotj
      gamtotk1 = gamtotk
      ijcase= iicase

c ---- VEGAS parameters ----

      iprint = 0 

cc      ivegas(1) = 300000
cc      ivegas(2) = 5
cc      ivegas(3) = 12000000
cc      ivegas(4) = 10

      if(iicase.eq.0) then
         ivegas(1) = 50000
         ivegas(2) = 5
         ivegas(3) = 40000
         ivegas(4) = 10
      else
         ivegas(1) = 300000
         ivegas(2) = 5
         ivegas(3) = 12000000
         ivegas(4) = 10
      endif

c ---- initialization of VEGAS ----

      call RSTART(12,34,56,78)

c ---- the off-shell decay Hi->Hj Hk ----
         
      call INTEG(hitohjhk_nmssm,2,iprint,ivegas,result,relative)

      htohhoff = result

      return

      end

c -------------------------------------------------------------------- c

      double precision function hitohjhk_nmssm(dum)

      implicit double precision (a-h,o-z)
      double precision dum(2),lamb

      common/DECPARAMETERS/ijcase,amhi,amhj,amhk,gamtotj,gamtotk

      lamb(x,y)=dsqrt((1.d0-x-y)**2-4.d0*x*y)
      pi=4d0*datan(1d0)

c      icase = 0

      if(dabs(amhi-amhj).le.5.D0.or.dabs(amhi-amhk).le.5.D0) then
         eps=1.D-02
      else
         eps=0.D0
      endif

      if(ijcase.eq.0)then
       yy = amhi**2-eps
       y1 = datan((yy-amhj**2)/amhj/gamtotj)
       y2 = -datan((amhj**2)/amhj/gamtotj)
       djac = y1-y2
       t1 = tan(y1*dum(1)+y2*(1.d0-dum(1)))
       sp = amhj**2 + amhj*gamtotj*t1
c       print*,'val1',sp/amhi**2
       yy = (amhi-dsqrt(sp))**2-eps
       y1 = datan((yy-amhk**2)/amhk/gamtotk)
       y2 = -datan((amhk**2)/amhk/gamtotk)
       djac = djac*(y1-y2)
       t2 = tan(y1*dum(2)+y2*(1.d0-dum(2)))
       sm = amhk**2 + amhk*gamtotk*t2
c       print*,'val2',sm/amhi**2
       am2=amhi**2
       gam = lamb(sp/am2,sm/am2)
c       print*,'val3',gam
       pro1 = sp
       pro2 = sm
       hitohjhk_nmssm = pro1*pro2*gam*djac/pi**2/amhj**2/amhk**2
c       print*,'val4',hitohjhk_nmssm
      
       else

c ---- the integration limits ----
c --- the x1 integration ---------
      x1m = 0.D0
      x1p = amhi**2
      x1 = (x1p-x1m)*dum(1)+x1m 
      sp = x1
c --- the x2 integration ---------
      x2m = 0.D0
      x2p = (amhi-dsqrt(x1))**2
      x2 = (x2p-x2m)*dum(2)+x2m
      sm = x2

      djac = amhi**2*(amhi-dsqrt(sp))**2/pi**2

      am2=amhi**2
      gam = lamb(sp/am2,sm/am2)
      pro1 = sp*gamtotj*amhj/((sp-amhj**2)**2+amhj**2*gamtotj**2)
      pro2 = sm*gamtotk*amhk/((sm-amhk**2)**2+amhk**2*gamtotk**2)
      hitohjhk_nmssm = pro1*pro2*gam*djac/amhj**2/amhk**2
c      print*,'vals',pro1,pro2,sp,sm,gamtotj,gamtotk
c      print*,'vals',pro1,pro2,gamtotj,gamtotk

      endif

      return

      end

c -------------------------------------------------------------------- c
C     ************************************************************
C     SUBROUTINE FOR H+ ---> BT* ---> BBW
C     ************************************************************
      subroutine ctott_hdec(amch,amt,amb,amw,tanb,ctt0)
      implicit real*8(a-z)
      integer ip,k
      common/prec1_hdec/ip
      external functt1_hdec
      common/iksy3_hdec/x1,x2,m1,m2,m3,ecm,s
      common/top3_hdec/amh3,amt3,amb3,amw3
      common/top5_hdec/tanb3
      amh3=amch
      amt3=amt
      amb3=amb
      amw3=amw
      tanb3=tanb
      ip=5
      m1=amb
      m2=amb
      m3=amw
c     first integrate over x2, i.e. (1+3) system
c        check whether enough phase space
      mastot=m1+m2+m3
      if(mastot.ge.amch) goto 12
      ecm=amch
      s=ecm**2
      u1=(ecm-m2)**2
      d1=(m1+m3)**2
      u=(s-d1+m2**2)/s
      d=(s-u1+m2**2)/s
      del=(u-d)/ip
      u=d+del
      xsec=0.d0
      do k=1,ip
      call qgaus1_hdec(functt1_hdec,d,u,ss)
      d=u
      u=d+del
      xsec=xsec+ss
      enddo
      ctt0=xsec
12    continue
      return
      end

c -------------------------------------------------------------------- c

      double precision function functt1_hdec(xl)
      implicit real*8(a-z)
      integer ip,i
      common/iksy3_hdec/x1,x2,m1,m2,m3,ecm,s
      common/prec1_hdec/ip
      external functt2_hdec
      x2=xl
      s13=s-s*x2+m2**2
      tem=2.d0*dsqrt(s13)
      e2s=(s-s13-m2**2)/tem
      e3s=(s13+m3**2-m1**2)/tem
c     second integral over x1, i.e. (2+3) system
      u1=(e2s+e3s)**2-(dsqrt(e2s**2-m2**2)-dsqrt(e3s**2-m3**2))**2
      d1=(e2s+e3s)**2-(dsqrt(e2s**2-m2**2)+dsqrt(e3s**2-m3**2))**2
      u=(s-d1+m1**2)/s
      d=(s-u1+m1**2)/s
      del=(u-d)/ip
      functt1_hdec=0.d0
      u=d+del
      do i=1,ip
      call qgaus2_hdec(functt2_hdec,d,u,ss)
      functt1_hdec=functt1_hdec+ss
      d=u
      u=d+del
      enddo
      return
      end

c -------------------------------------------------------------------- c

      double precision function functt2_hdec(xk)
      implicit real*8(a-z)
      common/iksy3_hdec/x1,x2,m1,m2,m3,ecm,s
      x1=xk
      call elemctt_hdec(ss)
      functt2_hdec=ss
      return
      end

c -------------------------------------------------------------------- c

      subroutine elemctt_hdec(res)

      implicit real*8(a-z)
      common/iksy3_hdec/x1,x2,m1,m2,m3,ecm,s
      common/top3_hdec/amch,amt,amb,amw
      common/wzwdth_hdec/gamc0,gamt0,gamt1,gamw,gamz
      common/top5_hdec/tanb
      gamt=gamt1**2*amt**2/amch**4
      w=amw**2/amch**2
      t=amt**2/amch**2
      b=amb**2/amch**2

c ---- with b-quark mass dependence ----
      res=(-2.d0*b*t*tanb**2*(1.d0+w+2.d0*b*w-2.d0*w**2-
     .     (2.d0+w)*x2+x2**2)+t**2*(2.d0*w**2+(-1.d0+x1)*(-1.d0+x2)+
     .     w*(-3.d0-2.d0*b+2.d0*x1+2.d0*x2))+b*tanb**4*(-2.d0*b**2*w-
     .     2.d0*w**2+(-1.d0+x2)**2*(-1.d0+x1+x2)+w*(-1.d0+x2)*
     .     (-3.d0+2.d0*x1+x2)+b*(2.d0*w**2+w*(3.d0-2.d0*x1)-
     .     (-1.d0+x2)*(-3.d0+x1+2.d0*x2))))/((1.d0-x2+b-t)**2+gamt)/t**2

      return
      end

c -------------------------------------------------------------------- c
c   *****************  integration routine ***********************
c    returns ss as integral of func from a to b, by 10-point gauss-
c    legendre integration
      subroutine qgaus1_hdec(func,a,b,ss)
      implicit real*8(a-z)
      integer j
      dimension x(5),w(5)
      external func
      data x/.1488743389d0,.4333953941d0,.6794095682d0
     .  ,.8650633666d0,.9739065285d0/
      data w/.2955242247d0,.2692667193d0,.2190863625d0
     .  ,.1494513491d0,.0666713443d0/
      xm=0.5d0*(b+a)
      xr=0.5d0*(b-a)
      ss=0.d0
      do 11 j=1,5
        dx=xr*x(j)
        ss=ss+w(j)*(func(xm+dx)+func(xm-dx))
11    continue
      ss=xr*ss
      return
      end

c -------------------------------------------------------------------- c
C -- Returns SS as integral of FUNC from A to B, by 10-point Gauss-
C -- Legendre integration

      subroutine qgaus2_hdec(func,a,b,ss)
      implicit real*8(a-z)
      integer j
      dimension x(5),w(5)
      external func
      data x/.1488743389d0,.4333953941d0,.6794095682d0
     .  ,.8650633666d0,.9739065285d0/
      data w/.2955242247d0,.2692667193d0,.2190863625d0
     .  ,.1494513491d0,.0666713443d0/
      xm=0.5d0*(b+a)
      xr=0.5d0*(b-a)
      ss=0.d0
      do 11 j=1,5
        dx=xr*x(j)
        ss=ss+w(j)*(func(xm+dx)+func(xm-dx))
11    continue
      ss=xr*ss
      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE AMHAMA_NMSSM_HDEC(ICASE,MH,TANB)
C--CALCULATION OF PSEUDOSCALAR HIGGS MASS FROM HIGGS MASS MH
C--ICASE=0: MH=PSEUDOSCALAR MASS
c maggie changed 4/9/2011
c--icase=1: mh=light scalar mass h1
c--icase=2: mh=heavy scalar mass h2
c--icase=3: mh=heavy scalar mass h3
c--icase=4: mh=charged Higgs mass
c end maggie changed 4/9/2011
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IMPLICIT REAL*8(A-H,L,M,O-Z)
      DIMENSION VH(2,2),M2(2,2),M2P(2,2)
c maggie changed 4/9/2011
c      COMMON/HMASS_HDEC/AMSM,AMA,AML,AMH,AMCH,AMAR
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
c end maggie changed 4/9/2011
      IF(ICASE.EQ.0)THEN
       MA = MH
      ELSE
       DEL0 = 1.D-4
       MA0 = 1.D0
       MA1 = 1.D4
1      MA = (MA0+MA1)/2
       AMA = MA
c maggie changed 4/9/2011
c       CALL SUSYCP_HDEC(TANB)
       CALL SUSYCP_NMSSM(TANB)
       if(icase.eq.1)then
        mx = amh1
       elseif(icase.eq.2)then
        mx = amh2
       elseif(icase.eq.3)then
        mx = amh3
       elseif(icase.eq.4)then
        mx = amch
       endif
c end maggie changed 4/9/2011
       DEL = DABS(MA1 - MA0)/MA
       IF(DEL.GT.DEL0) THEN
        IF(MX.GT.MH) MA1 = MA
        IF(MX.LT.MH) MA0 = MA
        GOTO 1
       ENDIF
       FAC = 1
       MAX = DINT(FAC*MA+0.5D0)/FAC
       AMA = MAX
c maggie changed 4/9/2011
c       CALL SUSYCP_HDEC(TANB)
       CALL SUSYCP_NMSSM(TANB)
       if(icase.eq.1)then
        mx = amh1
       elseif(icase.eq.2)then
        mx = amh2
       elseif(icase.eq.3)then
        mx = amh3
       elseif(icase.eq.4)then
        mx = amch
       endif
c end maggie changed 4/9/2011
       IF(MX.EQ.MH)THEN
        MA = MAX
       ELSE
        DEL0 = 1.D-8
2       MA = (MA0+MA1)/2
        AMA = MA
c maggie changed 4/9/2011
c        CALL SUSYCP_HDEC(TANB)
        CALL SUSYCP_NMSSM(TANB)
        if(icase.eq.1)then
         mx = amh1
        elseif(icase.eq.2)then
         mx = amh2
        elseif(icase.eq.3)then
         mx = amh3
        elseif(icase.eq.4)then
         mx = amch
        endif
c end maggie changed 4/9/2011
        DEL = DABS(MA1 - MA0)/MA
        IF(DEL.GT.DEL0) THEN
         IF(MX.GT.MH) MA1 = MA
         IF(MX.LT.MH) MA0 = MA
         GOTO 2
        ENDIF
       ENDIF
      ENDIF
      AMA = MA
c maggie changed 4/9/2011
c      CALL SUSYCP_HDEC(TANB)
      CALL SUSYCP_NMSSM(TANB)
c end maggie changed 4/9/2011
      RETURN
      END

c ======================================================================
c	  chargino and neutralino mass matrices and couplings
c ======================================================================
      subroutine gaugino_nmssm(mu,m2,b,mc,mn,xmn,ach1,ach2,ach3,aca1,
     .     aca2,anh1,anh2,anh3,ana1,ana2,acnl,acnr)            

      implicit real*8(a-h,k-z)
      double precision mc(2),mn(5)
      double precision achig(5,2,2),anhig(5,5,5)
      dimension xmn(5),z(5,5),zx(5,5),u(2,2),v(2,2)
      dimension ymn(5),yz(5,5)
      dimension slhaneut(5),slhaxneut(5),slhachar(2),slhau(2,2),
     .          slhav(2,2),slhaz(5,5),slhaxchar(2)
      dimension q(2,2),s(2,2),r(2,2),qq(5,5),ss(5,5),rr(5,5),qqa(5,5),
     .     ssa(5,5)
      dimension ach1(2,2),ach2(2,2),ach3(2,2),anh1(5,5),anh2(5,5),
     .     anh3(5,5),acnl(2,5),acnr(2,5),aca1(2,2),aca2(2,2),ana1(5,5),
     .     ana2(5,5)
      dimension slhahmix(3,3),slhaamix(2,2)

      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,mz,mw
      COMMON/GAUGINOMIX_NMSSM/zx,u,v
      COMMON/SLHA_M1_HDEC/am1
      COMMON/SLHA_GAUG_NMSSM/slhaneut,slhaxneut,slhachar,slhau,slhav,
     .                      slhaz,slhaxchar
      COMMON/NMSSMFLAG/inmssm,inmssmhi
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/VEV_NMSSM/vewsb
      COMMON/SHORTCOUPL3_NMSSM/achig,anhig

      cw=mw/mz
      sw=dsqrt(1-cw**2)
      pi=4.d0*datan(1.d0)
      sb=dsin(b)
      cb=dcos(b)
      tw=sw/cw
c      if(islhai.eq.0) then
c         m1=5.d0/3.d0*tw**2*m2
c      else
         m1 = am1
c      endif

c ================ neutralino masses and matrix elements ===============

c ======================= masses and couplings =========================

      do i=1,5,1
         xmn(i) = slhaxneut(i)
         mn(i)  = dabs(xmn(i))
         ymn(i) = xmn(i)
         do j=1,5,1
            zx(i,j) = slhaz(i,j)
            z(i,j) = zx(i,j)
         end do
         yz(i,1)=zx(i,1)
         yz(i,2)=zx(i,2)
         yz(i,3)=zx(i,3)
         yz(i,4)=zx(i,4)
         yz(i,5)=zx(i,5)
      end do

c ===============  neutralino couplings to Higgs bosons ================

      do i=1,5,1
         do j=1,5,1
            qq(i,j)=1.D0/2.D0*(z(i,3)*(z(j,2)-tw*z(j,1))+z(j,3)*
     .           (z(i,2)-tw*z(i,1)))
     .           -dsqrt(2.D0)*xlambdapar*vewsb/4.D0/mw*
     .           (z(i,5)*z(j,4)+z(j,5)*z(i,4))
            ss(i,j)=1.D0/2.D0*(z(i,4)*(z(j,2)-tw*z(j,1))+z(j,4)*
     .           (z(i,2)-tw*z(i,1)))
     .           +dsqrt(2.D0)*xlambdapar*vewsb/4.D0/mw*
     .           (z(i,5)*z(j,3)+z(j,5)*z(i,3))
            rr(i,j)=dsqrt(2.D0)/4.D0/mw*vewsb*
     .           (xlambdapar*(z(i,4)*z(j,3)+z(j,4)*z(i,3))
     .           -2.D0*xkappa*Z(i,5)*z(j,5))

            qqa(i,j)=1.D0/2.D0*(z(i,3)*(z(j,2)-tw*z(j,1))+z(j,3)*
     .           (z(i,2)-tw*z(i,1)))
     .           +dsqrt(2.D0)*xlambdapar*vewsb/4.D0/mw*
     .           (z(i,5)*z(j,4)+z(j,5)*z(i,4))
            ssa(i,j)=1.D0/2.D0*(z(i,4)*(z(j,2)-tw*z(j,1))+z(j,4)*
     .           (z(i,2)-tw*z(i,1)))
     .           -dsqrt(2.D0)*xlambdapar*vewsb/4.D0/mw*
     .           (z(i,5)*z(j,3)+z(j,5)*z(i,3))
         end do
      end do

      do i=1,5,1
         do j=1,5,1
            anh1(i,j)=slhahmix(1,1)*qq(i,j)-slhahmix(1,2)*ss(i,j)
     .           -slhahmix(1,3)*rr(i,j)
            anh2(i,j)=slhahmix(2,1)*qq(i,j)-slhahmix(2,2)*ss(i,j)
     .           -slhahmix(2,3)*rr(i,j)
            anh3(i,j)=slhahmix(3,1)*qq(i,j)-slhahmix(3,2)*ss(i,j)
     .           -slhahmix(3,3)*rr(i,j)
            ana1(i,j)=qqa(i,j)*dsin(b)*slhaamix(1,1)
     .           -ssa(i,j)*dcos(b)*slhaamix(1,1)
     .           +rr(i,j)*slhaamix(1,2)
            ana2(i,j)=qqa(i,j)*dsin(b)*slhaamix(2,1)
     .           -ssa(i,j)*dcos(b)*slhaamix(2,1)
     .           +rr(i,j)*slhaamix(2,2)            
         end do
      end do

      do i=1,5,1
         do j=1,5,1
            anhig(1,i,j) = anh1(i,j)
            anhig(2,i,j) = anh2(i,j)
            anhig(3,i,j) = anh3(i,j)
            anhig(4,i,j) = ana1(i,j)
            anhig(5,i,j) = ana2(i,j)
         end do
      end do

c ================ chargino masses and matrix elements =============== c

      mc(1) = slhachar(1)
      mc(2) = slhachar(2)
      do i=1,2,1
         do j=1,2,1
            u(i,j) = slhau(i,j)
            v(i,j) = slhav(i,j)
         end do
      end do

c ================= chargino couplings to Higgs bosons =================

      do i=1,2,1
         do j=1,2,1
            q(i,j)=dsqrt(1.d0/2.d0)*u(j,2)*v(i,1)
            s(i,j)=dsqrt(1.d0/2.d0)*u(j,1)*v(i,2)
            r(i,j)=dsqrt(1.d0/2.d0)*u(j,2)*v(i,2)*
     .           xlambdapar*vewsb/2.D0/mw
         end do
      end do

      do i=1,2,1
         do j=1,2,1
            ach1(i,j)=q(i,j)*slhahmix(1,1)+s(i,j)*slhahmix(1,2)+
     .           r(i,j)*slhahmix(1,3)
            ach2(i,j)=q(i,j)*slhahmix(2,1)+s(i,j)*slhahmix(2,2)+
     .           r(i,j)*slhahmix(2,3)
            ach3(i,j)=q(i,j)*slhahmix(3,1)+s(i,j)*slhahmix(3,2)+
     .           r(i,j)*slhahmix(3,3)
            aca1(i,j)=-(q(i,j)*dsin(b)*slhaamix(1,1)
     .           +s(i,j)*dcos(b)*slhaamix(1,1)
     .           -r(i,j)*slhaamix(1,2))
            aca2(i,j)=-(q(i,j)*dsin(b)*slhaamix(2,1)
     .           +s(i,j)*dcos(b)*slhaamix(2,1)
     .           -r(i,j)*slhaamix(2,2))
         end do
      end do

      do i=1,2,1
         do j=1,2,1
            achig(1,i,j) = ach1(i,j)
            achig(2,i,j) = ach2(i,j)
            achig(3,i,j) = ach3(i,j)
            achig(4,i,j) = aca1(i,j)
            achig(5,i,j) = aca2(i,j)
         end do
      end do

c ======= chargino-neutralino couplings to charged Higgs bosons ====== c
	do  i=1,2
	do  j=1,5
        acnl(i,j)=dcos(b)*(z(j,4)*v(i,1)+(z(j,2)+z(j,1)*tw)
     .       *v(i,2)/dsqrt(2.d0)) 
     .       + xlambdapar*dsin(b)*vewsb/2.D0/mw*z(j,5)*v(i,2) 
        acnr(i,j)=dsin(b)*(z(j,3)*u(i,1)-(z(j,2)+z(j,1)*tw)
     .       *u(i,2)/dsqrt(2.d0)) 
     .       + xlambdapar*dcos(b)*vewsb/2.D0/mw*z(j,5)*u(i,2)

         end do
      end do

       return
       end

c ==================================================================== c
c         subroutine for sfermion masses, mixing and couplings 
c ==================================================================== c

      subroutine sfermion_nmssm(mst,msb,msl,msu,msd,mse,msn,msnl,
     .     gh1ee,gh1tt,gh1bb,gh1elel,gh1nene,gh1uu,gh1dd,gh2ee,gh2tt,
     .     gh2bb,gh2elel,gh2nene,gh2uu,gh2dd,gh3ee,gh3tt,gh3bb,gh3elel,
     .     gh3nene,gh3uu,gh3dd,gha1ee,gha1tt,gha1bb,gha2ee,gha2tt,
     .     gha2bb,ghcen,ghctb,ghcelne,ghcud,al,at,ab,rmt,rmb)

      implicit real*8(a-h,k-z)
      double precision ghigtt(5,2,2),ghigbb(5,2,2),ghigee(5,2,2),
     .     ghiguu(3,2,2),ghigdd(3,2,2),ghigelel(3,2,2),ghignene(3,2,2),
     .     yhitt(3,2,2),yhaitt(2),yhibb(3,2,2),yhaibb(2)
      dimension mst(2),msb(2),msl(2),msu(2),msd(2),mse(2),msn(18),
     .     msnl(2),
     .     ghcen(2,2),ghctb(2,2),ghcelne(2,2),ghcud(2,2),gh1elel(2,2),
     .     gh1nene(2,2),gh1uu(2,2),gh1dd(2,2),gh1ee(2,2),gh1tt(2,2),
     .     gh1bb(2,2),gh2elel(2,2),gh2nene(2,2),gh2uu(2,2),gh2dd(2,2),
     .     gh2ee(2,2),gh2tt(2,2),gh2bb(2,2),gh3elel(2,2),gh3nene(2,2),
     .     gh3uu(2,2),gh3dd(2,2),gh3ee(2,2),gh3tt(3,3),gh3bb(3,3)
      dimension slhast(2),slhasb(2),slhasu(2),slhasd(2),slhase(2),
     .          slhasl(2),slhasn(18),slhasnl(2)
      dimension slhahmix(3,3),slhaamix(2,2)

      COMMON/GLUINO_NMSSM/amgluino,xmsb1,xmsb2,sthb,cthb,
     .     xh1bb(2,2),xh2bb(2,2),xh3bb(2,2),xha1bb(2,2),xha2bb(2,2),
     .     xmst1,xmst2,stht,ctht,xh1tt(2,2),xh2tt(2,2),xh3tt(2,2),
     .     xha1tt(2,2),xha2tt(2,2)
      COMMON/TAUMIX_HDEC/CL,SL
      COMMON/SLHA_sfer_HDEC/slhast,slhasb,slhasu,slhasd,slhase,slhasl,
     .                 slhasn,slhasnl,slhacot,slhasit,slhacob,slhasib,
     .                 slhacol,slhasil
      COMMON/SQUARKHIGGS_HDEC/THEB,AMG,JONSH,JDTH
      COMMON/SQNLO_NMSSM/ymsb(2),ysthb,ycthb,yh1bb(2,2),yh2bb(2,2),
     .     yh3bb(2,2),yha1bb,yha2bb,ymst(2),ystht,yctht,yh1tt(2,2),
     .     yh2tt(2,2),yh3tt(2,2),yha1tt,yha2tt
      COMMON/BREAKSCALE_HDEC/SUSYSCALE
      COMMON/SFMIXANG_NMSSM/sdthet,sdtheb,sdthel,cum,sum,cdm,sdm,cem,sem
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/VEV_NMSSM/vewsb
      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
      COMMON/MASSES_HDEC/AMS,AMC,AMB,AMT
      COMMON/SHORTCOUPL1_NMSSM/ghigtt,ghigbb,ghigee,ghiguu,ghigdd,
     .     ghigelel,ghignene
      COMMON/SHORTCOUPL4_NMSSM/yhitt,yhaitt,yhibb,yhaibb

      tb=dtan(b)

      amg = amgluino
      v = vewsb
      sw2 = 1.D0-amw**2/amz**2

      mt = rmt
      mb = rmb
      ml = amtau

      mw = amw
      mz = amz

c -- sfermion masses and mixing angles --

      do i=1,2,1
         msu(i) = slhasu(i)
         msd(i) = slhasd(i)
         mse(i) = slhase(i)
         msnl(i) = slhasnl(i)
         mst(i) = slhast(i)
         msb(i) = slhasb(i)
         msl(i) = slhasl(i)
      end do
         msn(1:18) = slhasn(1:18)
      ct = slhacot
      st = slhasit
      cb = slhacob
      sb = slhasib
      cl = slhacol
      sl = slhasil
      
      theb = sdtheb
      thet = sdthet
      thel = sdthel

c --- the NMSSM Higgs sfermion sfermion couplings --- c

c CP-even Higgs couplings to stops

      gh1tt(1,1)=(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (0.5d0*ct**2-2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(1,2)/dsin(b) 
     .     + mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(1,1)
     .             +at*slhahmix(1,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(1,3))

      gh1tt(2,2)=(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (0.5d0*st**2+2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(1,2)/dsin(b) 
     .     - mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(1,1)
     .             +at*slhahmix(1,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(1,3))
      gh1tt(1,2)=-2*
     .     (-1.D0)*(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(1,1)
     .     +at*slhahmix(1,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(1,3))
      gh1tt(2,1)=-2*
     .     (-1.D0)*(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(1,1)
     .     +at*slhahmix(1,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(1,3))

      gh2tt(1,1)=(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (0.5d0*ct**2-2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(2,2)/dsin(b) 
     .     + mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(2,1)
     .             +at*slhahmix(2,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(2,3))
      gh2tt(2,2)=(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (0.5d0*st**2+2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(2,2)/dsin(b) 
     .     - mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(2,1)
     .             +at*slhahmix(2,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(2,3))
      gh2tt(1,2)=-2*
     .     (-1.D0)*(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(2,1)
     .     +at*slhahmix(2,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(2,3))
      gh2tt(2,1)=-2*
     .     (-1.D0)*(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(2,1)
     .     +at*slhahmix(2,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(2,3))

      gh3tt(1,1)=(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (0.5d0*ct**2-2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(3,2)/dsin(b) 
     .     + mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(3,1)
     .             +at*slhahmix(3,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(3,3))
      gh3tt(2,2)=(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (0.5d0*st**2+2.d0/3.d0*sw2*dcos(2*thet)) 
     .     + mt**2/mz**2*slhahmix(3,2)/dsin(b) 
     .     - mt*st*ct/mz**2/dsin(b)*(-xlamvs*slhahmix(3,1)
     .             +at*slhahmix(3,2)
     .             -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(3,3))
      gh3tt(1,2)=-2*
     .     (-1.D0)*(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(3,1)
     .     +at*slhahmix(3,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(3,3))
      gh3tt(2,1)=-2*
     .     (-1.D0)*(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     st*ct*(2.d0/3.d0*sw2-0.25d0)
     .    + mt*dcos(2*thet)/2.d0/mz**2/dsin(b)*
     .     (-xlamvs*slhahmix(3,1)
     .     +at*slhahmix(3,2)
     .     -xlambdapar*v*dcos(b)/dsqrt(2.D0)*slhahmix(3,3))

c CP-even Higgs couplings to sbottoms

      gh1bb(1,1) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0*cb**2+1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(1,1)
     .     + mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3))
      gh1bb(2,2) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0*sb**2-1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(1,1)
     .     - mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3))
      gh1bb(1,2) = -2*(-1.D0)*
     .     (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3)) 
      gh1bb(2,1) = -2*(-1.D0)*
     .     (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3)) 

      gh2bb(1,1) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0*cb**2+1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(2,1)
     .     + mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3))
      gh2bb(2,2) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0*sb**2-1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(2,1)
     .     - mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3))
      gh2bb(1,2) = -2*(-1.D0)*
     .     (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3)) 
      gh2bb(2,1) = -2*(-1.D0)*
     .     (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3)) 

      gh3bb(1,1) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0*cb**2+1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(3,1)
     .     + mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3))
      gh3bb(2,2) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0*sb**2-1.D0/3.D0*sw2*dcos(2*theb)) 
     .     + mb**2/mz**2/dcos(b)*slhahmix(3,1)
     .     - mb*sb*cb/mz**2/dcos(b)*(ab*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3))
      gh3bb(1,2) = -2*(-1.D0)*
     .     (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3)) 
      gh3bb(2,1) = -2*(-1.D0)*
     .     (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     sb*cb*(-1.D0/3.D0*sw2+0.25D0)
     .     + mb*dcos(2*theb)/2.D0/mz**2/dcos(b)*(ab*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3)) 

c CP-even Higgs couplings to staus

      gh1ee(1,1) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0*cl**2+sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(1,1)
     .     + ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3))

      gh1ee(2,2) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0*sl**2-sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(1,1)
     .     - ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3))
      gh1ee(1,2) = -2*(-1.D0)*
     .     (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3)) 
      gh1ee(2,1) = -2*(-1.D0)*
     .     (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(1,1)
     .     -xlamvs*slhahmix(1,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(1,3)) 

      gh2ee(1,1) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0*cl**2+sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(2,1)
     .     + ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3))
      gh2ee(2,2) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0*sl**2-sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(2,1)
     .     - ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3))
      gh2ee(1,2) = -2*(-1.D0)*
     .     (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3)) 
      gh2ee(2,1) = -2*(-1.D0)*
     .     (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(2,1)
     .     -xlamvs*slhahmix(2,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(2,3)) 

      gh3ee(1,1) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0*cl**2+sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(3,1)
     .     + ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3))
      gh3ee(2,2) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0*sl**2-sw2*dcos(2*thel)) 
     .     + ml**2/mz**2/dcos(b)*slhahmix(3,1)
     .     - ml*sl*cl/mz**2/dcos(b)*(al*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3))
      gh3ee(1,2) = -2*(-1.D0)*
     .     (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3)) 
      gh3ee(2,1) = -2*(-1.D0)*
     .     (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     sl*cl*(-sw2+0.25D0)
     .     + ml*dcos(2*thel)/2.D0/mz**2/dcos(b)*(al*slhahmix(3,1)
     .     -xlamvs*slhahmix(3,2)
     .     -xlambdapar*v/dsqrt(2.D0)*dsin(b)*slhahmix(3,3)) 

c CP-even Higgs couplings to sups

      gh1uu(1,1)=(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (0.5d0-2.d0/3.d0*sw2) 
      gh1uu(2,2)=(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (2.d0/3.d0*sw2) 
      gh1uu(1,2)=0.D0
      gh1uu(2,1)=0.D0

      gh2uu(1,1)=(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (0.5d0-2.d0/3.d0*sw2) 
      gh2uu(2,2)=(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (2.d0/3.d0*sw2) 
      gh2uu(1,2)=0.D0
      gh2uu(2,1)=0.D0

      gh3uu(1,1)=(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (0.5d0-2.d0/3.d0*sw2) 
      gh3uu(2,2)=(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (2.d0/3.d0*sw2) 
      gh3uu(1,2)=0.D0
      gh3uu(2,1)=0.D0

c CP-even Higgs couplings to sdowns

      gh1dd(1,1) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0+1.D0/3.D0*sw2) 
      gh1dd(2,2) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-1.D0/3.D0*sw2) 
      gh1dd(1,2) = 0.D0
      gh1dd(2,1) = 0.D0

      gh2dd(1,1) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0+1.D0/3.D0*sw2) 
      gh2dd(2,2) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-1.D0/3.D0*sw2) 
      gh2dd(1,2) = 0.D0
      gh2dd(2,1) = 0.D0

      gh3dd(1,1) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0+1.D0/3.D0*sw2) 
      gh3dd(2,2) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-1.D0/3.D0*sw2) 
      gh3dd(1,2) = 0.D0
      gh3dd(2,1) = 0.D0

c CP-even Higgs couplings to selectrons

      gh1elel(1,1) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-0.5D0+sw2)
      gh1elel(2,2) = (slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (-sw2)
      gh1elel(1,2) = 0.D0
      gh1elel(2,1) = 0.D0

      gh2elel(1,1) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-0.5D0+sw2)
      gh2elel(2,2) = (slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (-sw2)
      gh2elel(1,2) = 0.D0
      gh2elel(2,1) = 0.D0

      gh3elel(1,1) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-0.5D0+sw2)
      gh3elel(2,2) = (slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (-sw2)
      gh3elel(1,2) = 0.D0
      gh3elel(2,1) = 0.D0

c CP-even Higgs couplings to sneutrinos

      gh1nene(1,1)=(slhahmix(1,1)*dcos(b)-slhahmix(1,2)*dsin(b))*
     .     (0.5d0)
      gh1nene(2,2)=0.D0
      gh1nene(1,2)=0.D0
      gh1nene(2,1)=0.D0

      gh2nene(1,1)=(slhahmix(2,1)*dcos(b)-slhahmix(2,2)*dsin(b))*
     .     (0.5d0)
      gh2nene(2,2)=0.D0
      gh2nene(1,2)=0.D0
      gh2nene(2,1)=0.D0

      gh3nene(1,1)=(slhahmix(3,1)*dcos(b)-slhahmix(3,2)*dsin(b))*
     .     (0.5d0)
      gh3nene(2,2)=0.D0
      gh3nene(1,2)=0.D0
      gh3nene(2,1)=0.D0

c pseudoscalar couplings

      gha1tt = mt/2.D0/mz**2*(xlamvs*slhaamix(1,1)
     .     +at/dtan(b)*slhaamix(1,1)
     .     +xlambdapar*v/dsqrt(2.D0)/dtan(b)*slhaamix(1,2))

      gha2tt = mt/2.D0/mz**2*(xlamvs*slhaamix(2,1)
     .     +at/dtan(b)*slhaamix(2,1)
     .     +xlambdapar*v/dsqrt(2.D0)/dtan(b)*slhaamix(2,2))

      gha1bb = mb/2.d0/mz**2*(xlamvs*slhaamix(1,1)
     .     +ab*dtan(b)*slhaamix(1,1)
     .     +xlambdapar*v/dsqrt(2.D0)*dtan(b)*slhaamix(1,2)) 

      gha2bb = mb/2.d0/mz**2*(xlamvs*slhaamix(2,1)
     .     +ab*dtan(b)*slhaamix(2,1)
     .     +xlambdapar*v/dsqrt(2.D0)*dtan(b)*slhaamix(2,2))

      gha1ee = ml/2.d0/mz**2*(xlamvs*slhaamix(1,1)
     .     +al*dtan(b)*slhaamix(1,1)
     .     +xlambdapar*v/dsqrt(2.D0)*dtan(b)*slhaamix(1,2)) 

      gha2ee = ml/2.d0/mz**2*(xlamvs*slhaamix(2,1)
     .     +al*dtan(b)*slhaamix(2,1)
     .     +xlambdapar*v/dsqrt(2.D0)*dtan(b)*slhaamix(2,2)) 
      do i=1,2,1
         do j=1,2,1
            ghigtt(1,i,j) = gh1tt(i,j)
            ghigtt(2,i,j) = gh2tt(i,j)
            ghigtt(3,i,j) = gh3tt(i,j)
            ghigbb(1,i,j) = gh1bb(i,j)
            ghigbb(2,i,j) = gh2bb(i,j)
            ghigbb(3,i,j) = gh3bb(i,j)
            ghigee(1,i,j) = gh1ee(i,j)
            ghigee(2,i,j) = gh2ee(i,j)
            ghigee(3,i,j) = gh3ee(i,j)

            ghiguu(1,i,j) = gh1uu(i,j)
            ghiguu(2,i,j) = gh2uu(i,j)
            ghiguu(3,i,j) = gh3uu(i,j)
            ghigdd(1,i,j) = gh1dd(i,j)
            ghigdd(2,i,j) = gh2dd(i,j)
            ghigdd(3,i,j) = gh3dd(i,j)

            ghigelel(1,i,j) = gh1elel(i,j)
            ghigelel(2,i,j) = gh2elel(i,j)
            ghigelel(3,i,j) = gh3elel(i,j)
            ghignene(1,i,j) = gh1nene(i,j)
            ghignene(2,i,j) = gh2nene(i,j)
            ghignene(3,i,j) = gh3nene(i,j)
         end do
      end do
      ghigtt(4,1,1) = 0.D0
      ghigtt(4,2,2) = 0.D0
      ghigtt(5,1,1) = 0.D0
      ghigtt(5,2,2) = 0.D0
      ghigtt(4,1,2) = gha1tt
      ghigtt(5,1,2) = gha2tt
      ghigbb(4,1,1) = 0.D0
      ghigbb(4,2,2) = 0.D0
      ghigbb(5,1,1) = 0.D0
      ghigbb(5,2,2) = 0.D0
      ghigbb(4,1,2) = gha1bb
      ghigbb(5,1,2) = gha2bb
      ghigee(4,1,1) = 0.D0
      ghigee(4,2,2) = 0.D0
      ghigee(5,1,1) = 0.D0
      ghigee(5,2,2) = 0.D0
      ghigee(4,1,2) = gha1ee
      ghigee(5,1,2) = gha2ee

c charged Higgs couplings to squarks, sleptons

      cll=(mw**2*dsin(2*b)-mt**2/dtan(b)-mb**2*dtan(b))
     .     /dsqrt(2.D0)/mw**2
      crr=-mt*mb*(1.D0/dtan(b)+dtan(b))/dsqrt(2.D0)/mw**2
      clr=-mb*(xlamvs+ab*dtan(b))/dsqrt(2.D0)/mw**2
      crl=-mt*(xlamvs+at/dtan(b))/dsqrt(2.D0)/mw**2
      ghctb(1,1)=+ct*cb*cll+st*sb*crr+ct*sb*clr+st*cb*crl
      ghctb(1,2)=-ct*sb*cll+st*cb*crr+ct*cb*clr-st*sb*crl
      ghctb(2,1)=-st*cb*cll+ct*sb*crr-st*sb*clr+ct*cb*crl
      ghctb(2,2)=+st*sb*cll+ct*cb*crr-st*cb*clr-ct*sb*crl

      cll=(mw**2*dsin(2*b))/dsqrt(2.D0)/mw**2
      ghcud(1,1)=cll
      ghcud(1,2)=0.D0
      ghcud(2,1)=0.D0
      ghcud(2,2)=0.D0

      cll=(mw**2*dsin(2*b)-ml**2*dtan(b))/dsqrt(2.D0)/mw**2
      clr=-ml*(xlamvs+al*dtan(b))/dsqrt(2.D0)/mw**2
      ghcen(1,1)=cl*cll+sl*clr
      ghcen(1,2)=-sl*cll+cl*clr
      ghcen(2,1)=0.D0
      ghcen(2,2)=0.D0 

      cll=(mw**2*dsin(2*b))/dsqrt(2.D0)/mw**2
      ghcelne(1,1)=cll
      ghcelne(1,2)=0.D0
      ghcelne(2,1)=0.D0
      ghcelne(2,2)=0.D0 

c --------------------------------------------------------------------- c

      ystht = st
      yctht = ct
      ysthb = sb
      ycthb = cb

      do i = 1,2
       ymst(i) = mst(i)
       ymsb(i) = msb(i)
       do j = 1,2
        yh1tt(i,j) = gh1tt(i,j)
        yh2tt(i,j) = gh2tt(i,j)
        yh3tt(i,j) = gh3tt(i,j)
        yh1bb(i,j) = gh1bb(i,j)
        yh2bb(i,j) = gh2bb(i,j)
        yh3bb(i,j) = gh3bb(i,j)
       enddo
      enddo
      do i=1,2,1
         do j=1,2,1
            yhitt(1,i,j) = gh1tt(i,j)
            yhitt(2,i,j) = gh2tt(i,j)
            yhitt(3,i,j) = gh3tt(i,j)
            yhibb(1,i,j) = gh1bb(i,j)
            yhibb(2,i,j) = gh2bb(i,j)
            yhibb(3,i,j) = gh3bb(i,j)
         end do
      end do

      yhaitt(1) = gha1tt
      yhaitt(2) = gha2tt
      yhaibb(1) = gha1bb
      yhaibb(2) = gha2bb

c      yhaitt(1) = gha1tt
c      yhaitt(2) = gha2tt
c      yhaibb(1) = gha1bb
c      yhaibb(2) = gha2bb

C--FILL COMMON BLOCK GLUINO_NMSSM FOR SUSY-QCD CORRECTIONS TO
C  HIGGS -> BB, SQUARKS

      xmst1 = mst(1)
      xmst2 = mst(2)
      xmsb1 = msb(1)
      xmsb2 = msb(2)
      stht = st
      ctht = ct
      sthb = sb
      cthb = cb

      do i=1,2
       do j=1,2
        xh1bb(i,j) = gh1bb(i,j)
        xh2bb(i,j) = gh2bb(i,j)
        xh3bb(i,j) = gh3bb(i,j)
        xh1tt(i,j) = gh1tt(i,j)
        xh2tt(i,j) = gh2tt(i,j)
        xh3tt(i,j) = gh3tt(i,j)

        xha1bb(i,j) = 0.D0
        xha2bb(i,j) = 0.D0
        xha1tt(i,j) = 0.D0
        xha2tt(i,j) = 0.D0
       enddo
      enddo

      xha1bb(1,2) = gha1bb
      xha1bb(2,1) = -gha1bb

      xha2bb(1,2) = gha2bb
      xha2bb(2,1) = -gha2bb

      xha1tt(1,2) = gha1tt
      xha1tt(2,1) = -gha1tt

      xha2tt(1,2) = gha2tt
      xha2tt(2,1) = -gha2tt

      return
111   stop
      end 

c -------------------------------------------------------------------- c

      double precision function runm_hdec(q,nf0)
      implicit double precision (a-h,o-z)
      parameter (nn=8)
      parameter (zeta3 = 1.202056903159594d0)
      dimension am(nn),ymsb(nn)
      common/als_hdec/xlambda,amca,amba,amta,n0a
      common/masses_hdec/ams,amc,amb,amt
      common/strange_hdec/amsb
      common/run_hdec/xmsb,xmhat,xkfac
      common/flag_hdec/ihiggs,nnlo
      save istrange

      b0(nf)=(33.d0-2.d0*nf)/12d0
      b1(nf) = (102d0-38d0/3d0*nf)/16d0
      b2(nf) = (2857d0/2d0-5033d0/18d0*nf+325d0/54d0*nf**2)/64d0
      g0(nf) = 1d0
      g1(nf) = (202d0/3d0-20d0/9d0*nf)/16d0
      g2(nf) = (1249d0-(2216d0/27d0+160d0/3d0*zeta3)*nf
     .       - 140d0/81d0*nf**2)/64d0
      c1(nf) = g1(nf)/b0(nf) - b1(nf)*g0(nf)/b0(nf)**2
      c2(nf) = ((g1(nf)/b0(nf) - b1(nf)*g0(nf)/b0(nf)**2)**2
     .       + g2(nf)/b0(nf) + b1(nf)**2*g0(nf)/b0(nf)**3
     .       - b1(nf)*g1(nf)/b0(nf)**2 - b2(nf)*g0(nf)/b0(nf)**2)/2d0
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
c     c1(nf) = 1.175d0
c     c2(nf) = 1.501d0
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      tran(x,xk)=1d0+4d0/3d0*alphas_hdec(x,3)/pi
     .              +xk*(alphas_hdec(x,3)/pi)**2
      cq(x,nf)=(2d0*b0(nf)*x)**(g0(nf)/b0(nf))
     .            *(1d0+loop2*c1(nf)*x+loop3*c2(nf)*x**2)
      data istrange/0/
c maggie changed 9/9/2011
      ism4 = 0
c end maggie changed
      nf = nf0
      loop = 3
      loop2 = 1
      loop3 = 1
      if(loop.le.2)loop3 = 0
      if(loop.le.1)loop2 = 0
      pi=4d0*datan(1d0)
      acc = 1.d-8
      am(1) = 0
      am(2) = 0
c--scale of strange msbar-mass
c     qqs = 1.d0
      qqs = 2.d0
c--------------------------------------------
      imsbar = 0
      if(imsbar.eq.1)then
       if(istrange.eq.0)then
c--strange pole mass from msbar-mass at qqs
        amsd = xlambda
        amsu = 1.d8
123     ams  = (amsu+amsd)/2
        am(3) = ams
        xmsb = ams/cq(alphas_hdec(ams,3)/pi,3)
     .            *cq(alphas_hdec(qqs,3)/pi,3)/tran(ams,0d0)
        dd = (xmsb-amsb)/amsb
        if(dabs(dd).ge.acc)then
         if(dd.le.0.d0)then
          amsd = am(3)
         else
          amsu = am(3)
         endif
         goto 123
        endif
        istrange=1
       endif
       am(3) = amsb
      else
       ams=amsb
       am(3) = ams
      endif
c--------------------------------------------
      am(3) = amsb
      am(4) = amc
      am(5) = amb
      am(6) = amt
      am(7) = 100*amt
      am(8) = 200*amt
c      if(ism4.ne.0)then
c       if(ambp.le.amtp)then
c        am(7) = ambp
c        am(8) = amtp
c       else
c        am(7) = amtp
c        am(8) = ambp
c        if(nf0.eq.7)then
c         nf = 8
c        else
c         nf = 7
c        endif
c       endif
c      endif
      xk = 16.11d0
      do 1 i=1,nf-1
       xk = xk - 1.04d0*(1.d0-am(i)/am(nf))
1     continue
      if(nf.ge.4)then
       xmsb = am(nf)/tran(am(nf),0d0)
       xmhat = xmsb/cq(alphas_hdec(am(nf),3)/pi,nf)
      else
       xmsb = 0
       xmhat = 0
      endif
      ymsb(3) = amsb
      if(nf.eq.3)then
       if(qqs.lt.amc)then
        ymsb(4) = ymsb(3)*cq(alphas_hdec(am(4),3)/pi,3)/
     .                    cq(alphas_hdec(qqs,3)/pi,3)
       else
        ymsb(4) = amsb*cq(alphas_hdec(am(4),3)/pi,4)/
     .                    cq(alphas_hdec(qqs,3)/pi,4)
        ymsb(3) = ymsb(4)
       endif
       ymsb(5) = ymsb(4)*cq(alphas_hdec(am(5),3)/pi,4)/
     .                   cq(alphas_hdec(am(4),3)/pi,4)
       ymsb(6) = ymsb(5)*cq(alphas_hdec(am(6),3)/pi,5)/
     .                   cq(alphas_hdec(am(5),3)/pi,5)
       ymsb(7) = ymsb(6)*cq(alphas_hdec(am(7),3)/pi,6)/
     .                   cq(alphas_hdec(am(6),3)/pi,6)
       ymsb(8) = ymsb(7)*cq(alphas_hdec(am(8),3)/pi,7)/
     .                   cq(alphas_hdec(am(7),3)/pi,7)
      elseif(nf.eq.4)then
       ymsb(4) = xmsb
       ymsb(3) = ymsb(4)*cq(alphas_hdec(qqs,3)/pi,3)/
     .                   cq(alphas_hdec(am(4),3)/pi,3)
       ymsb(5) = ymsb(4)*cq(alphas_hdec(am(5),3)/pi,4)/
     .                   cq(alphas_hdec(am(4),3)/pi,4)
       ymsb(6) = ymsb(5)*cq(alphas_hdec(am(6),3)/pi,5)/
     .                   cq(alphas_hdec(am(5),3)/pi,5)
       ymsb(7) = ymsb(6)*cq(alphas_hdec(am(7),3)/pi,6)/
     .                   cq(alphas_hdec(am(6),3)/pi,6)
       ymsb(8) = ymsb(7)*cq(alphas_hdec(am(8),3)/pi,7)/
     .                   cq(alphas_hdec(am(7),3)/pi,7)
      elseif(nf.eq.5)then
       ymsb(5) = xmsb
       ymsb(4) = ymsb(5)*cq(alphas_hdec(am(4),3)/pi,4)/
     .                   cq(alphas_hdec(am(5),3)/pi,4)
       ymsb(3) = ymsb(4)*cq(alphas_hdec(qqs,3)/pi,3)/
     .                   cq(alphas_hdec(am(4),3)/pi,3)
       ymsb(6) = ymsb(5)*cq(alphas_hdec(am(6),3)/pi,5)/
     .                   cq(alphas_hdec(am(5),3)/pi,5)
       ymsb(7) = ymsb(6)*cq(alphas_hdec(am(7),3)/pi,6)/
     .                   cq(alphas_hdec(am(6),3)/pi,6)
       ymsb(8) = ymsb(7)*cq(alphas_hdec(am(8),3)/pi,7)/
     .                   cq(alphas_hdec(am(7),3)/pi,7)
      elseif(nf.eq.6)then
       ymsb(6) = xmsb
       ymsb(5) = ymsb(6)*cq(alphas_hdec(am(5),3)/pi,5)/
     .                   cq(alphas_hdec(am(6),3)/pi,5)
       ymsb(4) = ymsb(5)*cq(alphas_hdec(am(4),3)/pi,4)/
     .                   cq(alphas_hdec(am(5),3)/pi,4)
       ymsb(3) = ymsb(4)*cq(alphas_hdec(qqs,3)/pi,3)/
     .                   cq(alphas_hdec(am(4),3)/pi,3)
       ymsb(7) = ymsb(6)*cq(alphas_hdec(am(7),3)/pi,6)/
     .                   cq(alphas_hdec(am(6),3)/pi,6)
       ymsb(8) = ymsb(7)*cq(alphas_hdec(am(8),3)/pi,7)/
     .                   cq(alphas_hdec(am(7),3)/pi,7)
      elseif(nf.eq.7)then
       ymsb(7) = xmsb
       ymsb(6) = ymsb(7)*cq(alphas_hdec(am(6),3)/pi,6)/
     .                   cq(alphas_hdec(am(7),3)/pi,6)
       ymsb(5) = ymsb(6)*cq(alphas_hdec(am(5),3)/pi,5)/
     .                   cq(alphas_hdec(am(6),3)/pi,5)
       ymsb(4) = ymsb(5)*cq(alphas_hdec(am(4),3)/pi,4)/
     .                   cq(alphas_hdec(am(5),3)/pi,4)
       ymsb(3) = ymsb(4)*cq(alphas_hdec(qqs,3)/pi,3)/
     .                   cq(alphas_hdec(am(4),3)/pi,3)
       ymsb(8) = ymsb(7)*cq(alphas_hdec(am(8),3)/pi,7)/
     .                   cq(alphas_hdec(am(7),3)/pi,7)
      elseif(nf.eq.8)then
       ymsb(8) = xmsb
       ymsb(7) = ymsb(8)*cq(alphas_hdec(am(7),3)/pi,7)/
     .                   cq(alphas_hdec(am(8),3)/pi,7)
       ymsb(6) = ymsb(7)*cq(alphas_hdec(am(6),3)/pi,6)/
     .                   cq(alphas_hdec(am(7),3)/pi,6)
       ymsb(5) = ymsb(6)*cq(alphas_hdec(am(5),3)/pi,5)/
     .                   cq(alphas_hdec(am(6),3)/pi,5)
       ymsb(4) = ymsb(5)*cq(alphas_hdec(am(4),3)/pi,4)/
     .                   cq(alphas_hdec(am(5),3)/pi,4)
       ymsb(3) = ymsb(4)*cq(alphas_hdec(qqs,3)/pi,3)/
     .                   cq(alphas_hdec(am(4),3)/pi,3)
      endif
      if(q.lt.amc)then
       n0=3
       q0 = qqs
      elseif(q.le.amb)then
       n0=4
       q0 = amc
      elseif(q.le.amt)then
       n0=5
       q0 = amb
      else
       n0=6
       q0 = amt
c       if(ism4.ne.0)then
c        if(q.gt.am(7))then
c         if(q.le.am(8))then
c          n0=7
c          q0 = am(7)
c         else
c          n0=8
c          q0 = am(8)
c         endif
c        endif
c       endif
      endif
      if(nnlo.eq.1.and.nf.gt.3)then
       xkfac = tran(am(nf),0d0)/tran(am(nf),xk)
      else
       xkfac = 1d0
      endif
      runm_hdec = ymsb(n0)*cq(alphas_hdec(q,3)/pi,n0)/
     .               cq(alphas_hdec(q0,3)/pi,n0)
     .       * xkfac
      return
      end

c -------------------------------------------------------------------- c

      double precision function alphas_hdec(q,n)
      implicit double precision (a-h,o-z)
      dimension xlb(6)
      common/alslam_hdec/xlb1(6),xlb2(6),xlb3(6)
      common/als_hdec/xlambda,amc,amb,amt,n0

      b0(nf)=33.d0-2.d0*nf
      b1(nf)=6.d0*(153.d0-19.d0*nf)/b0(nf)**2
      b2(nf)=27/2.d0*(2857-5033/9.d0*nf+325/27.d0*nf**2)/b0(nf)**3
      als1(nf,x)=12.d0*pi/(b0(nf)*dlog(x**2/xlb(nf)**2))
      als2(nf,x)=12.d0*pi/(b0(nf)*dlog(x**2/xlb(nf)**2))
     .          *(1.d0-b1(nf)*dlog(dlog(x**2/xlb(nf)**2))
     .           /dlog(x**2/xlb(nf)**2))
      als3(nf,x)=12.d0*pi/(b0(nf)*dlog(x**2/xlb(nf)**2))
     .          *(1.d0-b1(nf)*dlog(dlog(x**2/xlb(nf)**2))
     .           /dlog(x**2/xlb(nf)**2)
     .           +(b1(nf)**2*(dlog(dlog(x**2/xlb(nf)**2))**2
     .                      -dlog(dlog(x**2/xlb(nf)**2))-1)+b2(nf))
     .           /dlog(x**2/xlb(nf)**2)**2)
      pi=4.d0*datan(1.d0)
c     write(6,*)'als param: ',xlambda,amc,amb,amt,n0
      if(n.eq.1)then
       do 1 i=1,6
        xlb(i)=xlb1(i)
1      continue
      elseif(n.eq.2)then
       do 2 i=1,6
        xlb(i)=xlb2(i)
2      continue
      else
       do 3 i=1,6
        xlb(i)=xlb3(i)
3      continue
      endif
      if(q.lt.amc)then
       nf=3
      elseif(q.le.amb)then
       nf=4
      elseif(q.le.amt)then
       nf=5
      else
       nf=6
      endif
      if(n.eq.1)then
        alphas_hdec=als1(nf,q)
      elseif(n.eq.2)then
        alphas_hdec=als2(nf,q)
      else
        alphas_hdec=als3(nf,q)
      endif
      return
      end

c -------------------------------------------------------------------- c

      subroutine alsini_hdec(acc)
      implicit double precision (a-h,o-z)
      dimension xlb(6)
      common/alslam_hdec/xlb1(6),xlb2(6),xlb3(6)
      common/als_hdec/xlambda,amc,amb,amt,n0

      pi=4.d0*datan(1.d0)
      xlb1(1)=0d0
      xlb1(2)=0d0
      xlb2(1)=0d0
      xlb2(2)=0d0
      if(n0.eq.3)then
       xlb(3)=xlambda
       xlb(4)=xlb(3)*(xlb(3)/amc)**(2.d0/25.d0)
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
      elseif(n0.eq.4)then
       xlb(4)=xlambda
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
      elseif(n0.eq.5)then
       xlb(5)=xlambda
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
      elseif(n0.eq.6)then
       xlb(6)=xlambda
       xlb(5)=xlb(6)*(xlb(6)/amt)**(-2.d0/23.d0)
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
      endif
      do 1 i=3,6
       xlb1(i)=xlb(i)
1     continue
      if(n0.eq.3)then
       xlb(3)=xlambda
       xlb(4)=xlb(3)*(xlb(3)/amc)**(2.d0/25.d0)
     .             *(2.d0*dlog(amc/xlb(3)))**(-107.d0/1875.d0)
       xlb(4)=xiter_hdec(amc,xlb(3),3,xlb(4),4,acc)
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
     .             *(2.d0*dlog(amb/xlb(4)))**(-963.d0/13225.d0)
       xlb(5)=xiter_hdec(amb,xlb(4),4,xlb(5),5,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.4)then
       xlb(4)=xlambda
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
     .             *(2.d0*dlog(amb/xlb(4)))**(-963.d0/13225.d0)
       xlb(5)=xiter_hdec(amb,xlb(4),4,xlb(5),5,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter_hdec(amc,xlb(4),4,xlb(3),3,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.5)then
       xlb(5)=xlambda
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
     .             *(2.d0*dlog(amb/xlb(5)))**(963.d0/14375.d0)
       xlb(4)=xiter_hdec(amb,xlb(5),5,xlb(4),4,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter_hdec(amc,xlb(4),4,xlb(3),3,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.6)then
       xlb(6)=xlambda
       xlb(5)=xlb(6)*(xlb(6)/amt)**(-2.d0/23.d0)
     .            *(2.d0*dlog(amt/xlb(6)))**(321.d0/3703.d0)
       xlb(5)=xiter_hdec(amt,xlb(6),6,xlb(5),5,acc)
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
     .             *(2.d0*dlog(amb/xlb(5)))**(963.d0/14375.d0)
       xlb(4)=xiter_hdec(amb,xlb(5),5,xlb(4),4,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter_hdec(amc,xlb(4),4,xlb(3),3,acc)
      endif
      do 2 i=3,6
       xlb2(i)=xlb(i)
2     continue
      if(n0.eq.3)then
       xlb(3)=xlambda
       xlb(4)=xlb(3)*(xlb(3)/amc)**(2.d0/25.d0)
     .             *(2.d0*dlog(amc/xlb(3)))**(-107.d0/1875.d0)
       xlb(4)=xiter3_hdec(amc,xlb(3),3,xlb(4),4,acc)
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
     .             *(2.d0*dlog(amb/xlb(4)))**(-963.d0/13225.d0)
       xlb(5)=xiter3_hdec(amb,xlb(4),4,xlb(5),5,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter3_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.4)then
       xlb(4)=xlambda
       xlb(5)=xlb(4)*(xlb(4)/amb)**(2.d0/23.d0)
     .             *(2.d0*dlog(amb/xlb(4)))**(-963.d0/13225.d0)
       xlb(5)=xiter3_hdec(amb,xlb(4),4,xlb(5),5,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter3_hdec(amc,xlb(4),4,xlb(3),3,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter3_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.5)then
       xlb(5)=xlambda
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
     .             *(2.d0*dlog(amb/xlb(5)))**(963.d0/14375.d0)
       xlb(4)=xiter3_hdec(amb,xlb(5),5,xlb(4),4,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter3_hdec(amc,xlb(4),4,xlb(3),3,acc)
       xlb(6)=xlb(5)*(xlb(5)/amt)**(2.d0/21.d0)
     .            *(2.d0*dlog(amt/xlb(5)))**(-321.d0/3381.d0)
       xlb(6)=xiter3_hdec(amt,xlb(5),5,xlb(6),6,acc)
      elseif(n0.eq.6)then
       xlb(6)=xlambda
       xlb(5)=xlb(6)*(xlb(6)/amt)**(-2.d0/23.d0)
     .            *(2.d0*dlog(amt/xlb(6)))**(321.d0/3703.d0)
       xlb(5)=xiter3_hdec(amt,xlb(6),6,xlb(5),5,acc)
       xlb(4)=xlb(5)*(xlb(5)/amb)**(-2.d0/25.d0)
     .             *(2.d0*dlog(amb/xlb(5)))**(963.d0/14375.d0)
       xlb(4)=xiter3_hdec(amb,xlb(5),5,xlb(4),4,acc)
       xlb(3)=xlb(4)*(xlb(4)/amc)**(-2.d0/27.d0)
     .             *(2.d0*dlog(amc/xlb(4)))**(107.d0/2025.d0)
       xlb(3)=xiter3_hdec(amc,xlb(4),4,xlb(3),3,acc)
      endif
      do 3 i=3,6
       xlb3(i)=xlb(i)
3     continue
      return
      end

c -------------------------------------------------------------------- c

      double precision function xiter_hdec(q,xlb1,nf1,xlb,nf2,acc)
      implicit double precision (a-h,o-z)

      b0(nf)=33.d0-2.d0*nf
      b1(nf)=6.d0*(153.d0-19.d0*nf)/b0(nf)**2
      als2(nf,x,xlb)=12.d0*pi/(b0(nf)*dlog(x**2/xlb**2))
     .              *(1.d0-b1(nf)*dlog(dlog(x**2/xlb**2))
     .              /dlog(x**2/xlb**2))
      aa(nf)=12d0*pi/b0(nf)
      bb(nf)=b1(nf)/aa(nf)
      xit(a,b,x)=a/2.d0*(1d0+dsqrt(1d0-4d0*b*dlog(x)))
      pi=4.d0*datan(1.d0)
      xlb2=xlb
      ii=0
1     ii=ii+1
      x=dlog(q**2/xlb2**2)
      alp=als2(nf1,q,xlb1)
      a=aa(nf2)/alp
      b=bb(nf2)*alp
      xx=xit(a,b,x)
      xlb2=q*dexp(-xx/2.d0)
      y1=als2(nf1,q,xlb1)
      y2=als2(nf2,q,xlb2)
      dy=dabs(y2-y1)/y1
      if(dy.ge.acc) goto 1
       xiter_hdec=xlb2
      return
      end

c -------------------------------------------------------------------- c

      double precision function xiter3_hdec(q,xlb1,nf1,xlb,nf2,acc)
      implicit double precision (a-h,o-z)

      b0(nf)=33.d0-2.d0*nf
      b1(nf)=6.d0*(153.d0-19.d0*nf)/b0(nf)**2
      b2(nf)=27/2.d0*(2857-5033/9.d0*nf+325/27.d0*nf**2)/b0(nf)**3
      als3(nf,x,xlb)=12.d0*pi/(b0(nf)*dlog(x**2/xlb**2))
     .          *(1.d0-b1(nf)*dlog(dlog(x**2/xlb**2))
     .           /dlog(x**2/xlb**2)
     .           +(b1(nf)**2*(dlog(dlog(x**2/xlb**2))**2
     .                      -dlog(dlog(x**2/xlb**2))-1)+b2(nf))
     .           /dlog(x**2/xlb**2)**2)
      aa(nf)=12d0*pi/b0(nf)
      bb(nf)=b1(nf)/aa(nf)
      cc(nf)=b2(nf)/aa(nf)
      xit(a,b,c,x)=a/2.d0*(1d0+dsqrt(1d0-4d0*b*dlog(x)
     .          *(1-(a*b*(dlog(x)**2-dlog(x)-1)+c/b)/x/dlog(x))))
      pi=4.d0*datan(1.d0)
      xlb2=xlb
      ii=0
1     ii=ii+1
      x=dlog(q**2/xlb2**2)
      if(nf1.lt.nf2)then
       delta = 7*als3(nf1,q,xlb1)**2/pi**2/24
       alp=als3(nf1,q,xlb1)*(1+delta)
      else
       delta = 7*als3(nf1,q,xlb1)**2/pi**2/24
       alp=als3(nf1,q,xlb1)/(1+delta)
      endif
      a=aa(nf2)/alp
      b=bb(nf2)*alp
      c=cc(nf2)*alp
      xx=xit(a,b,c,x)
      xlb2=q*dexp(-xx/2.d0)
      if(nf1.lt.nf2)then
       delta = 7*als3(nf1,q,xlb1)**2/pi**2/24
       y1=als3(nf1,q,xlb1)*(1+delta)
       y2=als3(nf2,q,xlb2)
      else
       delta = 7*als3(nf1,q,xlb1)**2/pi**2/24
       y1=als3(nf1,q,xlb1)/(1+delta)
       y2=als3(nf2,q,xlb2)
      endif
      dy=dabs(y2-y1)/y1
      if(dy.ge.acc) goto 1
       xiter3_hdec=xlb2
      return
      end

c -------------------------------------------------------------------- c

      double precision function fint_hdec(z,xx,yy)
C--ONE-DIMENSIONAL CUBIC INTERPOLATION
C--Z  = WANTED POINT
C--XX = ARRAY OF 4 DISCRETE X-VALUES AROUND Z
C--YY = ARRAY OF 4 DISCRETE FUNCTION-VALUES AROUND Z
      implicit double precision (a-h,o-z)
      dimension xx(4),yy(4)

      x = dlog(z)
      x0=dlog(xx(1))
      x1=dlog(xx(2))
      x2=dlog(xx(3))
      x3=dlog(xx(4))
      y0=dlog(yy(1))
      y1=dlog(yy(2))
      y2=dlog(yy(3))
      y3=dlog(yy(4))
      a0=(x-x1)*(x-x2)*(x-x3)/(x0-x1)/(x0-x2)/(x0-x3)
      a1=(x-x0)*(x-x2)*(x-x3)/(x1-x0)/(x1-x2)/(x1-x3)
      a2=(x-x0)*(x-x1)*(x-x3)/(x2-x0)/(x2-x1)/(x2-x3)
      a3=(x-x0)*(x-x1)*(x-x2)/(x3-x0)/(x3-x1)/(x3-x2)
      fint_hdec=dexp(a0*y0+a1*y1+a2*y2+a3*y3)
      return
      end

c -------------------------------------------------------------------- c

      double precision function sp_hdec(x)
c--real dilogarithm (spence-function)
      implicit double precision (a-h,o-z)
      complex*16 cx,li2_hdec
      cx = dcmplx(x,0.d0)
      sp_hdec = dreal(li2_hdec(cx))
      return
      end
 
c -------------------------------------------------------------------- c

      complex*16 function li2_hdec(x)
c--complex dilogarithm (spence-function)
      implicit double precision (a-h,o-z)
      complex*16 x,y,cli2_hdec
      common/const_hdec/zeta2,zeta3
      zero=1.d-16
      xr=dreal(x)
      xi=dimag(x)
      r2=xr*xr+xi*xi
      li2_hdec=0
      if(r2.le.zero)then
        li2_hdec=x
        return
      endif
      rr=xr/r2
      if(r2.eq.1.d0.and.xi.eq.0.d0)then
        if(xr.eq.1.d0)then
          li2_hdec=dcmplx(zeta2)
        else
          li2_hdec=-dcmplx(zeta2/2.d0)
        endif
        return
      elseif(r2.gt.1.d0.and.rr.gt.0.5d0)then
        y=(x-1.d0)/x
        li2_hdec=cli2_hdec(y)+zeta2-cdlog(x)*cdlog(1.d0-x)
     .          +0.5d0*cdlog(x)**2
        return
      elseif(r2.gt.1.d0.and.rr.le.0.5d0)then
        y=1.d0/x
        li2_hdec=-cli2_hdec(y)-zeta2-0.5d0*cdlog(-x)**2
        return
      elseif(r2.le.1.d0.and.xr.gt.0.5d0)then
        y=1.d0-x
        li2_hdec=-cli2_hdec(y)+zeta2-cdlog(x)*cdlog(1.d0-x)
       return
      elseif(r2.le.1.d0.and.xr.le.0.5d0)then
        y=x
        li2_hdec=cli2_hdec(y)
        return
      endif
      end
 
c -------------------------------------------------------------------- c

      complex*16 function cli2_hdec(x)
c--taylor-expansion for complex dilogarithm (spence-function)
      implicit double precision (a-h,o-z)
      complex*16 x,z
      common/bernoulli_hdec/b2(18),b12(18),b3(18)
      common/poly_hdec/nber
      n=nber-1
      z=-cdlog(1.d0-x)
      cli2_hdec=b2(nber)
      do 111 i=n,1,-1
        cli2_hdec=z*cli2_hdec+b2(i)
111   continue
      cli2_hdec=z**2*cli2_hdec+z
      return
      end

c -------------------------------------------------------------------- c
 
      double precision function factrl_hdec(n)
c--double precision version of factorial
      implicit double precision (a-h,o-z)
      factrl_hdec=1.d0
      if(n.eq.0)return
      do 999 i=1,n
        factrl_hdec=factrl_hdec*dfloat(i)
999   continue
      return
      end
 
c -------------------------------------------------------------------- c

      subroutine bernini_hdec(n)
c--initialization of coefficients for polylogarithms
      implicit double precision (a-h,o-z)
      dimension b(18),pb(19)
      common/bernoulli_hdec/b2(18),b12(18),b3(18)
      common/const_hdec/zeta2,zeta3
      common/poly_hdec/nber
 
      nber=n
      pi=4.d0*datan(1.d0)
 
      b(1)=-1.d0/2.d0
      b(2)=1.d0/6.d0
      b(3)=0.d0
      b(4)=-1.d0/30.d0
      b(5)=0.d0
      b(6)=1.d0/42.d0
      b(7)=0.d0
      b(8)=-1.d0/30.d0
      b(9)=0.d0
      b(10)=5.d0/66.d0
      b(11)=0.d0
      b(12)=-691.d0/2730.d0
      b(13)=0.d0
      b(14)=7.d0/6.d0
      b(15)=0.d0
      b(16)=-3617.d0/510.d0
      b(17)=0.d0
      b(18)=43867.d0/798.d0
      zeta2=pi**2/6.d0
      zeta3=1.202056903159594d0
 
      do 995 i=1,18
        b2(i)=b(i)/factrl_hdec(i+1)
        b12(i)=dfloat(i+1)/factrl_hdec(i+2)*b(i)/2.d0
        pb(i+1)=b(i)
        b3(i)=0.d0
995   continue
      pb(1)=1.d0
      do i=1,18
      do j=0,i
        b3(i)=b3(i)+pb(j+1)*pb(i-j+1)/factrl_hdec(i-j)/factrl_hdec(j+1)
     .                                            /dfloat(i+1)
      enddo
      enddo
 
      return
      end

c -------------------------------------------------------------------- c

      double precision function qqint_hdec(rat,h1,h2)
      implicit double precision (a-h,o-z)
      n = 2
      qqint_hdec = rat**n * h1 + (1-rat**n) * h2
      return
      end

c -------------------------------------------------------------------- c

      double precision function xitla_hdec(no,alp,acc)
c--iteration routine to determine improved lambdas
      implicit double precision (a-h,o-z)
      common/param_hdec/gf,alph,amtau,ammuon,amz,amw
      b0(nf)=33.d0-2.d0*nf
      b1(nf)=6.d0*(153.d0-19.d0*nf)/b0(nf)**2
      b2(nf)=27/2.d0*(2857-5033/9.d0*nf+325/27.d0*nf**2)/b0(nf)**3
      als2(nf,x,xlb)=12.d0*pi/(b0(nf)*dlog(x**2/xlb**2))
     .              *(1.d0-b1(nf)*dlog(dlog(x**2/xlb**2))
     .              /dlog(x**2/xlb**2))
      als3(nf,x,xlb)=12.d0*pi/(b0(nf)*dlog(x**2/xlb**2))
     .          *(1.d0-b1(nf)*dlog(dlog(x**2/xlb**2))
     .           /dlog(x**2/xlb**2)
     .           +(b1(nf)**2*(dlog(dlog(x**2/xlb**2))**2
     .                      -dlog(dlog(x**2/xlb**2))-1)+b2(nf))
     .           /dlog(x**2/xlb**2)**2)
      aa(nf)=12d0*pi/b0(nf)
      bb(nf)=b1(nf)/aa(nf)
      cc(nf)=b2(nf)/aa(nf)
      xit(a,b,x)=a/2.d0*(1d0+dsqrt(1d0-4d0*b*dlog(x)))
      xit3(a,b,c,x)=a/2.d0*(1d0+dsqrt(1d0-4d0*b*dlog(x)
     .          *(1-(a*b*(dlog(x)**2-dlog(x)-1)+c/b)/x/dlog(x))))
      pi=4.d0*datan(1.d0)
      nf=5
      q=amz
      xlb=q*dexp(-aa(nf)/alp/2.d0)
      if(no.eq.1)goto 111
      ii=0
1     ii=ii+1
      x=dlog(q**2/xlb**2)
      a=aa(nf)/alp
      b=bb(nf)*alp
      c=cc(nf)*alp
      if(no.eq.2)then
       xx=xit(a,b,x)
      else
       xx=xit3(a,b,c,x)
      endif
      xlb=q*dexp(-xx/2.d0)
      y1=alp
      if(no.eq.2)then
       y2=als2(nf,q,xlb)
      else
       y2=als3(nf,q,xlb)
      endif
      dy=dabs(y2-y1)/y1
      if(dy.ge.acc) goto 1
111   xitla_hdec=xlb
      return
      end

c -------------------------------------------------------------------- c

      DOUBLE PRECISION FUNCTION COFSUSY_NMSSM(IHIGGS,AMB,RMB,QQ)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 C03_HDEC
      COMMON/PARAM_HDEC/GF,ALPH,AMTAU,AMMUON,AMZ,AMW
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
c      COMMON/HMASS_HDEC/AMSM,AMA,AMHL,AMHH,AMCH,AMAR
      COMMON/GLUINO_NMSSM/amg,amsb1,amsb2,sth,cth,
     .     gh1bb(2,2),gh2bb(2,2),gh3bb(2,2),gha1bb(2,2),gha2bb(2,2),
     .     amst1,amst2,stht,ctht,gh1tt(2,2),gh2tt(2,2),gh3tt(2,2),
     .     gha1tt(2,2),gha2tt(2,2)
c      COMMON/GLUINO_HDEC/AMG,AMSB1,AMSB2,STH,CTH,
c     .              GLBB(2,2),GHBB(2,2),GABB(2,2),
c     .              AMST1,AMST2,STHT,CTHT,
c     .              GLTT(2,2),GHTT(2,2),GATT(2,2)
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
c      COMMON/COUP_HDEC/GAT,GAB,GLT,GLB,GHT,GHB,GZAH,GZAL,
c     .            GHHH,GLLL,GHLL,GLHH,GHAA,GLAA,GLVV,GHVV,
c     .            GLPM,GHPM,B,A
      FC1(VI,VJ,AI,AJ,BI,BJ,BIJ,CIJ,AMI,AMJ) = -1.D0/4*
     .  ((VI*VJ+AI*AJ)/(AMH**2-4*AMB**2)*(BI+BJ-2*BIJ
     .                  +(AMI**2+AMJ**2-2*AMG**2-2*AMB**2)*CIJ)
     .  + AMG/RMB*(VI*VJ-AI*AJ)*CIJ)
      FCA(VI,VJ,AI,AJ,BI,BJ,BIJ,CIJ,AMI,AMJ) = -1.D0/4*(
     .   (VI*AJ+AI*VJ)/AMH**2*(BJ-BI+(AMI**2-AMJ**2)*CIJ)
     .  + AMG/RMB*(AI*VJ-VI*AJ)*CIJ
     .  )
      FC2(VI,AI,A0I,A0G,BI,BPI,AMI) = -1.D0/8*(
     .   (VI**2+AI**2)/2/AMB**2*(A0G-A0I+(AMB**2-AMG**2+AMI**2)*BI
     .                          +2*(AMB**2+AMG**2-AMI**2)*AMB**2*BPI)
     .  + 2*AMG*AMB*(VI**2-AI**2)*BPI
     .  )
      FC3(VI,AI,A0I,A0G,BI,AMI,Q2) = 1.D0/8*(
     .   (VI**2+AI**2)/2/Q2*(A0I-A0G+(Q2+AMG**2-AMI**2)*BI)
     .   +AMG/RMB*(VI**2-AI**2)*BI
     .  )
      FC4(VI,AI,A0I,A0G,BI,BPI,AMI) = 1.D0/8*(
     .   (VI**2+AI**2)/2*((A0G-A0I)/(AMG**2-AMI**2)
     .   +(AMG**2-AMI**2)*BPI)
     .   +AMG/RMB*(VI**2-AI**2)*BI
     .  )
      CF = 4.D0/3.D0
      PI = 4*DATAN(1.D0)

c maggie changed 16/8/2011
      if(ihiggs.eq.1)then
       amh = amh1
       glo = gh1b/amz**2
       g11 = gh1bb(1,1)/glo
       g12 = gh1bb(1,2)/glo
       g21 = gh1bb(2,1)/glo
       g22 = gh1bb(2,2)/glo

      elseif(ihiggs.eq.2)then
       amh = amh2
       glo = gh2b/amz**2
       g11 = gh2bb(1,1)/glo
       g12 = gh2bb(1,2)/glo
       g21 = gh2bb(2,1)/glo
       g22 = gh2bb(2,2)/glo

      elseif(ihiggs.eq.3)then
       amh = amh3
       glo = gh3b/amz**2
       g11 = gh3bb(1,1)/glo
       g12 = gh3bb(1,2)/glo
       g21 = gh3bb(2,1)/glo
       g22 = gh3bb(2,2)/glo

      elseif(ihiggs.eq.4)then
       amh = ama1
       glo = ga1b/amz**2
       g11 = gha1bb(1,1)/glo
       g12 = gha1bb(1,2)/glo
       g21 = gha1bb(2,1)/glo
       g22 = gha1bb(2,2)/glo

      elseif(ihiggs.eq.5)then
       amh = ama2
       glo = ga2b/amz**2
       g11 = gha2bb(1,1)/glo
       g12 = gha2bb(1,2)/glo
       g21 = gha2bb(2,1)/glo
       g22 = gha2bb(2,2)/glo

      endif
c end maggie changed 16/8/2011

      XMU = AMH
      V1 = CTH-STH
      V2 = -CTH-STH
      A1 = CTH+STH
      A2 = CTH-STH
      CC11  = DREAL(C03_HDEC(AMB**2,AMB**2,AMH**2,AMSB1,AMG,AMSB1))
      CC12  = DREAL(C03_HDEC(AMB**2,AMB**2,AMH**2,AMSB1,AMG,AMSB2))
      CC21  = DREAL(C03_HDEC(AMB**2,AMB**2,AMH**2,AMSB2,AMG,AMSB1))
      CC22  = DREAL(C03_HDEC(AMB**2,AMB**2,AMH**2,AMSB2,AMG,AMSB2))
      BB1 = B02_HDEC(AMB**2,AMG,AMSB1,XMU**2)
      BB2 = B02_HDEC(AMB**2,AMG,AMSB2,XMU**2)
      BB11 = B02_HDEC(AMH**2,AMSB1,AMSB1,XMU**2)
      BB12 = B02_HDEC(AMH**2,AMSB1,AMSB2,XMU**2)
      BB21 = B02_HDEC(AMH**2,AMSB2,AMSB1,XMU**2)
      BB22 = B02_HDEC(AMH**2,AMSB2,AMSB2,XMU**2)
      BP1 = BP02_HDEC(AMB**2,AMG,AMSB1,XMU**2)
      BP2 = BP02_HDEC(AMB**2,AMG,AMSB2,XMU**2)
      AA1 = AMSB1**2*(1+DLOG(XMU**2/AMSB1**2))
      AA2 = AMSB2**2*(1+DLOG(XMU**2/AMSB2**2))
      AAG = AMG**2*(1+DLOG(XMU**2/AMG**2))
      BCT1 = B02_HDEC(QQ**2,AMG,AMSB1,XMU**2)
      BCT2 = B02_HDEC(QQ**2,AMG,AMSB2,XMU**2)
      BPCT1 = BP02_HDEC(QQ**2,AMG,AMSB1,XMU**2)
      BPCT2 = BP02_HDEC(QQ**2,AMG,AMSB2,XMU**2)

c maggie changed 16/8/2011
      if(ihiggs.eq.4.or.ihiggs.eq.5)then
c end maggie changed 16/8/2011
       COF1 = G11*FCA(V1,V1,A1,A1,BB1,BB1,BB11,CC11,AMSB1,AMSB1)
     .      + G12*FCA(V1,V2,A1,A2,BB1,BB2,BB12,CC12,AMSB1,AMSB2)
     .      + G21*FCA(V2,V1,A2,A1,BB2,BB1,BB21,CC21,AMSB2,AMSB1)
     .      + G22*FCA(V2,V2,A2,A2,BB2,BB2,BB22,CC22,AMSB2,AMSB2)
      ELSE
       COF1 = G11*FC1(V1,V1,A1,A1,BB1,BB1,BB11,CC11,AMSB1,AMSB1)
     .      + G12*FC1(V1,V2,A1,A2,BB1,BB2,BB12,CC12,AMSB1,AMSB2)
     .      + G21*FC1(V2,V1,A2,A1,BB2,BB1,BB21,CC21,AMSB2,AMSB1)
     .      + G22*FC1(V2,V2,A2,A2,BB2,BB2,BB22,CC22,AMSB2,AMSB2)
      ENDIF
      COF2 = FC2(V1,A1,AA1,AAG,BB1,BP1,AMSB1)
     .     + FC2(V2,A2,AA2,AAG,BB2,BP2,AMSB2)
      IF(QQ.EQ.0.D0)THEN
       COF3 = FC4(V1,A1,AA1,AAG,BCT1,BPCT1,AMSB1)
     .      + FC4(V2,A2,AA2,AAG,BCT2,BPCT2,AMSB2)
      ELSE
       COF3 = FC3(V1,A1,AA1,AAG,BCT1,AMSB1,QQ**2)
     .      + FC3(V2,A2,AA2,AAG,BCT2,AMSB2,QQ**2)
      ENDIF
      COF1 = 2*CF*COF1
      COF2 = 2*CF*COF2
      COF3 = 2*CF*COF3
      COFSUSY_NMSSM = COF1 + COF2 + COF3

      RETURN
      END
 
c -------------------------------------------------------------------- c

      SUBROUTINE BOTSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,
     .     xgh2b,xgh3b,xga1b,xga2b,xghpm,tgbet,SCALE,IL)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      ICASE = 0

      CALL DMBAPP_NMSSM_HDEC(ICASE,dgh1b,dgh2b,dgh3b,dga1b,dga2b,dghpm,
     .     scale,il)
      
      xgh1b = gh1b*(1+dgh1b)
      xgh2b = gh2b*(1+dgh2b)
      xgh3b = gh3b*(1+dgh3b)
      xga1b = ga1b*(1+dga1b)
      xga2b = ga2b*(1+dga2b)
      xghpm = tgbet*(1+dghpm)

      RETURN
      END

c -------------------------------------------------------------------- c

      SUBROUTINE DMBAPP_NMSSM_HDEC(ICASE,dgh1b,dgh2b,dgh3b,dga1b,dga2b,
     .dghpm,SCALE,IL)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      dimension amneut(5),xmneut(5),amchar(2),xmst(2),xmsb(2),
     .     amsl(2),amsu(2),amsd(2),amse(2),amsn(18),amsnl(2),amnu(9)
      dimension slhahmix(3,3),slhaamix(2,2)
      COMMON/PARAM_HDEC/GF,ALPH,AMTAU,AMMUON,AMZ,AMW
      COMMON/MASSES_HDEC/AMS,AMC,AMB,AMT
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
      COMMON/GLUINO_NMSSM/amg,amsb1,amsb2,sth,cth,
     .     gh1bb(2,2),gh2bb(2,2),gh3bb(2,2),gha1bb(2,2),gha2bb(2,2),
     .     amst1,amst2,stht,ctht,gh1tt(2,2),gh2tt(2,2),gh3tt(2,2),
     .     gha1tt(2,2),gha2tt(2,2)
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/BREAK_NMSSM/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AU,AD,Alambda,
     .     Akappa,AMU,AM2
      COMMON/SMASS_NMSSM/AMNEUT,XMNEUT,AMCHAR,XMST,XMSB,AMSL,
     .              AMSU,AMSD,AMSE,AMSN,AMSNL,amnu
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/DAVID/QSUSY1,QSUSY2,LOOP
      COMMON/ALSLAM_HDEC/XLB1(6),XLB2(6),XLB3(6)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
c --- added 21/1/2014
      COMMON/M1andM2val/am1val,am2val
c --- end added 21/1/2014

      PI = 4*DATAN(1.D0)
c      V  = 1/DSQRT(2*DSQRT(2D0)*GF) 
      V  = 1/DSQRT(DSQRT(2D0)*GF)
      TANB = DTAN(B)

      vs = xlamvs/xlambdapar*dsqrt(2.D0)

      SB = TANB/DSQRT(1+TANB**2)
      CB = 1.D0/DSQRT(1+TANB**2)

      AT = AU
      AB = AD
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      SCALELW = QSUSY2*(AMST1+AMST2+DABS(AMU))/3
      SCALQCD = QSUSY1*(AMSB1+AMSB2+DABS(AMG))/3
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      RMTOP   = RUNM_HDEC(SCALELW,6)
      HT = RMTOP/V/SB*dsqrt(2.D0)
      STOP1 = AMST1
      STOP2 = AMST2
      SBOT1 = AMSB1
      SBOT2 = AMSB2
c --- added 21/1/2014 ---
      CW = AMW**2/AMZ**2
      SW = 1-CW
      AL2  = (2*AMW/V)**2/4/PI
      AL1  = AL2*SW/CW
c      AM1=5.D0/3.D0*SW/CW*AM2
      am1 = am1val
      am2 = am2val
c --- end added 21/1/2014 ---

      FELW = 1
      FQCD = 1
      IF(IL.EQ.2)THEN
       ASH = ALPHAS_HDEC(SCALELW,3)
       CELW = FELW_HDEC(SCALELW,AMU,AMG,SBOT1,SBOT2,STOP1,STOP2,AMT)
       FELW = 1+ASH/PI*CELW
       ASH = ALPHAS_HDEC(SCALQCD,3)
       XXT = AU - AMU/TANB
       CQCD = FQCD_HDEC(SCALQCD,AMT,AMG,SBOT1,SBOT2,STOP1,STOP2,
     .                  AMSU(1),AMSU(2),AMSD(1),AMSD(2),XXT)
       FQCD = 1+ASH/PI*CQCD
      ENDIF
 
      ASH = ALPHAS_HDEC(SCALQCD,3)

      IF(ICASE.EQ.0)THEN
c       DELTAMB = 2*ASH/3/PI*AMG*AMU*TANB*T_HDEC(SBOT1,SBOT2,AMG)*FQCD
c     *         /(1-2*ASH/3/PI*AMG*AB*T_HDEC(SBOT1,SBOT2,AMG))
c     * + HT**2/(4*PI)**2*AT*AMU*TANB*T_HDEC(STOP1,STOP2,AMU)*FELW
       DELTAMB = (2*ASH/3/PI*AMG*AMU*TANB*T_HDEC(SBOT1,SBOT2,AMG)
     *        +HT**2/(4*PI)**2*AT*AMU*TANB*T_HDEC(STOP1,STOP2,AMU)
c --- added 21/1/2014
     . - AL1/12/PI*AM1*AMU*TANB*(T_HDEC(SBOT1,SBOT2,AM1)/3
     .      + (CTH**2/2+STH**2)*T_HDEC(SBOT1,AM1,AMU)
     .      + (STH**2/2+CTH**2)*T_HDEC(SBOT2,AM1,AMU))
     . - AL2/4/PI*AM2*AMU*TANB*(CTHT**2*T_HDEC(STOP1,AM2,AMU)
     .                         +STHT**2*T_HDEC(STOP2,AM2,AMU)
     . +(CTH**2*T_HDEC(SBOT1,AM2,AMU)+STH**2*T_HDEC(SBOT2,AM2,AMU))/2))
c --- end added 21/1/2014
     *         /(1-2*ASH/3/PI*AMG*AB*T_HDEC(SBOT1,SBOT2,AMG))
c maggie changed 11/8/13
       dgh1b = -deltamb/(1+deltamb)*(1.D0+
     .      (-slhahmix(1,2)/slhahmix(1,1))/tanb
     .     +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb/(1+deltamb)*(1.D0+
     .      (-slhahmix(2,2)/slhahmix(2,1))/tanb
     .     +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb/(1+deltamb)*(1.D0+
     .      (-slhahmix(3,2)/slhahmix(3,1))/tanb
     .     +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb/(1+deltamb)*(1.D0+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb/(1+deltamb)*(1.D0+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb/(1+deltamb)*(1+1/tanb**2)
c end maggie changed 11/8/13
      ELSE
       DELTAMB = 2*ASH/3/PI*AMG*AMU*TANB*T_HDEC(SBOT1,SBOT2,AMG)

c maggie changed 15/8/2013
       dgh1b = -deltamb*(1+(-slhahmix(1,2)/slhahmix(1,1))/tanb
     .      +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb*(1+(-slhahmix(2,2)/slhahmix(2,1))/tanb
     .      +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb*(1+(-slhahmix(3,2)/slhahmix(3,1))/tanb
     .      +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb*(1+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb*(1+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb*(1+1/tanb**2)
c end maggie changed 15/8/2013

      ENDIF

      RETURN
      END

c -------------------------------------------------------------------- c

      SUBROUTINE STAUSUSY_NMSSM_HDEC(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,
     .     xgh2b,xgh3b,xga1b,xga2b,xghpm,tgbet,SCALE,IL)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      ICASE = 0
      CALL DMTAUAPP_NMSSM_HDEC(ICASE,dgh1b,dgh2b,dgh3b,dga1b,dga2b,
     .     dghpm,scale,il)

      xgh1b = gh1b*(1+dgh1b)
      xgh2b = gh2b*(1+dgh2b)
      xgh3b = gh3b*(1+dgh3b)
      xga1b = ga1b*(1+dga1b)
      xga2b = ga2b*(1+dga2b)
      xghpm = tgbet*(1+dghpm)

      RETURN
      END

c -------------------------------------------------------------------- c

      SUBROUTINE DMTAUAPP_NMSSM_HDEC(ICASE,dgh1b,dgh2b,dgh3b,dga1b,
     .     dga2b,dghpm,SCALE,IL)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      dimension amneut(5),xmneut(5),amchar(2),xmst(2),xmsb(2),
     .     amsl(2),amsu(2),amsd(2),amse(2),amsn(18),amsnl(2),amnu(9)
      dimension slhahmix(3,3),slhaamix(2,2)
      COMMON/PARAM_HDEC/GF,ALPH,AMTAU,AMMUON,AMZ,AMW
      COMMON/MASSES_HDEC/AMS,AMC,AMB,AMT
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
c ---- changed 21/1/2014
      COMMON/GLUINO_NMSSM/amg,amsb1,amsb2,sth0,cth0,
     .     gh1bb(2,2),gh2bb(2,2),gh3bb(2,2),gha1bb(2,2),gha2bb(2,2),
     .     amst1,amst2,stht0,ctht0,gh1tt(2,2),gh2tt(2,2),gh3tt(2,2),
     .     gha1tt(2,2),gha2tt(2,2)
c ---- end changed 2171/2014
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/BREAK_NMSSM/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AU,AD,Alambda,
     .     Akappa,AMU,AM2
      COMMON/SMASS_NMSSM/AMNEUT,XMNEUT,AMCHAR,XMST,XMSB,AMSL,
     .              AMSU,AMSD,AMSE,AMSN,AMSNL,AMNU
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/DAVID/QSUSY1,QSUSY2,LOOP
      COMMON/ALSLAM_HDEC/XLB1(6),XLB2(6),XLB3(6)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
      COMMON/M1andM2val/am1val,am2val
c --- added 21/1/2014
      COMMON/TAUMIX_HDEC/CL,SL
c --- end added 21/1/2014

      PI = 4*DATAN(1.D0)
c      V  = 1/DSQRT(2*DSQRT(2D0)*GF)
      V = 1/DSQRT(GF*DSQRT(2.d0)) 
      TANB = DTAN(B)
      SB = TANB/DSQRT(1+TANB**2)
      CB = 1.D0/DSQRT(1+TANB**2)

      vs = xlamvs/xlambdapar*dsqrt(2.D0)

      AT = AU
      AB = AD
      IF(IL.EQ.0)THEN
       STAU1 = AMSE(1)
       STAU2 = AMSE(2)
       SNEUT = AMSN(1)
c --- added 21/1/2014
       CTH = 1
       STH = 0
c --- end added 21/1/2014
      ELSE
       STAU1 = AMSL(1)
       STAU2 = AMSL(2)
       SNEUT = AMSNL(1)
c --- added 21/1/2014
       CTH = CL
       STH = SL
c --- end added 21/1/2014
      ENDIF
      CW = AMW**2/AMZ**2
      SW = 1-CW
c      AL2  = (2*AMW/V/DSQRT(2D0))**2/4/PI
      AL2  = (2*AMW/V)**2/4/PI 
      AL1  = AL2*SW/CW
c      AM1=5.D0/3.D0*SW/CW*AM2
      am1 = am1val
      am2 = am2val

      IF(ICASE.EQ.0)THEN
       DELTAMB = AL1/4/PI*AM1*AMU*TANB*(T_HDEC(STAU1,STAU2,AM1)
c --- added and corrected 21/1/2014
     . + (CTH**2/2-STH**2)*T_HDEC(STAU1,AM1,AMU)
     . + (STH**2/2-CTH**2)*T_HDEC(STAU2,AM1,AMU))
     .         - AL2/4/PI*AM2*AMU*TANB*(T_HDEC(SNEUT,AM2,AMU)
     . +(CTH**2*T_HDEC(STAU1,AM2,AMU)+STH**2*T_HDEC(STAU2,AM2,AMU))/2)
c --- end added and corrected 21/1/2014
       dgh1b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(1,2)/slhahmix(1,1))/tanb
     .     +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(2,2)/slhahmix(2,1))/tanb
     .     +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(3,2)/slhahmix(3,1))/tanb
     .     +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb/(1+deltamb)*(1+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb/(1+deltamb)*(1+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb/(1+deltamb)*(1+1/tanb**2)
      ELSE
       DELTAMB = AL1/4/PI*AM1*AMU*TANB*T_HDEC(STAU1,STAU2,AM1)
c --- corrected 21/1/2014
     .         - AL2/4/PI*AM2*AMU*TANB*T_HDEC(SNEUT,AM2,AMU)
c --- end corrected 21/1/2014
       dgh1b = -deltamb*(1+(-slhahmix(1,2)/slhahmix(1,1))/tanb
     .     +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb*(1+(-slhahmix(2,2)/slhahmix(2,1))/tanb
     .     +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb*(1+(-slhahmix(3,2)/slhahmix(3,1))/tanb
     .     +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb*(1+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb*(1+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb*(1+1/tanb**2)
      ENDIF
      RETURN
      END

c -------------------------------------------------------------------- c

      subroutine strsusy_nmssm_hdec(gh1b,gh2b,gh3b,ga1b,ga2b,xgh1b,
     .     xgh2b,xgh3b,xga1b,xga2b,xghpm,tgbet,scale,il)
      implicit double precision (a-h,o-z)
      icase = 0
      call dmsapp_nmssm_hdec(icase,dgh1b,dgh2b,dgh3b,dga1b,dga2b,
     .     dghpm,scale,il)

      xgh1b = gh1b*(1+dgh1b)
      xgh2b = gh2b*(1+dgh2b)
      xgh3b = gh3b*(1+dgh3b)
      xga1b = ga1b*(1+dga1b)
      xga2b = ga2b*(1+dga2b)
      xghpm = tgbet*(1+dghpm)

      return
      end
 
c -------------------------------------------------------------------- c

      subroutine dmsapp_nmssm_hdec(icase,dgh1b,dgh2b,dgh3b,dga1b,dga2b,
     .     dghpm,scale,il)
      implicit double precision (a-h,o-z)
      dimension amchar(2),amneut(5),xmneut(5),
     .          xmst(2),xmsb(2),amsl(2),
     .          amsu(2),amsd(2),amse(2),amsn(18),amsnl(2),amnu(9)
      dimension slhahmix(3,3),slhaamix(2,2)
      COMMON/PARAM_HDEC/gf,alph,amtau,ammuon,amz,amw
      COMMON/MASSES_HDEC/ams,amc,amb,amt
      COMMON/HMASS_NMSSM/ama1,ama2,amh1,amh2,amh3,amch,amar
c --- changed 2171/2014
      COMMON/GLUINO_NMSSM/amg,xmsb1,xmsb2,sth0,cth0,
     .     xh1bb(2,2),xh2bb(2,2),xh3bb(2,2),xha1bb(2,2),xha2bb(2,2),
     .     xmst1,xmst2,stht0,ctht0,xh1tt(2,2),xh2tt(2,2),xh3tt(2,2),
     .     xha1tt(2,2),xha2tt(2,2)
c --- end changed 21/1/2014
      COMMON/COUP_NMSSM/ga1t,ga1b,ga2t,ga2b,gh1t,gh1b,gh2t,gh2b,gh3t,
     .     gh3b,gza1h1,gza2h1,gza1h2,gza2h2,gza1h3,gza2h3,gh1h1h1,
     .     gh1h2h2,gh2h1h1,gh1h3h3,gh3h1h1,gh2h3h3,gh3h2h2,gh2h2h2,
     .     gh3h3h3,gh1a1a1,gh1a2a2,gh2a1a1,gh2a2a2,gh3a1a1,gh3a2a2,
     .     gh1h2h3,gh1a1a2,gh2a1a2,gh3a1a2,gh1vv,gh2vv,gh3vv,gh1pm,
     .     gh2pm,gh3pm,gh1hpwm,gh2hpwm,gh3hpwm,b
      COMMON/BREAK_NMSSM/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AU,AD,Alambda,
     .     Akappa,AMU,AM2
      COMMON/SMASS_NMSSM/amneut,xmneut,amchar,xmst,xmsb,amsl,
     .              amsu,amsd,amse,amsn,amsnl,amnu
      COMMON/SLHA_hmix_NMSSM/slhahmix,slhaamix
      COMMON/EXTPAR_NMSSM/xlambdapar,xkappa,Alam,Akap,xlamvs,xif,xis,
     .     xmuprime,amsph2,amsh2
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      COMMON/DAVID/qsusy1,qsusy2,loop
      COMMON/ALSLAM_HDEC/xlb1(6),xlb2(6),xlb3(6)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
c --- added 21/1/2014
      COMMON/M1andM2val/am1val,am2val
c --- end added 21/1/2014
      pi = 4*datan(1.d0)
c      v  = 1/dsqrt(2*dsqrt(2d0)*gf)
      v  = 1/dsqrt(dsqrt(2d0)*gf)
      tanb = dtan(b)
      cb = 1.d0/dsqrt(1+tanb**2)

      vs = xlamvs/xlambdapar*dsqrt(2.D0)

      at = au
      ab = ad
      sbot1 = amsd(1)
      sbot2 = amsd(2)
c --- added 21/1/2014
      STOP1 = AMSU(1)
      STOP2 = AMSU(2)
      CTHT = 1
      STHT = 0
      CTH  = 1
      STH  = 0
      CW = AMW**2/AMZ**2
      SW = 1-CW
      AL2  = (2*AMW/V)**2/4/PI 
      AL1  = AL2*SW/CW
      am1 = am1val
      am2 = am2val
c --- end added 21/1/2014

      felw = 1
      fqcd = 1
      scalqcd = (sbot1+sbot2+dabs(amg))/3
      ash = alphas_hdec(scalqcd,3)
      if(icase.eq.0)then
c --- changed 21/1/2014
       deltamb = (2*ash/3/pi*amg*amu*tanb*t_hdec(sbot1,sbot2,amg)*fqcd
     . - AL1/12/PI*AM1*AMU*TANB*(T_HDEC(SBOT1,SBOT2,AM1)/3
     .      + (CTH**2/2+STH**2)*T_HDEC(SBOT1,AM1,AMU)
     .      + (STH**2/2+CTH**2)*T_HDEC(SBOT2,AM1,AMU))
     . - AL2/4/PI*AM2*AMU*TANB*(CTHT**2*T_HDEC(STOP1,AM2,AMU)
     .                         +STHT**2*T_HDEC(STOP2,AM2,AMU)
     . +(CTH**2*T_HDEC(SBOT1,AM2,AMU)+STH**2*T_HDEC(SBOT2,AM2,AMU))/2))
     *         /(1-2*ASH/3/PI*AMG*AB*T_HDEC(SBOT1,SBOT2,AMG))
c --- end changed 21/1/2014
       dgh1b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(1,2)/slhahmix(1,1))/tanb
     .     +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(2,2)/slhahmix(2,1))/tanb
     .     +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb/(1+deltamb)*(1+
     .      (-slhahmix(3,2)/slhahmix(3,1))/tanb
     .     +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb/(1+deltamb)*(1+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb/(1+deltamb)*(1+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb/(1+deltamb)*(1+1/tanb**2)
      ELSE
       DELTAMB = 2*ASH/3/PI*AMG*AMU*TANB*T_HDEC(SBOT1,SBOT2,AMG)
       dgh1b = -deltamb*(1+(-slhahmix(1,2)/slhahmix(1,1))/tanb
     .     +(-slhahmix(1,3)/slhahmix(1,1))*v*CB/vs)
       dgh2b = -deltamb*(1+(-slhahmix(2,2)/slhahmix(2,1))/tanb
     .     +(-slhahmix(2,3)/slhahmix(2,1))*v*CB/vs)
       dgh3b = -deltamb*(1+(-slhahmix(3,2)/slhahmix(3,1))/tanb
     .     +(-slhahmix(3,3)/slhahmix(3,1))*v*CB/vs)
       dga1b = -deltamb*(1+1/tanb**2
     .      +slhaamix(1,2)/slhaamix(1,1)*v/vs/tanb)
       dga2b = -deltamb*(1+1/tanb**2
     .      +slhaamix(2,2)/slhaamix(2,1)*v/vs/tanb)
       dghpm = -deltamb*(1+1/tanb**2)
      endif
      return
      end
 
c -------------------------------------------------------------------- c
***********************************************************************
        function eta_hdec(c1,c2)
***********************************************************************
*       COMPLEX ETA-FUNKTION                                           
*---------------------------------------------------------------------*
*       8.06.90    ANSGAR DENNER                                       
***********************************************************************
        implicit   logical(a-z)                                        
        complex*16 eta_hdec,c1,c2
        real*8     pi,im1,im2,im12                                     
                                                                       
        pi     = 4d0*datan(1d0)                                        
        im1    = dimag(c1)                                             
        im2    = dimag(c2)                                             
        im12   = dimag(c1*c2)                                          
                                                                       
        if(im1.lt.0d0.and.im2.lt.0d0.and.im12.gt.0d0) then             
            eta_hdec = dcmplx(0d0,2d0*pi)
        else if (im1.gt.0d0.and.im2.gt.0d0.and.im12.lt.0d0) then       
            eta_hdec = dcmplx(0d0,-2d0*pi)
        else                                                           
            eta_hdec = dcmplx(0d0)
        end if                                                         
        end                                                            

***********************************************************************
        function etas_hdec(y,r,rs)
***********************************************************************
*       MODIFIED ETA-FUNKTION                                           
*---------------------------------------------------------------------*
*       18.1.94   SD                                       
***********************************************************************
        implicit   logical(a-z)                                        
        complex*16 eta_hdec,etas_hdec,y,r,rs
        real*8     pi,imy,imrs
                                                                       
        pi     = 4d0*datan(1d0)                                        

	if( dimag(r).ne.0d0 ) then
	    etas_hdec = eta_hdec(y,r)
	else	    
	    if( dreal(r).gt.0d0 ) then
		etas_hdec = dcmplx(0d0,0d0)
	    else
	 	imy  = dimag(y)
		imrs = dimag(rs)
		etas_hdec = 2d0*dcmplx(0d0,pi)*(
     *			(1d0+sign(1d0,-imy))*(1d0+sign(1d0,-imrs))-
     *			(1d0+sign(1d0, imy))*(1d0+sign(1d0, imrs))
     *					  )/4d0
	    endif
	endif
        end                                                            

***********************************************************************
        function sqe_hdec(a,b,c)
***********************************************************************
*       SOLUTION OF QUADRATIC EQUATION				      *
*---------------------------------------------------------------------*
*       13.1.92  SD						      *
***********************************************************************
        implicit real*8 (a-z)                                        
        complex*16 a,b,c,sqe_hdec,x1,x2

	x1=(-b+sqrt(b**2-4d0*a*c))/2d0/a
	x2=(-b-sqrt(b**2-4d0*a*c))/2d0/a

	if (abs(x1).gt.abs(x2)) then
	   sqe_hdec=x1
	else
	   sqe_hdec=x2
	endif

        end                                                            

************************************************************************
        function d04_hdec(p1,p2,p3,p4,p12,p23,m1,m2,m3,m4)
************************************************************************
*  SCALAR 4-POINT FUNCTION WITH AT LEAST ONE MASS ZERO                 *
*  P1,P2,P3,P4 = SQUARED EXTERNAL MOMENTA			       *
*  P12 = (p1+p2)**2,  P23 = (p2+p3)**2				       *
*----------------------------------------------------------------------*
*  2.1.92  SD	         					       *
************************************************************************
        implicit real*8 (a-z)
	real*8 m(4),p(4,4),k(4,4)
	complex*16 a1,a2,a3,a4,swap
	complex*16 ss(4), xx(2), x(2,4),rs(4,4)
	complex*16 s0(4),xx0(2),x0(2,4), r(4,4),g(2)
        complex*16 c04,d04_hdec,cspen_hdec,eta_hdec,sqe_hdec,etas_hdec
	complex*16 aa,bb,cc,dd,ieps,h,hh,l1,l2,l3,l4
        complex*16 z2,b,sc,tc,wp,wm,bs,xs
	integer gen,i,j

        pi = 4*datan(1.d0)

        mm1=m1
        mm2=m2
        mm3=m3
        mm4=m4
        m12=m1*m1
        m22=m2*m2
        m32=m3*m3
        m42=m4*m4
        q1=p1
        q2=p2
        q3=p3
	q4=p4
        q12=p12
        q23=p23

c	is at least one mass zero ???
	if (mm1*mm2*mm3*mm4.ne.0d0) goto 130

c	permutate until mm3=0d0
	goto 20
10	continue
	mm0=mm1
	mm1=mm2
	mm2=mm3
	mm3=mm4
	mm4=mm0
	m02=m12
	m12=m22
	m22=m32
	m32=m42
	m42=m02
	q00=q12
	q12=q23
	q23=q00
	q0=q1
	q1=q2
	q2=q3
	q3=q4
	q4=q0
20	if (mm3.ne.0d0) goto 10
c	only mm3 is zero
	if (mm1*mm2*mm4.ne.0d0) goto 30
c	only mm3 and mm4 are zero ==> 3->2, 4->3...
	if ((mm1*mm2.ne.0d0).and.(mm4.eq.0d0)) goto 10
c	only mm2 and mm3 are zero
	if ((mm1*mm4.ne.0d0).and.(mm2.eq.0d0)) goto 40
	write(*,*)'case of this special d0-function not implemented!'
	stop

c	****** no mass equal to zero ******
130	continue
	eps=1d-18
	ieps=dcmplx(0d0,eps)

	if( abs((mm1**2+mm3**2-q12)/mm1/mm3).lt.2d0 ) then
c	r13 would be not real. -> permutation! -> r(2,4) is not real.
	   m(1)=mm2
	   m(2)=mm3
	   m(3)=mm4
	   m(4)=mm1
	   p(1,2)=q2
	   p(1,3)=q23
	   p(1,4)=q1
	   p(2,3)=q3
	   p(2,4)=q12
	   p(3,4)=q4
	else
c	r(1,3) is real.
	   m(1)=mm1
	   m(2)=mm2
	   m(3)=mm3
	   m(4)=mm4
	   p(1,2)=q1
	   p(1,3)=q12
	   p(1,4)=q4
	   p(2,3)=q2
	   p(2,4)=q23
	   p(3,4)=q3
	endif

	do  j=2,4
	do  i=1,j-1
	k(i,j)=(m(i)**2+m(j)**2-p(i,j))/m(i)/m(j)
	r(i,j) =sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),0d0),
     *	            dcmplx(1d0,0d0))
	if( dimag(r(i,j)).eq.0d0 ) then
	   rs(i,j)=sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),eps),
     *	               dcmplx(1d0,0d0))
	else
	   rs(i,j)=r(i,j)
	endif
        enddo
        enddo

	ss(1)=rs(1,2)
	ss(2)=rs(2,3)
	ss(3)=rs(3,4)
	ss(4)=rs(1,4)
	s0(1)=r(1,2)
	s0(2)=r(2,3)
	s0(3)=r(3,4)
	s0(4)=r(1,4)
	aa=k(3,4)/r(2,4)+r(1,3)*k(1,2)-k(1,4)*r(1,3)/r(2,4)-k(2,3)
	bb=(r(2,4)-1d0/r(2,4))*(r(1,3)-1d0/r(1,3))
     *		+k(1,2)*k(3,4)-k(1,4)*k(2,3)
	cc=k(1,2)/r(1,3)+r(2,4)*k(3,4)-k(1,4)*r(2,4)/r(1,3)-k(2,3)
	dd=k(2,3)-r(1,3)*k(1,2)-r(2,4)*k(3,4)+r(1,3)*r(2,4)*k(1,4)
	xx(1)=sqe_hdec(aa,bb,cc+ieps*dd)
	xx(2)=(cc+ieps*dd)/aa/xx(1)
	xx0(1)=sqe_hdec(aa,bb,cc)
	xx0(2)=cc/aa/xx0(1)
c	if (abs(dreal(xx0(1)-xx(2))).lt.abs(dreal(xx0(1)-xx(1)))) then
	if (abs(xx0(1)-xx(2)).lt.abs(xx0(1)-xx(1))) then
	  swap  =xx0(1)
	  xx0(1)=xx0(2)
	  xx0(2)=swap
	endif

	do i=1,2
	g(i)  =sign( 1d0,dreal(aa*(xx(i)-xx(3-i))) )
	 x(i,1)= xx(i)/r(2,4)
	x0(i,1)=xx0(i)/r(2,4)
	 x(i,2)= xx(i)/r(2,4)*r(1,3)
	x0(i,2)=xx0(i)/r(2,4)*r(1,3)
	 x(i,3)= xx(i)*r(1,3)
	x0(i,3)=xx0(i)*r(1,3)
	 x(i,4)= xx(i)
	x0(i,4)=xx0(i)
        enddo

	d04_hdec = dcmplx(0d0,0d0)
	do  i=1,2
	do  j=1,4
	a1 = 1d0+x0(i,j)*s0(j) + abs(1d0+x0(i,j)*s0(j))*ieps*
     *				  sign(1d0,dimag(x(i,j)*ss(j)))
	a2 = 1d0+x0(i,j)/s0(j) + abs(1d0+x0(i,j)/s0(j))*ieps*
     *				  sign(1d0,dimag(x(i,j)/ss(j)))
	d04_hdec = d04_hdec + (-1d0)**(i+j)*(
     *		cspen_hdec(a1)+eta_hdec(-x(i,j),ss(j))*log(a1)
     *	       +cspen_hdec(a2)+eta_hdec(-x(i,j),1d0/ss(j))*log(a2))
       enddo
       enddo

	if( dimag(r(1,3)).eq.0d0 ) then
	do  i=1,2
	   a1 = (k(1,3)-2d0*r(1,3))/xx0(i)
     *		      -r(1,3)*k(1,4)+k(3,4)
     	   a2 = ((k(2,4)-2d0*r(2,4))*r(1,3)*xx0(i)
     *		      -r(2,4)*k(3,4)+k(2,3))/dd
	   a3 = (k(1,3)-2d0*r(1,3))*r(2,4)/xx0(i)
     *		      -r(1,3)*k(1,2)+k(2,3)
	   a4 = ((k(2,4)-2d0*r(2,4))*xx0(i)
     *		      -r(2,4)*k(1,4)+k(1,2))/dd
	   l1 = log( a1-abs(a1)*ieps )
     	   l2 = log( a2+abs(a2)*ieps*g(i)*sign(1d0,dreal(r(1,3))
     *				        	  *dimag(rs(2,4))) )
	   l3 = log( a3-abs(a3)*ieps )
	   l4 = log( a4+abs(a4)*ieps*g(i)*sign(1d0,dimag(rs(2,4))) )

	   d04_hdec = d04_hdec + (3d0-2d0*i)*(
     *		 etas_hdec(-xx(i),r(1,3),rs(1,3))
     *		   *( log(r(1,3)*xx(i)) + l1 + l2 )
     *		+etas_hdec(-xx(i),1d0/r(2,4),1d0/rs(2,4))
     *		   *( log(xx(i)/r(2,4)) + l3 + l4 )
     *		-( etas_hdec(-xx(i),r(1,3)/r(2,4),rs(1,3)/rs(2,4))
     *		  +eta_hdec(rs(1,3),1d0/rs(2,4)) )
     *		   *( log(xx(i)*r(1,3)/r(2,4)) + l3 + l2 )
     *	  	+eta_hdec(rs(1,3),1d0/rs(2,4))
     *		  *etas_hdec(-xx(i),-r(1,3)/r(2,4),-rs(1,3)/rs(2,4))   )
        enddo
	else
	do i=1,2
	   l1 = log( r(2,4)/xx0(i)+xx0(i)/r(2,4)+k(1,2)
     *		     -xx0(i)/r(2,4)*eps*bb*g(i) )
	   l2 = log( r(1,3)*xx0(i)+1d0/xx0(i)/r(1,3)+k(3,4)
     *		     -xx0(i)*r(1,3)*eps*bb*g(i) )
	   l3 = log( r(1,3)/r(2,4)*xx0(i)+r(2,4)/xx0(i)/r(1,3)+k(2,3)
     *		     -xx0(i)*r(1,3)/r(2,4)*eps*bb*g(i) )

	   d04_hdec = d04_hdec + (3d0-2d0*i)*(
     *		+eta_hdec(-xx(i),1d0/r(2,4))
     *		   *( log(xx(i)/r(2,4)) + l1 )
     *		+eta_hdec(-xx(i),r(1,3))
     *		   *( log(r(1,3)*xx(i)) + l2 )
     *		-( eta_hdec(-xx(i),r(1,3)/r(2,4))
     *		  +eta_hdec(r(1,3),1d0/r(2,4)) )
     *		   *( log(xx(i)*r(1,3)/r(2,4)) + l3 )
     *	  	+eta_hdec(r(1,3),1d0/r(2,4))
     *		   *eta_hdec(-xx(i),-r(1,3)/r(2,4))
     *		   *(1d0-g(i)*sign(1d0,dreal(bb)))	    )
        enddo
	endif

	d04_hdec = d04_hdec/m(1)/m(2)/m(3)/m(4)/aa/(xx(1)-xx(2))
	return


C--->	***************** SPEZIELL ( --> T.SACK-PROMOTION )
C	D1=Q12-M12
C	D2=Q2 -M22
C	D3=Q3 -M42
C	IF ((D1*D2.LE.0D0).OR.(D2*D3.LE.0D0)) THEN
C	   WRITE(*,*) 'THE CASE OF DIFFERENT SIGNS OF THE D1,D2,D3'
C	   WRITE(*,*) 'IN D04(...) IS NOT IMPLEMENTED !!!'
C	   STOP
C	ENDIF
C	NM1=ABS(MM1/D1)
C	NM2=ABS(MM2/D2)
C	NM3=ABS(MM4/D3)
C	NP1=Q2/D2**2+Q12/D1**2+(Q1-Q2-Q12)/D1/D2
C	NP2=Q2/D2**2+ Q3/D3**2+(Q23-Q2-Q3)/D2/D3
C	NP3=Q3/D3**2+Q12/D1**2+(Q4-Q3-Q12)/D1/D3
C	D04_HDEC=C04(NP1,NP2,NP3,NM1,NM2,NM3)/D1/D2/D3

C	*************** ALLGEMEIN


C	****** ONLY MM3 IS ZERO ******
30	continue
	eps=1d-17
	ieps=dcmplx(0d0,eps)
	m(1)=mm1
	m(2)=mm2
	m(3)=10d0
	m(4)=mm4
	p(1,2)=q1
	p(1,3)=q12
	p(1,4)=q4
	p(2,3)=q2
	p(2,4)=q23
	p(3,4)=q3
	do j=2,4
	do i=1,j-1
	k(i,j)=(m(i)**2+m(j)**2-p(i,j))/m(i)/m(j)
	if (i.eq.3) k(i,j)=k(i,j)-m(i)/m(j)
	if (j.eq.3) k(i,j)=k(i,j)-m(j)/m(i)
	r(i,j) =sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),0d0),
     *	            dcmplx(1d0,0d0))
	if( dimag(r(i,j)).eq.0d0 ) then
	   rs(i,j)=sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),eps),
     *	               dcmplx(1d0,0d0))
	else
	   rs(i,j)=r(i,j)
	endif
        enddo
        enddo
	ss(1)=rs(1,2)
	ss(2)=rs(2,3)
	ss(3)=rs(3,4)
	ss(4)=rs(1,4)
	aa=k(3,4)/r(2,4)-k(2,3)
	bb=k(1,3)*(1d0/r(2,4)-r(2,4))+k(1,2)*k(3,4)-k(1,4)*k(2,3)
	cc=k(1,2)*k(1,3)-k(1,3)*k(1,4)*r(2,4)+r(2,4)*k(3,4)-k(2,3)
	dd=k(2,3)-r(2,4)*k(3,4)
	xx(1)=sqe_hdec(aa,bb,cc+ieps*dd)
	xx(2)=(cc+ieps*dd)/aa/xx(1)
	do i=1,2
	x(i,1)=xx(i)/r(2,4)
	x(i,2)=xx(i)/r(2,4)*r(1,3)
	x(i,3)=xx(i)*r(1,3)
	x(i,4)=xx(i)
        enddo
	d04_hdec = dcmplx(0d0,0d0)
	do i=1,2
	d04_hdec = d04_hdec + (2d0*i-3d0)*(
     *		cspen_hdec(1d0+ss(4)*x(i,4))
     *	       -cspen_hdec(1d0+ss(1)*x(i,1))
     *	       +cspen_hdec(1d0+x(i,4)/ss(4))
     *	       -cspen_hdec(1d0+x(i,1)/ss(1))
     *	       +eta_hdec(-x(i,4),ss(4))*log(1d0+ss(4)*x(i,4))
     *	       -eta_hdec(-x(i,1),ss(1))*log(1d0+ss(1)*x(i,1))
     *	       +eta_hdec(-x(i,4),1d0/ss(4))*log(1d0+x(i,4)/ss(4))
     *	       -eta_hdec(-x(i,1),1d0/ss(1))*log(1d0+x(i,1)/ss(1))
     *	       -cspen_hdec(1d0+x(i,4)*(k(3,4)-ieps)/(k(1,3)-ieps))
     *	       +cspen_hdec(1d0+x(i,1)*(k(2,3)-ieps)/(k(1,3)-ieps))
     *	       -eta_hdec(-x(i,4),(k(3,4)-ieps)/(k(1,3)-ieps))
     *	           *log(1d0+x(i,4)*(k(3,4)-ieps)/(k(1,3)-ieps))
     *	       +eta_hdec(-x(i,1),(k(2,3)-ieps)/(k(1,3)-ieps))
     *	           *log(1d0+x(i,1)*(k(2,3)-ieps)/(k(1,3)-ieps))   )
	if (dimag(r(2,4)).ne.0d0) then
	   h=eta_hdec(-1d0/xx(i),r(2,4))
	else
	   h=dcmplx(0d0,0d0)
	   if (dreal(r(2,4)).lt.0d0) then
	      hh=-1d0/xx(i)
	      im1=dimag(hh)
	      im2=dimag(rs(2,4))
	      if ((im1.gt.0d0).and.(im2.gt.0d0)) then
	         h=-dcmplx(0d0,2d0*pi)
	      endif
	      if ((im1.lt.0d0).and.(im2.lt.0d0)) then
	         h=+dcmplx(0d0,2d0*pi)
	      endif
	   endif
	endif
	d04_hdec = d04_hdec + (2d0*i-3d0)*
     *	          h*( log( (k(1,2)-r(2,4)*k(1,4)
     *			  +xx(i)*(1d0/r(2,4)-r(2,4)))/dd )
     *		     +log(k(1,3)-ieps) )
        enddo
	d04_hdec = d04_hdec/m(1)/m(2)/m(3)/m(4)/aa/(xx(1)-xx(2))
	return

c	****** only mm2 and mm3 are zero ******
40	continue
	eps=1d-17
	ieps=dcmplx(0d0,eps)

	m(1)=mm1
	m(2)=10d0
	m(3)=10d0
	m(4)=mm4
	p(1,2)=q1
	p(1,3)=q12
	p(1,4)=q4
	p(2,3)=q2
	p(2,4)=q23
	p(3,4)=q3
	do j=2,4
	do i=1,j-1
	k(i,j)=(m(i)**2+m(j)**2-p(i,j))/m(i)/m(j)
	if (i.eq.2) k(i,j)=k(i,j)-m(i)/m(j)
	if (j.eq.2) k(i,j)=k(i,j)-m(j)/m(i)
	if (i.eq.3) k(i,j)=k(i,j)-m(i)/m(j)
	if (j.eq.3) k(i,j)=k(i,j)-m(j)/m(i)
	r(i,j) =sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),0d0),
     *	            dcmplx(1d0,0d0))
	if( dimag(r(i,j)).eq.0d0 ) then
	   rs(i,j)=sqe_hdec(dcmplx(1d0,0d0),dcmplx(-k(i,j),eps),
     *	               dcmplx(1d0,0d0))
	else
	   rs(i,j)=r(i,j)
	endif
        enddo
        enddo
	ss(1)=rs(1,2)
	ss(2)=rs(2,3)
	ss(3)=rs(3,4)
	ss(4)=rs(1,4)
	aa=k(2,4)*k(3,4)-k(2,3)
	bb=k(1,3)*k(2,4)+k(1,2)*k(3,4)-k(1,4)*k(2,3)
	cc=k(1,2)*k(1,3)-k(2,3)
	dd=k(2,3)
	xx(1)=sqe_hdec(aa,bb,cc+ieps*dd)
	xx(2)=(cc+ieps*dd)/aa/xx(1)
	do  i=1,2
	x(i,1)=xx(i)/r(2,4)
	x(i,2)=xx(i)/r(2,4)*r(1,3)
	x(i,3)=xx(i)*r(1,3)
	x(i,4)=xx(i)
        enddo
	d04_hdec = dcmplx(0d0,0d0)
	do i=1,2
	d04_hdec = d04_hdec + (2d0*i-3d0)*(
     *		cspen_hdec(1d0+ss(4)*x(i,4))
     *	       +cspen_hdec(1d0+x(i,4)/ss(4))
     *	       +eta_hdec(-x(i,4),ss(4))*log(1d0+ss(4)*x(i,4))
     *	       +eta_hdec(-x(i,4),1d0/ss(4))*log(1d0+x(i,4)/ss(4))
     *	       -cspen_hdec(1d0+xx(i)*(k(3,4)-ieps)/(k(1,3)-ieps))
     *	       -cspen_hdec(1d0+xx(i)*(k(2,4)-ieps)/(k(1,2)-ieps))
     *	       -eta_hdec(-xx(i),(k(3,4)-ieps)/(k(1,3)-ieps))
     *	           *log(1d0+xx(i)*(k(3,4)-ieps)/(k(1,3)-ieps))
     *	       -eta_hdec(-xx(i),(k(2,4)-ieps)/(k(1,2)-ieps))
     *	           *log(1d0+xx(i)*(k(2,4)-ieps)/(k(1,2)-ieps))
     *	       +log(-xx(i))*( log(k(1,2)-ieps)
     *			     +log(k(1,3)-ieps)-log(k(2,3)-ieps) ) )
        enddo
	d04_hdec = d04_hdec/m(1)/m(2)/m(3)/m(4)/aa/(xx(1)-xx(2))

	return

	end

************************************************************************
        function c03_hdec(p1,p2,p3,m1,m2,m3)
************************************************************************
*  SCALAR 3-POINT FUNCTION                                             *
*  P1,P2,P3 = SQUARED EXTERNAL MOMENTA  			       *
*----------------------------------------------------------------------*
*  5.12.96  M. SPIRA    					       *
************************************************************************
      implicit real*8 (a-h,o-z)
      real*8 m1,m2,m3
      real*8 r(0:2)
      complex*16 c03_hdec,cspen_hdec,eta_hdec,ieps,im
      complex*16 alp(0:2),x(0:2,2),y0(0:2),y(0:2,2)
      complex*16 cdum
c     real*8 kappa
      complex*16 kappa
c     kappa(a,b,c) = dsqrt(a**2+b**2+c**2-2*(a*b+a*c+b*c))
c     kappa(a,b,c) = dsqrt(dabs(a**2+b**2+c**2-2*(a*b+a*c+b*c)))
c     kappa(a,b,c) = cdsqrt(dcmplx(a**2+b**2+c**2-2*(a*b+a*c+b*c)))
      kappa(a,b,c,d) = cdsqrt((a**2+b**2+c**2-2*(a*b+a*c+b*c))
     .               * (1+ieps*d))
      eps = 1.d-8*(p1+p2+p3)
      im = dcmplx(0.d0,1.d0)
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      ieps = dcmplx(0.d0,1.d-17)
c     ieps = dcmplx(0.d0,1.d-20)
      pi = 4*datan(1.d0)
      xx = 0.d0
c     if(p1.lt.0.d0.or.p2.lt.0.d0.or.p3.lt.0.d0) xx=1.d0
      if(p1.ne.0.d0.or.xx.ne.0.d0)then
       q10 = p1
      else
       q10 = eps
      endif
      if(p3.ne.0.d0.or.xx.ne.0.d0)then
       q20 = p3
      else
       q20 = eps
      endif
      if(p2.ne.0.d0.or.xx.ne.0.d0)then
       q21 = p2
      else
       q21 = eps
      endif
      r(0) = p2
      r(1) = p3
      r(2) = p1
      sm0 = m1**2
      sm1 = m2**2
      sm2 = m3**2
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
c     alpha  = kappa(q10,q21,q20)
c     alp(0) = kappa(q21,sm1,sm2)*(1+ieps*q21)
c     alp(1) = kappa(q20,sm2,sm0)*(1+ieps*q20)
c     alp(2) = kappa(q10,sm0,sm1)*(1+ieps*q10)
      alpha  = kappa(q10,q21,q20,1.d0)
      alp(0) = kappa(q21,sm1,sm2,dsign(1.d0,q21))
      alp(1) = kappa(q20,sm2,sm0,dsign(1.d0,q20))
      alp(2) = kappa(q10,sm0,sm1,dsign(1.d0,q10))
      x(0,1) = (q21 - sm1 + sm2 + alp(0))/2/q21
      x(0,2) = (q21 - sm1 + sm2 - alp(0))/2/q21
      x(1,1) = (q20 - sm2 + sm0 + alp(1))/2/q20
      x(1,2) = (q20 - sm2 + sm0 - alp(1))/2/q20
      x(2,1) = (q10 - sm0 + sm1 + alp(2))/2/q10
      x(2,2) = (q10 - sm0 + sm1 - alp(2))/2/q10
      y0(0) = (q21*(q21-q20-q10+2*sm0-sm1-sm2) - (q20-q10)*(sm1-sm2)
     .      + alpha*(q21-sm1+sm2))/2/alpha/q21
      y0(1) = (q20*(q20-q10-q21+2*sm1-sm2-sm0) - (q10-q21)*(sm2-sm0)
     .      + alpha*(q20-sm2+sm0))/2/alpha/q20
      y0(2) = (q10*(q10-q21-q20+2*sm2-sm0-sm1) - (q21-q20)*(sm0-sm1)
     .      + alpha*(q10-sm0+sm1))/2/alpha/q10
      y(0,1) = y0(0) - x(0,1)
      y(0,2) = y0(0) - x(0,2)
      y(1,1) = y0(1) - x(1,1)
      y(1,2) = y0(1) - x(1,2)
      y(2,1) = y0(2) - x(2,1)
      y(2,2) = y0(2) - x(2,2)
      cdum=0.d0
      do i=0,2
       do j=1,2
        cdum = cdum + cspen_hdec((y0(i)-1)/y(i,j))
     .              - cspen_hdec(y0(i)/y(i,j))
        cx = eta_hdec(1-x(i,j),1/y(i,j))
        if(cx.ne.0.d0)then
         cdum = cdum + cx*cdlog((y0(i)-1)/y(i,j))
        endif
        cy = eta_hdec(-x(i,j),1/y(i,j))
        if(cy.ne.0.d0)then
         cdum = cdum - cy*cdlog(y0(i)/y(i,j))
        endif
       enddo
       cx = eta_hdec(-x(i,1),-x(i,2))
       if(cx.ne.0.d0)then
        cdum = cdum - cx*cdlog((1-y0(i))/(-y0(i)))
       endif
       cy = eta_hdec(y(i,1),y(i,2))
       if(cy.ne.0.d0)then
        cdum = cdum + cy*cdlog((1-y0(i))/(-y0(i)))
       endif
       a = -r(i)
       b = -dimag(y(i,1)*y(i,2))
       if(a.gt.0.d0.and.b.gt.0.d0) then
        cdum = cdum + 2*pi*im*cdlog((1-y0(i))/(-y0(i)))
       endif
      enddo
      c03_hdec = cdum/alpha
      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C        SUBROUTINE CALCULATING THE FINITE REAL PART OF THE           C
C          GENERAL MASSIVE TWO POINT FUNCTION                         C
C                                                                     C
C           B02(P.P,M1,M2,MU**2)                                      C
C           BP02(P.P,M1,M2,MU**2)                                     C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c ---------------------------------------------------------------------
      real*8 function B02_HDEC(s,m1,m2,mu2)

      implicit none 

      real*8     s,m1,m2,mu2,m12,m22 
      complex*16 zkappa,x1,x2 

      m12 = m1**2 
      m22 = m2**2 

      zkappa=cdsqrt(dcmplx(s**2+m12**2+m22**2
     &                     -2.D0*(s*m12+s*m22+m12*m22)))

      if (s.eq.0.D0) then
         if (m12.eq.m22) then
            B02_HDEC=-dlog(m12/mu2) 
         else
          if (m22.eq.0.D0) then
           if (m12.eq.0.D0) then
            B02_HDEC=1.D0
           else
            B02_HDEC=1.D0 - m12/(m12-m22)*dlog(m12/mu2)
           endif
          else
           if (m12.eq.0.D0) then
            B02_HDEC=1.D0 + m22/(m12-m22)*dlog(m22/mu2) 
           else
            B02_HDEC=1.D0 - m12/(m12-m22)*dlog(m12/mu2)
     &                 + m22/(m12-m22)*dlog(m22/mu2) 
           endif
          endif
         endif
      else 
         if ((m12.eq.0.D0).and.(m22.eq.0.D0)) then 
            B02_HDEC=2.D0 - dlog(s/mu2)
         elseif ((m12.eq.s).and.(m22.eq.0.D0)) then 
            B02_HDEC=2.D0 - dlog(m12/mu2)
         elseif ((m22.eq.s).and.(m12.eq.0.D0)) then 
            B02_HDEC=2.D0 - dlog(m22/mu2)
         elseif (m12.eq.0.D0) then
            B02_HDEC=2.D0 - (s-m22)/s*dlog( dabs(m22-s)/m22 )
     &                 - dlog(m22/mu2)
         elseif (m22.eq.0.D0) then
            B02_HDEC=2.D0 - (s-m12)/s*dlog( dabs(m12-s)/m12 ) 
     &                 - dlog(m12/mu2)
         else
            x1=dcmplx( (s-m22+m12+zkappa)/(2.D0*s) )
            x2=dcmplx( (s-m22+m12-zkappa)/(2.D0*s) )
            B02_HDEC=dreal( 2.D0+ dlog(mu2/m22) 
     &                       + x1*cdlog(1.D0-1.D0/x1) 
     &                       + x2*cdlog(1.D0-1.D0/x2))
         endif
      endif 

      return
      end



c ---------------------------------------------------------------------
      real*8 function BP02_HDEC(s,m1,m2,mu2)
      
      implicit none 

      real*8     s,m1,m2,mu2,m12,m22 
      complex*16 zkappa,x1,x2
      
      m12 = m1**2
      m22 = m2**2 

      zkappa=cdsqrt(dcmplx(s**2+m12**2+m22**2
     &                    -2.D0*(s*m12+s*m22+m12*m22)))

      if (s.eq.0.D0) then
         if (m12.eq.m22) then
            BP02_HDEC=1.D0/(6.D0*m12)
         else
            BP02_HDEC=( (m12+m22)/2.D0 
     &        - m12*m22/(m12-m22)*dlog(m12/m22) )/(m12-m22)**2 
         endif
      elseif ((s.eq.m12).and.(m22.eq.0.D0)) then 
         BP02_HDEC=( -1.D0 + dlog(m12/mu2)/2.D0 )/m12
      elseif ((s.eq.m22).and.(m12.eq.0.D0)) then 
         BP02_HDEC=( -1.D0 + dlog(m22/mu2)/2.D0 )/m22
      else 
         x1=dcmplx( (s-m22+m12+zkappa)/(2.D0*s) )
         x2=dcmplx( (s-m22+m12-zkappa)/(2.D0*s) )
         BP02_HDEC=dreal( -1.D0 + ( x1*(1.D0-x1)*cdlog(1.D0-1.D0/x1)
     &                     - x2*(1.D0-x2)*cdlog(1.D0-1.D0/x2) )  
     &                                                  /(x1-x2) )/s
      endif 

      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
        function cspen_hdec(z)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       SPENCE-FUNKTION KOMPLEX, FREI NACH HOLLIK                     C
C---------------------------------------------------------------------C
C       20.07.83    LAST CHANGED 10.05.89        ANSGAR DENNER        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
        complex*16 cspen_hdec,w,sum,z,u
        real*8 rz,az,a1
        real*8 b(9)/
     1   0.1666666666666666666666666667d0,
     2  -0.0333333333333333333333333333d0,
     3   0.0238095238095238095238095238d0,
     4  -0.0333333333333333333333333333d0,
     5   0.0757575757575757575757575758d0,
     6  -0.2531135531135531135531135531d0,
     7   1.1666666666666666666666666667d0,
     8  -7.09215686274509804d0         ,
     9  54.97117794486215539d0         /
C     BEACHTE:                 B(N)=B2N
C     B(1)=1./6.
C     B(2)=-1./30.
C     B(3)=1./42.
C     B(4)=-1./30.
C     B(5)=5./66.
C     B(6)=-691./2730.
C     B(7)=7./6.
C     B(8)=-3617./510.
C     B(9)=43867./798.
C     B(10)=-174611./330.
C     B(11)=854513./138.
C     PI=3.1415926535897932384
C     PI*PI/6.=1.6449..., PI*PI/3=3.28986...
C
c      write(*,*) 'z:',z
      z =z*dcmplx(1d0)
      rz=dreal(z)
      az=cdabs(z)
      a1=cdabs(1d0-z)
c      write(*,*)'z, rz, az, a1:',z,rz,az,a1
C     IF((SNGL(RZ) .EQ. 0.0) .AND. (SNGL(DIMAG(Z)) .EQ. 0.0)) THEN
C ---> CHANGED  10.5.89
      if(az .lt. 1d-20) then
        cspen_hdec=-cdlog(1d0-z)
c        write(*,*) 'cspen:', cspen_hdec
        return
      end if
      if((sngl(rz) .eq. 1.0) .and. (sngl(dimag(z)) .eq. 0.0)) then
        cspen_hdec=1.64493406684822643d0
c        write(*,*) 'cspen:', cspen_hdec
        return
      end if
      if(rz.gt.5d-1) goto 20
      if(az.gt.1d0) goto 10
      w=-cdlog(1d0-z)
      sum=w-0.25d0*w*w
      u=w
      if(cdabs(u).lt.1d-10) goto 2
c      write(*,*) 'u:',u
c      write(*,*) 'sum:',sum
      do 1 k=1,9
      u=u*w*w/dfloat(2*k*(2*k+1))
      if(cdabs(u*b(k)/sum).lt.1d-20) goto 2
      sum=sum+u*b(k)
 1    continue
 2    cspen_hdec=sum
c        write(*,*) 'cspen:', cspen_hdec
      return
10    w=-cdlog(1d0-1d0/z)
      sum=w-0.25d0*w*w
      u=w
      if(cdabs(u).lt.1d-10) goto 12

      do 11 k=1,9
      u=u*w*w/dfloat(2*k*(2*k+1))
      if(cdabs(b(k)*u/sum).lt.1d-20) goto 12
      sum=sum+u*b(k)
11    continue
12    cspen_hdec=-sum-1.64493406684822643d0-.5d0*cdlog(-z)**2
c        write(*,*) 'cspen:', cspen_hdec
      return
20    if(a1.gt.1d0) goto 30
      w=-cdlog(z)
      sum=w-0.25d0*w*w
      u=w
      if(cdabs(u).lt.1d-10) goto 22
      do 21 k=1,9
      u=u*w*w/dfloat(2*k*(2*k+1))
      if(cdabs(u*b(k)/sum).lt.1d-20) goto 22
      sum=sum+u*b(k)
21    continue
22    cspen_hdec=-sum+1.64493406684822643d0-cdlog(z)*cdlog(1d0-z)
c        write(*,*) 'cspen:', cspen_hdec
      return
30    w=cdlog(1d0-1d0/z)
      sum=w-0.25d0*w*w
      u=w
      if(cdabs(u).lt.1d-10) goto 32
      do 31 k=1,9
      u=u*w*w/dfloat(2*k*(2*k+1))
      if(cdabs(u*b(k)/sum).lt.1d-20) goto 32
      sum=sum+u*b(k)
31    continue
32    cspen_hdec=sum+3.28986813369645287d0
     *               +.5d0*cdlog(z-1d0)**2-cdlog(z)*cdlog(1d0-z)
50    continue
c        write(*,*) 'cspen:', cspen_hdec
      end

c -------------------------------------------------------------------- c

      function t_hdec(x,y,z)
      implicit real*8(a-h,l,m,o-z)
      if(x.eq.y) x = x - 0.00001
      if(x.eq.z) x = x - 0.00002
      if(y.eq.z) y = y - 0.00003
c       write(*,*) 'xyz',x,y,z
      t_hdec = (x**2*y**2*log(x**2/y**2) + x**2*z**2*log(z**2/x**2)
     * + y**2*z**2*log(y**2/z**2))/((x**2-y**2)*(y**2-z**2)*(x**2-z**2))
      return
      end

